/*-----------------------------------------------------------------------------
 * Dogen -- the Data Object GENerator
 * Copyright 1999 Mindgarden.com, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *
 *      This product includes Dogen software developed by Mindgarden.com, Inc.
 *
 * 4. The name of Mindgarden.com, Inc. may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MINDGARDEN.COM, INC. ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL MINDGARDEN.COM BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: MetaAttribute.java,v 1.1.1.1 1999/09/23 13:42:20 andy Exp $
 *-----------------------------------------------------------------------------
 */

package dogen.generator;

import java.io.*;
import dogen.types.*;

public class MetaAttribute {
    protected String name;
    protected MetaType metaType;
    protected boolean isKey;
    protected boolean canBeNull;
    protected boolean isUnique;
    protected boolean noAccessMethod;
    protected MetaAttribute foreignAttribute;
    protected MetaClass foreignClass;
    protected int size;

    public MetaAttribute(String name) {
	this.name = name;
	this.metaType = null;
	this.isKey = false;
	this.canBeNull = true;
	this.isUnique = false;
	this.foreignAttribute = null;
	this.foreignClass = null;
	this.size = -1;
	this.noAccessMethod = false;
    }

    public String getName() {
	return name;
    }

    public void setName(String name) {
	this.name = name;
    }

    public MetaType getMetaType() {
	return metaType;
    }

    public void setMetaType(MetaType type) {
	metaType = type;
    }

    public String getSqlType() {
	return metaType.getSqlType(this);
    }

    public boolean isKey() {
	return isKey;
    }

    public void setIsKey(boolean value) {
	isKey = value;
    }

    public boolean canBeNull() {
	return canBeNull;
    }

    public void setCanBeNull(boolean value) {
	canBeNull = value;
    }

    public MetaAttribute getForeignAttribute() {
	return foreignAttribute;
    }

    public void setForeignAttribute(MetaAttribute attr) {
	foreignAttribute = attr;
    }

    public MetaClass getForeignClass() {
	return foreignClass;
    }

    public void setForeignClass(MetaClass c) {
	foreignClass = c;
    }

    public void setForeignKey(MetaAttribute attr, MetaClass c) {
	foreignClass = c;
	foreignAttribute = attr;
    }

    public boolean isForeignKey() {
	return (foreignClass != null) && (foreignAttribute != null);
    }

    public boolean isUnique() {
	return isUnique;
    }

    public void setIsUnique(boolean value) {
	isUnique = value;
    }

    public void emitSqlCode(CodeWriter s) {
	s.print(name + " " + metaType.getSqlType(this));
	
	if(isUnique()) {
	    s.print(" UNIQUE");
	}

	if(isKey() || !canBeNull()) {
	    s.print(" NOT NULL");
	}

	if(isKey()) {
	    s.print(", PRIMARY KEY(" + name + ")");
	}
    }

    public boolean hasConstraints() {
	return isForeignKey();
    }

    public String getConstraintSqlCode() {
	if(isForeignKey()) {
	    return "ADD CONSTRAINT " + name + "REF " +
		   "FOREIGN KEY(" + name + ") " +
		   "REFERENCES " + foreignClass.getName() +
		       "(" + foreignAttribute.getName() + ")";
	} else {
	    return null;
	}
    }

    public void emitJavaDeclaration(CodeWriter cw) {
	if(noAccessMethod == false) {
	    cw.println("    protected " +
		       metaType.getJavaType(this) + " " + name + ";");
	}
    }

    protected String upcaseName() {
	return Character.toUpperCase(name.charAt(0)) + name.substring(1);
    }

    public void emitJavaGetMethod(CodeWriter cw) {
	if(noAccessMethod == false) {
	    cw.println("    public " + metaType.getJavaType(this) +
		              " get" + upcaseName() + "()");
	    cw.println("        throws DataObjectException");
	    cw.println("    {");
	    cw.println("        validateAttributes();");
	    cw.println("        return " + name + ";");
	    cw.println("    }");
	}
    }

    public void emitJavaSetMethod(CodeWriter cw) {
	if(noAccessMethod == false) {
	    cw.println("    public void set" + upcaseName() +
		       "(" + metaType.getJavaType(this) + " value) " +
		       "throws DataObjectException {");
	    cw.println("        validateAttributes();");

	    if(metaType.isString()) {
		String max = Integer.toString(size);

		cw.println("        if(value.length() >= " + max + ") {");
		cw.println("            this." + name + " = value.substring(0, "
			                                    + max + ");");
		cw.println("        } else {");
		cw.println("            this." + name + " = value;");
		cw.println("        }");
	    } else {
		cw.println("        this." + name + " = value;");
	    }

	    cw.println("        dirtyAttributes();");
	    cw.println("    }");
	}
    }

    public void emitUnpackCode(CodeWriter cw,
			       String resultSet,
			       String offset)
    {
	if(noAccessMethod == false) {
	    String result;

	    if(metaType.isReference()) {
		result = resultSet + "." + metaType.getUnpackMethod(this) +
		    "(" + offset + ")";

		result = "toOID(" + result + ")";
	    } else {
		result = resultSet + "." + metaType.getUnpackMethod(this) +
		    "(" + offset + ")";
	    }

	    cw.print(name + " = ");

	    if(isForeignKey()) {
		String c = foreignClass.getFullyQualifiedName();

		cw.println("(" + c + ") " + c + ".lookup(" + result + ");");
	    } else {
		cw.println(result + ";");
	    }
	}
    }

    public void emitPackCode(CodeWriter cw, String offset, String value) {
        if(metaType.isReference()) {
	    cw.print(metaType.getPackMethod(this) +
		     "(" + offset + ", " +
		     "DataObject.getOIDAsLong(" + value + "))");
	} else {
	    cw.print(metaType.getPackMethod(this) +
		     "(" + offset + ", " + value + ")");
	}
    }

    public int getSize() {
	return size;
    }

    public void setSize(int value) {
	size = value;
    }

    public boolean getNoAccessMethod() {
	return noAccessMethod;
    }

    public void setNoAccessMethod(boolean value) {
	noAccessMethod = value;
    }
}

