/*-----------------------------------------------------------------------------
 * Dogen -- the Data Object GENerator
 * Copyright 1999 Mindgarden.com, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *
 *      This product includes Dogen software developed by Mindgarden.com, Inc.
 *
 * 4. The name of Mindgarden.com, Inc. may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MINDGARDEN.COM, INC. ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL MINDGARDEN.COM BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: MetaClass.java,v 1.1.1.1 1999/09/23 13:42:20 andy Exp $
 *-----------------------------------------------------------------------------
 */

package dogen.generator;

import dogen.types.*;

import java.io.*;
import java.util.*;
import java.lang.reflect.*;

/**
 * Used to represent a class of persistent objects for which a Java
 * implementation will be generated. 
 * @author Andy Turk (andy_turk@hotmail.com)
 */
public class MetaClass extends CodeGenerator {
    /*
     * A table of all the MetaClass instances known to the code generator
     */
    protected static Hashtable metaClasses;

    /*
     * The java.lang.Object class
     */
    protected static Class nullDeclarationClass;

    /*
     * An instance of the root meta type class
     */
    protected static Class typeClassRoot;

    /*
     * The T_Reference class
     */
    protected static Class referenceClass;

    /*
     * A table which maps native Java types to MetaTypes.
     */
    protected static Hashtable nativeTypes;

    /*
     * The declaration which "generates" this MetaClass
     */
    protected Object declaration;

    /*
     * A table of all the attributes in this class
     */
    protected Hashtable attributesByName;

    /*
     * The name of the cache variable in the generated class
     */
    protected String cacheName;

    /*
     * The MetaClass corresponding to the superclass of the declaration
     * (if there is one)
     */
    protected MetaClass superClass;

    /*
     * The Schema this class belongs to
     */
    protected MetaSchema metaSchema;

    /*
     * A vector of attributes
     */
    protected Vector attributes;

    /*
     * The name of the superclass for objects of this MetaClass
     * at runtime. The default is null which means that the generated
     * superclass will be used (if necessary). Set this by adding a
     * "$superclassclass" modifier to a class declaration. 
     */
    protected String runtimeSuperClass;

    /*
     * Whether or not code will be generated for this MetaClass. Code
     * shouldn't be generated for MetaClasses which
     * are only present for "housekeeping" purposes. Set this by adding
     * a "$suppressCode" modifier to a class declaration.
     */
    protected boolean suppressCode = false;

    /*
     * This string contains some SQL code that will be executed after all
     * the tables have been created. Use the $postCreateSQL directive to
     * set it.
     */
    protected String postCreateSQL = null;

    /*
     * Whether or not this class should be abstract when generated. Classes
     * are abstract if they aren't explicitly declared in a schema declaration.
     * When this occurs, the generated code won't contain any SQL methods.
     */
    protected boolean isAbstract = true;

    /*
     * Dumps the names of all the know MetaClass objects to stdout. Used
     * for debugging purposes only.
     */
    protected static void dumpMetaClasses() {
	for(Enumeration e = metaClasses.keys(); e.hasMoreElements(); ) {
	    System.out.println(((String) e.nextElement()) +
			       " is a known Declaration");
	}
    }

    /*
     * Prints a debugging message on stdout
     */
    protected static void debug(String msg) {
	System.out.println("MetaClass: " + msg);
    }

    /*
     * This code block initializes all the shared static variables.
     */
    static {
	metaClasses = new Hashtable();

	try {
	    nullDeclarationClass = Class.forName("java.lang.Object");
	    typeClassRoot = Class.forName("dogen.generator.MetaType");
	    referenceClass = Class.forName("dogen.types.T_Reference");

	    setTypeMapping(new DefaultTypeMapping());
	} catch (Exception e) {
	    debug(e.getMessage());
	    throw new Error(e.toString());
	}
    }

    /**
     * Creates a new MetaClass instance. This method
     * is protected because we only want one instance of a particular
     * MetaClass to be kept track of. These instances are generated by the
     * <code>forName</code> method.
     *
     * @param decl the declaration to convert to a MetaClass
     * @exception MetaException if the superclass of the declaration can't be
     *                       converted to a MetaClass
     */
    protected MetaClass(Object decl) throws MetaException {
	super(decl.getClass().getName());

	declaration = decl;
	Class classDeclClass = declaration.getClass();

	metaSchema = MetaSchema.getCurrentSchema();
	attributes = new Vector();
	attributesByName = new Hashtable();

	/*
	 * Add this MetaClass to the table right away so that it's
	 * available if we need it during the processing of any attributes
	 * and/or superclasses.
	 */

	metaClasses.put(classDeclClass.getName(), this);

	Class classDeclSuper = classDeclClass.getSuperclass();
	if((classDeclSuper != null) &&
	   (classDeclSuper != nullDeclarationClass)) 
	{
	    String superName = classDeclClass.getSuperclass().getName();
	    superClass = MetaClass.forName(superName);
	} else {
	    superClass = null;
	}

	cacheName = name + "$cache";

	Field[] fieldArray = classDeclClass.getDeclaredFields();

	extractClassModifiers(fieldArray);
	extractAttributes(fieldArray);
	extractAttributeModifiers(fieldArray);
    }

    /**
     * Returns an enumeration of all the MetaClass objects.
     * @return an enumeration
     */
    public static Enumeration getMetaClasses() {
	return metaClasses.elements();
    }

    /**
     * Sets whether or not this MetaClass will generate an abstract
     * class.
     * @param value false if the MetaClass is not abstract
     */
    public void setAbstract(boolean value) {
	isAbstract = value;
    }

    /**
     * Returns whether or not this MetaClass will generate an abstract
     * class. The default is true.
     * @return true if this class is abstract
     */
    public boolean getAbstract() {
	return isAbstract;
    }
    
    /**
     * Returns the superclass (i.e., a <code>MetaClass</code>) of this
     * <code>MetaClass</code>, if any.
     * @return the superclass or <code>null</code> if there isn't one
     */
    public MetaClass getSuperClass() {
	return superClass;
    }

    /**
     * Returns a string of SQL code that should be executed after all tables
     * have been created.
     * @return a string containing SQL or <code>null</code>
     */
    public String getPostCreateSQL() {
	return postCreateSQL;
    }

    /**
     * Defines a function which maps native types like <code>int</code> and
     * <code>boolean</code> to instances of MetaType. The fields of the
     * passed object are paired sequentially. The type of the first element
     * of each pair is the native type that appears in a DECL class. The
     * type of the second element is the MetaType. Since the fields are
     * paired, the number of fields should always be even.
     * @see dogen.types.DefaultTypeMapping
     * @param map an object with an even number of fields
     */
    public static void setTypeMapping(Object map)
	throws InstantiationException, IllegalAccessException
    {
	Hashtable table = new Hashtable();

	setTypeMapping(map.getClass(), table);
	nativeTypes = table;
    }

    protected static void setTypeMapping(Class mapClass, Hashtable table)
	throws InstantiationException, IllegalAccessException
    {
	if(mapClass.getSuperclass() != null) {
	    setTypeMapping(mapClass.getSuperclass(), table);
	}

	Field[] mapFields = mapClass.getDeclaredFields();

	for(int i = 0; i < mapFields.length; i += 2) {
	    Class nativeType = mapFields[i].getType();
	    Class metaType = mapFields[i + 1].getType();

	    table.put(nativeType, metaType.newInstance());
	}
    }

    /**
     * Returns whether or not code generation should be suppressed for
     * this MetaClass.
     * @return a boolean
     */
    public boolean getSuppressCode() {
	return suppressCode;
    }

    /**
     * Returns whether this object is the root of the <code>MetaClass</code>
     * tree.
     * @return true if this is the root
     * @see #getSuperClass
     */
    public boolean isRoot() {
	return (superClass == null);
    }

    /**
     * Returns whether this object can benefit from adding constraints to the
     * SQL definition of its tables. If this method returns true, then at
     * least one attribute can benefit from a constraint.
     * @return true if at least on attribute can benefit from a constraint
     * @see dogen.generator.MetaAttribute.hasConstraints
     */
    public boolean hasSqlConstraints() {
	for(Enumeration e = attributes.elements(); e.hasMoreElements(); ) {
	    MetaAttribute attr = (MetaAttribute) e.nextElement();

	    if(attr.hasConstraints()) {
		return true;
	    }
	}

	return false;
    }

    /**
     * Searches through the attributes for this declaration and (recursively)
     * the superclass looking for an attribute which is declared as being
     * key. We assume that the key is a single attribute. 
     *
     * @return the key attribute or <code>null</code> if there isn't one
     */
    public MetaAttribute getIDAttribute() {
	MetaAttribute retval = null;

	for(Enumeration e = attributes.elements(); e.hasMoreElements(); ) {
	    MetaAttribute attr = (MetaAttribute) e.nextElement();

	    if(attr.isKey()) {
		retval = attr;
		break;
	    }
	}

	if(retval != null) {
	    return retval;
	} else if(superClass != null) {
	    return superClass.getIDAttribute();
	} else {
	    return null;
	}
    }

    /**
     * Returns the name of the nearest ancestor class that wasn't generated.
     * This is used as a supertype that all generated classes belong to.
     *
     * @return the name of the nearest shared ancestor class
     */
    public String getSharedAncestorClass() {
	if(runtimeSuperClass != null) {
	    return runtimeSuperClass;
	} else if(superClass != null) {
	    return superClass.getSharedAncestorClass();
	} else {
	    return super.getFullyQualifiedName();
	}
    }

    /**
     * Returns the name defined in the $class modifier, if any.
     *
     * @return a string
     */
    protected String getruntimeSuperClass() {
	return runtimeSuperClass;
    }

    /**
     * Returns the instance of <code>MetaClass</code> which corresponds
     * to a declaration <code>Class</code> .
     * @param c a declaration <code>Class</code>
     * @exception MetaException 
     * @return the <code>MetaClass</code>
     */
    public static MetaClass forDeclaration(Class c) throws MetaException {
	return forName(c.getName());

	//	if(nullDeclarationClass.isAssignableFrom(c)) {
	//	    return forName(c.getName());
	//	} else {
	//	    throw error(c.getName() + " is not a valid declaration");
	//	}
    }

    /**
     * Returns true if the specified class has a corresponding MetaClass.
     * @param c a Class
     * @return true of the class already has a MetaClass
     */
    public static boolean hasMetaClass(Class c) {
	return metaClasses.containsKey(c.getName());
    }

    /**
     * Returns the instance of <code>MetaClass</code> which corresponds
     * to an instance of a declaration
     * @param d a declaration
     * @return the <code>MetaClass</code>
     */
    public static MetaClass forDeclaration(Object d)
	throws MetaException
    {
	return forName(d.getClass().getName());
    }

    /**
     * Returns the instance of <code>MetaClass</code> which corresponds
     * to a declaration
     * @param name the fully qualified name of the declaration class
     * @exception MetaException if the argument isn't the name of a
     *            declaration class
     * @return the <code>MetaClass</code>
     */
    public static MetaClass forName(String name) throws MetaException {
	if(metaClasses.containsKey(name)) {
	    return (MetaClass) metaClasses.get(name);
	} else {
	    try {
		Class classDeclClass = Class.forName(name);
		return new MetaClass(classDeclClass.newInstance());
	    } catch (ClassNotFoundException e) {
		throw error(e.getMessage());
	    } catch (IllegalAccessException e) {
		throw error(e.getMessage());
	    } catch (InstantiationException e) {
		throw error(e.getMessage());
	    }
	}
    }

    /**
     * Iterates over an array of fields and processes any which are
     * meant to describe or modify the generation of the
     * <code>MetaClass</code>. 
     * Any fields which are class modifiers are nulled out in the
     * array so that future passes over the same array
     * won't see them.
     * @param fieldArray an array of fields
     * @exception MetaException if one of the fields is a class modifier
     *            but can't be accessed due to a protection violation.
     */
    protected void extractClassModifiers(Field[] fieldArray) 
	throws MetaException
    {
	for(int i = 0; i < fieldArray.length; ++i) {
	    Field field = fieldArray[i];

	    if(field != null) {
		String fieldName = fieldArray[i].getName();

		if(fieldName.charAt(0) == '$') {
		    processClassModifier(field);
		    fieldArray[i] = null;
		}
	    }
	}		
    }

    protected void processClassModifier(Field field) throws MetaException {
	try {
	    String name = field.getName();

	    if(name.equals("$superclass")) {
		Object val = field.get(declaration);

		if(val instanceof java.lang.String) {
		    runtimeSuperClass = (String) val;
		} else {
		    throw error("$superclass should be declared as a String");
		}
	    } else if(name.equals("$suppressCode")) {
		suppressCode = field.getBoolean(declaration);
	    } else if(name.equals("$postCreateSQL")) {
		Object val = field.get(declaration);

		if(val instanceof java.lang.String) {
		    postCreateSQL = (String) field.get(declaration);
		} else {
		    throw error("postCreateSQL should be declared as a String");
		}
	    } else {
		throw error("illegal class modifier: " + name);
	    }
	} catch (IllegalAccessException e) {
	    throw error("can't access " + field.getName());
	}
    }

    /**
     * Iterates over an array of fields and processes any which are
     * meant to describe or modify the generation of an attribute.
     * Any fields which are attribute modifiers are
     * nulled out in the array so that future passes over the same array
     * won't see them.
     * @param fieldArray an array of fields
     * @exception MetaException if one of the fields is a class modifier
     *            but can't be accessed due to a protection violation.
     */
    protected void extractAttributeModifiers(Field[] fieldArray)
	throws MetaException
    {
	for(int i = 0; i < fieldArray.length; ++i) {
	    if(fieldArray[i] != null) {
		String fieldName = fieldArray[i].getName();
		int delimiter = fieldName.indexOf((int) '$');

		if(delimiter > 0) {
		    String attrName = fieldName.substring(0, delimiter);
		    String modifierName = fieldName.substring(delimiter + 1);

		    if(attributesByName.containsKey(attrName)) {
			MetaAttribute attr =
			    (MetaAttribute) attributesByName.get(attrName);
			processAttributeModifier(attr,
						 modifierName,
						 fieldArray[i]);
		    } else {
			throw error(fieldName +
				    " references unknown attribute '" +
				    attrName + "'");
		    }
		} else {
		    throw error("invalid MetaClass field: " + fieldName);
		}

		fieldArray[i] = null;
	    }
	}
    }

    /**
     * Does the actual work of modifying an attribute.
     * @param attr the attribute to be modified
     * @param modifierName the name of the modification (e.g., "size" or
     *                     "isUnique")
     * @param modifierDeclaration the field from which this modification
     *                     came
     * @exception MetaException modifierName doesn't correspond to a valid
     *            modifier or if the value of the field can't be accessed.
     */
    protected void processAttributeModifier(MetaAttribute attr,
					    String modifierName,
					    Field modifierDeclaration)
	throws MetaException
    {
	try {
	    if(modifierName.equals("size")) {
		attr.setSize(modifierDeclaration.getInt(declaration));
	    } else if(modifierName.equals("isKey")) {
		attr.setIsKey(modifierDeclaration.getBoolean(declaration));
	    } else if(modifierName.equals("isUnique")) {
		attr.setIsUnique(modifierDeclaration.getBoolean(declaration));
	    } else if(modifierName.equals("canBeNull")) {
		attr.setCanBeNull(modifierDeclaration.getBoolean(declaration));
	    } else if(modifierName.equals("noAccess")) {
		attr.setNoAccessMethod(modifierDeclaration.getBoolean(declaration));
	    } else {
		throw error("unknown attribute modifier in: " +
			    modifierDeclaration.getName());
	    }
	} catch (IllegalAccessException e) {
	    throw error("can't access " + modifierDeclaration.getName());
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    protected void dumpTypeMap() {
	for(Enumeration e = nativeTypes.keys(); e.hasMoreElements(); ) {
	    Class key = (Class) e.nextElement();
	    MetaType t = (MetaType) nativeTypes.get(key);

	    System.out.println(key.getName() + " maps to " +
			       t.getClass().getName());
	}
    }

    /*
     * This function computes the appropriate MetaType for an attirbute
     * declaration based on the current type mapping.
     */
    protected MetaType mapType(Class declaredType) throws MetaException {
	if(isObjectReference(declaredType)) {
	    MetaClass foreignClass = MetaClass.forDeclaration(declaredType);
	    
	    /* we get a prototype instance of the correct type
	       with this nativeTypes.get(). However, since it's a
	       parameterized type, we need to create another
	       instance which knows about the type we're
	       referring to.
	    */

	    Class refTypeClass =
		(Class) nativeTypes.get(referenceClass).getClass();

	    MetaType mappedReferenceType =
		(MetaType) nativeTypes.get(referenceClass);

	    Class classOfMappedReferenceType =
		mappedReferenceType.getClass();

	    try {
		T_Reference refType =
		    (T_Reference) classOfMappedReferenceType.newInstance();

		refType.setReferencedClass(foreignClass);

		return refType;
	    } catch (Exception e) {
		throw error("caught: " + e.toString());
	    }
	} else if(isMetaType(declaredType)) {
	    try {
		MetaType mappedType = (MetaType) nativeTypes.get(declaredType);

		if(mappedType != null) {
		    return (MetaType) mappedType.getClass().newInstance();
		} else {
		    return (MetaType) declaredType.newInstance();
		}
	    } catch(Exception e) {
		throw error("caught: " + e.toString());
	    }
	} else {
	    MetaType metaType = (MetaType) nativeTypes.get(declaredType);

	    if(metaType != null) {
		return metaType;
	    } else {
		// dumpTypeMap();
		throw error("unknown type: " + declaredType.getName() +
			    " in declaration of " + name);
	    }
	}
    }

    /**
     * Iterates over an array of fields and extracts the ones which will
     * will correspond to attributes of the generated class. Such fields
     * will be removed from the array so that future passes over the same
     * array won't see them.
     * @param fieldArray an array of fields
     * @exception MetaException if the type of the intended attribute can't
     *                          be processed.
     */
    protected void extractAttributes(Field[] fieldArray) throws MetaException {
	for(int i = 0; i < fieldArray.length; ++i) {
	    if(fieldArray[i] != null) {
		Field field = fieldArray[i];

		if(isAttributeDeclaration(field)) {
		    Class fieldType = field.getType();
		    MetaAttribute attr = new MetaAttribute(field.getName());
		    MetaType t = mapType(field.getType());

		    attr.setMetaType(t);

		    if(isObjectReference(fieldType)) {
			MetaClass foreignClass =
			    MetaClass.forDeclaration(fieldType);

			attr.setForeignKey(foreignClass.getIDAttribute(),
					   foreignClass);
		    }

		    attributesByName.put(field.getName(), attr);
		    attributes.addElement(attr);

		    fieldArray[i] = null;
		}
	    }
	}
    }

    /**
     * Returns whether the specified <code>Class</code> corresponds to
     * one of the native Java types that is valid for an attribute.
     * The valid native types are: <code><br>
     * byte<br>
     * short<br>
     * int<br>
     * long<br>
     * char<br>
     * boolean<br>
     * float<br>
     * double<br>
     * String<br></code>
     * @param c a class
     * @return <code>true</code> if the class is derived from
     *         <code>MetaType</code>.
     * @see dogen.generator.MetaType
     */
    protected static boolean isNativeType(Class c) {
	return nativeTypes.containsKey(c);
    }

    /**
     * Returns whether the specified <code>Class</code> is derived from
     * <code>dogen.generator.MetaType</code>
     * @param c a class
     * @return <code>true</code> if the class is derived from
     *         <code>MetaType</code>.
     * @see dogen.generator.MetaType
     */
    protected static boolean isMetaType(Class c) {
	return MetaType.isMetaType(c);
    }

    /**
     * Returns whether the specified <code>Class</code> represents a reference
     * to another class. By default, any type which isn't a MetaType and isn't
     * a native type (as defined by the current type map) is defined to
     * be an object reference.
     * @param c a class
     * @return <code>true</code> if the class references another class
     */
    protected static boolean isObjectReference(Class c) {
	return !isMetaType(c) && !isNativeType(c);
    }

    /**
     * Determines whether the field is a declaration of an attribute.
     * This is done one the basis of the type of the field, which is
     * assumed not to be a class modifier or an attribute modifier.
     * If the type of the field is either a <code>MetaType</code>
     * (e.g., <code>T_int</code>) or an object reference, then
     * the field is an attribute.
     * @param field a field of a class
     * @return true if the field will be an attribute
     * @see dogen.generator.MetaType
     */
    protected boolean isAttributeDeclaration(Field field) {
	Class type = field.getType();

	if(field.getName().indexOf((int) '$') == -1) {
	    return isMetaType(type) ||
		isObjectReference(type) ||
		isNativeType(type);
	} else {
	    return false;
	}
    }

    /**
     * A helper method for the 0-ary getAllAttributes
     * @param attrs a vector of <code>MetaAttribute</code>s to which the
     * attributes of this class will be appended after adding the attributes
     * of any superclasses.
     */
    protected void getAllAttributes(Vector attrs) {
	if(superClass != null) {
	    superClass.getAllAttributes(attrs);
	}

	for(Enumeration e = attributes.elements(); e.hasMoreElements(); ) {
		attrs.addElement(e.nextElement());
	}
    }

    /**
     * Flattens the inheritance hierarchy by returning a vector of all the
     * attributes of this <code>MetaClass</code> including all the attributes
     * of any superclasses. The first attributes in the vector will the those
     * which were declared highest in the inheritance hierarchy.
     * @return a vector of attributes
     */
    protected Vector getAllAttributes() {
	Vector retval = new Vector();

	getAllAttributes(retval);

	return retval;
    }

    /**
     * Generates a complete class implementation for this
     * <code>MetaClass</code>. A file will be created in the appropriate
     * location using the name of the declaration without the trailing
     * "DECL".
     * @exception IOException if something goes wrong in writing the file
     * @see dogen.runtime.DataObject
     */
    public void generateJavaCode() throws IOException {
	super.generateJavaCode(); // for the class defintion
	generateQueryCode();
    }

    /**
     * Generates a complete Query class for this
     * <code>MetaClass</code> that will return fully-formed instances from
     * the database.
     * @exception IOException if something goes wrong in writing the file
     * @see dogen.runtime.DataObjectQuery
     */
    protected void generateQueryCode() throws IOException {
	MetaAttribute id = getIDAttribute();

	if((id != null) && !isAbstract) {
	    CodeWriter cw = outputTo(getSourceFile("Query"));
	    emitJavaQueryCode(cw);
	    cw.close();

	    cw = outputTo(getSourceFile("IDQuery"));
	    emitJavaIDQueryCode(cw);
	    cw.close();
	}
    }


    /**
     * Emits the complete query class implementation for a
     * <code>MetaClass</code>.
     * @param cw the output stream for the generated code
     */
    public void emitJavaQueryCode(CodeWriter cw) {
	emitPackage(cw);
	cw.println("import java.sql.*;");
	cw.println("import com.lutris.appserver.server.sql.*;");
	cw.println("import dogen.runtime.*;\n");

	cw.beginComment();
	cw.println("THIS IS PROGRAM GENERATED CODE. DO NOT EDIT.");
	cw.println("");
	cw.println("The instances of this class represent queries against");
	cw.println("the " + name.toUpperCase() + " table. Create subclasses ");
	cw.println("by implementing the <code>getSqlTemplate</code> method ");
	cw.println("which returns a complete SQL query that selects all");
	cw.println("columns from " + name.toUpperCase());
	cw.println("@see " + declaration.getClass().getName());
	cw.endComment();

	cw.println("public class " + name + "Query extends DataObjectQuery {");

	cw.println("    protected static final String DBTABLE = \""
		   + name + "\";\n");
	cw.println("");

	MetaAttribute id = getIDAttribute();

	if(id != null) {
	    cw.println("    protected static final String KEY = \"" +
		   id.getName() + "\";\n");
	    cw.println("");
	}

	cw.println("    public String getSqlTemplate() {");
	cw.println("        return \"SELECT * FROM \" + DBTABLE;");
	cw.println("    }");
	cw.println("");
	cw.println("    protected DataObject buildResult(ResultSet rs, " +
		                                        "int offset)");
	cw.println("        throws DataObjectException");
	cw.println("    {");
	cw.println("        return " + name + ".lookup(rs, offset);");
	cw.println("    }");

	String schemaName = metaSchema.getFullyQualifiedName();

	cw.println("    public Schema getSchema() {");
	cw.println("        return " + schemaName + ".instance;");
	cw.println("    }\n");

	cw.println("}");
    }

    /**
     * Emits the complete query class implementation for a
     * <code>MetaClass</code>.
     * @param cw the output stream for the generated code
     */
    public void emitJavaIDQueryCode(CodeWriter cw) {
	emitPackage(cw);
	cw.println("import java.sql.*;");
	cw.println("import com.lutris.appserver.server.sql.*;");
	cw.println("import dogen.runtime.*;\n");

	cw.beginComment();
	cw.println("THIS IS PROGRAM GENERATED CODE. DO NOT EDIT.");
	cw.println("");
	cw.println("The instances of this class represent queries against");
	cw.println("the " + name.toUpperCase() + " table by OID.");
	cw.println("@see " + declaration.getClass().getName());
	cw.endComment();

	cw.println("public class "
		   + name + "IDQuery extends " + name + "Query {");
	cw.println("    protected String[] oids;");
	cw.println("");

	MetaAttribute id = getIDAttribute(); // better have an id!

	cw.println("    protected static final String KEY = \"" +
		   id.getName() + "\";");
	cw.println("");

	cw.println("    public " + name + "IDQuery(String[] oids) {");
	cw.println("        this.oids = oids;");
	cw.println("    }");
	cw.println("");

	cw.println("    public " + name + "IDQuery(String oid) {");
	cw.println("        this.oids = new String[1];");
	cw.println("        this.oids[0] = oid;");
	cw.println("    }");
	cw.println("");

	String key = id.getName();

	cw.println("    public String getSqlTemplate() {");
	cw.println("        StringBuffer sql = new StringBuffer();");
	cw.println("        sql.append(\"SELECT * FROM \" + DBTABLE);");
	cw.println("        sql.append(\" WHERE " + key + " IN (\");");
	cw.println("");
	cw.println("        if((oids == null) || (oids.length == 0)) {");
	cw.println("            sql.append(\"0)\");");
	cw.println("        } else {");
	cw.println("            for(int i = 0; i < oids.length; ++i) {");
	cw.println("                sql.append(oids[i]);");
	cw.println("");
	cw.println("                if(i < (oids.length - 1)) {");
	cw.println("                    sql.append(\",\");");
	cw.println("                }");
	cw.println("            }");
	cw.println("            sql.append(\")\");");
	cw.println("        }");
	cw.println("        return sql.toString();");
	cw.println("    }");

	cw.println("}");
    }

    /**
     * Emits the complete class implementation for a
     * <code>MetaClass</code>.
     * @param cw the output stream for the generated code
     */
    public void emitJavaCode(CodeWriter cw) {
	emitPackage(cw);
	emitImports(cw);

	cw.beginComment();
	cw.println("THIS IS PROGRAM GENERATED CODE. DO NOT EDIT.");
	cw.println("");
	cw.println("This instances of this class represent rows from ");
	cw.println("the " + name.toUpperCase() + " table.");
	cw.println("@see " + declaration.getClass().getName());
	cw.endComment();

	emitDefinition(cw);
	emitClassVariables(cw);
	emitInstanceVariables(cw);
	emitStaticBlock(cw);
	emitFactoryMethods(cw);
	emitConstructors(cw);
	emitStatementMethods(cw);
	emitGetMethods(cw);
	emitSetMethods(cw);
	emitUtilityMethods(cw);

	cw.println("}");
    }

    /**
     * Emits the import declarations for a generated subclass of
     * <code>DataObject</code>
     * @param cw the output stream for the generated code
     */
    protected void emitImports(CodeWriter cw) {
	cw.println("import dogen.runtime.*;");
	cw.println("import java.util.*;");
	cw.println("import java.math.*;");
	cw.println("import java.sql.*;");
	cw.println("import com.lutris.appserver.server.sql.*;");
	cw.println("import org.w3c.dom.*;");
	cw.println("import dogen.runtime.*;\n");
    }

    /**
     * Emits the class definition for a generated subclass of
     * <code>DataObject</code>.
     * @param cw the output stream for the generated code
     */
    protected void emitDefinition(CodeWriter cw) {
	String sup = null;

	if(runtimeSuperClass != null) {
	    sup = runtimeSuperClass;
	} else if(superClass != null) {
	    sup = superClass.getFullyQualifiedName();
	} else {
	    sup = null;
	}

	if(sup != null) {
	    sup = " extends " + sup;
	} else {
	    sup = "";
	}

	String abs = "abstract ";

	if(!isAbstract) {
	    abs = "";
	}

	cw.println("public " + abs + "class " + name + sup + " {");
    }

    /**
     * Emits the class variables for a generated subclass of
     * <code>DataObject</code>.
     * @param cw the output stream for the generated code
     */
    protected void emitClassVariables(CodeWriter cw) {
	MetaAttribute id = getIDAttribute();

	cw.println("    protected static " +
		            metaSchema.getFullyQualifiedName() + " $schema;");

	if((id != null) & !isAbstract) {
	    cw.println("    protected static Hashtable " + cacheName + ";");
	}

	cw.println("    public static final String[] $columnNames = {");
	
	Vector attrs = getAllAttributes();

	for(int i = 0; i < attrs.size(); ++i) {
	    MetaAttribute attr = (MetaAttribute) attrs.elementAt(i);
	    cw.print("        \"" + attr.getName() + "\"");

	    if(i < (attrs.size() - 1)) {
		cw.println(",");
	    } else {
		cw.println();
	    }
	}

	cw.println("    };\n");
    }

    /**
     * Emits the instance variables for a generated subclass of
     * <code>DataObject</code>.
     * @param cw the output stream for the generated code
     */
    protected void emitInstanceVariables(CodeWriter cw) {
	for(Enumeration e = attributes.elements();
	    e.hasMoreElements(); )
	{
	    MetaAttribute attr = (MetaAttribute) e.nextElement();

	    attr.emitJavaDeclaration(cw);
	}
	cw.println();
    }

    /**
     * Emits the static code block for a generated subclass of
     * <code>DataObject</code>.
     * @param cw the output stream for the generated code
     */
    protected void emitStaticBlock(CodeWriter cw) {
	MetaAttribute id = getIDAttribute();

	if((id != null) & !isAbstract) {
	    cw.println("    static {");
	    cw.println("        " + cacheName + " = new Hashtable();");
	    cw.println("        $schema = " +
		       metaSchema.getFullyQualifiedName() +
		       ".getInstance();");
	    cw.println("    }\n");
	}
    }

    /**
     * Emits the typed <code>lookup</code> methods for a generated subclass of
     * <code>DataObject</code>.
     * @param cw the output stream for the generated code
     */
    protected void emitFactoryMethods(CodeWriter cw) {
	MetaAttribute id = getIDAttribute();

	// if there's no ID attribute, then we can't look it up by ID
	if((id != null) && !isAbstract) {
	    cw.println("    public static DataObject lookup(OID oid) " +
		       "throws DataObjectException {");
	    cw.println("        synchronized (" + cacheName + ") {");
	    cw.println("            " + name + " object = (" + name + ") " +
		       cacheName + ".get(oid);");
	    cw.println("            if(object == null) {");
	    cw.println("                object = new " + name + "(oid);");
	    cw.println("                " + cacheName + ".put(oid, object);");
	    cw.println("            }");
	    cw.println("            return object;");
	    cw.println("        }");
	    cw.println("    }\n");


	    cw.println("    public static DataObject lookup(ResultSet rs)");
	    cw.println("        throws DataObjectException");
	    cw.println("    {");
	    cw.println("        return lookup(rs, 0);");
	    cw.println("    }\n");

	    cw.println("    public static DataObject" +
		       " lookup(ResultSet rs, int offset)");
	    cw.println("        throws DataObjectException");
	    cw.println("    {");

	    cw.println("        OID oid = extractOID(rs, " +
		       "offset);");
	    cw.println("        synchronized (" + cacheName + ") {");
	    cw.println("            " + name + " object = (" + name + ") " +
		       cacheName + ".get(oid);");
	    cw.println("            if(object == null) {");
	    cw.println("                object = new " + name + "(oid);");
	    cw.println("                " + cacheName + ".put(oid, object);");
	    cw.println("             }");
	    cw.println("            if(!object.wasModified()) {");
	    cw.println("                object.unpack(rs, offset);");
	    cw.println("            }");
	    cw.println("            return object;");
	    cw.println("        }");
	    cw.println("    }\n");

	    // even if there's no ID attribute, we can still unpack the object
	    cw.println("    protected void unpack(ResultSet rs, int offset)");
	    cw.println("        throws DataObjectException");
	    cw.println("    {");
	    cw.println("        try {");

	    int index = 1;

	    for(Enumeration e = getAllAttributes().elements();
		e.hasMoreElements(); )
		{
		    MetaAttribute attr = (MetaAttribute) e.nextElement();

		    if(attr.getNoAccessMethod() == false) {
			cw.print("            ");
		    }

		    attr.emitUnpackCode(cw, "rs", ("offset + " + index++));
		}

	    cw.println("            attributeState = VALID;");
	    cw.println("        } catch (SQLException e) {");
	    cw.println("            throw new DataObjectException(e);");
	    cw.println("        }");
	    cw.println("    }\n");
	}
    }

    /**
     * Emits the constructors for a generated subclass of
     * <code>DataObject</code>.
     * @param cw the output stream for the generated code
     */
    protected void emitConstructors(CodeWriter cw) {
	MetaAttribute id = getIDAttribute();
	
	cw.println("    public " + name + "()" +
		   " throws DataObjectException {");
	cw.println("        super();");
	cw.println("    }\n");

	// if there's no ID, then skip this constructor
	if(id != null) {
	    cw.println("    protected " + name + "(OID oid) " +
		       "throws DataObjectException {");
	    cw.println("        super(oid);");
	    cw.println("    }\n");
	}
    }

    /**
     * Emits the get methods for a generated subclass of
     * <code>DataObject</code>.
     * @param cw the output stream for the generated code
     */
    protected void emitGetMethods(CodeWriter cw) {
	String schemaName = metaSchema.getFullyQualifiedName();

	cw.println("    public Schema getSchema() {");
	cw.println("        return " + schemaName + ".instance;");
	cw.println("    }\n");

	MetaAttribute id = getIDAttribute();

	if((superClass == null) && (id != null)) {
	    cw.println("    public OID get$OID() {");
	    cw.println("        return " + id.getName() + ";");
	    cw.println("    }\n");
	}

	if(id != null) {
	    for(Enumeration e = attributes.elements(); e.hasMoreElements();) {
		MetaAttribute attr = (MetaAttribute) e.nextElement();
		attr.emitJavaGetMethod(cw);
	    }
	}
    }

    /**
     * Emits the set methods for a generated subclass of
     * <code>DataObject</code>.
     * @param cw the output stream for the generated code
     */
    protected void emitSetMethods(CodeWriter cw) {
	MetaAttribute id = getIDAttribute();

	if((superClass == null) && (id != null)) {
	    cw.println("    public void set$OID(OID id) {");
	    cw.println("    " + id.getName() + " = id;");
	    cw.println("    }\n");
	}

	if(id != null) {
	    for(Enumeration e = attributes.elements(); e.hasMoreElements(); ) {
		MetaAttribute attr = (MetaAttribute) e.nextElement();
		attr.emitJavaSetMethod(cw);
	    }
	}
    }

    /**
     * Emits the methods which create SQL statements for a generated
     * subclass of <code>DataObject</code>.
     * @param cw the output stream for the generated code
     */
    protected void emitStatementMethods(CodeWriter cw) {
	if(isAbstract) {
	    return;
	}

	cw.println("    public PreparedStatement getInsertStatement(DBConnection conn)");
	cw.println("        throws SQLException");
	cw.println("    {");
	cw.print(  "        String sql =  \"INSERT INTO " + name + " VALUES (");

	Vector allAttrs = getAllAttributes();
	int attrCount = allAttrs.size();

	for(int i = attrCount; i > 0; --i) {
	    if(i > 1) {
		cw.print("?,");
	    } else {
		cw.print("?");
	    }
	}

	cw.println(")\";");

	cw.println("        PreparedStatement stmt = conn.prepareStatement(sql);\n");

	int index = 1;
	for(Enumeration e = allAttrs.elements(); e.hasMoreElements(); ) {
	    MetaAttribute attr = (MetaAttribute) e.nextElement();

	    cw.print("        stmt.");
	    attr.emitPackCode(cw, Integer.toString(index++), attr.getName());
	    cw.println(";");
	}

	cw.println("\n        return stmt;");
	cw.println("    }\n");

	MetaAttribute id = getIDAttribute();

	cw.println("    public PreparedStatement getDeleteStatement(DBConnection conn)");
	cw.println("        throws SQLException");
	cw.println("    {");
	
	if(id != null) {
	    cw.println("        String sql = \"DELETE FROM " + name +
		       " WHERE " + id.getName() + " = ?\";");
	    cw.println("        PreparedStatement stmt = conn.prepareStatement(sql);");
	    cw.print(  "        stmt.");
	    id.emitPackCode(cw, "1", id.getName());
	    cw.println(";");
	    cw.println("        return stmt;");
	} else {
	    cw.println("        return null;");
	}
	cw.println("    }\n");

	    cw.println("    public PreparedStatement getUpdateStatement(DBConnection conn)");
	    cw.println("        throws SQLException");
	    cw.println("    {");

	if(id != null) {
	    cw.print(  "        String sql = \"UPDATE " + name + " SET ");

	    for(int i = 0; i < attrCount; ++i) {
		MetaAttribute attr = (MetaAttribute) allAttrs.elementAt(i);

		if(attr != id) {
		    cw.print(attr.getName() + " = ?");

		    if(i < (attrCount - 1)) {
			cw.print(", ");
		    }
		}
	    }
	
	    cw.println(" WHERE " + id.getName() + " = ?\";");
	    cw.println("        PreparedStatement stmt = conn.prepareStatement(sql);\n");

	    for(int i = 0; i < attrCount; ++i) {
		MetaAttribute attr = (MetaAttribute) allAttrs.elementAt(i);

		if(attr != id) {
		    cw.print("        stmt.");

		    /* this code passes "i" as the offset rather than "i+1"
		     * because the first parameter set in the statement will
		     * actually be the second attribute--the one after the OID.
		     * We never update the OID since that's what we're using to
		     * select the row in the first place.
		     */
		    attr.emitPackCode(cw, Integer.toString(i), attr.getName());
		    cw.println(";");
		}
	    }

	    cw.print("        stmt.");
	    id.emitPackCode(cw, Integer.toString(attrCount), id.getName());
	    cw.println(";");

	    cw.println("\n        return stmt;");
	} else {
	    cw.println("        return null;");
	}

	cw.println("    }\n");

	cw.println("    public static String getDropTableSQL() {");
	cw.println("        return  \"DROP TABLE " + name + "\";");
	cw.println("    }\n");

	cw.println("    public static String getCreateTableSQL() {");
	cw.println("        return \"CREATE TABLE " + name + " (\" +");

	for(int i = 0; i < attrCount; ++i) {
	    MetaAttribute attr = (MetaAttribute) allAttrs.elementAt(i);

	    cw.print("            \"");
	    attr.emitSqlCode(cw);

	    if(i < (attrCount - 1)) {
		cw.println(", \" +");
	    } else {
		cw.println("\" +\n            \")\";");
	    }
	}

	cw.println("    }\n");

	cw.println("    public static String getTableConstraintSQL() {");

	if(hasSqlConstraints()) {
	    cw.println("        return \"ALTER TABLE " + name + " \" +");

	    Vector constraints = new Vector();

	    for(int i = 0; i < attrCount; ++i) {
		MetaAttribute attr = (MetaAttribute) allAttrs.elementAt(i);

		if(attr.hasConstraints()) {
		    constraints.addElement(attr.getConstraintSqlCode());
		}
	    }

	    for(int i = 0; i < constraints.size(); ++i) {
		String constraint = (String) constraints.elementAt(i);

		cw.print("            \"");
		cw.print(constraint);

		if(i < (constraints.size() - 1)) {
		    cw.println("\" +");
		} else {
		    cw.println("\" +\n            \"\";");
		}
	    }
	} else {
	    cw.println("        return null;");
	}

	cw.println("    }\n");

	cw.println("    protected synchronized void fetchAttributes() " +
		   "throws DataObjectException {");

	/*
	 * This conditional is a hack. If this class doesn't have an ID
	 * attribute, then we really shouldn't be generating a
	 * fetchAttributes() method at all. However, we need a stub because
	 * dogen.runtime.DataObject wants it to be there.
	 */
	if(id != null) {
	    cw.println("        try {");
	    cw.println("            DBConnection conn = " +
		       "$schema.allocateConnection();");
	    cw.println("            String sql = \"SELECT * FROM " +
		       name + " WHERE " +
		       id.getName() + " = ?\";\n");
	    
	    cw.println("            PreparedStatement stmt = " +
		       "conn.prepareStatement(sql);");
	    cw.print(  "            stmt.");

	    id.emitPackCode(cw, "1", id.getName());
	    cw.println(";\n");
	    cw.println("            ResultSet rs = conn.executeQuery(stmt, " +
		       "\"fetchAttrs\");\n");
	    cw.println("            if(rs.next()) {");
	    cw.println("                 unpack(rs, 0);");
	    cw.println("            } else {");
	    cw.println("                throw new DataObjectException(" +
		       "\"no such object\");");
	    cw.println("            }");
	    cw.println("         } catch (DataObjectException e) {");
	    cw.println("              throw e;");
	    cw.println("         } catch (Exception e) {");
	    cw.println("              throw new DataObjectException(e);");
	    cw.println("         }");
	}

	cw.println("    }\n");
    }

    /**
     * Emits the methods which do useful things.
     * @param cw the output stream for the generated code
     */
    protected void emitUtilityMethods(CodeWriter cw) {
	MetaAttribute ident = getIDAttribute();
	if(isAbstract || (ident == null)) {
	    return;
	}

	cw.println("    public String toString() {");
	cw.println("        StringBuffer s = new StringBuffer();");
	cw.println("        s.append(\"[\" + " +
		   "(wasModified() ? \"*\" : \"\") + \"" +
		   name + "\");");

	Vector attrs = getAllAttributes();

	for(Enumeration e = attrs.elements(); e.hasMoreElements(); ) {
	    MetaAttribute attr = (MetaAttribute) e.nextElement();

	    cw.println("        s.append(\" " + attr.getName() + "=\");");

	    if(attr.isKey()) {
		cw.println("        s.append(" + attr.getName() + ");");
	    } else if(attr.isForeignKey()) {
		String refClass = attr.getForeignClass().getName();
		String id = "getOIDAsString(" +attr.getName() + ")";

		cw.println("        s.append(\"" + refClass + "#\" + " +
			                           id + ");");
	    } else {
		cw.println("        s.append(\"\" + " + attr.getName() + ");");
	    }
	}

	cw.println("        s.append(\"]\");");
	cw.println("        return s.toString();");
	cw.println("    }\n");

	cw.println("    public String toHTML() {");
	cw.println("        StringBuffer s = new StringBuffer();");
	cw.println("        s.append(\"<tr>\");");

	for(Enumeration e = attrs.elements(); e.hasMoreElements(); ) {
	    MetaAttribute attr = (MetaAttribute) e.nextElement();

	    cw.println("        s.append(\"<td>\");");

	    if(attr.isKey()) {
		cw.println("        s.append(" + attr.getName() +
			                  ".toString());");
	    } else if(attr.isForeignKey()) {
		String refClass = attr.getForeignClass().getName();
		String id = "getOIDAsString(" +attr.getName() + ")";

		cw.println("        s.append(\"" + refClass + "#\" + " +
			                           id + ");");
	    } else {
		cw.println("        s.append(\"\" + " + attr.getName() + ");");
	    }

	    cw.println("        s.append(\"</td>\");");
	}

	cw.println("        s.append(\"</tr>\\n\");");
	cw.println("        return s.toString();");
	cw.println("    }\n");

	cw.println("    public Element toRowElement(Document doc) {");
	cw.println("        Element tr = doc.createElement(\"TR\");");
	cw.println("        Element td;");
	cw.println("        Node text;");

	for(Enumeration e = attrs.elements(); e.hasMoreElements(); ) {
	    MetaAttribute attr = (MetaAttribute) e.nextElement();

	    cw.println("        td = doc.createElement(\"TD\");");
	    cw.print(  "        text = doc.createTextNode(");

	    if(attr.isForeignKey()) {
		String tableName = attr.getForeignClass().getName();
		cw.println("\"" + tableName + "#\" + " + 
			   "getOIDAsString(" + attr.getName() +
			   "));");
	    } else {
		cw.println("\"\" + " + attr.getName() + ");");
	    }
	    cw.println("        td.appendChild(text);");
	    cw.println("        tr.appendChild(td);");
	}	
	cw.println("        return tr;");
	cw.println("    }\n");

	cw.println("    public String[] getColumnNames() {");
	cw.println("        return $columnNames;");
	cw.println("    }\n");

	cw.println("    public static String toHTML(Vector objects) {");
	cw.println("        return toHTML($columnNames, objects);");
	cw.println("    }\n");

	cw.println("    public static Element toTableElement(Document doc, " +
		                                            "Vector objects) {");
	cw.println("        return toHTMLTable(doc, $columnNames, objects);");
	cw.println("    }\n");
    }
}


