/*-----------------------------------------------------------------------------
 * Dogen -- the Data Object GENerator
 * Copyright 1999 Mindgarden.com, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *
 *      This product includes Dogen software developed by Mindgarden.com, Inc.
 *
 * 4. The name of Mindgarden.com, Inc. may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MINDGARDEN.COM, INC. ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL MINDGARDEN.COM BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: MetaSchema.java,v 1.1.1.1 1999/09/23 13:42:20 andy Exp $
 *-----------------------------------------------------------------------------
 */

package dogen.generator;

import java.util.*;
import java.io.*;
import java.lang.reflect.*;

public class MetaSchema extends CodeGenerator {
    protected static void debug(String msg) {
	System.out.println("MetaSchema: " + msg);
    }

    /*
     * If this variable is true, then generated table creation code (SQL) will
     * include constraints (i.e., foreign keys have to reference actual
     * rows). These constraints are implemented using "ALTER TABLE ADD
     * CONSTRAINT" which works for Oracle, but probably not much else.
     * Set this variable to true if you're running Oracle.
     */
    public static boolean emitSQLConstraints = false;

    protected Object declaration;

    /*
     * metaClasses holds all the MetaClasses that are referenced directly
     * or indirectly by the schema.
     */
    protected Hashtable metaClasses;

    /*
     * metaClassesToGenerate has only those MetaClasses that appear in the
     * MetaSchema declaration itself.
     */
    protected Vector metaClassesToGenerate;
    protected static MetaSchema current = null;

    public MetaSchema(Object decl) throws MetaException {
	super(decl.getClass().getName());

	declaration = decl;
	metaClasses = new Hashtable();
	metaClassesToGenerate = new Vector();

	current = this;
	processDeclaration(decl);
    }

    public void generateClasses() throws IOException {
	generateJavaCode();

	for(Enumeration e = metaClasses.elements(); e.hasMoreElements(); ) {
	    MetaClass c = (MetaClass) e.nextElement();
	    c.generateJavaCode();
	}
    }

    public static void setCurrentSchema(MetaSchema schema) {
	current = schema;
    }

    public static MetaSchema getCurrentSchema() {
	return current;
    }

    public Object getDeclaration() {
	return declaration;
    }

    protected void processDeclaration(Object decl) throws MetaException {
	Class declClass = decl.getClass();

	Field[] fieldArray = declClass.getDeclaredFields();

	extractSchemaModifiers(fieldArray);
	extractClasses(fieldArray);
    }

    protected void extractSchemaModifiers(Field[] fieldArray)
	throws MetaException
    {
	for(int i = 0; i < fieldArray.length; ++i) {
	    Field field = fieldArray[i];

	    if(field != null) {
		String fieldName = fieldArray[i].getName();

		if(fieldName.charAt(0) == '$') {
		    processSchemaModifier(field);
		    fieldArray[i] = null;
		}
	    }
	}
    }

    protected void processSchemaModifier(Field field)
	throws MetaException
    {
	try {
	    if(field.getName().equals("$typemap")) {
		MetaClass.setTypeMapping(field.getType().newInstance());
	    } else if(field.getName().equals("$emitSQLConstraints")) {
		emitSQLConstraints = field.getBoolean(declaration);
	    } else {
		throw error("unrecognized schema modifier: " +
			    field.getName());
	    }

	} catch (MetaException e) {
	    throw e;
	} catch (Exception e2) {
	    throw error("caught: " + e2.toString());
	}
	    
    }

    protected void extractClasses(Field[] fieldArray) throws MetaException {
	for(int i = 0; i < fieldArray.length; ++i) {
	    if(fieldArray[i] != null) {
		Field field = fieldArray[i];

		Class fieldType = field.getType();
		MetaClass metaClass = MetaClass.forDeclaration(fieldType);

		metaClass.setAbstract(false);
		addSchemaClass(metaClass);
		metaClassesToGenerate.addElement(metaClass);

		fieldArray[i] = null;
	    }
	}
    }

    protected void addSchemaClass(Object decl) throws MetaException {
	addSchemaClass(MetaClass.forDeclaration(decl));
    }

    protected boolean containsClass(MetaClass metaClass) {
	if(metaClass.isRoot() || metaClass.getSuppressCode()) {
	    return true;
	} else {
	    String metaClassName = metaClass.getFullyQualifiedName();
	    return metaClasses.containsKey(metaClassName);
	}
    }

    protected void addSchemaClass(MetaClass metaClass) throws MetaException {
	if(metaClass != null) {
	    if(!containsClass(metaClass)) {
		metaClasses.put(metaClass.getFullyQualifiedName(), metaClass);

		MetaClass parent = metaClass.getSuperClass();

		if(parent != null) {
		    addSchemaClass(parent);
		}

		/*
		 * add all references in the attributes of this class
		 */
		for(Enumeration e = metaClass.getAllAttributes().elements();
		    e.hasMoreElements(); )
		{
		    MetaAttribute attr = (MetaAttribute) e.nextElement();
			
		    if(attr.isForeignKey()) {
			addSchemaClass(attr.getForeignClass());
		    }
		}
	    }
	}
    }

    public void emitJavaCode(CodeWriter cw) {
	emitPackage(cw);
	emitImports(cw);
	emitDefinition(cw);
	emitVariables(cw);
	cw.println("");
	emitMethods(cw);
	cw.println("}");
    }

    protected void emitImports(CodeWriter cw) {
	cw.println("import java.sql.*;");
	cw.println("import com.lutris.appserver.server.sql.*;");
	cw.println("import com.lutris.util.*;");
	cw.println("import dogen.runtime.*;\n");
    }

    protected void emitDefinition(CodeWriter cw) {
	cw.println("public final class " + name + " extends Schema {");
    }

    protected void emitVariables(CodeWriter cw) {
	cw.println("    public static " + name + " instance = null;");
    }

    protected void emitMethods(CodeWriter cw) {
	emitConstructor(cw);
	emitStaticMethods(cw);
	emitDropTablesMethod(cw);
	emitCreateTablesMethod(cw);
	emitUtilityMethods(cw);
    }

    protected void emitConstructor(CodeWriter cw) {
	cw.println("    public " + name + "(Config config, String database)");
	cw.println("        throws DataObjectException");
	cw.println("    {");
	cw.println("        super(config, database);");
	cw.println("        instance = this;");
	cw.println("    }");
    }

    protected void emitStaticMethods(CodeWriter cw) {
	cw.println("    public static void begin() throws DataObjectException {");
	cw.println("        instance.beginTransaction();");
	cw.println("    }\n");

	cw.println("    public static void end() throws DataObjectException {");
	cw.println("        instance.endTransaction();");
	cw.println("    }\n");

	cw.println("    public static void abort() throws DataObjectException {");
	cw.println("        instance.abortTransaction();");
	cw.println("    }\n");

	cw.println("    public static " + name + " getInstance() {");
	cw.println("        return instance;");
	cw.println("    }");

	cw.println("    public static boolean exists()");
	cw.println("        throws ConfigException, SQLException, SchemaException");
	cw.println("    {");
	cw.println("        return (instance != null) && "+
		            "instance.schemaExists();");
	cw.println("    }\n");

	cw.println("    public static void create() throws ConfigException, " +
		   "SQLException, SchemaException {");
	cw.println("        instance.createSchema();");
	cw.println("    }\n");

	cw.println("    public static void destroy() throws ConfigException, " +
		   "SQLException, SchemaException {");
	cw.println("         instance.destroySchema();");
	cw.println("    }\n");
    }

    protected void emitCreateTablesMethod(CodeWriter cw) {
	cw.println("    protected void createTables()");
	cw.println("        throws SQLException, ConfigException");
	cw.println("    {");
	cw.println("        super.createTables();\n");

	for(Enumeration e = metaClassesToGenerate.elements();
	    e.hasMoreElements();
	    )
	{
	    MetaClass metaClass = (MetaClass) e.nextElement();

	    if(!metaClass.getSuppressCode()) {
		cw.println("        userSQL(" +
			   metaClass.getFullyQualifiedName() +
			   ".getCreateTableSQL());");
	    }
	}

	if(emitSQLConstraints) {
	    for(Enumeration e = metaClassesToGenerate.elements();
		e.hasMoreElements();)
	    {
		MetaClass metaClass = (MetaClass) e.nextElement();

		if(metaClass.hasSqlConstraints()) {
		    cw.println("        userSQL(" +
			       metaClass.getFullyQualifiedName() +
			       ".getTableConstraintSQL());");
		}
	    }
	}

	for(Enumeration e = metaClassesToGenerate.elements();
	    e.hasMoreElements(); )
	{
	    MetaClass metaClass = (MetaClass) e.nextElement();

	    if(metaClass.getPostCreateSQL() != null) {
		cw.println("        userSQL(\"" +
			   metaClass.getPostCreateSQL() + "\");");
	    }
	}

	cw.println("    }");
    }

    protected void emitDropTablesMethod(CodeWriter cw) {
	cw.println("    protected void dropTables()");
	cw.println("        throws SQLException, ConfigException");
	cw.println("    {");
	cw.println("        super.dropTables();\n");

	for(Enumeration e = metaClassesToGenerate.elements();
	    e.hasMoreElements(); )
	{
	    MetaClass metaClass = (MetaClass) e.nextElement();

	    if(!metaClass.getSuppressCode()) {
		cw.println("        userSQL(" +
			   metaClass.getFullyQualifiedName() +
			   ".getDropTableSQL());");
	    }
	}

	cw.println("    }");
    }

    public void generateDeclarations() throws IOException {
	for(Enumeration e = metaClasses.elements(); e.hasMoreElements(); ) {
	    MetaClass metaClass = (MetaClass) e.nextElement();

	    metaClass.generateJavaCode();
	}
    }

    protected void emitUtilityMethods(CodeWriter cw) {
    }
}
