/*-----------------------------------------------------------------------------
 * Dogen -- the Data Object GENerator
 * Copyright 1999 Mindgarden.com, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *
 *      This product includes Dogen software developed by Mindgarden.com, Inc.
 *
 * 4. The name of Mindgarden.com, Inc. may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MINDGARDEN.COM, INC. ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL MINDGARDEN.COM BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DataObject.java,v 1.1.1.1 1999/09/23 13:42:20 andy Exp $
 *-----------------------------------------------------------------------------
 */

package dogen.runtime;

import java.sql.*;
import java.math.*;
import java.util.*;

import com.lutris.appserver.server.sql.*;
import org.w3c.dom.*;

public abstract class DataObject {
    public static final
	short PRENATAL      = 1; // needs to be created in db
    public static final
	short IDENTIFIED    = 2; // already exists in db
    public static final
	short DELETED       = 3; // has already been deleted

    public static final
	short INVALID       = 5; // attributes have not yet been read
    public static final
	short VALID         = 6; // attributes have been read
    public static final
	short MODIFIED      = 7; // attributes have been read and modified

    protected static Schema schema;

    protected short idState;
    protected short attributeState;

    public abstract OID get$OID();
    public abstract void set$OID(OID oid);
	
    protected static void debug(String msg) {
	System.out.println("DataObject: " + msg);
    }

    protected DataObject() throws DataObjectException {
	try {
	    this.idState = PRENATAL;
	    this.attributeState = VALID;
	    set$OID(getSchema().allocateOID());
	    getSchema().insert(this);
	} catch (Exception e) {
	    throw new DataObjectException(e);
	}
    }

    protected DataObject(OID oid) {
	this.idState = IDENTIFIED;
	this.attributeState = INVALID;
	set$OID(oid);
    }

    public static void setSchema(Schema schema) {
	DataObject.schema = schema;
    }

    protected static String getOIDAsString(DataObject obj) {
	if(obj != null) {
	    return obj.get$OID().toString();
	} else {
	    return "null";
	}
    }

    protected static long getOIDAsLong(DataObject obj) {
	if(obj != null) {
	    return obj.get$OID().toLong();
	} else {
	    return (long) 0;
	}
    }

    protected static long getOIDAsLong(OID obj) {
	if(obj != null) {
	    return obj.toLong();
	} else {
	    return (long) 0;
	}
    }

    public abstract Schema getSchema();

    short getIdState() {
	return idState;
    }

    public void invalidateAttributes() {
	attributeState = INVALID;
    }

    public void delete() throws DataObjectException {
	switch(idState) {
	case PRENATAL :
	case IDENTIFIED :
	    getSchema().delete(this);
	    break;

	case DELETED :
	    // should really throw an exception here
	    break;

	default :
	    throw new DataObjectException("unknown idState");
	}
    }

    public void uncreate() throws DataObjectException {
	switch(idState) {
	case IDENTIFIED :
	    idState = PRENATAL;
	    break;

	case PRENATAL :
	    break;

	case DELETED :
	    // should really throw an exception here
	    break;

	default :
	    throw new DataObjectException("unknown idState");
	}
    }

    protected abstract void fetchAttributes() throws DataObjectException;

    protected void validateAttributes() throws DataObjectException {
	if(attributeState == INVALID) {
	    fetchAttributes();
	}
    }

    protected void dirtyAttributes() throws DataObjectException {
	switch(attributeState) {
	case VALID :
	    attributeState = MODIFIED;
	    getSchema().update(this);
	    break;

	case MODIFIED :
	    // do nothing
	    break;

	case INVALID :
	default :
	    throw new DataObjectException("can't modify invalid attributes");
	}
    }

    protected static OID toOID(long value) {
	if(value == 0) {
	    return OID.nullOID;
	} else {
	    return new OID(value);
	}
    }

    protected static OID extractOID(ResultSet rs, int offset)
	throws DataObjectException
    {
	try {
	    long id = rs.getLong(offset + 1);

	    if(id == 0) {
		return null;
	    } else {
		return new OID(id);
	    }
	} catch (Exception e) {
	    throw new DataObjectException(e);
	}
    }

    public boolean wasModified() {
	return false;
    }

    public abstract PreparedStatement getInsertStatement(DBConnection c)
	throws SQLException;

    public void executeInsert(DBConnection conn) throws SQLException {
	if(idState == IDENTIFIED) {
	    executeUpdate(conn);
	} else {
	    conn.executeUpdate(getInsertStatement(conn), "executeInsert");
	}
    }

    public void finalizeInsert(boolean success) {
	idState = IDENTIFIED;
	attributeState = VALID;
    }

    public abstract PreparedStatement getUpdateStatement(DBConnection c)
	throws SQLException;

    public void executeUpdate(DBConnection conn) throws SQLException {
	conn.executeUpdate(getUpdateStatement(conn), "executeUpdate");
    }

    public void finalizeUpdate(boolean success) {
	idState = IDENTIFIED;
	attributeState = VALID;
    }

    public abstract PreparedStatement getDeleteStatement(DBConnection c)
	throws SQLException;

    public void executeDelete(DBConnection conn) throws SQLException {
	conn.executeUpdate(getDeleteStatement(conn), "DataObject.executeDelete()");
    }

    public void finalizeDelete(boolean success) {
	idState = DELETED;
    }

    public abstract String toHTML();
    public abstract String[] getColumnNames();

    protected static String toHTML(String[] columnNames, Vector objects) {
	StringBuffer s = new StringBuffer();

	s.append("<table><tr>");

	for(int i = 0; i < columnNames.length; ++i) {
	    s.append("<td>" + columnNames[i] + "</td>");
	}

	s.append("</tr>\n");

	for(Enumeration e = objects.elements(); e.hasMoreElements(); ) {
	    DataObject o = (DataObject) e.nextElement();

	    s.append(o.toHTML());
	}

	s.append("</table>");
	return s.toString();
    }

    protected abstract Element toRowElement(Document doc);

    protected static Element toHTMLTable(Document doc,
				      String[] columnNames,
				      Vector objects)
    {
	Element table = doc.createElement("TABLE");
	Element header = doc.createElement("TR");

	for(int i = 0; i < columnNames.length; ++i) {
	    Element td = doc.createElement("TD");
	    Node text = doc.createTextNode(columnNames[i]);

	    td.appendChild(text);
	    header.appendChild(td);
	}

	table.appendChild(header);

	for(Enumeration rows = objects.elements(); rows.hasMoreElements(); ) {
	    DataObject object = (DataObject) rows.nextElement();

	    try {
		table.appendChild(object.toRowElement(doc));
	    } catch (Exception e) {
		e.printStackTrace();
	    }
	}

	return table;
    }
}
