/*-----------------------------------------------------------------------------
 * Dogen -- the Data Object GENerator
 * Copyright 1999 Mindgarden.com, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *
 *      This product includes Dogen software developed by Mindgarden.com, Inc.
 *
 * 4. The name of Mindgarden.com, Inc. may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MINDGARDEN.COM, INC. ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL MINDGARDEN.COM BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: Schema.java,v 1.1.1.1 1999/09/23 13:42:20 andy Exp $
 *-----------------------------------------------------------------------------
 */

package dogen.runtime;

import java.util.*;
import java.sql.*;

import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.util.*;

public class Schema {
    protected static final String OID_TABLE = "objectid";

    protected Connection userConnection;
    protected Hashtable transactions;
    protected DatabaseManager dbManager;
    protected Config config;
    protected Config logicalConfig;
    protected Config databaseConfig;
    protected String name;

    protected void debug(String msg) {
	System.out.println("Schema: " + msg);
    }

    protected Schema(Config configuration, String logicalDatabase)
	throws DataObjectException
    {
	try {
	    config = configuration;
	    databaseConfig = config.getConfig("DatabaseManager");

	    if((logicalDatabase == null) || logicalDatabase.equals("")) {
		name = configuration.getString("DatabaseManager.DefaultDatabase");
	    } else {
		name = logicalDatabase;
	    }

	    logicalConfig = config.getConfig("DatabaseManager.DB." + name);
	    
	    Class.forName(logicalConfig.getString("JdbcDriver"));

	    transactions = new Hashtable();

	    Config dbConfig = config.getConfig("DatabaseManager");
	    this.dbManager = new StandardDatabaseManager(dbConfig);
	    DataObject.setSchema(this);
	} catch (Exception e) {
	    throw new DataObjectException(e);
	}
    }

    protected DatabaseManager getDatabaseManager() {
	return dbManager;
    }

    public String getName() {
	return name;
    }

    protected void getUserConnection() throws ConfigException, SQLException {
	if(userConnection == null) {
	    String url = logicalConfig.getString("Connection.Url");
	    String user = logicalConfig.getString("Connection.User");
	    String password = logicalConfig.getString("Connection.Password");

	    userConnection = DriverManager.getConnection(url, user, password);
	}
    }

    public void shutdown() throws SQLException {
	if(dbManager != null) {
	    dbManager.shutdown();
	    dbManager = null;
	}

	if(userConnection != null) {
	    userConnection.close();
	    userConnection = null;
	}
    }

    public DBConnection getEnhydraDBConnection()
	throws DatabaseManagerException, DataObjectException, SQLException
    {
	return dbManager.allocateConnection();
    }

    public boolean schemaExists() throws ConfigException, SQLException {
	getUserConnection();

	boolean retval;
	ResultSet rs = null;

	try {
	    DatabaseMetaData metaData = userConnection.getMetaData();
	    String user = logicalConfig.getString("Connection.User");

	    String types[] = new String[1];
	    types[0] = "TABLE";

	    rs = metaData.getTables(null, user.toUpperCase(), null, types);

	    if(rs.next()) {
		retval = true;
	    } else {
		retval = false;
	    }
	} catch (SQLException e) {
	    System.out.println("caught: " + e.toString());
	    retval = false;
	} finally {
	    try {
		if(rs != null) {
		    rs.close();
		}
	    } catch (SQLException e) {
	    }
	}

	return retval;
    }

    public void createSchema() throws SQLException, ConfigException {
	if(!schemaExists()) {
	    createTables();
	}
    }

    public void destroySchema() throws SQLException, ConfigException {
	if(schemaExists()) {
	    dropTables();
	}
    }

    protected void printConnectionWarnings(Connection c) throws SQLException {
	SQLWarning w;

	for(w = c.getWarnings(); w != null; w = w.getNextWarning()) {
	    debug("SQLWarning: " + w.getMessage());
	}

	c.clearWarnings();
    }

    protected void userSQL(String sql) throws SQLException, ConfigException {
	getUserConnection();
	userConnection.setAutoCommit(true);

	Statement s = userConnection.createStatement();

	// debug("userSql: " + sql);
	s.executeUpdate(sql);
	printConnectionWarnings(userConnection);
	s.close();
    }


    protected void dropTables() throws SQLException, ConfigException {
    }

    protected void createTables()
	throws SQLException, ConfigException
    {
    }

    public DBConnection allocateConnection()
	throws DataObjectException
    {
	try {
	    return dbManager.allocateConnection(name);
	} catch (Exception e) {
	    throw new DataObjectException(e);
	}
    }

    public OID allocateOID() throws SchemaException {
	try {
	    ObjectId oid = dbManager.allocateObjectId();
	    OID retval = new OID(oid.toBigDecimal().longValue());
	    return retval;
	} catch (Exception e) {
	    throw new SchemaException(e);
	}
    }

    public void beginTransaction() {
	Thread t = Thread.currentThread();
	SchemaTransaction current = (SchemaTransaction) transactions.get(t);
	SchemaTransaction newtrans = new SchemaTransaction(current);

	transactions.put(t, newtrans);
    }

    public void endTransaction() throws DataObjectException {
	Thread t = Thread.currentThread();
	SchemaTransaction current = (SchemaTransaction) transactions.get(t);

	if(current != null) {
	    try {
		SchemaTransaction parent = current.getParent();

		if(parent != null) {
		    transactions.put(t, parent);
		} else {
		    transactions.remove(t);

		    current.commit();
		}
	    } catch(Exception e) {
		e.printStackTrace();
		throw new DataObjectException(e);
	    }
	} else {
	    throw new DataObjectException("no open transaction");
	}
    }

    public void abortTransaction() throws DataObjectException {
	Thread t = Thread.currentThread();
	SchemaTransaction current = (SchemaTransaction) transactions.get(t);

	if(current != null) {
	    try {
		SchemaTransaction parent = current.getParent();

		if(parent != null) {
		    transactions.put(t, parent);
		} else {
		    transactions.remove(t);

		    current.abort();
		}
	    } catch(Exception e) {
		e.printStackTrace();
		throw new DataObjectException(e);
	    }
	} else {
	    throw new DataObjectException("no open transaction");
	}
    }

    void insert(DataObject obj) throws DataObjectException {
	Thread t = Thread.currentThread();
	SchemaTransaction current = (SchemaTransaction) transactions.get(t);

	if(current != null) {
	    current.insert(obj);
	} else {
	    throw new DataObjectException("no open transaction");
	}
    }

    void update(DataObject obj) throws DataObjectException {
	Thread t = Thread.currentThread();
	SchemaTransaction current = (SchemaTransaction) transactions.get(t);

	if(current != null) {
	    current.update(obj);
	} else {
	    throw new DataObjectException("no open transaction");
	}
    }

    void delete(DataObject obj) throws DataObjectException {
	Thread t = Thread.currentThread();
	SchemaTransaction current = (SchemaTransaction) transactions.get(t);

	if(current != null) {
	    current.delete(obj);
	} else {
	    throw new DataObjectException("no open transaction");
	}
    }
}
