/*-----------------------------------------------------------------------------
 * Dogen -- the Data Object GENerator
 * Copyright 1999 Mindgarden.com, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *
 *      This product includes Dogen software developed by Mindgarden.com, Inc.
 *
 * 4. The name of Mindgarden.com, Inc. may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MINDGARDEN.COM, INC. ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL MINDGARDEN.COM BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: SchemaTransaction.java,v 1.1.1.1 1999/09/23 13:42:20 andy Exp $
 *-----------------------------------------------------------------------------
 */

package dogen.runtime;

import java.sql.*;
import com.lutris.appserver.server.sql.*;

public class SchemaTransaction {
    protected static final int NULL   = 0;
    protected static final int INSERT = 1;
    protected static final int UPDATE = 2;
    protected static final int DELETE = 3;
    protected static final int SUBTRANS = 4;

    protected static final int INITIALSIZE = 5;

    protected SchemaTransaction parent;
    protected Object[] components;
    protected int[] componentTypes;
    protected int componentCount;

    public SchemaTransaction(SchemaTransaction parent) {
	components = new Object[INITIALSIZE];
	componentTypes = new int[INITIALSIZE];
	componentCount = 0;

	if(parent != null) {
	    parent.appendComponent(this, SUBTRANS);
	}
    }

    public SchemaTransaction getParent() {
	return parent;
    }

    protected void appendComponent(Object component, int type) {
	if(componentCount == components.length) {
	    growComponentArray();
	}

	components[componentCount] = component;
	componentTypes[componentCount] = type;
	componentCount += 1;
    }

    protected void removeComponentAt(int index) {
	if((index >= 0) && (index < componentCount)) {
	    System.arraycopy(components, index + 1,
			     components, index,
			     componentCount - index);

	    System.arraycopy(componentTypes, index + 1,
			     componentTypes, index,
			     componentCount - index);
	}
    }

    protected void growComponentArray() {
	int newLength = components.length*2;

	Object[] newComponents = new Object[newLength];
	int[] newComponentTypes = new int[newLength];

	System.arraycopy(components, 0,
			 newComponents, 0,
			 componentCount);

	System.arraycopy(componentTypes, 0,
			 newComponentTypes, 0,
			 componentCount);

	components = newComponents;
	componentTypes = newComponentTypes;
    }

    public void insert(DataObject obj) {
	appendComponent(obj, INSERT);
    }

    public void update(DataObject obj) {
	appendComponent(obj, UPDATE);
    }

    public void delete(DataObject obj) {
	appendComponent(obj, DELETE);
    }

    public void abort() throws DataObjectException {
	for(int i = 0; i < componentCount; ++i) {
	    switch(componentTypes[i]) {
	    case SUBTRANS : {
		SchemaTransaction sub = (SchemaTransaction) components[i];

		sub.abort();
		break;
	    }

	    case INSERT : {
		DataObject obj = (DataObject) components[i];

		obj.uncreate();
		break;
	    }

	    case UPDATE : {
		// abort means invalidate modified attributes
		DataObject obj = (DataObject) components[i];

		obj.invalidateAttributes();
		break;
	    }

	    case DELETE : {
		// nothing to do
		break;
	    }

	    default :
		// can't happen
	    }

	    components[i] = null;
	    componentTypes[i] = NULL;
	}
    }

    protected void notifyComponents(boolean success) {
	for(int i = 0; i < componentCount; ++i) {
	    switch(componentTypes[i]) {
	    case SUBTRANS : {
		SchemaTransaction sub = (SchemaTransaction) components[i];

		sub.notifyComponents(success);
		break;
	    }

	    case INSERT : {
		DataObject obj = (DataObject) components[i];
		obj.finalizeInsert(success);
		break;
	    }

	    case UPDATE : {
		DataObject obj = (DataObject) components[i];
		obj.finalizeUpdate(success);
		break;
	    }

	    case DELETE : {
		DataObject obj = (DataObject) components[i];
		obj.finalizeDelete(success);
		break;
	    }

	    default :
		// can't happen
	    }

	    components[i] = null;
	    componentTypes[i] = NULL;
	}
    }

    public void commit() throws DataObjectException {
	DBConnection conn;

	try {
	    conn = DataObject.schema.getEnhydraDBConnection();
	    conn.setAutoCommit(false);

	    boolean success = false;

	    try {
		commit(conn);
		conn.commit();
		success = true;
	    } catch (SQLException e) {
		throw new DataObjectException(e);
	    } finally {
		conn.release();
		notifyComponents(success);
	    }
	} catch (SQLException e) {
	    throw new DataObjectException(e);
	} catch (DatabaseManagerException e) {
	    throw new DataObjectException(e);
	}
    }

    protected void commit(DBConnection conn) throws SQLException {
	for(int i = 0; i < componentCount; ++i) {
	    switch(componentTypes[i]) {
	    case SUBTRANS : {
		SchemaTransaction sub = (SchemaTransaction) components[i];
		sub.commit(conn);
		break;
	    }

	    case INSERT : {
		DataObject obj = (DataObject) components[i];
		obj.executeInsert(conn);
		break;
	    }

	    case UPDATE : {
		DataObject obj = (DataObject) components[i];
		obj.executeUpdate(conn);
		break;
	    }

	    case DELETE : {
		DataObject obj = (DataObject) components[i];
		obj.executeDelete(conn);
		break;
	    }

	    default :
		// can't happen
	    }
	}
    }
}
