/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * /root/test/poker/data/DODS_GameData/GameDataQuery.java
 *-----------------------------------------------------------------------------
 */

package poker.data.DODS_GameData;

import dods.builder.sourceGenerators.Query.*;

//import com.lutris.business.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.session.*;

import java.sql.*;
import java.util.*;
import java.util.Date;  // when I say Date, I don't mean java.sql.Date

/**
 * GameDataQuery is used to query the GameData table in the database.
 * It returns objects of type GameDataDO.
 *
 * General usage:
 *
 *     First, prepare the query:
 *
 *             DogQuery dq = new DogQuery();
 *             dq.setName( "Rex" );
 *             if ( there will never be another dog like Rex )
 *                 dq.requireUniqueInstance();
 *
 *     Then, get the query results one of two ways:
 *
 *         #1:
 *             DogDO[] dogs = dq.getDOArray();
 *             for ( int i = 0; i < dogs.length; i++ ) {
 *                 dog[i].feed();
 *             }
 *
 *      or #2:
 *             DogDO dog;
 *             while ( null != ( dog = dq.getNextDO() ) ) {
 *                 dog.walk();
 *             }
 *
 *     Note:   If requireUniqueInstance() was called,
 *             then getDOArray() or getNextDO()
 *             will throw an exception if more than one "Rex" was found.
 *
 *     Note:   Results of the query will come from the Data Object cache if:
 *             -  The cache is available.
 *             -  Matches were found in the cache.
 *             -  No other tables (Data Objects of other types) were involved
 *                in the query.
 *                This can happen if you extend the DogQuery class
 *                and you make calls to the QueryBuilder object
 *                to add SQL involving other tables.
 *             If any of these conditions is not true,
 *             then any results from the query will come from the database.
 *
 *     To reuse the query object, call:
 *
 *             dq.reset();
 *
 * @author <AUTHOR>
 * @version $Revision: 1.2 $
 */
final public class GameDataQuery implements Query {

    private QueryBuilder builder;

    /**
     * Public constructor.
     */
    public GameDataQuery() { 
	builder = new QueryBuilder( "GameData", "GameData.*" );
	builder.setDatabaseVendor( "Standard JDBC" );
	reset();
    }

    private ResultSet		resultSet	= null;
    private boolean 		uniqueInstance	= false;
    private GameDataDO[]	DOs		= null;
    private int			arrayIndex	= -1;
    private boolean		needToRun	= true;
    private Vector		cacheHits	= null;

    /**
     * Return the Querybuilder
     */
    public QueryBuilder getQueryBuilder() {
        return builder;
    }

    /**
     * Perform the query on the database, and prepare the 
     * array of returned DO objects.
     *
     * @exception DataObjectException If a database access error occurs. 
     * @exception NonUniqueQueryException If too many rows were found. 
     */
    private void runQuery()
    throws DataObjectException, NonUniqueQueryException {
	needToRun = false;
	arrayIndex = -1;
	DBQuery dbQuery = null; 

	try {
	    dbQuery = LBS.getDatabaseManager().createQuery();
	    dbQuery.query( this );    // invokes executeQuery
	    Vector results;
	    if ( null == resultSet ) {
		results = cacheHits;	 // executeQuery saw cache hits
	    } else {
	        results = new Vector();  // executeQuery queried the database
	        while ( resultSet.next() ) {
		    results.addElement( new GameDataDO ( resultSet ) );
	        }
	    }
	    if ( results.size() > 1 && uniqueInstance )
		throw new NonUniqueQueryException( 
		    "Too many rows returned from database" );
	    DOs = new GameDataDO [ results.size() ];
	    for ( int i = 0; i < results.size(); i++ ) {
		DOs[i] = ( GameDataDO ) results.elementAt( i );
	    }
	    arrayIndex = 0;
	} catch ( SQLException se ) {
	    if (	se.getSQLState().startsWith("02") &&
			se.getErrorCode() == 100 ) {
		throw new DataObjectException( 
		    "Update or delete DO is out of synch", se );
	    } else if (	se.getSQLState().equals("S1000") &&
			se.getErrorCode() == -268 ) {
		throw new DataObjectException( 
		    "Integrity constraint violation", se );
	    } else {
		throw new DataObjectException( "Data Object Error", se );
	    }
	} catch ( ObjectIdException oe ) { 
	    throw new DataObjectException( "Object ID Error", oe );
	} catch ( DatabaseManagerException de ) {
	    throw new DataObjectException( "Database connection Error", de );
	} finally {
	     dbQuery.release();	    
	}

    }


    /**
     * Return array of DOs constructed from ResultSet returned by query.
     * @exception DataObjectException If a database access error occurs. 
     * @exception NonUniqueQueryException If too many rows were found. 
     */
    public GameDataDO[] getDOArray()
    throws DataObjectException, NonUniqueQueryException {
	Thread.currentThread().dumpStack();
	if ( needToRun )
	    runQuery();
	return DOs;
    }

    /**
     * Return successive DOs from array built from ResultSet returned by query.
     * @exception DataObjectException If a database access error occurs. 
     * @exception NonUniqueQueryException If too many rows were found. 
     */
    public GameDataDO getNextDO()
    throws DataObjectException, NonUniqueQueryException {
	if ( needToRun )
	    runQuery();
	if ( null == DOs ) {
	    /* This should never happen.  
	     * runQuery() should either throw an exception
	     * or create an array of DOs, possibly of zero length.
	     */
	    return null;
	}
	if ( arrayIndex < DOs.length )
	    return DOs[ arrayIndex++ ];
	return null;
    }


    /**
     * Set the OID to query.
     * WARNING!  This method assumes that table <CODE>GameData</CODE>
     * has a column named <CODE>"oid"</CODE>.
     * This method is called from the DO classes to retrieve an object by id.
     *
     * @param oid The object id to query.
     */
    public void setQueryOId(ObjectId oid) {
	builder.addWhereClause( "oId",  oid.toBigDecimal(),
						QueryBuilder.NOT_NULL );
    }


    /**
     * Set "unique instance" assertion bit.
     * The first call to the next() method will throw an exception 
     * if more than one object was found.
     */
    public void requireUniqueInstance()
    {
	uniqueInstance = true;
    }

    /**
     * Reset the query parameters.
     */
    public void reset() {
	cacheHits	= new Vector();
	Enumeration cacheDOs = GameDataDO.getCacheContents();
	if ( null != cacheDOs ) 
	    while ( cacheDOs.hasMoreElements() )
		cacheHits.addElement( cacheDOs.nextElement() );
	DOs		= null;
	uniqueInstance	= false;
	needToRun	= true;
        builder.reset();
    }

    /**
     * Return the appropriate QueryBuilder flag for selecting
     * exact matches (SQL '=') or inexact matches (SQL 'matches').
     */
    private boolean exactFlag( boolean exact ) {
        return exact ? QueryBuilder.EXACT_MATCH : QueryBuilder.NOT_EXACT;
    }



    //
    // Implementation of Query interface
    //

    /**
     * Method to query objects from the database.
     * The following call in runQuery()
     *	    dbQuery.query( this );
     * causes the dbQuery object to invoke 
     *      executeQuery()
     *
     * @param conn Handle to database connection.
     * @exception java.sql.SQLException If a database access error occurs.
     */
    public ResultSet executeQuery(DBConnection conn)
        throws SQLException
    {
	if ( cacheHits.size() > 0 ) {
	    // The setQuery methods build up the cacheHits.
	    // If the cache had our desired objects, 
	    // we don't query the database, so we have no ResultSet.
	    resultSet = null; 
        } else {
	    // If the cache doesn't exist, or could not handle the query,
	    // then we actually query the database.
	    resultSet = builder.executeQuery( conn );
	}
        return resultSet;
    }
 
 
    /**
     * WARNING!  This method is disabled.
     * It's implementation is forced by the Query interface.
     * This method is disabled for 2 reasons:
     * 1)  the getDOArray() and getNextDO() methods are better
     *     because they return DOs instead of JDBC objects.
     * 2)  the createExisting() method throws an exception
     *     that we cannot reasonable handle here,
     *     and that we cannot throw from here.
     *
     * @param rs JDBC result set from which the next object
     *   will be instantiated.
     * @exception java.sql.SQLException
     *   If a database access error occurs.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an invalid object id was queried from the database.
     */
    public Object next(ResultSet rs)
    throws SQLException, ObjectIdException {
	/*
        if (!rs.next())
            return null;
 
        return GameDataDO.createExisting(rs);
	*/
	throw new ObjectIdException( 
	    "next() should not be used.  Use getNextDO() instead." );
	//return null;
    }


    /**
     * Set the Name to query.
     *
     * @param x The Name of the GameData to query.
     * @param exact to use matches or not
     */
    public void setQueryName(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size(); i++ ) {
	    GameDataDO DO = ( GameDataDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String s = DO.getName();
		if ( null == s && null == x ) {
		    equals = true;
		} else if ( null != s && null != x ) {
		    if ( exact ) 
			equals = s.equals( x );
		    else {
			equals = ( -1 != s.toLowerCase().indexOf(
					 x.toLowerCase() ) );
		    }
		} else {  // one is null, the other isn't
		    equals = false;
		}
	    
	    /*
		BigDecimal bd = DO.getName();
		if ( null == bd && null == x ) {
		    equals = true;
		} else if ( null != bd && null != x ) {
		    equals = bd.equals( x );
		} else {  // one is null, the other isn't
		    equals = false;
		}
	    */
	    /*
		Date d = (Date) DO.getName();
		if ( null == d && null == x ) {
		    equals = true;
		} else if ( null != d && null != x ) {
		    equals = d.equals( x );
		} else {  // one is null, the other isn't
		    equals = false;
		}
	    */
	    /*
		byte[] ba = DO.getName();
		if ( ! (	null == ba && null == x ) ) {
		    equals = false;
		} else if ( ba.length != x.length ) {
		    equals = false;
		} else {
		    for ( int j = 0; j < x.length; j++ ) {
			if ( ba[j] != x[j] ) {
			    equals = false;
			    break;
			}
		    }
		}
	    */
	    /*
		// primitive types are compared using the == operator.
		equals = ( DO.getName() == x );
	    */
	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	builder.addWhereClause( "Name", x, "VARCHAR",
                QueryBuilder.NOT_NULL, exactFlag( exact ) );
    }

    /**
     * Set the Name to query
     * @param x The Name of the GameData to query.
     */
    public void setQueryName( 
				String x )
    throws DataObjectException {
	setQueryName( x, true );
    }

}
