package com.lutris.xml.xmlc.validate;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import com.lutris.xml.xmlc.html.*;


/**
 *  This class validates HTML for use in a project with XMLC.
 *  In one mode it examines compiled HTML objects and creates a script.
 *  In the other mode it reads the script and ensures that the right tags exist.
 *  This is usefull after the HTML has been edited: if you are about to hand
 *  off the HTML to a designer, you can run the validator to generate a script,
 *  then when you get the HTML back you can check the HTML against the script
 *  to verify that the designer did not accidentally delete any tags.
 *  <p>
 *  Usage:
 *  <pre>
 *      Validate gen <i>class</i> [<i>class class class</i>....]
 *      Validate <i>scriptname</i>
 *  </pre><P>
 *
 *  The script is read one line at a time. Each line is a command, directing
 *  the validator to take some action. The syntax of the commands is as follows:
 *  <pre>
 *  &nbsp;&nbsp;&nbsp;&nbsp;<b>page</b> <i>classname</i> 
 *  &nbsp;&nbsp;&nbsp;&nbsp;<b>id</b> <i>tag-type</i>|*  
 *  &nbsp;&nbsp;&nbsp;&nbsp;<b>class</b> <i>class</i> n|<i>tag-type</i>|*
 *  &nbsp;&nbsp;&nbsp;&nbsp;<b>in id</b> <i>id</i>|*
 *  &nbsp;&nbsp;&nbsp;&nbsp;<b>in class</b> <i>class</i>|*
 *  &nbsp;&nbsp;&nbsp;&nbsp;<b>only ids</b> <i>id id id...</i>
 *  &nbsp;&nbsp;&nbsp;&nbsp;<b>only classes</b> <i>class class class...</i>
 *  &nbsp;&nbsp;&nbsp;&nbsp;<b>#</b> comment
 *  </pre><p>
 *  The command "page" switches to a new page. Use a full classname. 
 *  The page must have already been compiled with XLMC (in the future
 *  the ability to read files will be supported).<p>
 *
 *  The command "id" requres that there be a tag with the given id and type. 
 *  If the type is "*" then any type is allowed. <p>
 * 
 *  The command "class" takes a class name and either an integer or a
 *  tag type. If an integer <i>N</i> is specified, then it requires that
 *  there be <i>N</i> tags with the given class name. 
 *  If <i>N</i> is "*", then there must be one or more. 
 *  If a tag type is specified, then there must me exactly one tag with
 *  the given class, and it's type must match (e.g. "span").<p>
 *
 *  The command "in id" restricts the search scope of the "id" and "class" 
 *  commands to the tag with the given id. The current search scope is used
 *  to find the tag with the given id, so the search scope is always
 *  reduced to a subset of the current scope, unless id is "*".
 *  If id is "*" then the search scope is reset to the whole document. <p> 
 *
 *  The command "in class" restricts the search scope of the "id" and "class"
 *  commands to the first tag found with the given class. The current search
 *  scope is used to find the tag with the given class, so the search scope
 *  is always reduced to a subset of the current scope, unless class is "*".
 *  If class is "*" then the search scope is reset to the whole document. <p> 
 *
 *  The command "only ids" requires that at the current scope there must
 *  exist only the specified id names. If any other ids exist it will cause
 *  an error and their id names will be listed. <p>
 * 
 *  The command "only classes" requires that at the current scope there must
 *  exist only the specified class names. If any other classes exist it will
 *  cause an error and their class names will be listed. <p>
 *  <p>
 *  
 *  The recommended usage for the validator is to add the following
 *  lines to your makefile in the directory where your compiled HTML
 *  resides (where the makefile that lists all the html classes is):
 *  <pre>
 *  generate:
 *          &amp;[ ! -f validate.script ] || echo "validate.script exists."
 *          &amp; ! -f validate.script ] 
 *          &amp;CLASSPATH=$(CLASSPATH) ; PATH=$(PATH) ; export CLASSPATH PATH  ; \
 *          set -x ; \
 *          ($(JAVA) com.lutris.xml.xmlc.validate.Validate gen $(HTML_CLASSES:%=$(PACKAGE).%) > validate.script; [ -f validate.extra ] && cat validate.extra >> validate.script; true) || exit 1
 *  <br>
 *  validate:
 *          &amp; -f validate.script ] || echo "no file validate.script";
 *          &amp; -f validate.script ] 
 *          &amp;CLASSPATH=$(CLASSPATH) ; PATH=$(PATH) ; export CLASSPATH PATH  ; \
 *          set -x ; \
 *          $(JAVA) com.lutris.xml.xmlc.validate.Validate validate.script
 *  </pre>
 *  This will add two rules to your makefile. When your HTML is at an
 *  appropriate state, for example right before you hand it out to someone
 *  else to edit, type "<code>make generate</code>" (remove the file
 *  validate.script, if it exists, first). This will generate a file
 *  validate.script which will be used later to validate all the compiled HTML
 *  in the current directory. <p>
 *
 *  When you wish to validate the HTML, for example after you get it back
 *  from someone else, you must first put it in the tree and compile it
 *  (do a normal "make"). After it all compiles, then type
 *  "<code>make validate</code>" and the validator will process the script
 *  validate.script and check the HTML. <p>
 *
 *  The tags in your HTML will probably have additional contstraints that
 *  the validator program does not know about. In this situation you use
 *  the validate.extra file. It is appened onto the end of the resulting
 *  validate.script file whenever it is generated. You can add your own
 *  additional checks here and they will not be overwritten when you
 *  generate a new validate.script file. The validate.script file may
 *  be edited also, but then you must be carefull to save it under a new
 *  name.
 *  
 *   
 *  @author Andy John
 */
public class Validate {

    // Do not instantiate - use main().
    private Validate() {
    }

    /**
     *  There are two modes: if you pass in one arg it is assumed to
     *  be the filename of the script to read. Otherwise a script is
     *  generated and written to stdout. The first argument is assumed
     *  to be "gen" and is skipped. The rest of the arguments are the
     *  class names of the pages to examine.
     *  
     *  @param args the initial arguments. 
     *  @exception Exception If an error occurs.
     */
    public static void main(String[] args) throws Exception {
        if (args.length == 0) {
            System.out.println("Usage: Validate filename");
            System.out.println("       Validate gen class [class] [class]...");
            return;
        } else if (args.length == 1) {
            runScript(args[0]);
        } else {
            for (int i=1; i<args.length; i++)
                parseHtmlObject(args[i]);
        }
    }
     

    /*
     *  Read a script and process the commands one at a time.
     *  The results are printed to System.out.
     */       
    private static void runScript(String filename) throws Exception {
        // Setup initial context.
        HTMLObject currentDoc = null;
        String currentDocName = null;
        Node currentNode = null;
        String currentScope = ""; // Human readable description of scope.
        int errors = 0;  
        boolean skipInSection = false; // Should we skip commands for this file?

        // Open the script.
        FileInputStream fin = new FileInputStream(filename);
        InputStreamReader isr = new InputStreamReader(fin);
        LineNumberReader lnr = new LineNumberReader(isr); 

        // Process the commands one at a time.
        while (true) {
            try {
                String cmd = lnr.readLine();
                if (cmd == null) 
                    break; // EOF
                else if (cmd.startsWith("page ")) {
                    /*
                     *  Select a new compiled HTML page class.
                     */
                    String classname = cmd.substring(5);
                    currentDocName = classname;
                    Class c = Class.forName(classname);
                    currentDoc = (HTMLObject) c.newInstance();
                    currentNode = currentDoc;  
                    currentScope = "";
                    skipInSection = false;
                    System.out.println("Processing class " + classname);
                } else if (cmd.startsWith("in id *")) {
                    /*
                     *  Reset the current node to the top of the tree.
                     */
                    currentNode = currentDoc;
                    currentScope = "";
                    skipInSection = false;
                } else if (cmd.startsWith("in class *")) {
                    /*
                     *  Reset the current node to the top of the tree.
                     */
                    currentNode = currentDoc;
                    currentScope = "";
                    skipInSection = false;
                } else if (skipInSection)
                    /*
                     *  There was an error setting the search scope,
                     *  so skip commands until the next file, or the search
                     *  scope is reset.
                     */
                    continue;
                else if (cmd.startsWith("#")) 
                    /*
                     *  Skip comments.
                     */
                    continue;
                else if (cmd.equals("")) 
                    /*
                     *  Skip blank lines.
                     */
                    continue;
                else if (cmd.startsWith("id ")) {
                    /*
                     *  Check that an id exists.
                     */
                    String cmd2 = cmd.substring(3);
                    int i = cmd2.indexOf(" ");
                    String id = cmd2.substring(0, i);
                    String type = cmd2.substring(i+1);
                    Node target = findNodeById(currentNode, id);
                    if (target == null) {
                        System.out.print("*** <" + type + " id=" + id + 
                                         " ...> not found.");
                        if (currentScope.length() > 0) 
                            System.out.print("Searching" + currentScope + ". ");
                        System.out.println("Line " + lnr.getLineNumber() + ". ");
                        errors++;
                        continue;
                    }
                    if (!(target instanceof Element)) {
                        System.out.println("Internal Error: id " + id + 
                                           " is not an Element. Line " +
                                           lnr.getLineNumber() + ". Halting.");
                        errors++;
                        break;
                    }
                    Element e = (Element) target;
                    if (!type.equals("*")) { 
                        /*
                         *  Ok, the tag exists, but does is it of the right
                         *  type?
                         */
                        if (!type.equalsIgnoreCase(e.getTagName())) {
                            System.out.print("*** Expected <" + type +
                                             " id=" + id + " ...> but found <" + 
                                             e.getTagName() + " id=" +
                                             id + " ...>. ");
                            if (currentScope.length() > 0) 
                                System.out.print("Searching" +
                                                 currentScope + ". ");
                            System.out.println("Line " +
                                               lnr.getLineNumber() + ". ");
                            errors++;
                            continue;
                        }
                    }
                } else if (cmd.startsWith("class ")) {
                    /*
                     *  Check that a class exists.
                     */
                    String cmd2 = cmd.substring(6);
                    int i = cmd2.indexOf(" ");
                    String cl = cmd2.substring(0, i);
                    String ns = cmd2.substring(i+1);
                    int n = -1;
                    boolean checkType = false;
                    if (!ns.equals("*")) {
                        try {
                            n = Integer.parseInt(ns);
                        } catch (NumberFormatException nfe) {
                            checkType = true;
                        }
                    } 
                    int nf = countNodesByClass(currentNode, cl);
                    if (!checkType && (n == -1) && (nf == 0)) {
                        System.out.print("*** Expected one or more " + 
                                         " <* class=" + cl + 
                                         " ...> but found none. ");
                        if (currentScope.length() > 0) 
                            System.out.print("Searching" + currentScope + ". ");
                        System.out.println("Line " + lnr.getLineNumber() + ".");
                        errors++;
                        continue;
                    }
                    if (!checkType && (n != -1) && (nf != n)) {
                        System.out.print("*** Expected " + n +
                                         " <* class=" + cl + 
                                         " ...> but found " + nf + ". ");
                        if (currentScope.length() > 0) 
                            System.out.print("Searching" + currentScope + ". ");
                        System.out.println("Line " + lnr.getLineNumber() + ".");
                        errors++;
                        continue;
                    }
                    if (checkType) {
                        /*
                         *  The class exists, but now check that the tag type
                         *  is what was expected.
                         */
                        String type = ns;
                        if (nf != 1) {
                            System.out.print("*** Expected one <" + type +  
                                             " class=" + cl + 
                                             " ...> but found " + nf + ". ");
                            if (currentScope.length() > 0) 
                                System.out.print("Searching" +
                                                 currentScope + ". ");
                            System.out.println("Line " +
                                               lnr.getLineNumber() + ".");
                            errors++;
                            continue;
                        }
                        Node target = findNodeByClass(currentNode, cl);
                        if (!(target instanceof Element)) {
                            System.out.println("Internal Error: class " + cl + 
                                               " is not an Element. Line " +
                                               lnr.getLineNumber() + ". Halting.");
                            errors++;
                            break;
                        }
                        Element e = (Element) target;
                        if (!type.equalsIgnoreCase(e.getTagName())) {
                            System.out.print("*** Expected <" + type +  
                                             " class=" + cl + 
                                             " ...> but found <" +
                                             e.getTagName() + " class=" +
                                             cl + "...>. ");
                            if (currentScope.length() > 0) 
                                System.out.print("Searching" +
                                                 currentScope + ". ");
                            System.out.println("Line " +
                                               lnr.getLineNumber() + ".");
                            errors++;
                            continue;
                        }
                    }
                } else if (cmd.startsWith("in id ")) {
                    /*
                     *  Reduce the search scope to a subset of the current
                     *  scope, by finding a child with the given id.
                     */
                    String id = cmd.substring(6);
                    currentNode = findNodeById(currentNode, id);
                    if (currentNode == null) {
                        System.out.print("in id: *** <* id=" + id +
                                         " ...> not found");
                        if (findNodeById(currentDoc, id) != null)
                            System.out.print(" in current context");
                        if (currentScope.length() > 0) 
                            System.out.print(". Searching" + currentScope);
                        System.out.println(". Line " + lnr.getLineNumber() + 
                                           ". Skipping lines...");
                        errors++;
                        skipInSection = true;
                        continue;
                    }
                    currentScope = " within <* id=" + id + " ...>" + currentScope;
                } else if (cmd.startsWith("in class ")) {
                    /*
                     *  Reduce the search scope to a subset of the current
                     *  scope, by finding a child with the given class.
                     */
                    String cl = cmd.substring(9);
                    currentNode = findNodeByClass(currentNode, cl);
                    if (currentNode == null) {
                        System.out.print("*** in class: <* class=" + cl +
                                         " ...> not found");
                        if (findNodeByClass(currentDoc, cl) != null)
                            System.out.print(" in current context");
                        if (currentScope.length() > 0) 
                            System.out.print(". Searching" + currentScope);
                        System.out.println(". Line " + lnr.getLineNumber() + 
                                           ". Skipping lines...");
                        errors++;
                        skipInSection = true;
                        continue;
                    }
                    currentScope = " within <* class=" + cl + " ...>" +
                                   currentScope;
                } else if (cmd.startsWith("only ids")) {
                    /*
                     *  Check that there are no other ids in the current
                     *  search scope other than those listed.
                     */
                    String ids = cmd.substring(8);
                    Hashtable allowed = new Hashtable();
                    Hashtable found = allIdNames(currentNode);
                    Hashtable extra = new Hashtable();
                    // What are the allowed ids?
                    StringTokenizer stok = new StringTokenizer(ids, " ");
                    while (stok.hasMoreTokens()) {
                        String id = stok.nextToken();
                        if (!allowed.containsKey(id))
                                allowed.put(id, allowed);
                    }
                    // Is each id in the page allowed?
                    Enumeration e = found.keys();
                    while (e.hasMoreElements()) {
                        String id = (String) e.nextElement(); 
                        if (!allowed.containsKey(id) && !extra.containsKey(id))
                            extra.put(id, extra);
                    }
                    // Where there any extra ids found? 
                    e = extra.keys();
                    if (e.hasMoreElements()) {
                        System.out.print("*** only ids: found unexpected ids: ");
                        while (e.hasMoreElements()) { 
                            String id = (String) e.nextElement(); 
                            System.out.print(id);
                            if (e.hasMoreElements())
                                System.out.print(" ");
                        }
                        System.out.print(". ");
                        if (currentScope.length() > 0) 
                            System.out.print("Searching" + currentScope + ". ");
                        System.out.println("Line " + lnr.getLineNumber() + ".");
                        errors++;
                    }
                } else if (cmd.startsWith("only classes")) {
                    /*
                     *  Check that there are no other classes in the current
                     *  search scope other than those listed.
                     */
                    String classes = cmd.substring(12);
                    Hashtable allowed = new Hashtable();
                    Hashtable found = allClassNames(currentNode);
                    Hashtable extra = new Hashtable();
                    // What are the allowed classes?
                    StringTokenizer stok = new StringTokenizer(classes, " ");
                    while (stok.hasMoreTokens()) {
                        String cl = stok.nextToken();
                        if (!allowed.containsKey(cl))
                                allowed.put(cl, allowed);
                    }
                    // Is each class in the page allowed?
                    Enumeration e = found.keys();
                    while (e.hasMoreElements()) {
                        String cl = (String) e.nextElement(); 
                        if (!allowed.containsKey(cl) && !extra.containsKey(cl))
                            extra.put(cl, extra);
                    }
                    // Where there any extra classes found? 
                    e = extra.keys();
                    if (e.hasMoreElements()) {
                        System.out.print("*** only classes: found unexpected classes: ");
                        while (e.hasMoreElements()) { 
                            String cl = (String) e.nextElement(); 
                            System.out.print(cl);
                            if (e.hasMoreElements())
                                System.out.print(" ");
                        }
                        System.out.print(". ");
                        if (currentScope.length() > 0) 
                            System.out.print("Searching" + currentScope + ". ");
                        System.out.println("Line " + lnr.getLineNumber() + ".");
                        errors++;
                    }
                } else {
                    /*
                     *  Since it's not one of the above, it must be an 
                     *  illegal command.
                     */
                    System.out.println("*** Illegal cmd: \"" + cmd + 
                                       "\" (line " + lnr.getLineNumber() + 
                                       ").");
                    errors++;
                    continue;
                }
            } catch (Throwable t) {
                System.out.println("Exception thrown (line " +
                                   lnr.getLineNumber() + "): " + t);
                errors++;
                break;
            }
        }
 
        /*
         *  All done! Report how many problems were found.
         */
        System.out.println(errors + " errors detected.");
        lnr.close();
        isr.close();
        fin.close();
    }


    private static Node findNodeById(Node root, String key) {
        return findNodeById(root, key, false);
    }

    private static Node findNodeById(Node root, String key, boolean rootOK) {
        // Is this it?
        if (rootOK && (root instanceof Element)) {
            Element e = (Element) root;
            String id = e.getAttribute("id");
            if ((id != null) && id.equals(key)) 
                return root;
        } 
        // Search kids
        NodeList nl = root.getChildNodes();
        int n = nl.getLength();
        for (int i=0; i<n; i++) {
            Node result = findNodeById(nl.item(i), key, true);
            if (result != null)
                return result;
        }
        // Not found.
        return null;
    }

    private static Node findNodeByClass(Node root, String key) {
        return findNodeByClass(root, key, false);
    }

    private static Node findNodeByClass(Node root, String key, boolean rootOK) {
        // Is this it?
        if (rootOK && (root instanceof Element)) {
            Element e = (Element) root;
            String cl = e.getAttribute("class");
            if ((cl != null) && cl.equals(key)) 
                return root;
        } 
        // Search kids
        NodeList nl = root.getChildNodes();
        int n = nl.getLength();
        for (int i=0; i<n; i++) {
            Node result = findNodeByClass(nl.item(i), key, true);
            if (result != null)
                return result;
        }
        // Not found.
        return null;
    }

    private static int countNodesByClass(Node root, String key) {
        return countNodesByClass(root, key, true);
    }
    private static int countNodesByClass(Node root,
                                         String key,
                                         boolean rootOK) {
        int result = 0;
        // Is this one?
        if (rootOK && (root instanceof Element)) {
            Element e = (Element) root;
            String cl = e.getAttribute("class");
            if ((cl != null) && cl.equals(key))
                result++;
        }
        // Search kids
        NodeList nl = root.getChildNodes();
        int n = nl.getLength();
        for (int i=0; i<n; i++) 
            result += countNodesByClass(nl.item(i), key, true);
        // Done
        return result;
    }








    /*
     *  Examine a compiled HTML object, and generate a validation script
     *  based on the ids and classes found. The results are writtent to
     *  System.out.
     */
    private static void parseHtmlObject(String classname) throws Exception {
        Class c = Class.forName(classname);
        HTMLObject ho = (HTMLObject) c.newInstance();
        System.out.println("#");
        System.out.println("# ---------------------------------------------");
        System.out.println("#");
        System.out.println("page " + classname);
        System.out.println("#");
        System.out.println("#    Global id list");
        System.out.println("#");
        dumpIds(ho);
        System.out.println("#");
        System.out.println("#    Global class list");
        System.out.println("#");
        dumpClasses(ho);
        System.out.println("#");
        System.out.println("#    Classes per containing id");
        System.out.println("#");
        dumpTemplates(ho);
        System.out.println("#");
        System.out.println("#    All id names");
        System.out.println("#");
        dumpAllIds(ho);
        System.out.println("#");
        System.out.println("#    All class names");
        System.out.println("#");
        dumpAllClasses(ho);
    }

     
    private static void dumpIds(Node root) {
        if (root instanceof Element) {
            Element e = (Element) root;
            String type = e.getTagName();
            String id = e.getAttribute("id");
            if ((id != null) && (id.length() > 0))
                System.out.println("id " + id + " " + type.toLowerCase());
        }
        // Search kids
        NodeList nl = root.getChildNodes();
        int n = nl.getLength();
        for (int i=0; i<n; i++) 
            dumpIds(nl.item(i));
    }


    private static void dumpClasses(Node root) {
        Enumeration e = findAllClassNames(root);
        while (e.hasMoreElements()) {
            String cl = (String) e.nextElement();
            int n = countNodesByClass(root, cl);
            if (n == 1) {
                Element el = (Element) findNodeByClass(root, cl);
                System.out.println("class " + cl + " " +
                                   el.getTagName().toLowerCase());
            } else
                System.out.println("class " + cl + " " + n);
        }
    }

    private static void dumpTemplates(Node root) {
        Enumeration e = findAllClassElements(root);
        Hashtable idContainers = new Hashtable();
        while (e.hasMoreElements()) {
            Element el = (Element) e.nextElement();
            Element id = findParentId(el);
            if ((id != null) && !idContainers.containsKey(id))
                idContainers.put(id, idContainers);
        }
        e = idContainers.keys();
        boolean empty = !e.hasMoreElements();
        while (e.hasMoreElements()) {
            Element el = (Element) e.nextElement();
            System.out.println("in id *");
            System.out.println("in id " + el.getAttribute("id"));
            dumpClasses(el);
        }
        if (!empty)
            System.out.println("in id *");
    }


    private static void dumpAllIds(Node root) {
        Hashtable ids = allIdNames(root);
        System.out.println("in id *");
        System.out.print("only ids");
        Enumeration e = ids.keys();
        while (e.hasMoreElements()) {
            String name = (String) e.nextElement();
            System.out.print(" " + name);
        }
        System.out.println();
    }


    private static void dumpAllClasses(Node root) {
        Hashtable classes = allClassNames(root);
        System.out.println("in class *");
        System.out.print("only classes");
        Enumeration e = classes.keys();
        while (e.hasMoreElements()) {
            String name = (String) e.nextElement();
            System.out.print(" " + name);
        }
        System.out.println();
    }


    private static Enumeration findAllClassElements(Node root) {
        Hashtable results = new Hashtable();
        findAllClassElements(root, results);
        return results.keys();
    }
    private static void findAllClassElements(Node root, Hashtable results) {
        if (root instanceof Element) {
            Element e = (Element) root;
            String cl = e.getAttribute("class");
            if ((cl != null) && (cl.length() > 0))
                results.put(e, results);
        }
        // Search kids
        NodeList nl = root.getChildNodes();
        int n = nl.getLength();
        for (int i=0; i<n; i++) 
            findAllClassElements(nl.item(i), results);
    }


    /*
     *  Returns an Enumeration of all the class names found at or below
     *  a given Node.
     */
    private static Enumeration findAllClassNames(Node root) {
        Hashtable results = new Hashtable();
        Enumeration e = findAllClassElements(root);
        while (e.hasMoreElements()) {
            Element el = (Element) e.nextElement();
            String cl = el.getAttribute("class");
            if (!results.containsKey(cl))
                results.put(cl, results);
        }
        return results.keys();
    }


    /*
     *  Find the first parent of the given node that has an ID attribute
     *  assigned to it, or null if none is found.
     */
    private static Element findParentId(Node node) {
        if (node == null)
            return null;
        if (node instanceof Element) {
            Element e = (Element) node;
            String id = e.getAttribute("id");
            if ((id != null) && (id.length() > 0))
                return e;
        }
        return findParentId(node.getParentNode());
    }



    /*
     *  Returns a hashtable of all the ID names assigned to the given node
     *  and all it's children. Use the key names of the resulting Hashtable.
     */
    private static Hashtable allIdNames(Node node) {
        Hashtable results = new Hashtable();
        allAttributeNames(node, "id", results, false);
        return results;
    }
    /*
     *  Returns a hashtable of all the class names assigned to the given node
     *  and all it's children. Use the key names of the resulting Hashtable.
     */
    private static Hashtable allClassNames(Node node) {
        Hashtable results = new Hashtable();
        allAttributeNames(node, "class", results, false);
        return results;
    }
    private static void allAttributeNames(Node root,
                                          String attribute,
                                          Hashtable soFar, 
                                          boolean rootOk) {
        if (rootOk && (root instanceof Element)) {
            Element e = (Element) root;
            String attr = e.getAttribute(attribute);
            if ((attr != null) &&
                (attr.length() > 0) &&
                !soFar.containsKey(attr)) 
                soFar.put(attr, soFar);
        }
        // Search kids
        NodeList nl = root.getChildNodes();
        int n = nl.getLength();
        for (int i=0; i<n; i++) 
            allAttributeNames(nl.item(i), attribute, soFar, true);
    }




}


