/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DOgraphNode.java,v 1.2 1999/06/08 01:06:48 jay Exp $
 *-----------------------------------------------------------------------------
 */
 
/*
 *   DOgraphNode.java
 *
 * @author    Jay Gunter
 * @since     LBS1.8
 * @version   $Revision: 1.2 $
 *
 */

package dods.builder.engine;

import dods.data.*;

import  java.io.File;
import  java.util.Vector;
import  java.util.Hashtable;
import  java.util.Enumeration;


/**
 * This class is used by the Builder to generate two lists:
 * 1) a list of generated .sql files in a valid execution order.
 * 2) a list of "drop table" SQL commands in a valid execution order.
 * The 2nd list is in the reverse order of the 1st list.
 * The order is determined by the inter-tables references (foreign keys).
 * The tables with no references are created first and dropped last.
 */
class DOgraphNode {
    static private Hashtable set;
    static private Vector sqls;
    static private Vector tables;

    static public void reset() {
	set = new Hashtable();
	sqls = null;
	tables = null;
    }
    static {
	reset();
    }
    static public void addNode( DataObject DO ) {
	DOgraphNode node = new DOgraphNode( DO );
	set.put( DO, node );
    }
    static public String getCreateOrder() {
	String ret = "";
	process();
	for ( int i = 0; i < sqls.size(); i++ ) {
	    ret += (String) sqls.elementAt( i ) + "\n";
	}
	return ret;
    }
    static public String[] getOrderedFilenames() {
	process();
	String[] ret = new String[ sqls.size() ];
	for ( int i = 0; i < sqls.size(); i++ ) {
	    ret[ i ] = (String) sqls.elementAt( i );
	}
	return ret;
    }
    static public String getDropCommands() {
	String ret = "";
	process();
	for ( int i =  tables.size() - 1; 0 <= i; i-- ) {
	    String table = (String) tables.elementAt( i );
	    ret += "drop table " + table + ";\n";
	}
	return ret;
    }

    static private void process() {
	if ( null != sqls ) 
	    return;
	sqls = new Vector();
	tables = new Vector();
	fillRefs();
	Enumeration DOs;
	for ( DOs = set.keys(); ! set.isEmpty(); DOs = set.keys() ) {
//System.err.println("========================================");
	    while ( DOs.hasMoreElements() ) {
		DataObject DO = (DataObject) DOs.nextElement();
		DOgraphNode node = (DOgraphNode) set.get( DO );
//System.err.println("Looking at node ="+node );
		if ( node.noRefs() ) {
		    String outDir = Builder.fixOutputDir( DO );
		    String pkg = DO.getAdjustedPackageName();
		    String subDir = pkg.replace( '.', File.separatorChar );
		    sqls.addElement( outDir + subDir + File.separatorChar + 
						DO.getName() + "SQL.sql" );
		    tables.addElement( DO.getDBTableName() );
		    zapRef( node );
		    set.remove( DO );
		}
	    }
	}
    }
    static private void zapRef( DOgraphNode ref ) {
//System.err.println("zapping ref ="+ref );
	Enumeration nodes = set.elements();
	while ( nodes.hasMoreElements() ) {
	    DOgraphNode node = (DOgraphNode) nodes.nextElement();
	    node.removeRef( ref );
	}
    }
    static private void fillRefs() {
	Enumeration DOs = set.keys();
	while ( DOs.hasMoreElements() ) {
	    DataObject DO = (DataObject) DOs.nextElement();
	    DOgraphNode node = (DOgraphNode) set.get( DO );
	    Vector attrs = DO.getAttributes();
	    for ( int i = 0; i < attrs.size(); i++ ) {
		Attribute attr = (Attribute) attrs.elementAt( i );
		if ( attr.isObjectRef() ) {
		    DataObject refDO    = attr.getObjectRefDO();
		    DOgraphNode refNode = (DOgraphNode) set.get( refDO );
		    node.addRef( refNode );
		}
	    }
	}
    }

    DataObject DO;
    Hashtable references;
    private DOgraphNode( DataObject o ) {
	DO = o;
	references = new Hashtable();
    }
    private boolean noRefs() {
	return references.isEmpty();
    }
    private void addRef( DOgraphNode ref ) {
//System.err.println("Node " + toString() + ": adding ref to " + ref.toString() );
	if ( this != ref )
	    references.put( ref, "" );
    }
    private void removeRef( DOgraphNode ref ) {
//if ( references.containsKey( ref ) )
//System.err.println("Node " + toString() + ": removing ref to " + ref.toString() );
	references.remove( ref );
    }
    public String toString() {
	return DO.getName() + "/" + DO.getDBTableName();
    }
}
