/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: GenericDO.java,v 1.12 1999/06/17 23:45:29 jay Exp $
 *-----------------------------------------------------------------------------
 */



/*
 * GenericDO.java
 *
 * This class is the parent of all the generated DO classes produced 
 * by the SourceGenerator_DO.  The template file
 *           SourceGenerator_DO/templates/writeFileHeader.template
 * contains the line
 *           class <CLASS_NAME> <EXTENDS> {
 * and the EXTENDS tag is replaced by either
 *           extends GenericDO
 * or, if the DataObject is derived from another DataOBject, the replacement is
 *           extends ThatOtherDO
 */

package dods.builder.sourceGenerators.DO;

import com.lutris.appserver.server.Enhydra;
import com.lutris.appserver.server.sql.*;
import com.lutris.logging.Logger;
import java.sql.*;
import java.math.BigDecimal;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;

abstract public class GenericDO extends CloneableDO {

  // True if this object has been changed with respect to
  // its persistent representation in the database.
  protected boolean dirty = false;
  private boolean updateInProgress = false;

  /**
   * Public constructor.
   */
  public GenericDO() throws ObjectIdException, DatabaseManagerException {
    super();
    //System.err.println("GenericDO ctor...");
    //Thread.currentThread().dumpStack();
    setOId(Enhydra.getDatabaseManager().allocateObjectId());
    //System.err.println("GenericDO ctor past setOId");
    markNewValue();
    //System.err.println("GenericDO ctor past markNewValue");
  }

  /**
   * Public constructor.
   * @param rs a result set.
   * @exception SQLException if an error occurs while instantiating this
   * object from the result set.
   * @exception ObjectIdException if the object id for this object is
   * invalid.
   */
  public GenericDO(ResultSet rs)
  throws SQLException, ObjectIdException {
    super(rs);
    setNewVersion(getVersion());
  }

  /**
   * Creates a clone of the object, but ensures that
   * a new and unique object id is created for the object
   * and that the version number is set to zero.
   * @exception DatabaseManagerException if an error occurs while
   * allocation a new object id from the default logical database.
   * @exception ObjectIdException if a new object id could not be
   * allocated.
   */
  public synchronized Object cloneUnique()
    throws DatabaseManagerException, ObjectIdException {
      GenericDO dataObj = (GenericDO)super.cloneUnique();
      // It's important to do the following.  Otherwise two
      // commits in a row on a cloned object will fail.
      dataObj.setNewVersion(0);
      dataObj.markNewValue(); // make sure it gets commited.
      return dataObj;
    }

  /**
   * Updates the contents of this object in the database
   * but only if the datab object is dirty.  This requires
   * that all set methods in the data objects keep track
   * if a value has changed.
   *
   * @param conn the database connection.
   * @exception java.sql.SQLException If a database access error
   *  occurs.
   */
  public synchronized void executeUpdate(DBConnection conn)
    throws SQLException {
      if (dirty && !updateInProgress) {
        Enhydra.getApplication().getLogChannel().
        write(Logger.DEBUG2,
              getClass().getName() + ".executeUpdate:" + persistent + ": "
              + getClass().getName() + ": "
              + getOId().toString()
              + ": " + getVersion());
        super.executeUpdate(conn);
        updateInProgress = true;

      }
    }

  /**
   * Updates the contents of this object in the database
   * but only if the datab object is dirty.  This requires
   * that all set methods in the data objects keep track
   * if a value has changed.
   *
   * @param conn the database connection.
   * @exception java.sql.SQLException If a database access error
   *  occurs.
   */
  public synchronized void executeInsert(DBConnection conn)
    throws SQLException {
      if (!updateInProgress) {
        Enhydra.getApplication().getLogChannel().
        write(Logger.DEBUG2,
              getClass().getName() + ".executeInsert():" + persistent + ": "
              + getOId().toString()
              + ": " + getVersion() + ": " + getNewVersion());
        super.executeInsert(conn);
        updateInProgress = true;
      }
    }

  /**
   * Deletes this object from the database.
   *
   * @param conn Database connection.
   * @exception java.sql.SQLException If a database access error
   *  occurs.
   */
  public void executeDelete(DBConnection conn)
  throws SQLException {
    PreparedStatement stmt = getDeleteStatement(conn);
    if (isPersistent()) {
      Enhydra.getApplication().getLogChannel().
      write(Logger.DEBUG2,
            getClass().getName() + ".executeDelete(): "
            + persistent + ": "
            + getOId().toString()
            + ": " + getVersion());
      super.executeDelete(conn);
    }
  }

  /**
   * If transaction succeeded marks this object as clean.
   * @param success true if the transaction succeeded
   *   and this object was successfully inserted into the database.
   */
  public void finalizeInsert(boolean success) {
    Enhydra.getApplication().getLogChannel().
    write(Logger.DEBUG2,
          getClass().getName() + ".finalizeInsert(" + success + ") "
          + CoreDO.NAME_OBJECTID + "=" + getOId().toString() + " "
          + CoreDO.NAME_VERSION + "=" + getVersion());

    boolean p = persistent;
    super.finalizeInsert(success);
    if (p == true) {
      persistent = true; // fix bug in CoreDO.
    }

    if (success) {
      dirty = false;
    }
    updateInProgress = false;
  }

  /**
   * If transaction succeeded marks this object as clean.
   * @param success true if the transaction succeeded
   *   and this object was successfully updated in the database.
   */
  public void finalizeUpdate(boolean success) {
    Enhydra.getApplication().getLogChannel().
    write(Logger.DEBUG2,
          getClass().getName() + ".finalizeUpdate:" + success + ": "
          + getOId().toString()
          + ": " + getVersion());
    super.finalizeUpdate(success);
    if (success) {
      dirty = false;
    }
    updateInProgress = false;
  }

  /**
   * Sets this object's identifier.
   * @param oId this object's identifier.
   */
  protected void setOId(ObjectId oId) {
    super.setOId(markNewValue(getOId(), oId));
  }

  /**
   * <CODE>markClean()</CODE> with no arguments is used to indicate that
   * this object is marked as "clean"
   * (it does not need to be updated to the database).<p>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   */
  protected void markClean() {
    dirty = false;
  }

  /**
   * <CODE>markNewValue()</CODE> with no arguments is used to indicate that
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   */
  protected void markNewValue() {
    dirty = true;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value,
   *                                  max_length, nullOK )</CODE>
   *
   * If <CODE>nullOK</CODE> is false and <CODE>new_field_value</CODE>
   * is null, <CODE>this.field</CODE> is set to <CODE>""</CODE>.
   *
   * @param current_string The current value of the String data member. 
   * @param new_string The new value of the String data member. 
   * @param max_length The maximum allowed length of the String value.
   * If the new_string is longer than max_length, new_string is truncated.
   * If max_length is less than 1, no maximum is imposed.
   * @return new_string if it is not null, otherwise "".
   */
  protected String markNewValue(String current_string, String new_string,
                                int max_length,
                                boolean nullOK ) {
    /*
           if ( new_string == null ) 
               if ( ! nullOK ) 
                   return current_string;
               else
                   new_string = "";
           if ((max_length > 00) && (new_string.length() > max_length)) {
               new_string = new_string.substring( 0, max_length );
           }
           if (!new_string.equals(current_string)) {
               dirty = true;
           }
           return new_string;
    */
    // FIX delete stuff above after testing line below
    return markNewValue(current_string, new_string, 0, max_length, nullOK);

  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value,
   *                                  min_length, max_length, nullOK )</CODE>
   *
   * If <CODE>min_length > 0</CODE> and
   * <CODE>nullOK</CODE> is false and 
   * <CODE>new_field_value</CODE> is null, 
   * <CODE>this.field</CODE> is returned 
   * (the <CODE>new_field_value</CODE> is ignored.)
   *
   * @param current_string The current value of the String data member. 
   * @param new_string The new value of the String data member. 
   * @param min_length The minimum allowed length of the String value.
   * If the new_string is shorter than min_length, new_string is ignored.
   * If min_length is less than 0, no minimum is imposed.
   * @param max_length The maximum allowed length of the String value.
   * If the new_string is longer than max_length, new_string is truncated.
   * If max_length is less than 1, no maximum is imposed.
   * @return new_string if it is not null, otherwise "".
   */
  protected String markNewValue(String current_string, String new_string,
                                int min_length, int max_length,
                                boolean nullOK ) {
    if ( ! nullOK && current_string == null )  // fix bogus current value
      current_string = "";

    if ( new_string == null ) {
      if ( ! nullOK ) {  // reject new value
        return current_string;
      } else {
        if ( current_string != null )
          dirty = true;
        return null;
      }
    }

    if ( min_length >= 0 && new_string.length() < min_length )
      new_string = current_string; // reject new_string
    if ( max_length > 00 && new_string.length() > max_length ) {
      new_string = new_string.substring( 0, max_length );
    }
    if ( ! new_string.equals( current_string ) )
      dirty = true;
    return new_string;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value )</CODE>
   *
   * @param current_OId The current value of the ObjectId data member. 
   * @param new_OId The new value of the ObjectId data member. 
   * @return new_OId
   */
  protected ObjectId markNewValue(ObjectId current_OId, ObjectId new_OId) {
    if ( current_OId == null ) {
      if ( new_OId != null )
        dirty = true;
    } else {
      if ( new_OId == null )
        dirty = true;
      else if ( ! current_OId.toString().equals( new_OId.toString() ) )
        dirty = true;
    }
    return new_OId;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value )</CODE>
   *
   * @param current_date The current value of the Date data member. 
   * @param new_date The new value of the Date data member. 
   * @return new_date
   */
  protected java.util.Date markNewValue(
    java.util.Date current_date, java.util.Date new_date) {
    if ( current_date == null ) {
      if ( new_date != null )
        dirty = true;
    } else {
      if ( new_date == null )
        dirty = true;
      else if ( ! current_date.toString().equals( new_date.toString() ) )
        dirty = true;
    }
    return new_date;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value )</CODE>
   *
   * @param current_bd The current value of the BigDecimal data member. 
   * @param new_bd The new value of the BigDecimal data member. 
   * @return new_bd
   */
  protected BigDecimal markNewValue(BigDecimal current_bd, BigDecimal new_bd) {
    if ( current_bd == null ) {
      if ( new_bd != null )
        dirty = true;
    } else {
      if ( new_bd == null )
        dirty = true;
      else if ( ! current_bd.equals( new_bd ) )
        dirty = true;
    }
    return new_bd;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value )</CODE>
   *
   * @param curr_val The current value of the byte array data member. 
   * @param new_val The new value of the byte array data member. 
   * @return new_val
   */
  protected byte[] markNewValue(byte[] current_val, byte[] new_val) {
    if ( current_val == null ) {
      if ( new_val != null )
        dirty = true;
    } else {
      if ( new_val == null )
        dirty = true;
      else {
        if ( current_val.length != new_val.length ) {
          // FIX what to do?
        }
        else {
          for ( int i = 0; i < current_val.length; i++ )
            if ( current_val[i] != new_val[i] ) {
              dirty = true;
              break;
            }
        }
      }
    }
    return new_val;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value )</CODE>
   *
   * @param current_val The current value of the java.sql.Date data member. 
   * @param new_val The new value of the java.sql.Date data member. 
   * @return new_val
   */
  protected java.sql.Date markNewValue(
    java.sql.Date current_val, java.sql.Date new_val) {
    if ( current_val == null ) {
      if ( new_val != null )
        dirty = true;
    } else {
      if ( new_val == null )
        dirty = true;
      else if ( ! current_val.equals( new_val ) )
        dirty = true;
    }
    return new_val;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value )</CODE>
   *
   * @param current_val The current value of the java.sql.Time data member. 
   * @param new_val The new value of the java.sql.Time data member. 
   * @return new_val
   */
  protected java.sql.Time markNewValue(
    java.sql.Time current_val, java.sql.Time new_val) {
    if ( current_val == null ) {
      if ( new_val != null )
        dirty = true;
    } else {
      if ( new_val == null )
        dirty = true;
      else if ( ! current_val.equals( new_val ) )
        dirty = true;
    }
    return new_val;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value )</CODE>
   *
   * @param current_val The current value of the java.sql.Timestamp data member. 
   * @param new_val The new value of the java.sql.Timestamp data member. 
   * @return new_val
   */
  protected java.sql.Timestamp markNewValue(
    java.sql.Timestamp current_val, java.sql.Timestamp new_val) {
    if ( current_val == null ) {
      if ( new_val != null )
        dirty = true;
    } else {
      if ( new_val == null )
        dirty = true;
      else if ( ! current_val.equals( new_val ) )
        dirty = true;
    }
    return new_val;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value )</CODE>
   *
   * @param current_int The current value of the int data member. 
   * @param new_int The new value of the int data member. 
   * @return new_int
   */
  protected int markNewValue(int current_int, int new_int) {
    if ( current_int != new_int )
      dirty = true;
    return new_int;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value )</CODE>
   *
   * @param current_float The current value of the float data member. 
   * @param new_float The new value of the float data member. 
   * @return new_float
   */
  protected float markNewValue(float current_float, float new_float) {
    if ( current_float != new_float )
      dirty = true;
    return new_float;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value )</CODE>
   *
   * @param current_double The current value of the double data member. 
   * @param new_double The new value of the double data member. 
   * @return new_double
   */
  protected double markNewValue(double current_double, double new_double) {
    if ( current_double != new_double )
      dirty = true;
    return new_double;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value )</CODE>
   *
   * @param current_boolean The current value of the boolean data member. 
   * @param new_boolean The new value of the boolean data member. 
   * @return new_boolean
   */
  protected boolean markNewValue(boolean current_boolean, boolean new_boolean) {
    if ( current_boolean != new_boolean )
      dirty = true;
    return new_boolean;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value )</CODE>
   *
   * @param current_char The current value of the char data member. 
   * @param new_char The new value of the char data member. 
   * @return new_char
   */
  protected char markNewValue(char current_char, char new_char) {
    if ( current_char != new_char )
      dirty = true;
    return new_char;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value )</CODE>
   *
   * @param current_byte The current value of the byte data member. 
   * @param new_byte The new value of the byte data member. 
   * @return new_byte
   */
  protected byte markNewValue(byte current_byte, byte new_byte) {
    if ( current_byte != new_byte )
      dirty = true;
    return new_byte;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value )</CODE>
   *
   * @param current_short The current value of the short data member. 
   * @param new_short The new value of the short data member. 
   * @return new_short
   */
  protected short markNewValue(short current_short, short new_short) {
    if ( current_short != new_short )
      dirty = true;
    return new_short;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value )</CODE>
   *
   * @param current_long The current value of the long data member. 
   * @param new_long The new value of the long data member. 
   * @return new_long
   */
  protected long markNewValue(long current_long, long new_long) {
    if ( current_long != new_long )
      dirty = true;
    return new_long;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value )</CODE>
   *
   * @param current_long The current value of the long data member. 
   * @param new_long The new value of the long data member. 
   * @return new_long
   */
  protected GenericDO markNewValue(GenericDO current_DO, GenericDO new_DO) {
    if ( current_DO != new_DO )
      dirty = true;
    return new_DO;
  }

  /**
   * If the current data member value differs from the new value,
   * this object is marked as "dirty" (in need of an update to the database).
   * There is no way to mark this object as "clean";<BR> only the
   * <CODE>executeInsert()</CODE> and <CODE>executeUpdate()</CODE>
   * methods can make that so.<P>
   *
   * This method is protected since only derived Data Objects (DO's)
   * "<CODE>set</CODE>" methods will be calling it.<P>
   *
   * This method returns the new value as a convenience to the calling
   * "<CODE>set</CODE>" method.  The usage is:
   * <CODE>this.field = markNewValue( this.field, new_field_value )</CODE>
   *
   * @param current_ts The current value of the long data member. 
   * @param new_ts The new value of the long data member. 
   * @return new_ts
   */
      protected Object markNewValue(Object current_obj, Object new_obj)
      {
          if ( ! current_obj.equals( new_obj ) )
              dirty = true;
          return new_obj;
      }




  protected void setPrepStmtParam_DO(
    PreparedStatement stmt, int[] paramIndex, GenericDO value )
  throws java.sql.SQLException {
    if ( null == value )
      stmt.setNull( paramIndex[0]++, Types.DECIMAL );
    else {
      ObjectId oid = value.getOId();
      if ( null == oid )
        stmt.setNull( paramIndex[0]++, Types.DECIMAL );
      else
        setPrepStmtParam_BigDecimal( stmt, paramIndex,
                                     oid.toBigDecimal() );
    }
  }

  protected void setPrepStmtParam_String(
    PreparedStatement stmt, int[] paramIndex, String value )
  throws java.sql.SQLException {
    if ( null == value )
      stmt.setNull( paramIndex[0]++, Types.VARCHAR );
    else
      stmt.setString( paramIndex[0]++, value );
  }

  protected void setPrepStmtParam_Float(
    PreparedStatement stmt, int[] paramIndex, float value )
  throws java.sql.SQLException {
    stmt.setFloat( paramIndex[0]++, value );
  }

  protected void setPrepStmtParam_Int(
    PreparedStatement stmt, int[] paramIndex, int value )
  throws java.sql.SQLException {
    stmt.setInt( paramIndex[0]++, value );
  }

  protected void setPrepStmtParam_java_math_BigDecimal(
    PreparedStatement stmt, int[] paramIndex, BigDecimal value )
  throws java.sql.SQLException {
    setPrepStmtParam_BigDecimal(stmt, paramIndex, value);
  }

  protected void setPrepStmtParam_BigDecimal(
    PreparedStatement stmt, int[] paramIndex, BigDecimal value )
  throws java.sql.SQLException {
    if ( null == value )
      stmt.setNull( paramIndex[0]++, Types.DECIMAL );
    else
      stmt.setBigDecimal( paramIndex[0]++, value );
  }

  protected void setPrepStmtParam_java_sql_Date(
    PreparedStatement stmt, int[] paramIndex, java.sql.Date value )
  throws java.sql.SQLException {
    setPrepStmtParam_Date(stmt, paramIndex, value);
  }

  protected void setPrepStmtParam_Date(
    PreparedStatement stmt, int[] paramIndex, java.sql.Date value )
  throws java.sql.SQLException {
    if ( null == value )
      stmt.setNull( paramIndex[0]++, Types.DATE );
    else
      stmt.setDate( paramIndex[0]++, value );
  }

  protected void setPrepStmtParam_java_sql_Time(
    PreparedStatement stmt, int[] paramIndex, java.sql.Time value )
  throws java.sql.SQLException {
    setPrepStmtParam_Time(stmt, paramIndex, value);
  }

  protected void setPrepStmtParam_Time(
    PreparedStatement stmt, int[] paramIndex, java.sql.Time value )
  throws java.sql.SQLException {
    if ( null == value )
      stmt.setNull( paramIndex[0]++, Types.TIME );
    else
      stmt.setTime( paramIndex[0]++, value );
  }

  protected void setPrepStmtParam_java_sql_Timestamp(
    PreparedStatement stmt, int[] paramIndex, java.sql.Timestamp value )
  throws java.sql.SQLException {
    setPrepStmtParam_Timestamp(stmt, paramIndex, value);
  }

  protected void setPrepStmtParam_Timestamp(
    PreparedStatement stmt, int[] paramIndex, java.sql.Timestamp value )
  throws java.sql.SQLException {
    if ( null == value )
      stmt.setNull( paramIndex[0]++, Types.TIMESTAMP );
    else
      stmt.setTimestamp( paramIndex[0]++, value );
  }

  protected void setPrepStmtParam_Bytes(
    PreparedStatement stmt, int[] paramIndex, byte[] value )
  throws java.sql.SQLException {
    if ( null == value )
      stmt.setNull( paramIndex[0]++, Types.VARBINARY );
    else
      stmt.setBytes( paramIndex[0]++, value );
  }

  protected void setPrepStmtParam_Double(
    PreparedStatement stmt, int[] paramIndex, double value )
  throws java.sql.SQLException {
    stmt.setDouble( paramIndex[0]++, value );
  }

  protected void setPrepStmtParam_Long(
    PreparedStatement stmt, int[] paramIndex, long value )
  throws java.sql.SQLException {
    stmt.setLong( paramIndex[0]++, value );
  }

  protected void setPrepStmtParam_Short(
    PreparedStatement stmt, int[] paramIndex, short value )
  throws java.sql.SQLException {
    stmt.setShort( paramIndex[0]++, value );
  }

  protected void setPrepStmtParam_Byte(
    PreparedStatement stmt, int[] paramIndex, byte value )
  throws java.sql.SQLException {
    stmt.setByte( paramIndex[0]++, value );
  }

  protected void setPrepStmtParam_Boolean(
    PreparedStatement stmt, int[] paramIndex, boolean value )
  throws java.sql.SQLException {
    stmt.setString( paramIndex[0]++, value ? "1" : "0" );
  }

}
