/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: QueryBuilder.java,v 1.14 1999/07/16 22:25:56 jay Exp $
 *-----------------------------------------------------------------------------
 */




package dods.builder.sourceGenerators.Query;

//import kinkobahn.data.utilities.*;
import com.lutris.appserver.server.sql.*;
import java.sql.*;
import java.util.*;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;

import dods.builder.sourceGenerators.DO.GenericDO;


/**
 *	A QueryBuilder is a helper-object used by the xxxQuery classes.
 *
 *	Stand-alone usage:
 *
 *		QueryBuilder qb;
 *		qb = new QueryBuilder( "cats" );
 *			or
 *		qb = new QueryBuilder( "cats", "name, age" );
 *			...
 *		qb.addWhereClause( "owner.name", "Smith", 
 *					QueryBuilder.NOT_NULL,
 *					QueryBuilder.EXACT_MATCH );
 *		qb.addWhereClause( "paws", 3, QueryBuilder.GREATER_THAN );
 *              qb.addReferentialWhereClause( "cat.ownerId", "owner.Id" );
 *		ResultSet rs = qb.executeQuery( conn );
 *			...
 *		qb.reset();
 *
 * @author Jay Gunter
 * @version $Revision: 1.14 $
 */
public class QueryBuilder {

// Comparison operator passed as 3rd arg to addWhereClause( int|float|double )
//
public static final String	EQUAL			= "=";
public static final String	NOT_EQUAL		= "!=";
public static final String	LESS_THAN		= "<";
public static final String	LESS_THAN_OR_EQUAL	= "<=";
public static final String	GREATER_THAN		= ">";
public static final String	GREATER_THAN_OR_EQUAL	= ">=";

// Null-acceptability indicator passed as 3rd arg to addWhereClause( String )
//
public static final boolean	NULL_OK			= true;
public static final boolean	NOT_NULL		= false;

// Wild-card          indicator passed as 4th arg to addWhereClause( String )
//
public static final boolean	EXACT_MATCH		= true;
public static final boolean	NOT_EXACT		= false;

// Default inexact string match keyword (e.g. can be MATCHES or LIKE)
public static final String DEFAULT_MATCHES_KEYWORD = "MATCHES";

// Default inexact string wildcard (e.g. can be * or %)
public static final String DEFAULT_WILDCARD = "*";

// See debug() method.
static private boolean debugAllSQL = false;
private boolean debugSQL = false;

/**
 * Construct a QueryBuilder object that will return all fields of a table.
 *
 * @param table Name of the table on which to perform the SQL 'select'.
 */
public QueryBuilder( String tableName ) {
    this( tableName, tableName + ".*" );
}

/**
 * Construct a QueryBuilder object that will return specified fields of a table.
 *
 * @param table     Name of the table on which to perform the SQL 'select'.
 * @param fieldList Comma-separated string of field names to retrieve.
 */
public QueryBuilder( String tableName, String fieldList  ) {
    debugSQL = debugAllSQL;
    reset();
/*     mainTableName = tableName.toLowerCase(); */
    mainTableName = tableName;
    storeTableName( mainTableName );
    selectClause = "select " + fieldList;
}

/**
 * Determine table name of column.
 * For example, we return 'tableX' from the following column specification:
 *     UPPER(tableX.columnY)
 * WARNING: trickier column specifications will cause this method to fail.
 *
 * @param column     A column (field) name, possibly with a leading table name.
 */
private String getTableName( String column ) {
    int offset = 0;
    int end = 0;
    String str = column;
    int index = str.lastIndexOf( "(" );
    if ( -1 != index ) 
//	str = str.substring( index + 1, 1000 );
	str = str.substring( index + 1);
    index = str.indexOf( ")" );
    if ( -1 != index ) 
	str = str.substring( 0, index );
    index = str.indexOf( "." );
    if ( -1 == index )
	return null;
    return str.substring( 0, index );
}

/**
 * Store all table names that occur as prefix of column specifications.
 * All table names must appear in the "from" clause.
 *
 * @param column     A column (field) name, possibly with a leading table name.
 */
private void storeTableNameForColumn( String column ) {
    String table = getTableName( column );
    storeTableName( table );
}

/**
 * Store a table name.
 * All table names must appear in the "from" clause.
 *
 * @param table name.
 */
private void storeTableName( String table ) {
    if ( null != table )
        tableNames.put( table.toLowerCase(), "" );
}

/**
 * Add a where-clause to the SQL command.
 * E.g., we want to find cats with too many paws:
 *      column1 = "cat.paws"
 *      column2 = "animalNorms.catPaws"
 *      cmp_op  = QueryBuilder.GREATER_THAN
 * generates the where-clause:
 *      cat.paws > animalNorms.catPaws
 * and remembers the table names (cat, animalNorms) 
 * for inclusion in the from-clause.
 *
 * @param column1  A column (field) name to use as a search constraint.
 * @param column2  Another column to compare against.
 * @param cmp_op   The comparison operator to use:  QueryBuilder.EQUALS, ...
 */
public void addTwoColumnWhereClause(
			       String column1, String column2, String cmp_op )
{
    storeTableNameForColumn( column1 );
    storeTableNameForColumn( column2 );

    whereClauses.addElement( column1 + " " + cmp_op + " " + column2 );
}

/**
 * Add an inter-table reference where-clause to the SQL command.
 * The two columns specified will be compared using the EQUALS operator.
 *
 * column1 and column2 should refer to different tables.
 *
 * E.g.:
 *      column1 = "cat.ownerId"
 *      column2 = "owner.Id"
 * generates the where-clause:
 *      cat.ownerId = owner.Id
 * and remembers the table names for inclusion in the from-clause.
 *
 * @param column1  A column (field) name, possibly with a leading table name.
 * @param column2  A column (field) name, possibly with a leading table name.
 * @return Whether a where-clause was generated.  
 * True only if column1 and column2 refer to different tables.
 */
public boolean addReferentialWhereClause( String column1, String column2 ) {
    String table1 = getTableName( column1 );
    if ( null == table1 )
        table1 = mainTableName;

    String table2 = getTableName( column2 );
    if ( null == table2 )
        table2 = mainTableName;

    if ( table1.equals( table2 ) )
	return false;

    storeTableName( table1 );
    storeTableName( table2 );

    refWhereClauses.put( column1 + " = " + column2, "" );

    return true;
}

/**
 * Add a where-clause that compares a column against a string value
 * to the SQL command.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 * @param cmp_op   The comparison operator to use:  QueryBuilder.EQUALS, ...
 */
public void addWhereClause(	String column, String value, String cmp_op )
{
    storeTableNameForColumn( column );

    whereClauses.addElement( column + " " + cmp_op + " ?" );
    parms.addElement( value );
}

/**
 * Add a where-clause that compares a column against a string value
 * to the SQL command.  The 3rd and 4th params allow checking to be done
 * on the string value before the where-clause is generated.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 * @param nullOk   Whether a value of NULL is acceptable.
 * @param exact    Whether the value must match exactly (no wildcards).
 * @return Whether a where-clause was generated.
 */
public boolean addWhereClause(	String column, String value, 
				boolean nullOk, boolean exact )
{
    storeTableNameForColumn( column );

    String append = "";
    boolean added = false;

    if ( value == null && nullOk ) {
        append = " " + column + " is null";
        added = true;
    } else {
        String pattern = value.trim();
        if ( pattern.length() > 0 && ! pattern.equals( wildcard ) ) {
            if ( exact )
                append = " " + column + " = ?";
            else {
                append = " " + column + " " + matchKeyword + " ?";
                fuzzies.set( parms.size() );
	    }
            added = true;
	    parms.addElement( pattern );
        }
    }
    if ( added )
        whereClauses.addElement( append );

    return added;
}

/**
 * Add a where-clause to the SQL command.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 * @param nullOk   Whether a value of NULL is acceptable.
 * @return Whether a where-clause was generated.
 */
public boolean addWhereClause(	String column, BigDecimal value,
				boolean nullOk )
{
    storeTableNameForColumn( column );

    String append = "";
    boolean added = false;

    if ( value == null && nullOk ) {
        append = " " + column + " is null";
        added = true;
    } else {
        append = " " + column + " = ?";
        added = true;
	parms.addElement( value );
    }
    if ( added )
        whereClauses.addElement( append );
 
    return added;
}

/**
 * Add a where-clause to the SQL command.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 */
public void addWhereClause(	String column, int value )
{
    addWhereClause( column, value, "=" );
}

/**
 * Add a where-clause to the SQL command.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 * @param cmp_op   The comparison operator to use:  QueryBuilder.EQUALS, ...
 */
public void addWhereClause(	String column, int value, String cmp_op )
{
    storeTableNameForColumn( column );

    whereClauses.addElement( column + " " + cmp_op + " ?" );
    parms.addElement( new Integer( value ) );
}

/**
 * Add a where-clause to the SQL command.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 */
public void addWhereClause(	String column, float value )
{
    addWhereClause( column, value, "=" );
}

/**
 * Add a where-clause to the SQL command.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 * @param cmp_op   The comparison operator to use:  QueryBuilder.EQUALS, ...
 */
public void addWhereClause(	String column, float value, String cmp_op )
{
    storeTableNameForColumn( column );

    whereClauses.addElement( column + " " + cmp_op + " ?" );
    parms.addElement( new Float( value ) );
}

/**
 * Add a where-clause to the SQL command.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 */
public void addWhereClause(	String column, double value )
{
    addWhereClause( column, value, "=" );
}

/**
 * Add a where-clause to the SQL command.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 * @param cmp_op   The comparison operator to use:  QueryBuilder.EQUALS, ...
 */
public void addWhereClause(	String column, double value, String cmp_op )
{
    storeTableNameForColumn( column );

    whereClauses.addElement( column + " " + cmp_op + " ?" );
    parms.addElement( new Double( value ) );
}

/**
 * Add a where-clause to the SQL command.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 */
public void addWhereClause(	String column, boolean value )
{
    storeTableNameForColumn( column );

    whereClauses.addElement( column + " = ?" );
    parms.addElement( new Integer( value ? 1 : 0 ) );
}

/**
 * Add a trailing clause (sort by, group by, etc.) to the SQL command.
 *
 * @param clause     The clause to append to the SQL command.
 */
public void addEndClause( String clause ) {
    endClauses.put( clause, "" );
}

/**
 * Append to the 'select' portion of the SQL command.
 * This has been deprecated.  Use addEndClause instead.
 *
 * @param str     The string to append to the SQL command.
 */
public void add( String str ) {
    addEndClause( str );
}

/**
 * Sets debug flag to print the SQL statement just before its execution.
 * Affects just this instance of QueryBuilder.
 */
 public void debug() {
    debugSQL = true;
 }

/**
 * Sets debug flag to print the SQL statement just before its execution.
 * Affects all instances of QueryBuilder.
 */
 static public void debugAll() {
    debugAllSQL = true;
 }

/**
 * Generate a JDBC PreparedStatement using the values passed 
 * for the where-clauses.
 *
 * @param conn   The DBConnection object to use for the search.
 */
private void prepareStatement( DBConnection conn )
		throws SQLException {

    String sql = selectClause + " from ";
    Enumeration tables = tableNames.keys();
    for ( int i = 0; tables.hasMoreElements(); i++ ) {
	if ( i > 0 ) 
	    sql += ", ";
	sql += tables.nextElement();
    }

    String glueWord = " where ";
    for ( int i = 0; i < whereClauses.size(); i++ ) {
	sql += glueWord + whereClauses.elementAt( i );
	glueWord = " and ";
    }

    Enumeration refClauses = refWhereClauses.keys();
    while ( refClauses.hasMoreElements() ) {
	sql += glueWord + refClauses.nextElement();
	glueWord = " and ";
    }

    /*
     * ORDER BY code added by Chris Ryan (cryan@plugged.net.au)
     * The following code adds the ORDER BY 'xxx[, xxx]' to the 
     * statement for querying.  
     */
    int nr_OrderFields = orderFields.size();

//	System.err.println("\nQueryBuilder.nr_OrderFields="+nr_OrderFields+"\n");
    // ORDER BY required?
    if (nr_OrderFields > 0) {

      // yes, fill out the SQL...
      sql += " ORDER BY ";
      for ( int i = 0; i< nr_OrderFields; i++ ) {
            if ( i > 0 ) 
              sql += ", ";
            sql += orderFields.elementAt(i)+" "+orderDirections.elementAt(i);
      }

    }

    Enumeration endings = endClauses.keys();
    while ( endings.hasMoreElements() )
	sql += endings.nextElement();

    stmt = conn.prepareStatement( sql );

    if ( debugSQL )
	System.err.println("\nQueryBuilder.prepareStatement sql="+sql+"\n");

    int param;
    for ( int i = 0; i < parms.size(); i++ ) {
	param = i + 1;
//System.out.println("\n\nprepareStatement param=" + param);
        Object    o = parms.elementAt( i );
	if      ( o instanceof String ) {
/* System.out.println("\n\nprepareStatement str=" + (String)o); */

        if ( fuzzies.get( i ) ) {

/* System.out.println("\n\nmatchKeyword str=" + matchKeyword); */

          if (matchKeyword.equals(DEFAULT_MATCHES_KEYWORD)) {

            stmt.setString(param, getRegExpMatch( (String) o ) );

          } else {

            stmt.setString(param, (String) o );

          }

        } else {

         stmt.setString(param, (String) o );

        }

      } else if ( o instanceof BigDecimal ) {
/* System.out.println("\n\nprepareStatement bd=" +  (BigDecimal) o); */
		  stmt.setBigDecimal( param,  (BigDecimal) o );
	} else if ( o instanceof Integer )    {
/* System.out.println("\n\nprepareStatement int=" + (Integer)o); */
		  stmt.setInt(        param, ((Integer)    o).intValue() );
	} else if ( o instanceof Float )    {
/* System.out.println("\n\nprepareStatement fl=" +  (Float)o); */
		  stmt.setFloat(      param, ((Float)      o).floatValue() );
	} else if ( o instanceof Double )    {
/* System.out.println("\n\nprepareStatement df=" +  (Double)o); */
		  stmt.setDouble(     param, ((Double)      o).doubleValue() );
	} else if ( o instanceof Boolean )    {
/* System.out.println("\n\nprepareStatement bool=" + (Boolean)o); */
		  stmt.setBoolean(    param, ((Boolean)    o).booleanValue() );
    	}
    }
}

/**
 * Return the JDBC PreparedStatement generated by prepareStatement().
 * @return The JDBC PreparedStatement.
 */
public PreparedStatement getStatement() {
    // Would anyone ever want this?
    return stmt;
}

/**
 * Execute the PreparedStatement.
 * @return The ResultSet object generated by JDBC for the search.
 */
/*
public ResultSet executeQuery() {
    conn = new DBConnection();
    return executeQuery( conn );
}
*/

/**
 * Execute the PreparedStatement and return a JDBC ResultSet.
 * @param conn   The DBConnection object to use for the search.
 * @return The ResultSet object generated by JDBC for the search.
 */
public ResultSet executeQuery( DBConnection conn )
		throws SQLException {
    prepareStatement( conn );
    return stmt.executeQuery();
}

/**
 * Reset the where-clause information.  
 * The same fields specified in the QueryBuilder constructor,
 * can be retrieved for a different set of where-clauses.
 */
public void reset() {
    whereClauses = new Vector(); // keep the select, forget the where-clauses
    parms   = new Vector();	// discard parms for previous where-clause
    fuzzies = new BitSet();	// forget special string handling

    // Clear the ORDER BY Vectors
    orderFields = new Vector(); 
    orderDirections = new Vector(); 

    if ( null != conn )	// if this class created the connection, close it.
	conn.close();
    conn    = null;
}

/**
 * Returns a regular expression, as supported by "MATCHES"
 * that will will perform a case insensitive match of a substring.
 * @param base   The base String to convert to a regular expression.
 * @return       The generated regular expression.
 */
private String getRegExpMatch( String base )
{
    int len = base.length();
    char [] baseChars = new char[len];
    base.getChars(0, len, baseChars, 0);
 
    // The result will never exceed 5 * the length of
    // the original string.
    char [] result = new char[len * 5];
    int resultIdx = 0;
    for (int idx = 0; idx < baseChars.length; idx++) {
        if (Character.isSpaceChar(baseChars[idx])) {
            // Convert spaces to ' *'
            if ((resultIdx > 0) && (result[resultIdx-1] != '*')) {
                result[resultIdx++] = ' ';
                //result[resultIdx++] = '*';
                result[resultIdx++] = '%';
            }
        }
        else if (Character.isLetter(baseChars[idx])) {
            // Convert letters to [Aa]
            result[resultIdx++] = '[';
            result[resultIdx++] = Character.toUpperCase(baseChars[idx]);
            result[resultIdx++] = Character.toLowerCase(baseChars[idx]);
            result[resultIdx++] = ']';
        }
        else {
            // Leave everything else as is.
            result[resultIdx++] = baseChars[idx];
        }
    }
    // Match anything at end of line
    result[resultIdx++] = '*';
    return new String(result);
}

private String mainTableName;
private Hashtable tableNames = new Hashtable();
private Hashtable refWhereClauses = new Hashtable();
private Hashtable endClauses = new Hashtable();
private String selectClause;
private Vector whereClauses = new Vector();
private Vector orderFields = new Vector();
private Vector orderDirections = new Vector();
private PreparedStatement stmt;
private Vector parms;
private BitSet fuzzies;
private int param = 0;
private DBConnection conn = null;

/* 
 * The following methods made the implementation of the query template easier.
 * They allow one template to handle all Java and database types.
 * They do this by ignoring some of the parameters passed.
 * The addWhereClause() for String     ignores dbtype.
 * The addWhereClause() for byte[]     ignores dbtype.
 * The addWhereClause() for boolean    ignores            nullOk and exact.
 * The addWhereClause() for int        ignores dbtype,    nullOK and exact.
 * The addWhereClause() for float      ignores dbtype,    nullOK and exact.
 * The addWhereClause() for double     ignores dbtype,    nullOK and exact.
 * The addWhereClause() for BigDecimal ignores dbtype            and exact.
 * The addWhereClause() for Date       ignores dbtype            and exact.
 * The addWhereClause() for Time       ignores dbtype            and exact.
 * The addWhereClause() for Timestamp  ignores dbtype            and exact.
 * The addWhereClause() for GenericDO  ignores dbtype            and exact.
 *
 * An alternative solution would be to implement 4 different templates
 * and have SourceGenerator_Query create filters for each template
 * and use the appropriate template according to the type of each Attribute.
 * While that approach would not be wildly complex, it would be less
 * well-contained than implementing the methods below.
 */

/**
 * Add a where-clause to the SQL command.
 * NOTE: for use by SourceGenerator_Query and writeMemberStuff.template.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 */
public void addWhereClause(	String column, String value, String dbtype,
				boolean nullOk, boolean exact )
{
    addWhereClause( column, value, nullOk, exact );
}

/**
 * Add a where-clause to the SQL command.
 * NOTE: for use by SourceGenerator_Query and writeMemberStuff.template.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 */
public void addWhereClause(	String column, byte[] value, String dbtype,
				boolean nullOk, boolean exact )
{
/*     String s = new String( value, 0 );	// FIX check this:  hibyte==0? */
    String s = new String( value );
    addWhereClause( column, s, nullOk, exact );
}

/**
 * Add a where-clause to the SQL command.
 * NOTE: for use by SourceGenerator_Query and writeMemberStuff.template.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 */
public void addWhereClause(	String column, boolean value, String dbtype,
				boolean nullOk, boolean exact )
{
    boolean guess = false;
    if ( dbtype == null ) {
	guess = true;
    } else if ( dbtype.equals( "BIT" ) ) {
	parms.addElement( new Integer( value ? 1 : 0 ) );
    } else if ( dbtype.equals( "INTEGER" ) ) {
	parms.addElement( new Integer( value ? 1 : 0 ) );
    } else if ( dbtype.equals( "CHAR" ) ) {
	parms.addElement( value ? "1" : "0" );
    } else if ( dbtype.equals( "VARCHAR" ) ) {
	parms.addElement( value ? "1" : "0" );
    } else {
	guess = true;
    }

    if ( guess ) {
	addWhereClause( column, value );
    } else {
	storeTableNameForColumn( column );

	whereClauses.addElement( column + " = ?" );
    }
}

/**
 * Add a where-clause to the SQL command.
 * NOTE: for use by SourceGenerator_Query and writeMemberStuff.template.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 */
public void addWhereClause(	String column, int value, String dbtype,
				boolean nullOk, boolean exact )
{
    addWhereClause( column, value );
}

/**
 * Add a where-clause to the SQL command.
 * NOTE: for use by SourceGenerator_Query and writeMemberStuff.template.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 */
public void addWhereClause(	String column, float value, String dbtype,
				boolean nullOk, boolean exact )
{
    addWhereClause( column, value );
}

/**
 * Add a where-clause to the SQL command.
 * NOTE: for use by SourceGenerator_Query and writeMemberStuff.template.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 */
public void addWhereClause(	String column, double value, String dbtype,
				boolean nullOk, boolean exact )
{
    addWhereClause( column, value );
}

/**
 * Add a where-clause to the SQL command.
 * NOTE: for use by SourceGenerator_Query and writeMemberStuff.template.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 */
public void addWhereClause(	String column, BigDecimal value, String dbtype,
				boolean nullOk, boolean exact )
{
    addWhereClause( column, value, nullOk );
}

/**
 * Set the Database vendor name.
 * Each vendor has a date format that must be used for searching.
 *
 * @param vendor Name of the database vendor; e.g. "Oracle".
 */
private final String dateFormatInformix = "yyyy-MM-dd HH:mm:ss";
private final String dateFormatSybase   = "MMM dd yyyy hh:mma";
private final String dateFormatOracle   = "dd-MMM-yyyy";
private final String dateFormatMSQL     = "MMM dd yyyy hh:mma"; // FIX
private final String dateFormatStandard = "MMM dd yyyy hh:mma"; // FIX
private final String dateFormatPostgreSQL = "MMM dd yyyy hh:mma"; // FIX
private       String dateFormat = dateFormatInformix;
private SimpleDateFormat df = new SimpleDateFormat( dateFormat );
public void setDatabaseVendor( String vendor ) {
    if (      vendor.toLowerCase().equals( "informix" ) )
	df = new SimpleDateFormat( dateFormatInformix );
    else if ( vendor.toLowerCase().equals( "sybase" ) )
	df = new SimpleDateFormat( dateFormatSybase );
    else if ( vendor.toLowerCase().equals( "oracle" ) )
	df = new SimpleDateFormat( dateFormatOracle );
    else if ( vendor.toLowerCase().equals( "msql" ) )
	df = new SimpleDateFormat( dateFormatMSQL );
    else if ( vendor.toLowerCase().equals( "standard jdbc" ) )
	df = new SimpleDateFormat( dateFormatStandard );
    else if ( vendor.toLowerCase().equals( "postgresql" ) )
	df = new SimpleDateFormat( dateFormatPostgreSQL );
    else {
	System.out.println(
		"ERROR in QueryBuilder!  Unimplemented Database Vendor: " +
		vendor );
    }
}


/**
 * Set the string matching details for inexact string matches
 * See dods.conf for details.
 * Code added by Chris Ryan (cryan@plugged.net.au)
 *
 * @param match_keyword  Whether the DB can use "MATCHES" or "LIKE"
 * @param wildcard  The wildcard character for string searches
 */
private String matchKeyword = DEFAULT_MATCHES_KEYWORD;
private String wildcard = DEFAULT_WILDCARD;
public void setStringMatchDetails( String match_keyword, String wildcard ) {

  this.matchKeyword = match_keyword;
  this.wildcard = wildcard;

}


/**
 * Add a where-clause to the SQL command.
 * NOTE: for use by SourceGenerator_Query and writeMemberStuff.template.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 */
public void addWhereClause(	String column, java.sql.Date value,
				String dbtype, boolean nullOk, boolean exact )
{
    String strValue = df.format( value );
    addWhereClause( column, strValue, nullOk, true );
}

/**
 * Add a where-clause that compares a column against a date value
 * to the SQL command.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to compare against.
 * @param cmp_op   The comparison operator to use:  QueryBuilder.EQUALS, ...
 */
public void addWhereClause(	String column, java.sql.Date value, String cmp_op )
{
    storeTableNameForColumn( column );

    String strValue = df.format( value );
    whereClauses.addElement( column + " " + cmp_op + " '" + strValue + "' " );
}

/**
 * Add a where-clause to the SQL command.
 * NOTE: for use by SourceGenerator_Query and writeMemberStuff.template.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 */
public void addWhereClause(	String column, java.sql.Time value,
				String dbtype, boolean nullOk, boolean exact )
{
    String strValue = df.format( value );
    addWhereClause( column, strValue, nullOk, true );
}

/**
 * Add a where-clause to the SQL command.
 * NOTE: for use by SourceGenerator_Query and writeMemberStuff.template.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 */
public void addWhereClause(	String column, java.sql.Timestamp value,
				String dbtype, boolean nullOk, boolean exact )
{
    String strValue = df.format( value );
    addWhereClause( column, strValue, nullOk, true );
}

/**
 * Add a where-clause to the SQL command.
 * NOTE: for use by SourceGenerator_Query and writeMemberStuff.template.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 */
public void addWhereClause(	String column,
				GenericDO DO,
				String dbtype, boolean nullOk, boolean exact )
{
    if ( null == DO ) {
	if ( nullOk )
	    addWhereClause( column, null, nullOk );
	// ignore this call if DO is null and ! nullOk
    } else {
        BigDecimal id = DO.getOId().toBigDecimal();
        addWhereClause( column, id, nullOk );
    }
}

/**
 * Add a where-clause to the SQL command.
 * NOTE: for use by SourceGenerator_Query and writeMemberStuff.template.
 *
 * @param column   The column (field) name to use as a search constraint.
 * @param value    The value to search for.
 */
/*
public void addWhereClause(	String column,
				Object x,
				String dbtype, boolean nullOk, boolean exact )
{
    if ( null == x ) {
	if ( nullOk )
	    addWhereClause( column, null, nullOk );
	// ignore this call if x is null and ! nullOk
    } else {
        addWhereClause( column, x.toString(), nullOk );
    }
}
*/

/**
 * Add another field to the ORDER BY list. 
 * NOTE: is used by SourceGenerator_Query and writeMemberStuff.template.
 *
 * This method added by Chris Ryan (cryan@plugged.net.au) to
 * allow easier access to the ORDER BY part of the SQL query.
 *
 * @param column     The column to order by
 * @param direction  ASCENDING or DESCENDING order
 */
public void addOrderByColumn(String column, String direction) {
  orderFields.addElement(column);  
  orderDirections.addElement(direction);  
}

} // end QueryBuilder class
