/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: SourceGenerator_SQL.java,v 1.15 1999/06/03 01:46:49 jay Exp $
 *-----------------------------------------------------------------------------
 */

package dods.builder.sourceGenerators.SQL;

import dods.data.*;
import dods.util.*;
import dods.gui.tree.*;

import dods.builder.engine.*;
import dods.builder.sourceGenerators.*;

import com.lutris.appserver.server.sql.*;

import java.io.File;
import java.io.StringWriter;
import java.util.Vector;

public class SourceGenerator_SQL extends SourceGenerator {
   
      final String REFERENCES	= "REFERENCES";	// FOREIGN KEY constraint
      final String KEY		= "KEY";		// PRIMARY KEY
      final String DEF_VALUE	= "DEF_VALUE";
      final String NOT_NULL	= "NOT_NULL";
      final String INDEXES	= "INDEXES";
      final String REFERENCE_TYPE	= "REFERENCE_TYPE";
      final String ON_DELETE_CASCADE	= "ON_DELETE_CASCADE";
   
		//Leigh Dodds (ldodds@ingenta.com) - added to allow unique columns
		final String UNIQUE = "UNIQUE";

      final String OID_DB_TYPE	= "OIDtype";
   
    // Variables used to build the value of the INDEXES tag.
      private String tablename;
      private String indexes;
      private int    index_num;
   
      private DODSProject project = null;
      private DODSTypes dodsTypes = null;
   
      public SourceGenerator_SQL() throws Exception {
        dodsTypes = DODSTypes.getInstance();
      }
   
   /**
     * Adds tag values to the <CODE>TagSet</CODE>.
     * This method invokes 
     *    <CODE>TemplateFilter.process( tagSet )</CODE>
     * which will create the output source file.
     *
     * @param o
     *   DataObject for which source code will be generated by
     *    <CODE>writeFileHeader</CODE> and
     *    <CODE>writeFileFooter</CODE>
     * @exception Exception
     *   If an unrecoverable error occurs.
     */
      public void objectPrep(	GeneratorInfo sgi,
      DODSProject proj,
      DataObject o )
      throws Exception {
         super.objectPrep( sgi, proj, o );
         project = proj;
         indexes = "";
         index_num = 1;
         tablename = o.getDBTableName();
         tagSet.setTagValue( REFERENCE_TYPE, oidDbType );
      }
   
/**
 * Prepares each 'member' for code generation.
 *
 * @param a  The attribute or member currently worked on.
 */
  public void memberPrep(  Attribute a ) throws Exception {

    super.memberPrep( a );
      
    tagSet.setTagValue( KEY, "" );

    // In this implementation, oid is the primary key of every table.
    //tagSet.setTagValue( KEY,      a.isIndex()   ? "PRIMARY KEY" : "" );
      
    tagSet.setTagValue( NOT_NULL, a.canBeNull() ? "" : "NOT NULL" );

    //Leigh Dodds (ldodds@ingenta.com) - added to allow unique columns
    tagSet.setTagValue( UNIQUE, a.isUnique() ? "UNIQUE" : "" ) ;

    // Get the initial DB value
    String defValue = a.getInitialDBValue();

    // get the type information for this attribute
    String dodsType = a.getDODSType();

    // get the code to generate the default value
    String dbInitial = dodsTypes.getDBInitial(dodsType);

    // handle the default value
    if (null != defValue && 0 != defValue.length() && !"".equals(defValue)) {

      if (dbInitial != null) {

        // replace the special DB_VALUE variable with the default value.
        defValue = Util.replace(dbInitial, DODSTypes.DB_VALUE, defValue);

      }

      defValue = "DEFAULT " + defValue;

    }

    if (defValue != null)
      tagSet.setTagValue( DEF_VALUE,  defValue );
    else
      tagSet.setTagValue( DEF_VALUE,  "" );
      
    // If necessary, append (size) to database type.
    if (dodsTypes.isSizeAllowed(dodsType)) {

      String size  = a.getSize();

      // The following should not occur - if a size is allowed,
      // then the gui should force a valid entry.
      if ( null == size || 0 == size.length() )
        size = "32";

      String dbtype = tagSet.getTagValue( MEMBER_DB_TYPE );
      dbtype += "(" + size + ")";
      tagSet.setTagValue( MEMBER_DB_TYPE, dbtype );

    }
      
    if ( a.isIndex() ) {

      //using ON conforms to standard SQL, fixes problems with some db's
      //forgot in increment counter - Open Source contribution by Joel Boehland
      indexes += "CREATE INDEX I" + index_num + "_" + tablename +
                 " ON " + tablename +
                 "(" + a.getName() + ");\n";
      index_num++;

    }
      
    // by default there's no 'ON DELETE CASCADE'
    tagSet.setTagValue( ON_DELETE_CASCADE, "" ); 

    if ( a.isObjectRef() || a.isRefConstraint() ) {

      /* 
       * When the user defined Attribute 'A'
       * of Data Object 'D1' (table 'd1'),
       * he marked the Attribute (class member, table column)
       * as a referential constraint.
       * The Attribute is an reference to
       * some other Data Object 'D2' (table 'd2').
       * This means that when column 'xx' of table 'd1' is not NULL,
       * the value must be a valid index into table 'd2'.
       * Within the 'CREATE TABLE' SQL command for table 'd1',
       * this referential constraint is specified as part of
       * the definition of column 'xx'.  E.g.:
       *    CREATE TABLE d1 (
       *        xx DECIMAL(19,0) REFERENCES d2(CoreDO.NAME_OBJECTID)
       *
       * We must learn the name of the table ('d2') that 
       * corresponds to the class 'D2'.
       * So, 'D2' must be a Data Object that is defined in
       * the DODSProject that we are processing.
       */
      tagSet.setTagValue( MEMBER_DB_TYPE, oidDbType );
      tagSet.setTagValue( REFERENCES, "REFERENCES " + 
        getTableName( a ) + "("+ CoreDO.NAME_OBJECTID+")" );

      if ( cascadeDelete )
        tagSet.setTagValue( ON_DELETE_CASCADE, "ON DELETE CASCADE" ); 

    } else {

      tagSet.setTagValue( REFERENCES, "" );
      tagSet.setTagValue( ON_DELETE_CASCADE, "" ); 

    }

  }
   
      private String getTableName( Attribute a ) throws Exception {
      // FIX?  we are searching the entire project because
      // Attribute does not have a method to return the 
      // DataObject to which it belongs.
      // Rather inelegant.
         String table = null;
         String classname = (String) a.getDODSType();
      // classname is something like pkgX.AaaDO
         int    len = classname.length();
         String last2 = classname.substring( len - 2, len );
/*          System.err.println("getTableName: a.getName()="+a.getName()); */
      /* System.err.println("last2 ="+last2 ); */
         if ( ! last2.equals( "DO" ) ) {
         // The Attribute is not a Data Object created by DODS,
         // so we don't know what table it references.
            throw new Exception(
               "ERROR:  Attribute '" + a.getName() +
               "':  type '" + classname + "' was not created by DODS.\n" +
               "        The corresponding table is not known." );
         }
         classname = classname.substring( 0, len - 2 );
      // classname is something like pkgX.Aaa
         int lastDot = classname.lastIndexOf( '.' );
         classname = classname.substring( lastDot + 1, classname.length() );
      // classname is something like Aaa.  
      // Now we can go looking for the Data Object named 'Aaa'.
      // When we find it, we return the table name for that Data Object.
      /* System.err.println("classname ="+classname ); */
         DOTreeNode treeRoot = new DOTreeNode( project, true );
         Vector children = treeRoot.getChildren();
         for ( int k = 0; k < children.size(); k++ ) {
            DOTreeNode node = (DOTreeNode) children.elementAt(k);
            if ( null != ( table = searchChildrenForTable( node, classname ) ) )
               return table;
         }
         return table;
      }
   
      private String searchChildrenForTable( DOTreeNode node, String classname )
      throws Exception {
         String table = null;
         DataObject DO = node.getDataObject();
      /* System.err.println("DO.getName() ="+DO .getName()); */
         if ( ! ( DO instanceof DODSProject || DO instanceof DOPackage ) ) {
            if ( DO.getName().equals( classname ) )
               return DO.getDBTableName();
         }
         Vector children = node.getChildren();
         if ( null != children ) {
            for ( int k = 0; k < children.size(); k++ ) {
               table = searchChildrenForTable( 
                  (DOTreeNode) children.elementAt( k ),
                  classname );
               if ( null != table )
                  return table;
            }
         }
         return table;
      }
   
      public void writeFileHeader(  DataObject o ) throws Exception {
	 /*
         tagSet.setTagValue( BEGIN_COMMENT, getBeginCommentMark() );
         tagSet.setTagValue( END_COMMENT,   getEndCommentMark() );
	 */
         defaultWriteFileHeader( o );
      
         Vector ancestors = o.getAncestors();
      /* System.err.println("ancestors.size()="+ancestors.size()); */
         if ( null != ancestors ) {
            for ( int i = 0; i < ancestors.size(); i++ ) {
               DataObject p = (DataObject) ancestors.elementAt( i );
            /* System.err.println("p.getName()="+p.getName()); */
               output.write(
                  getBeginCommentMark() + " class " + p.getName() + " " +
                  getEndCommentMark() + "\n" );
               if ( null != p ) {
                  Vector members = p.getAttributes();
                  for ( int j = 0; j < members.size(); j++ ) {
                     Attribute attr = (Attribute) members.elementAt( j );
                  /* System.err.println("attr.getName()="+attr.getName()); */
                     memberPrep( attr );
                     writeMemberStuff( attr );
                  }
               }
            }
         }
      
         output.write(	getBeginCommentMark() + " class " + o.getName() + " " +
            getEndCommentMark() + "\n" );
      }
   
      public void writeMemberStuff( Attribute  a ) throws Exception {
         if ( a.isConstant() ) 
            return;
      
      // FIX should this warning be in SourceGenerator_DO as well?
      // It is not as necessary there because the generated DO code
      // will not compile, but that's an ugly way to alert the user.
         String name = a.getName().toLowerCase();
         if ( name.equals( CoreDO.NAME_OBJECTID ) || name.equals( CoreDO.NAME_VERSION ) )
            throw new Exception( "ERROR:  the field " + name +
               " is already defined by the class GenericDO." );
      
         if ( a.isArray() ) {
         // FIX complete implementation?
         // arrays are handled as separate tables
         } 
         else {
            defaultWriteMemberStuff( a );
         }
      }
   
      public void writeFileFooter(  DataObject o ) throws Exception {
         tagSet.setTagValue( INDEXES, indexes );
         defaultWriteFileFooter( o );
      }
   
      private String getBeginCommentMark() throws Exception {
         String ret = null;
         if ( db_vendor == null ) {
            throw new Exception( "No Database Vendor set." );
         } 
         ret = tagSet.getTagValue( BEGIN_COMMENT );
         return ret;
      }
   
      private String getEndCommentMark() throws Exception {
         String ret = null;
         if ( db_vendor == null ) {
            throw new Exception( "No Database Vendor set." );
         } 
         ret = tagSet.getTagValue( END_COMMENT );
         return ret;
      }
   
}
