package com.lutris.xml.xmlc;

import com.lutris.xml.xmlc.XMLObject;
import java.io.File;
import java.io.FilenameFilter;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Vector;

/*
 * @author Hani Suleiman (hani@fate.demon.co.uk)
 */
public class XMLCLoader extends Thread
{
  /**
   * The class file base directory.
   */
  private File dir;
  
  /**
   * Internal mapping of interfaces to concrete classes.
   */
  private HashMap classes = new HashMap();
  
  /**
   * List of classnames mapped to html files.
   */
  private HashMap mappings = new HashMap();
  
  private long pause = 60000;
  
  private String[] xmlcArgs;
  
  private boolean monitoring;
  
  private class Page
  {
    String html;
    String className;

    public Page(String className, String html)
    {
      this.className=className;
      this.html=html;
    }    
  }
  
  public XMLCLoader(String loadDir)
  {
    dir = new File(loadDir);
    if(!dir.exists() || !dir.isDirectory()) throw new IllegalArgumentException("Invalid directory " + loadDir);
    setName("XMLCLoaderUpdater");
    setPriority(Thread.MIN_PRIORITY);
  }
  
  /**
   * Set the class name to html mappings.
   * @ param maps A hashtable with fully qualified class names as keys, and html files as values.
   */
  public void setMappings(HashMap maps)
  {
    //create the object mappings hash
    mappings = new HashMap();
    Iterator iter = maps.entrySet().iterator();
    while(iter.hasNext())
    {
       Map.Entry entry = (Map.Entry)iter.next();
      mappings.put(entry.getKey(), new Page((String)entry.getKey(), (String)entry.getValue()));
    }
    if(xmlcArgs!=null && !monitoring)
    {
      monitoring=true;
      start();
    }
  }
  
  /**
   * Set the base arguments to pass to the XMLC compiler.
   */
  public void setXMLCBaseArgs(String[] args)
  {
    xmlcArgs = args;
    if(mappings!=null && !monitoring)
    {
      monitoring=true;
      start();
    }
  }
  
  /**
   * Set the time in ms in between checks for changes html pages.
   */
  public void setPollingInterval(long ms)
  {
    pause = ms;
  }
  
  /**
   * Get the time in ms in between checks for changes in html pages.
   */
  public long getPollingInterval()
  {
    return pause;
  }
  
  /**
   * Get a page, given a fully qualified interface name.
   */
  public XMLObject getPage(String className)
  {
    Page page = (Page)mappings.get(className);
    if(page==null) throw new IllegalArgumentException("Unmapped page " + className + " requested!");
    Class c = (Class)classes.get(className);
    if(c==null)
    {
      synchronized(page)
      {
        try
        {
          //System.out.println("Waiting on " + className + "...");
          page.wait();
          c = (Class)classes.get(className);
          //System.out.println("Found " + c + " for " + className);
        }
        catch(InterruptedException ex)
        {}
      }
    }
    try
    {
      return (XMLObject)c.newInstance();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
    return null;
  }
  
  public void run()
  {
    while(monitoring)
    {
      //Check all mapped files
      Iterator iter = mappings.values().iterator();
      while(iter.hasNext())
      {
        Page page = (Page)iter.next();
        String className = page.className;
    
        //Get the html file, so we can check the modified time
        File htmlFile = new File(page.html);
        //Ensure the mapped classname exists, compile it if it doesnt.
        //Note that once the classloader loads the interface,
        //it won't be recompiled during the jvm's lifetime
        try
        {
          Class.forName(className);
        }
        catch(ClassNotFoundException ex)
        {
          //Generate the stub interface
          System.out.println("stub not found for " + className + ", recompiling...");
          String[] myArgs = new String[xmlcArgs.length+4];
          myArgs[0] = "-stub";
          for(int i=0;i<xmlcArgs.length;i++)
          {
            myArgs[i+1] = xmlcArgs[i];
          }
          myArgs[xmlcArgs.length+1] = "-class";
          myArgs[xmlcArgs.length+2] = className;
          myArgs[xmlcArgs.length+3] = htmlFile.toString();
//          for(int i=0;i<myArgs.length;i++)
//         {
//            System.out.print(myArgs[i] + " ");
//          }
//          System.out.println();
          com.lutris.xml.xmlc.commands.xmlc.XMLC.main(myArgs);
          System.out.println("stub compilation for " + className + " complete!");
        }
        if(htmlFile.exists())
        {
          String result = getImpl(className, htmlFile);          
          if(result==null)
          {
//            System.out.println("Result is null, need to compile html file ");
            //need to recompile the html file
            System.out.println("Recompiling " + htmlFile);
            String[] myArgs = new String[xmlcArgs.length+4];
            myArgs[0] = "-stamp";
            for(int i=0;i<xmlcArgs.length;i++)
            {
              myArgs[i+1] = xmlcArgs[i];
            }
            myArgs[xmlcArgs.length+1] = "-class";
            myArgs[xmlcArgs.length+2] = className;
            myArgs[xmlcArgs.length+3] = htmlFile.toString();
/*            System.out.print("Invoking xmlc with args:");
            for(int i=0;i<myArgs.length;i++)
            {
              System.out.print(myArgs[i] + " ");
            }
            System.out.println();
            */
            com.lutris.xml.xmlc.commands.xmlc.XMLC.main(myArgs);
            System.out.println("Compilation of " + className + " impl complete");
            result = getImpl(className, htmlFile);
          }
          
          // assume compilation succeeded?
          Class pageClass = null;
          try
          {
            pageClass = Class.forName(result);
          }
          catch(ClassNotFoundException ex)
          {
            System.out.println("Class not found: " + result + ":compilation failed?");
          }
          if(pageClass!=null)
          {
            classes.put(className, pageClass);
          }
          
          //Should we notify if we haven't put in a new class? Hrm, let's do it and blowup later.
          synchronized(page)
          {
            page.notifyAll();
          }
        }
        else
        {
          System.out.println("Uhoh, html file " + htmlFile + " doesn't exist!");
          //What to do if html doesn't exists?!
        }
      }
      try
      {
        sleep(pause);
      }
      catch(InterruptedException ex) {}
    }
  }
  
  private String getImpl(String className, File htmlFile)
  {
    String unqualClassName = className.substring(className.lastIndexOf('.')+1, className.length());
    String[] filenames = getFilesForClass(className);
    long htmlModified = htmlFile.lastModified();
    String result = null;
    String oldFile = null;
    for(int i=0;i<filenames.length;i++)
    {
      String time = filenames[i].substring(filenames[i].lastIndexOf(unqualClassName) + unqualClassName.length(), filenames[i].length()-6);
      long mod = Long.parseLong(time);
      if(mod>=htmlModified)
      {
        //htmlFile.setLastModified(System.currentTimeMillis());
        result = className + mod;
      }
      else
      {
        oldFile = className + mod;
        
        if(htmlModified-mod>(1000*60*60*24))
        {
          //Old files, wipe em?
          File toDel = new File(filenames[i]);
          toDel.delete();
          System.out.println("Deleted old file " + filenames[i]);
        }
      }
      if(result!=null) return result;
    }
    return oldFile!=null ? oldFile : null;
  }
  
  private String[] getFilesForClass(String className)
  {
    String unqualClassName = className.substring(className.lastIndexOf('.')+1, className.length());
    String myPackage = className.substring(0, className.lastIndexOf(unqualClassName));
    File currentDir = dir;
    StringTokenizer st = new StringTokenizer(myPackage, ".");
    //descend into the murky depths of the package tree...
    while(st.hasMoreTokens())
    {
      currentDir = new File(currentDir + File.separator + st.nextToken());
      if(!currentDir.exists() || !currentDir.isDirectory())
      {
        return new String[0];
      }
    }
    //we're there! Now lets grab the .class file list
    String[] list = currentDir.list(new ClassFileFilter(unqualClassName));
    
    //yuck, this is a crappy way of making sure we have paths relative to our root
    for(int i=0;i<list.length;i++)
    {
      StringBuffer buf = new StringBuffer(currentDir.toString());
      buf.append(File.separator);
      buf.append(list[i]);
      list[i] = buf.toString();
    }
    return list;
  }
    

  static class ClassFileFilter implements FilenameFilter
  {
    private String clazz;
    
    public ClassFileFilter(String clazz)
    {
      this.clazz = clazz;
    }
    
    public boolean accept(File f, String s)
    {
      if(s.startsWith(clazz) && s.endsWith(".class") && s.length()>6+clazz.length())
      {
        String extra = s.substring(s.indexOf(clazz)+clazz.length(), s.lastIndexOf(".class"));
        //Check it's all numbers, so it is something we generated.
        char[] chars = extra.toCharArray();
        for(int i=0;i<chars.length;i++)
        {
          if(chars[i]<'0' || chars[i]>'9')
          {
            return false;
          }
        }
        return true;
      }
      return false;
    }
  }

/*	public static void main(String[] argv)
  {
    XMLCLoader loader = new XMLCLoader("/projects/DIY/servlets/classes");
    HashMap hash = new HashMap();
    String htmlRoot = "/projects/DIY/pages/html" + "/mydiy/";
    HashMap mappings = new HashMap();
    String pagePackage = "com.sunshinenyc.web.pages.mydiy.";
    mappings.put(pagePackage + "Home", htmlRoot + "mydiy_home.htm");
    mappings.put(pagePackage + "Login", htmlRoot + "mydiy_login.htm");
    mappings.put(pagePackage + "UpdateFolder", htmlRoot + "mydiy_foldersedit.htm");
    mappings.put(pagePackage + "LoginError", htmlRoot + "mydiy_loginerror.htm");
    mappings.put(pagePackage + "ReclaimPassword1", htmlRoot + "mydiy_passwdreclam1.htm");
    mappings.put(pagePackage + "ReclaimPassword2", htmlRoot + "mydiy_passwdreclam2.htm");
    mappings.put(pagePackage + "ReclaimPassword3", htmlRoot + "mydiy_passwdreclam3.htm");
    mappings.put(pagePackage + "ReclaimPassword4", htmlRoot + "mydiy_passwdreclam4.htm");
    mappings.put(pagePackage + "Register1", htmlRoot + "mydiy_reg1.htm");
    mappings.put(pagePackage + "Register1Error", htmlRoot + "mydiy_reg1error.htm");
    mappings.put(pagePackage + "Register2", htmlRoot + "mydiy_reg2.htm");
    mappings.put(pagePackage + "RegisterConfirm", htmlRoot + "mydiy_regconfirm.htm");
    mappings.put(pagePackage + "PrefUpdate", htmlRoot + "mydiy_prefupdate.htm");
    mappings.put(pagePackage + "FolderView", htmlRoot + "mydiy_projfolder.htm");
    loader.setMappings(mappings);
    String[] xmlcArgs = {"-keep", "-workdir", "e:/temp", "-javac","e:/JRun/bin/jikesw","-d","/projects/DIY/servlets/classes","-classpath","/projects/DIY/servlets/classes;/projects/XMLC/build/enhydra.jar;e:/jdk1.3/jre/lib/rt.jar"};
    loader.setXMLCBaseArgs(xmlcArgs);
    TesterThread tester = new TesterThread(loader);
    tester.start();
  }
  
  static class TesterThread extends Thread
  {
    XMLCLoader loader;
    
    public TesterThread(XMLCLoader loader)
    {
      this.loader=loader;
    }
    
    public void run()
    {
      while(true)
      {
        com.sunshinenyc.web.pages.mydiy.Login login = (com.sunshinenyc.web.pages.mydiy.Login)loader.getPage("com.sunshinenyc.web.pages.mydiy.Login");
        System.out.println(login.toDocument());
        try
        {
          sleep(7000);
        }
        catch(InterruptedException ex)
        {}
      }
    }
  }*/
}

