/*
 * Enhydra XMLC
 * Copyright (C) 1999, Lutris Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * For more information:
 * http://www.enhydra.org
 * http://www.lutris.com
 * mailto:info@lutris.com
 *
 * $Id: XMLC.java,v 1.11 1999/10/23 08:21:42 markd Exp $
 */

package com.lutris.xml.xmlc.commands.xmlc;

import com.lutris.xml.xmlc.*;
import com.lutris.xml.xmlc.compiler.*;
import com.lutris.xml.xmlc.commands.options.*;
import com.lutris.xml.xmlc.dom.XMLCDom;
import com.lutris.xml.xmlc.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import java.io.*;

/**
 * XMLC compiler program.  See the user manual for details on the
 * command line.
 */
public class XMLC {
    /*
     * Object containing the parsed standard options.
     */
    private XMLCOptions options;

    /*
     * Document being compiled.
     */
    private Document document;

    /*
     * Error output.
     */
    ErrorReporter errorReporter;

    /*
     * Trace and verbose output.
     */
    private PrintWriter traceOut = new PrintWriter(System.err, true);

    /*
     * XML DOM management object.
     */
    private XMLCDom xmlcDom;

    /*
     * Print out a stack trace.  This works around bugs in SAXException that
     * cause printing a stack trace to generate a null pointer exception.  We
     * can't just fix this in the OpenXML source (we did), because other
     * parsers (IBM) include that same broken class.
     */
    private void printStackTrace(Throwable except) {
        if (except instanceof SAXException) {
            Exception except2 = ((SAXException)except).getException();
            if (except2 != null) {
                except2.printStackTrace();
                return;
            }
        }
        except.printStackTrace();
    }
    
    /*
     * Handle an exception, printing the approriate error
     * message.
     */
    public void printException(Throwable except) {
        // Must check that options have been parsed before using them.
        if (except instanceof XMLCException) {
            System.err.println("Error: " + except.getMessage());
            if ((options != null) && (options.verbose)) {
                printStackTrace(except);
            }

            // If this is result of a java.lang exception, print stack.
            Throwable cause = ((XMLCException)except).getCause();
            if ((cause != null)
                && cause.getClass().getName().startsWith("java.lang.")) {
                printStackTrace(cause);
            }
        } else if ((except instanceof FileNotFoundException)
                   || (except instanceof IOException)) {
            System.err.println("Error: " +except.getClass().getName() + ": "
                               + except.getMessage());
            if ((options != null) && (options.verbose)) {
                printStackTrace(except);
            }
        } else {
            System.err.println("Error: " + except.getClass().getName() + ": "
                               + except.getMessage());
            printStackTrace(except);
        }
    }

    /*
     * Parse the page into the DOM and perform various checks and edits.
     */
    private void parsePage()
        throws XMLCException, FileNotFoundException, IOException, SAXException {
        if (options.verbose) {
            traceOut.println(">>> parsing " + options.sourceDocument);
        }

        Parse parser = new Parse(errorReporter,
                                (options.parseInfo ? traceOut : null));
        xmlcDom = parser.parse(options.sourceDocument,
                               options.parser,
                               options.parserOptions,
                               options.documentClassName,
                               options.extendClassName,
                               options.htmlProprietaryTags);
        document = xmlcDom.getDocument();

        if (options.printInfo) {
            if (options.verbose) {
                traceOut.println(">>> printing document info");
            }
            PrintInfo info = new PrintInfo(document, xmlcDom);
            info.printInfo(traceOut);
        }


        EditDOM domEditor = new EditDOM();
        if (options.urlMappings != null) {
            domEditor.addURLMappings(options.urlMappings);
        }
        if (options.urlRegExpMappings != null) {
            domEditor.addURLRegexpMappings(options.urlRegExpMappings);
        }
        if (options.urlSettings != null) {
            domEditor.addURLSettings(options.urlSettings);
        }
        if (options.urlSettings != null) {
            domEditor.addURLSettings(options.urlSettings);
        }
        if (options.deleteElementClasses != null) {
            domEditor.addDeleteClasss(options.deleteElementClasses);
        }
        domEditor.edit(xmlcDom);

        if (options.dump) {
            if (options.verbose) {
                traceOut.println(">>> dumping document hierarchy");
            }
            XMLCUtil.printNode("DOM hierarchy", document, 
                               XMLCUtil.PRINT_ALL,
                               traceOut);
        }
    }

    /*
     * Generate the Java source file.
     */
    private void generateJavaSource() throws XMLCException, IOException {
        if (options.verbose) {
            traceOut.println(">>> generating code");
        }
        if (options.stamp) {
            long stamp = System.currentTimeMillis();
            options.unqualClassName = options.unqualClassName + stamp;
            String s = options.javaSource.toString();
            String outputFile = s.substring(0, s.lastIndexOf('.')) + stamp + s.substring(s.lastIndexOf('.'), s.length());
            options.javaSource = new File(outputFile);
        }
        CodeGenerator codeGen =
            new CodeGenerator(xmlcDom,
                              (options.printAccessMethods ? traceOut : null));

        PrintWriter javaOut = new PrintWriter
            (new BufferedOutputStream(new FileOutputStream(options.javaSource)));
        try {
            if(options.stub)
            {
              codeGen.writeInterface(javaOut, options.packageName, options.unqualClassName,
                               options.getImplements());
            }
            else
            {
              codeGen.writeClass(javaOut, options.packageName, options.unqualClassName,
                               options.getImplements());
            }
        } finally {
            javaOut.close();
        }

    }

  /*
     * Generate method information without generating code.
     */
    private void generateJavaSourceInfo() throws XMLCException {
        if (options.verbose) {
            traceOut.println(">>> generating code info without generating source");
        }
        CodeGenerator codeGen =
            new CodeGenerator(xmlcDom,
                              (options.printAccessMethods ? traceOut : null));
    }

    /*
     * Compile the generate source.
     */
    private void compileJavaSource() throws XMLCException {
        if (options.verbose) {
            traceOut.println(">>> compiling code");
        }
        Javac javac = new Javac(options.javac);
        javac.compile(options.javaSource,
                      errorReporter,
                      options.verbose ? traceOut : null,
                      options.getJavacArgs());
    }

    /*
     * Write the DOM to a file.
     */
    private void writeDOM() throws IOException {
        String dir = options.docOut.getParent();
        if (dir != null) {
            new File(dir).mkdirs();
        }
        PrintWriter domOut = new PrintWriter
            (new BufferedOutputStream(new FileOutputStream(options.docOut)));
        try {
            domOut.println(xmlcDom.toDocument());
        } finally {
            domOut.close();
        }
    }

    /*
     * Generate the Jave source and Compile the page.
     */
    private void compilePage()
        throws XMLCException, FileNotFoundException, IOException {
        try {
            generateJavaSource();
            if (!options.noCompile) {
                compileJavaSource();
            }
        } finally {
            if (!options.keep) {
                options.javaSource.delete();
            }
        }
    }

    /*
     * Print the XMLC version number.
     */
    private void printVersion() {
//				System.out.println("Enhydra XMLC version " + XMLCVersion.VERSION);
        System.out.println("See http://www.enhydra.org for latest distribution");
    }

    /*
     * Parse arguments and compile the page.
     */
    public void compile(String[] args)
        throws XMLCException, FileNotFoundException, IOException, SAXException {
        
        // Setup and document parsing.
        options = new XMLCOptions();
        OptionsParser optParser = new OptionsParser(options,
                                                    args);
        if (options.printVersion) {
            printVersion();
            if (options.sourceDocument == null) {
                return;
            }
        }

        errorReporter = new ErrorReporter(new PrintWriter(System.err, true),
                                          options.sourceDocument.getPath());
        parsePage();
        if (options.docOut == null) {
            compilePage();
        } else {
            if (options.printAccessMethods) {
                generateJavaSourceInfo();
            }
            writeDOM();
        }
        if (options.verbose) {
            traceOut.println(">>> completed");
        }
    }

    /*
     * Parse arguments and compile the page.  Print errors that
     * are thrown.
     */
    private void compileHandleErrors(String[] args) {
        try {
            compile(args);
        } catch (Exception except) {
            printException(except);
        }
        
    }

    /*
     * Program entry point.
     */
    public static void main(String[] args) {
        XMLC xmlc = new XMLC();
        xmlc.compileHandleErrors(args);
    }
}
