/*
 * Enhydra XMLC
 * Copyright (C) 1999, Lutris Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * For more information:
 * http://www.enhydra.org
 * http://www.lutris.com
 * mailto:info@lutris.com
 *
 * $Id: XMLCOptions.java,v 1.9 1999/10/23 08:21:43 markd Exp $
 */

package com.lutris.xml.xmlc.commands.xmlc;

import com.lutris.xml.xmlc.XMLCException;
import com.lutris.xml.xmlc.compiler.*;
import com.lutris.xml.xmlc.commands.options.*;
import com.lutris.xml.xmlc.parsers.XMLCParser;
import com.lutris.xml.xmlc.html.HTMLProprietaryTags;
import java.io.File;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;

/**
 * OptionSet for xmlc command.
 */
public class XMLCOptions implements OptionSet {
    /**
     * Valid parsers.
     */
    public static final String[] validParsers = {
        Parse.OPENXML_PARSER,
        Parse.IBM_PARSER,
        Parse.HTML_SWING_PARSER,
        Parse.HTML_TIDY_PARSER
    };

    /**
     * Usage message.
     */
    private static final String USAGE = "wrong # args: xmlc [options] [optfile.xmlc ...] src.ext";

    /**
     * Source file to compile (a positional argument).
     */
    public File sourceDocument;
    
    /**
     * Parser to use.
     */
    public String parser;

    /**
     * Keep generate java source.
     */
    public boolean keep;

    /**
     * If true, don't compile the generate java source.
     */
    public boolean noCompile = false;

    /**
     * Print information about the page.
     */
    public boolean printInfo;

    /**
     * Dump out the DOM tree.
     */
    public boolean dump;

    /*
     * Name of the generated java source file.
     */
    public File javaSource;

    /**
     * Fully qualified class name, or null if not specified.
     */
    public String className;

    /*
     * List of interfaces the class will implement.
     */
    public Vector willImplement;

    /**
     * Package name, maybe null.
     */
    public String packageName;

    /**
     * Unqualified class name.
     */
    public String unqualClassName;

    /**
     * Verbose output.
     */
    public boolean verbose;

    /**
     * Information about parsing.
     */
    public boolean parseInfo;

    /**
     * Information about access methods that are generated.
     */
    public boolean printAccessMethods;

    /**
     * A HTML frameset is being compiled. 
     */
    public boolean oldHtmlFrameSet;

    /**
     * Table of URL mappings, indexed by original URL.
     */
    public Hashtable urlMappings;

    /**
     * Vector of regexp url mappings (URLRegexpMapping).
     */ 
    public Vector urlRegExpMappings;

    /**
     * Table of URL setting, indexed by id.
     */
    public Hashtable urlSettings;

    /**
     * Table of element classes that are to be deleted.
     */
    public Vector deleteElementClasses;

    /**
     * If not null, write a static document to this file instead
     * of generating a java class.
     */
    public File docOut = null;

    public File workDir = new File(".");
  
    /**
     * Java compiler program.
     */
    public String javac = "javac";

    /**
     * Print version number.
     */
    public boolean printVersion = false;

    /**
     * Parser options.
     */
    public int parserOptions = 0;

    /*
     * Options and arguments for java compiler.  Each element is an
     * argument to the compiler.
     */
    private Vector javacArgs;

    /**
     * Ordered list of Option objects associated with this OptionSet.
     */
    private Vector optionList = new Vector();

    /**
     * Hash table of valid Option objects.
     */
    private Hashtable options = new Hashtable();

    /**
     * Should the parser do validation.  If null, its not specified.
     * Value used to set parserOptions.
     */
    public Boolean validateDocument = null;

    /**
     * Class the generated class should extend.
     */
    public String extendClassName = null;

    /**
     * DOM Document class to use.
     */
    public String documentClassName = null;

    /**
     * stamp the classes with a unique ID.
     */
    public boolean stamp = false;
  
    /**
     * Generate interfaces.
     */
    public boolean stub = false;
  
    /**
     * Proprietary tags and attribute definitions.
     */
    public HTMLProprietaryTags htmlProprietaryTags = null;

    /**
     * Class for javac flags that don't take arguments.
     */
    private class JavacFlag extends Option {
        /**
         * Constructor.
         */
        public JavacFlag(String option, String optHelp) {
            super(XMLCOptions.this, option, 0, false, optHelp);
        }

        /**
         * Parse an instance of the option.
         */
        protected void parse(String[] args) throws XMLCException {
            if (javacArgs == null) {
                javacArgs = new Vector();
            }
            javacArgs.addElement(getName());
        }
    }

    /**
     * Class for javac option that takes one argument.
     */
    private class JavacOption extends Option {
        /**
         * Constructor.
         */
        public JavacOption(String option, String optHelp) {
            super(XMLCOptions.this, option, 1, false, optHelp);
        }

        /**
         * Parse an instance of the option.
         */
        protected void parse(String[] args) throws XMLCException {
            if (javacArgs == null) {
                javacArgs = new Vector();
            }
            javacArgs.addElement(getName());
            javacArgs.addElement(args[0]);
        }
    }

    /**
     * Class for implementing the options that pass arbitrary arguments on to
     * javac.
     */
    private class JavacArbitraryOption extends Option {
        /**
         * Constructor.
         */
        public JavacArbitraryOption(String option, int numArgs, String optHelp) {
            super(XMLCOptions.this, option, numArgs, true, optHelp);
        }

        /**
         * Parse an instance of the option.
         */
        protected void parse(String[] args) throws XMLCException {
            if (javacArgs == null) {
                javacArgs = new Vector();
            }
            for (int idx = 0; idx < numArgs; idx++) {
                javacArgs.addElement(args[idx]);
            }
        }
    }

    /**
     * Class for -urlregexpmapping option.
     */
    private class UrlRegExpMappingOption extends Option {
        /**
         * Constructor.
         */
        public UrlRegExpMappingOption () {
            super(XMLCOptions.this, "-urlregexpmapping", 2, true,
                  "regexp replace - Specify a regular expression to match URLs and a replacement pattern");
        }

        /**
         * Parse an instance of the option.
         */
        protected void parse(String[] args) throws XMLCException {
            if (urlRegExpMappings == null) {
                urlRegExpMappings = new Vector();
            }
            urlRegExpMappings.addElement(new URLRegexpMapping(args[0], args[1]));
        }
    }

    /**
     * Class for the implements option.
     */
    private class ImplementsOption extends Option {
        /**
         * Constructor.
         */
        public ImplementsOption() {
            super(XMLCOptions.this, "-implements", 1, true,
                "interface - Specify an interface the generate class will implement");
        }

        /**
         * Parse an instance of the option.
         */
        protected void parse(String[] args) throws XMLCException {
            if (willImplement == null) {
                willImplement = new Vector();
            }
            willImplement.addElement(args[0]);
        }
    }

    /**
     * Class for deprecated -html:frameset option.
     * FIXME: delete this after several releases, added in 1.1.
     */
    private class HTMLFrameSetOption extends FlagOption {
        /**
         * Construct a new object.
         *
         * @param optSet The option set this option is associated with.
         */
        public HTMLFrameSetOption(OptionSet optSet) {
            super(optSet, "-html:frameset", "oldHtmlFrameSet",
                  "- Deprecated and ignored.");
        }
        
        /**
         * Parse an instance of the option and set the field in the Options
         * object using Java reflection.
         *
         * @param args The option's arguments.  Should have none.
         */
        public void parse(String[] args) throws XMLCException {
            System.err.println("Notice: The -html:frameset option is deprecated and ignored");
            super.parse(args);
        }
    }

    /**
     * Class for the -html:addtagset option.
     */
    private class HTMLAddTagSetOption extends Option {
        /**
         * Construct a new object.
         *
         * @param optSet The option set this option is associated with.
         */
        public HTMLAddTagSetOption(OptionSet optSet) {
            super(XMLCOptions.this, "-html:addtagset", 1, true,
                "tagsetname - Add a predefined set of proprietary tags to the list of valid HTML tags.");
        }
        
        /**
         * Parse an instance of the option and set the field in the Options
         * object using Java reflection.
         *
         * @param args The option's arguments.  Should have one.
         */
        public void parse(String[] args) throws XMLCException {
            if (htmlProprietaryTags == null) {
                htmlProprietaryTags = new HTMLProprietaryTags();
            }
            htmlProprietaryTags.addTagSet(args[0]);
        }
    }

    /**
     * Class for the -html:addtag option.
     */
    private class HTMLAddTagOption extends Option {
        /**
         * Construct a new object.
         *
         * @param optSet The option set this option is associated with.
         */
        public HTMLAddTagOption(OptionSet optSet) {
            super(XMLCOptions.this, "-html:addtag", 2, true,
                "tagname flags - Add a proprietary tag to the list of valid HTML tags.");
        }

        /**
         * Parse the flags representing the content model.
         */
        private int parseFlags(String flagsStr) throws XMLCException {
            // Split list at commas and convert to bit set.
            int flags = 0;
            int startIdx = 0;
            while (startIdx < flagsStr.length()) {
                int nextIdx = flagsStr.indexOf(',', startIdx);
                if (nextIdx < 0) {
                    nextIdx = flagsStr.length(); // End of string
                }
                String flagStr = flagsStr.substring(startIdx, nextIdx);
                if (flagStr.equalsIgnoreCase("inline")) {
                    flags |= HTMLProprietaryTags.TAG_CM_INLINE;
                } else if (flagStr.equalsIgnoreCase("block")) {
                    flags |= HTMLProprietaryTags.TAG_CM_BLOCK;
                } else if (flagStr.equalsIgnoreCase("empty")) {
                    flags |= HTMLProprietaryTags.TAG_CM_EMPTY;
                } else if (flagStr.equalsIgnoreCase("opt")) {
                    flags |= HTMLProprietaryTags.TAG_CM_OPT;
                } else {
                    throw new XMLCException("Invaild flag for " + name
                                            + " \"" + flagStr + "\"");
                }
                startIdx = nextIdx+1;
            }
            return flags;
        }

        /**
         * Parse an instance of the option and set the field in the Options
         * object using Java reflection.
         *
         * @param args The option's arguments.  Should have one.
         */
        public void parse(String[] args) throws XMLCException {
            if (htmlProprietaryTags == null) {
                htmlProprietaryTags = new HTMLProprietaryTags();
            }
            htmlProprietaryTags.addTag(args[0],parseFlags(args[1]));
        }
    }

    /**
     * Class for the -html:addattr option.
     */
    private class HTMLAddAttrOption extends Option {
        /**
         * Construct a new object.
         *
         * @param optSet The option set this option is associated with.
         */
        public HTMLAddAttrOption(OptionSet optSet) {
            super(XMLCOptions.this, "-html:addattr", 1, true,
                "attrname - Add a proprietary attribute the list of valid HTML attributes.");
        }
        
        /**
         * Parse an instance of the option and set the field in the Options
         * object using Java reflection.
         *
         * @param args The option's arguments.  Should have one.
         */
        public void parse(String[] args) throws XMLCException {
            if (htmlProprietaryTags == null) {
                htmlProprietaryTags = new HTMLProprietaryTags();
            }
            htmlProprietaryTags.addAttribute(args[0]);
        }
    }

    /**
     * Add an option to the table.
     */
    private void addOption(Option opt) {
        optionList.addElement(opt);
        options.put(opt.getName(), opt);
    }

    /**
     * Constructor.  Initializes all option objects.
     */
    protected XMLCOptions() {
        addOption(new StringOption(this, "-parser", "parser",
                                   "type - Specify the parser to use (openxml, ibm, or swing)."));
        addOption(new FlagOption(this, "-keep", "keep",
                                 "- Keep the generate java source, do not delete it."));
        addOption(new FlagOption(this, "-nocompile", "noCompile",
                                 "- If specified, don't compile the generate java file."));
        addOption(new FlagOption(this, "-info", "printInfo",
                                 "- Print useful information about the page object, such as ids and URLs."));
        addOption(new FlagOption(this, "-dump", "dump",
                                 "- Dump out the DOM tree for the page."));
        addOption(new StringOption(this, "-class", "className",
                                   "class - Fully qualified class name for the generated class."));
        addOption(new FlagOption(this, "-verbose", "verbose",
                                 "- Generate useful output about the compiler process."));
        addOption(new FlagOption(this, "-parseinfo", "parseInfo",
                                 "- Print detailed information about the page parsing."));
        addOption(new FlagOption(this, "-methods", "printAccessMethods",
                                 "- Print the signature of each generated access method."));
        addOption(new HTMLFrameSetOption(this));

        addOption(new StringHashOption(this, "-urlmapping", "urlMappings", false,
                                       "orgURL newURL - Specify a URL to modify and its new value"));
        addOption(new StringHashOption(this, "-urlsetting", "urlSettings", false,
                                       "id newURL - Specify a URL to set given a tag id"));
  addOption (new UrlRegExpMappingOption());

        addOption(new StringVectorOption(this, "-implements", "willImplement",
                                         "interface - Specify an interface the generate class will implement"));

        addOption(new StringVectorOption(this, "-delete-class", "deleteElementClasses",
                                         "classname - Delete all elements are in the specified class (element class attribute, not Java class)"));

        addOption(new FileOption(this, "-docout", "docOut",
                                 "outfile - Write a static document to the outfile instead of generating a class"));

        addOption(new FileOption(this, "-workdir", "workDir",
                                 "workdir - Create java sources in the specified work directory, defaults to the current directory"));
      
        addOption(new StringOption(this, "-javac", "javac",
                                   "prog - Specify the java compiler to use."));
        addOption(new JavacFlag("-g",
                                "- Passed to javac"));
        addOption(new JavacFlag("-O",
                                "- Passed to javac"));
        addOption(new JavacOption("-classpath",
                                "path - Passed to javac"));
        addOption(new JavacOption("-d",
                                  "dir - Passed to javac"));
        addOption(new JavacArbitraryOption("-javacflag", 1,
                                           "flag - Aribitrary flag to pass to the javac program; including leading `-' or `+' character"));
        addOption(new JavacArbitraryOption("-javacopt", 2,
                                           "opt value - Aribitrary option and value to pass to the javac program; including leading `-' or `+' character"));
        addOption(new FlagOption(this, "-version", "printVersion",
                                 "- Print the XMLC version number and exit."));
        addOption(new BooleanOption(this, "-validate", "validateDocument",
                                    "yes|no - Changes the default documentvalidation mode of the parser"));
        addOption(new StringOption(this, "-docclass", "documentClassName",
                                   "classname - DOM Document class for the generated object"));
        addOption(new StringOption(this, "-extends", "extendClassName",
                                   " classname - Class the generated document extends"));
        addOption(new FlagOption(this, "-stamp", "stamp",
                                 "- Generate class names stamped with System.currentTimeMillis()."));
        addOption(new FlagOption(this, "-stub", "stub",
                                 "- Generate stub interfaces rather than classes."));
        addOption(new HTMLAddTagSetOption(this));
        addOption(new HTMLAddTagOption(this));
        addOption(new HTMLAddAttrOption(this));
    }

    /**
     * Get displayable string of valid options for use in error messages.
     */
    public String getOptionsMsg() {
        StringBuffer msg = new StringBuffer();
        Enumeration opts = getOptions();

        while (opts.hasMoreElements()) {
            Option opt = (Option)opts.nextElement();
            msg.append("    ");
            msg.append(opt.getHelp());
            msg.append("\n");
        }
        return msg.toString();
    }

    /**
     * Get an enumeration of all options in the set.
     */
    public Enumeration getOptions() {
        return optionList.elements();
    }

    /**
     * Lookup an option in the table.
     *
     * @param name The option name, including the leading `-'.
     * @return The Option object or null if name is not found. 
     */
    public Option findOption(String name) {
        return (Option)options.get(name);
    }

    /**
     * Determine if a String is in a String array, ignoring case.
     */
    private boolean containsIgnoreCase(String value,
                                       String[] valueList) {
        for (int idx = 0; idx < valueList.length; idx++) {
            if (value.equalsIgnoreCase(valueList[idx])) {
                return true;
            }
        }
        return false;
    }

    /**
     * Generated a expected one of message from a list of strings.
     */
    private String expectedOneOf(String value,
                                 String[] valueList) {
        StringBuffer buf = new StringBuffer();
        buf.append(", got \"");
        buf.append(value);
        buf.append("\", expected one of: ");
        for (int idx = 0; idx < valueList.length; idx++) {
            if (idx == valueList.length-1) {
                buf.append(", or ");
            } else if (idx > 0) {
                buf.append(", ");
            }
            buf.append(valueList[idx]);
        }
        return buf.toString();
    }

    /**
     * Parse the positional arguments.
     */
    public void processPositionalArgs(String[] posArgs) throws XMLCException {
        // No filename is required if print version is specified, otherwise
        // must have an input (although it is allowed with -version).
        if (posArgs.length > 1) {
            throw new XMLCException(USAGE);
        }
        if (posArgs.length > 0) {
            sourceDocument = new File(posArgs[0]);
        }
    }

    /**
     * Default any options in the set that have not been specified.
     * Determination of document type, and hence validation and defaulting of
     * the parser base on document type, is handled outside of this class.
     */
    public void complete() throws XMLCException {
        if (sourceDocument == null) {
            if (printVersion) {
                return;  // Only print version, no other checking.
            }
            throw new XMLCException(USAGE);
        }

        // If source is being created, we need a package and class name.
        if (docOut == null) {
            setupClassName();
        }

        // Check for a valid parser.
        if (parser != null) {
            if (!containsIgnoreCase(parser, validParsers)) {
                throw new XMLCException("Invalid parser: "
                                        + expectedOneOf(parser, validParsers));
            }
            parser = parser.toLowerCase();
        }

        // Set parser options.
        if (validateDocument != null) {
            parserOptions |= (validateDocument.booleanValue() ? 
                              XMLCParser.OPT_VALIDATE_DOCUMENT :
                              XMLCParser.OPT_NO_VALIDATE_DOCUMENT);
        }
    }

    /**
     * Handle class, package and source file names.
     */
    private void setupClassName() throws XMLCException {
        // If class does not exist, use file name if legal.
        if (className == null) {
            String srcName = sourceDocument.getName();
            int dotIdx = srcName.lastIndexOf('.');
            if (dotIdx >= 0) {
                className = srcName.substring(0, dotIdx);
            } else {
                className = srcName;
            }
            if (!JavaLang.legalJavaIdentifier(className)) {
                throw new XMLCException("Can't use source file base name \""
                                        + className
                                        + "\" as class name: "
                                        + "not a legal Java identifier");
            }
            unqualClassName = className;
        } else {
            int idx = className.lastIndexOf('.');
            if (idx < 0) {
                packageName = null;
                unqualClassName = className;
            } else {
                packageName = className.substring(0, idx);
                unqualClassName = className.substring(idx+1);
            }
        }
        javaSource = new File(workDir + File.separator + unqualClassName + ".java");
    }

    /**
     * Get the options for javac.
     */
    public String[] getJavacArgs() {
        String[] args;
        if (javacArgs == null) {
            args = new String[0];
        } else {
            args =  new String[javacArgs.size()];
            javacArgs.copyInto(args);
        }
        return args;
    }

    /**
     * Get the interfaces the class will implement or null if none.
     */
    public String[] getImplements() {
        if(stamp && !stub) {
            if(willImplement==null) willImplement = new Vector();
            willImplement.add(className);
        }
        if (willImplement==null || willImplement.size()==0) {
            return null;
        }
        String[] impls = new String[willImplement.size()];
        willImplement.copyInto(impls);
        return impls;
    }

    /**
     * Get values of object as a printable string.
     */
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("sourceDocument = " + sourceDocument + "\n");
        buf.append("parser = " + parser + "\n");
        buf.append("keep = " + keep + "\n");
        buf.append("noCompile = " + noCompile + "\n");
        buf.append("printInfo = " + printInfo + "\n");
        buf.append("dump = " + dump + "\n");
        buf.append("javaSource = " + javaSource + "\n");
        buf.append("className = " + className + "\n");
        buf.append("packageName = " + packageName + "\n");
        buf.append("unqualClassName = " + unqualClassName + "\n");
        buf.append("verbose = " + verbose + "\n");
        buf.append("parseInfo = " + parseInfo + "\n");
        buf.append("printAccessMethods = " + printAccessMethods + "\n");
        buf.append("docOut = " + docOut + "\n");
        buf.append("javac = " + javac + "\n");
        buf.append("stub = " + stub + "\n");
        buf.append("stamp = " + stamp + "\n");
        buf.append("printVersion = " + printVersion + "\n");
        buf.append("validateDocument = " + validateDocument + "\n");
        buf.append("extendClassName = " + extendClassName + "\n");
        buf.append("documentClassName = " + documentClassName + "\n");
        buf.append("willImplement =\n");
        if (willImplement != null) {
            Enumeration impl = willImplement.elements();
            while (impl.hasMoreElements()) {
                buf.append("    " + (String)impl.nextElement());
            }
        }

        buf.append("urlMappings =\n");
        if (urlMappings != null) {
            Enumeration mappings = urlMappings.keys();
            while (mappings.hasMoreElements()) {
                String key = (String)mappings.nextElement();
                buf.append("    " + (String)key + "="
                       + (String)urlMappings.get(key) + "\n");
            }
        }
        buf.append("urlSettings =\n");
        if (urlSettings != null) {
            Enumeration settings = urlSettings.keys();
            while (settings.hasMoreElements()) {
                String key = (String)settings.nextElement();
                buf.append("    " + (String)key + "="
                           + (String)urlSettings.get(key) + "\n");
            }
        }

        buf.append(" =\n");
        if (deleteElementClasses != null) {
            Enumeration deleteClasses = deleteElementClasses.elements();
            while (deleteClasses.hasMoreElements()) {
                buf.append("    " + (String)deleteClasses.nextElement()
                           + "\n");
            }
        }
        buf.append("javacArgs =\n");
        if (javacArgs != null) {
            for (int idx = 0; idx < javacArgs.size(); idx++) {
                buf.append("    " + (String)javacArgs.elementAt(idx) + "\n");
            }
        }

        return buf.toString();
    }
}

