/*
 * Enhydra XMLC
 * Copyright (C) 1999, Lutris Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * For more information:
 * http://www.enhydra.org
 * http://www.lutris.com
 * mailto:info@lutris.com
 *
 * $Id: AccessMethods.java,v 1.3 1999/10/10 09:17:56 markd Exp $
 */

package com.lutris.xml.xmlc.compiler;

import com.lutris.xml.xmlc.*;
import com.lutris.xml.xmlc.dom.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.*;
import org.w3c.dom.*;

/**
 * Data collection and code generation for element access methods.
 */
class AccessMethods {
    /**
     * Container for document.
     */
    private XMLCDom xmlcDom;

    /**
     * Information collected about each element id.
     */
    private ElementIdTable elementIdTable;

    /**
     * Constructor.
     * @param xmlcDomObj Must contain parsed document.
     */
    public AccessMethods(XMLCDom xmlcDomObj) {
        xmlcDom = xmlcDomObj;
        elementIdTable = new ElementIdTable(xmlcDom);
    }
    
    /**
     * Get the name of the field that will store an element id.
     */
    private String getElementIdField(String id) {
        return "$element" + id;
    }

    /**
     * Process an element.  If the element has an id, record information
     * and return initialization code for document builder.
     *
     * @param element The element being processed.
     * @param elemVarName The name of a variable that will contain
     *  the element during document construction at the point that
     *  the element is created.
     * @return A Java statement that will initialize the access method
     *  fields, given the variable name.  Returns null if no statement
     *  is needed (element doesn't have an id).
     */
    public String processElement(Element element,
                                 String elemVarName) throws XMLCException {
        String id = elementIdTable.recordElementId(element);

        if (id == null) {
            return null;
        } else {
            // Generate code to initialize the field when DOM is being built.
            return getElementIdField(id) + " = ("
                + elementIdTable.getElementClassName(id) + ")"
                + elemVarName + ";\n";
        }
    }

    /**
     * Create an access method definitions.
     */
    private void createAccessMethod(IndentPrinter out,
                                    String id,
                                    PrintWriter methodInfoOutput, boolean sigOnly) {
        String elementClass = elementIdTable.getElementClassName(id);
        String adjustedId = elementIdTable.adjustElementId(id);
        String fieldName = getElementIdField(id);

        String sig = "public " + elementClass + " getElement" + adjustedId + "()";

        out.println("/**");
        out.println(" * Get the element with id <CODE>" + id + "</CODE>.");
        out.println(" * @see " + elementClass);
        out.println(" */");
        if(sigOnly) out.println(sig + ";");
        else {
            out.println(sig + " {");
            out.enter();
            out.println("return " + fieldName +  ";");
            out.leave();
            out.println("}");
            out.println("private " + elementClass + " " + fieldName + ";");
            out.println();
        }
        // Output the signature information, if requested.
        if (methodInfoOutput != null) {
            methodInfoOutput.println(sig + ";");
        }
    }

    /**
     * Create the set method for a text child of an element.
     */
    private void createSetTextMethod(IndentPrinter out,
                                     String id,
                                     PrintWriter methodInfoOutput, boolean sigOnly) {
        String adjustedId = elementIdTable.adjustElementId(id);
        String fieldName = getElementIdField(id);

        String sig = "public void setText" + adjustedId + "(String text)";
        out.println("/**");
        out.println(" * Get the value of text child of element <CODE>" + id + "</CODE>.");
        out.println(" * @see org.w3c.dom.Text");
        out.println(" */");
        if(sigOnly) out.println(sig + ";");
        else {
            out.println(sig + " {");
            out.enter();
            out.println("XMLCUtil.getFirstText(" + fieldName +  ").setData(text);");
            out.leave();
            out.println("}");
            out.println();
        }
        // Output the signature information, if requested.
        if (methodInfoOutput != null) {
            methodInfoOutput.println(sig + ";");
        }
    }

    /**
     * Generate access methods and field definitions.
     */
    public void createAccessMethods(IndentPrinter out,
                                    PrintWriter methodInfoOutput, boolean sigOnly) {
        Enumeration ids = elementIdTable.getIds();
        while (ids.hasMoreElements()) {
            String id = (String)ids.nextElement();
            createAccessMethod(out, id, methodInfoOutput, sigOnly);
            if (elementIdTable.hasSetText(id)) {
                createSetTextMethod(out, id, methodInfoOutput, sigOnly);
            }
        }
    }

    /**
     * Create code for initializing id fields from an enumeration of ids.
     */
    private void createIdFieldInits(IndentPrinter out,
                                    Enumeration ids) {
        while (ids.hasMoreElements()) {
            String id = (String)ids.nextElement();

            out.println("} else if (id.equals(" + JavaLang.createStringConst(id) + ")) {");
            out.enter();
            out.println(getElementIdField(id) + " = ("
                        + elementIdTable.getElementClassName(id) + ")node;");
            out.leave();
        }
    }

    /**
     * Create body of code to set all element access fields to null.
     */
    private void createNullAccessFieldsInit(IndentPrinter out) {
        Enumeration ids = elementIdTable.getIds();
        while (ids.hasMoreElements()) {
            String id = (String)ids.nextElement();
            out.println(getElementIdField(id) + " = null;");
        }
        out.println();
    }

    /**
     * Create body of field initialization method for global id attribute
     * mode.
     */
    private void createGlobalIdModeInit(IndentPrinter out) {
        String idAttr = xmlcDom.getGlobalIdAttribute();
        out.println("if (node instanceof Element) {");
        out.enter();
        out.println("String id = ((Element)node).getAttribute("
                    + JavaLang.createStringConst(idAttr) + ");");
        out.println("if (id.length() == 0) {");  // Block contains empty statement.
        
        createIdFieldInits(out, elementIdTable.getIds());

        out.println("}");  // Closes last else-if
        out.leave();
        out.println("}");
    }

    /**
     * Create body of field initialization method for multple id attribute
     * mode.
     */
    private void createMultipleIdModeInit(IndentPrinter out) {
        out.println("if (node instanceof Element) {");
        out.enter();
        out.println("Element element = (Element)node;");
        out.println("String tag = element.getTagName();");
        out.println();
        
        // For each element tag name, generate code for all ids associated
        // with it.
        Enumeration tags = elementIdTable.getTagNames();
        while (tags.hasMoreElements()) {
            String tag = (String)tags.nextElement();
            String idAttr = elementIdTable.getIdAttributeName(tag);
            out.println("if (tag.equals(" + JavaLang.createStringConst(tag) + ")) {");
            out.enter();
            out.println("String id = element.getAttribute("
                        + JavaLang.createStringConst(elementIdTable.getIdAttributeName(tag)) + ");");
            out.println("if (id.length() == 0) {");  // Block contains empty statement.
            
            // Generate code for each id field for this tag.
            createIdFieldInits(out, elementIdTable.getIds(tag));
            out.println("}");  // Closes last else-if
            out.leave();
            out.println("}");  // Closes tag.equals(xxx) block
        }
        out.leave();
        out.println("}");
    }

    /**
     * Generate access method field initialization function used in clone.
     * There are two versions of the code that can be generated:
     * <UL>
     * <LI> Global id attribute mode.  One attribute names is used for
     *      all ids and is not used anywhere else.  That is,
     *      <CODE>XMLCDom.getGlobalIdAttribute()</CODE> returns a name.
     * <LI> Multiple id attribute mode.  Multiple attributes are declared
     *      of type ID.
     * </UL>
     */
    public void createAccessMethodSync(IndentPrinter out) {
        // Create recursive method to do initialization
        out.println("/**");
        out.println(" * Recursize function to do set access method fields from the DOM.");
        out.println(" * Missing ids have fields set to null.");
        out.println(" */");
        out.println("protected void syncWithDocument(Node node) {");
        out.enter();

        // Generate body to initialize.
        if (xmlcDom.getGlobalIdAttribute() != null) {
            createGlobalIdModeInit(out);
        } else {
            createMultipleIdModeInit(out);
        }
        
        // Generate recursion down the tree.

        out.println("Node child = node.getFirstChild();");
        out.println("while (child != null) {");
        out.enter();
        out.println("syncWithDocument(child);");
        out.println("child = child.getNextSibling();");
        out.leave();
        out.println("}");

        out.leave();
        out.println("}");
        out.println();
    }

    /**
     * Print ids that didn't have accessors generated due to
     * being illegal Java identifiers.
     */
    public void printOmittedIds(PrintWriter out) {
        Enumeration ids = elementIdTable.getInvalidElementIds();
        if (ids != null) {
            out.println("Access methods not created for these element IDs");
            out.println("since they are not legal Java identifiers:");
            while (ids.hasMoreElements()) {
                out.println("    " + (String)ids.nextElement());
            }
        }
    }
}
