/*
 * Enhydra XMLC
 * Copyright (C) 1999, Lutris Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * For more information:
 * http://www.enhydra.org
 * http://www.lutris.com
 * mailto:info@lutris.com
 *
 * $Id: CodeGenerator.java,v 1.39 1999/10/10 09:17:56 markd Exp $
 */

package com.lutris.xml.xmlc.compiler;

import com.lutris.xml.xmlc.*;
import com.lutris.xml.xmlc.dom.*;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.*;
import org.w3c.dom.*;

/**
 * Generate code for the a an XML class.
 */
public class CodeGenerator {
    /*
     * XMLC DOM management object.
     */
    private XMLCDom xmlcDom;

    /*
     * If not null, write information about accessor methods to this
     * file.
     */
    private PrintWriter methodInfoOutput;

    /*
     * List of imports for generate code.
     */
    private static final String[] IMPORT_LIST = {
        "org.w3c.dom.*",
        "com.lutris.xml.xmlc.XMLCUtil",
        "com.lutris.xml.xmlc.XMLCError",
    };

    /*
     * Name of variable in init function containing Document.
     */
    private static final String DOCUMENT_VAR = "document";

    /*
     * Name of variable in init function that holds document type object.
     */
    private static final String DOC_TYPE_VAR = "documentType";

    /*
     * Access method creator.
     */
    AccessMethods accessMethods;

    /*
     * Access constants creator.
     */
    AccessConsts accessConsts;

    /*
     * Document initialization code.
     */
    private String docInit;

    /*
     * Code that builds the DOM object.
     */
    private StringBuffer domInitDefs = new StringBuffer();

    /*
     * Table of tmp node variable names.
     */
    private VarNames nodeVarNames = new VarNames("Node", "$node");

    /*
     * Table of tmp Element node variable names.
     */
    private VarNames elementVarNames = new VarNames("Element", "$elem");

    /*
     * Table of tmp attribute variable names.
     */
    private VarNames attrVarNames = new VarNames("Attr", "$attr");

    /**
     * Generate code to construct a node.  Returned string is an R-value
     * to use in an assignment.  Can't construct top-level documents.
     */
    private String getNodeConstructor(Node node) throws XMLCException {
        switch (node.getNodeType()) {
        case Node.ATTRIBUTE_NODE:
            return DOCUMENT_VAR + ".createAttribute("
                + JavaLang.createStringConst(((Attr)node).getName()) + ");";

        case Node.CDATA_SECTION_NODE:
            return DOCUMENT_VAR + ".createCDATASection("
                + JavaLang.createStringConst(((CDATASection)node).getData()) + ");";

        case Node.COMMENT_NODE:
            return DOCUMENT_VAR + ".createComment("
                + JavaLang.createStringConst(((Comment)node).getData()) + ");";

        case Node.DOCUMENT_FRAGMENT_NODE:
        case Node.DOCUMENT_NODE:
        case Node.DOCUMENT_TYPE_NODE:
            throw new XMLCException("Creating node of this type not supported: "
                                    + node.getClass().getName());

        case Node.ELEMENT_NODE:
            return DOCUMENT_VAR + ".createElement("
                + JavaLang.createStringConst(((Element)node).getTagName()) + ");";

        case Node.ENTITY_NODE:
            return xmlcDom.getDomCodeGen().generateCreateEntity(DOC_TYPE_VAR,
                                                                (Entity)node);

        case Node.ENTITY_REFERENCE_NODE:
            return DOCUMENT_VAR + ".createEntityReference("
                + JavaLang.createStringConst(((EntityReference)node).getNodeName()) + ");";

        case Node.NOTATION_NODE:
            Notation notation = (Notation)node;
            return DOCUMENT_VAR + ".createNotation("
                + JavaLang.createStringConst(notation.getNodeName()) + ", "
                + JavaLang.createStringConst(notation.getPublicId()) + ", "
                + JavaLang.createStringConst(notation.getSystemId()) + ");";

        case Node.PROCESSING_INSTRUCTION_NODE:
            ProcessingInstruction procInstr = (ProcessingInstruction)node;
            return DOCUMENT_VAR + ".createProcessingInstruction("
                + JavaLang.createStringConst(procInstr.getNodeName()) + ", "
                + JavaLang.createStringConst(procInstr.getData()) + ");";

        case Node.TEXT_NODE:
            return DOCUMENT_VAR + ".createTextNode("
                + JavaLang.createStringConst(((Text)node).getData()) + ");";

        default:
            throw new XMLCException("Unknown node type: " + node.getNodeType());
        }
    }

    /**
     * Generate code to to initialize an attribute.
     */
    private void processAttr(String elementVarName,
                             Attr attr,
                             int level)
            throws XMLCException {
        String attrVarName = attrVarNames.getVarName(level);
        if (attr.getSpecified()) {
            domInitDefs.append(attrVarName + " = "
                               + getNodeConstructor(attr)
                               + "\n");
            domInitDefs.append(attrVarName + ".setValue("
                               + JavaLang.createStringConst(attr.getValue())
                               + ");\n");
            domInitDefs.append(elementVarName + ".setAttributeNode(" + attrVarName + ");\n\n");
        }
    }

    /**
     * Generate code to to initialize a attribute.
     */
    private void processNodeAttrs(String elementVarName,
                                  Node node,
                                  int level)
            throws XMLCException {

        NamedNodeMap attrList = node.getAttributes();
        if (attrList != null) {
            for (int idx = 0; idx < attrList.getLength(); idx++) {
                processAttr(elementVarName, (Attr)attrList.item(idx), level);
            }
        }
    }

    /**
     * Process a node, generating code to recreate it.
     */
    private void processNode(String parentVarName,
                             Node node,
                             int level)
            throws XMLCException {
        String nodeVarName;

        // Special handling for Elements; allow for attribute setting
        // without requiring (slower) type casting in generate code.
        if (node instanceof Element) {
            nodeVarName = elementVarNames.getVarName(level);
        } else {
            nodeVarName = nodeVarNames.getVarName(level);
        }
        
        // Create object.
        domInitDefs.append(nodeVarName + " = " + getNodeConstructor(node) + "\n");
            
        // Add to parent
        domInitDefs.append(parentVarName + ".appendChild(" + nodeVarName + ");\n\n");

        // If this is an element with an special attributes (id, class, etc),
        // create a n access methods.
        if (node instanceof Element) {
            Element element = (Element)node;
            String createCode = accessMethods.processElement(element,
                                                             nodeVarName);
            if (createCode != null) {
                domInitDefs.append(createCode);
            }
            accessConsts.processElement(element);
        }

        // Handle attributes.
        if (node instanceof Element) {
            processNodeAttrs(nodeVarName, node, level);
        }
        processChildNodes(nodeVarName, node, level+1);
    }

    /**
     * Process children nodes.
     */
    private void processChildNodes(String nodeVarName,
                                   Node node,
                                   int level)
            throws XMLCException {
        Node child = node.getFirstChild();
        while (child != null) {
            processNode(nodeVarName, child, level);
            child = child.getNextSibling();
        }
    }

    /**
     * Construct an object, converting a DOM into code
     * fragments.
     *
     * @param xmlcDom XMLC DOM object containing the document.
     * @param methodOutput Write information about the generated 
     *  methods to this file if not NULL.
     * @exception XMLCException If an error is detected.
     */
    public CodeGenerator(XMLCDom xmlcDom,
                         PrintWriter methodOutput)
            throws XMLCException {
        this.xmlcDom = xmlcDom;
        methodInfoOutput = methodOutput;
        accessMethods = new AccessMethods(xmlcDom);
        accessConsts = new AccessConsts(xmlcDom);

        // Initialize document and optionally document type objects.
        docInit = xmlcDom.getDomCodeGen().generateDocumentInit(DOCUMENT_VAR,
                                                               DOC_TYPE_VAR);

        Document document = xmlcDom.getDocument();
        DocumentType docType = document.getDoctype();

        // Generate code for document type declaration, if needed.
        if (docType != null) {
            processChildNodes(DOC_TYPE_VAR, docType, 0);
        }

        // Generate code for documents
        processChildNodes(DOCUMENT_VAR, document, 0);

        // Output information about omitted accessors.
        if (methodOutput != null) {
            accessMethods.printOmittedIds(methodOutput);
            accessConsts.printOmittedConstants(methodOutput);
        }
    }

    /**
     * Generate and return the class.
     * @param outWriter Print stream to write generate source to.
     * @param packageName Package name for the class.  If null, then
     *  no package will be specifed.
     * @param unqualClassName Unqualified class name for the generate class.
     * @param willImplement Interfaces the class will implement, or null if
     *  none.
     */
    public void writeClass(PrintWriter outWriter,
                           String packageName,
                           String unqualClassName,
                           String[] willImplement) {
        IndentPrinter out = new IndentPrinter(outWriter);
        out.println("/*");
        out.println(" ************************************");
        out.println(" * XMLC GENERATED CODE, DO NOT EDIT *");
        out.println(" ************************************");
        out.println(" */");

        // Set up package decl and import list
        if (packageName != null) {
            out.println("package " + packageName + ";");
        }
        for (int idx = 0; idx < IMPORT_LIST.length; idx++) {
            out.println("import " + IMPORT_LIST[idx] + ";");
        }
        out.println();

        // Start out the class
        out.print("public class " + unqualClassName + " extends "
                    + xmlcDom.getObjectBaseClass());
        if (willImplement != null) {
            out.print(" implements ");
            for (int idx = 0; idx < willImplement.length; idx++) {
                if (idx > 0) {
                    out.print(", ");
                }
                out.print(willImplement[idx]);
            }
        }
        out.println(" {");

        out.enter();

        // Constants, fields and access methods.
        accessConsts.createAccessConstants(out, methodInfoOutput);
        accessMethods.createAccessMethods(out, methodInfoOutput, false);

        // Define method that initializes DOM
        out.println("/**");
        out.println(" * Create document as a DOM and initialize accessor method fields.");
        out.println(" */");
        out.println("private Document buildDocument() {");
        out.enter();
        out.println(nodeVarNames.getVarDefs());
        out.println(elementVarNames.getVarDefs());
        out.println(attrVarNames.getVarDefs());
        out.println();
        out.println(docInit);
        out.println(domInitDefs.toString());
        out.println("return " + DOCUMENT_VAR  + ";");
        out.leave();
        out.println("}");
        out.println();

        // Code for initialization of access method fields after
        // clone.
        accessMethods.createAccessMethodSync(out);

        // Constructor
        out.println("/**");
        out.println(" * Constructor.");
        out.println(" */");
        out.println("public " + unqualClassName + "() {");
        out.enter();
        out.println("super.setDocument(buildDocument());");
        out.leave();
        out.println("}");
        out.println();

        // Copy constructor
        out.println("/**");
        out.println(" * Copy constructor.");
        out.println(" */");
        out.println("public " + unqualClassName + "(" + unqualClassName + " src) {");
        out.enter();
        out.println("super.setDocument((Document)src.getDocument().cloneNode(true));");
        out.println("syncAccessMethods();");
        out.leave();
        out.println("}");
        out.println();

        // Clone method
        out.println("/**");
        out.println("  * Clone the document.");
        out.println("  * @param deep Must be true, only deep clone is supported.");
        out.println("  */");
        out.println("public Node cloneNode(boolean deep) {");
        out.enter();
        out.println("cloneDeepCheck(deep);");
        out.println("return new " + unqualClassName + "(this);");
        out.leave();
        out.println("}");
        out.println();

        // End of class!
        out.leave();
        out.println("}");
        out.done();
    }

    public void writeInterface(PrintWriter outWriter, String packageName,
                                String unqualClassName, String[] willImplement) {
        IndentPrinter out = new IndentPrinter(outWriter);
        out.println("/*");
        out.println(" ************************************");
        out.println(" * XMLC GENERATED CODE, DO NOT EDIT *");
        out.println(" ************************************");
        out.println(" */");

        // Set up package decl and import list
        if (packageName != null) {
            out.println("package " + packageName + ";");
        }
        for (int idx = 0; idx < IMPORT_LIST.length; idx++) {
            out.println("import " + IMPORT_LIST[idx] + ";");
        }

        // Start out the class
        out.print("public interface " + unqualClassName);
        if (willImplement != null) {
            out.print(" extends ");
            for (int idx = 0; idx < willImplement.length; idx++) {
                if (idx > 0) {
                    out.print(", ");
                }
                out.print(willImplement[idx]);
            }
        }
        out.println(" {");
        out.enter();
                                  
        // access constants
        accessConsts.createAccessConstants(out, methodInfoOutput);
        // Access methods
        accessMethods.createAccessMethods(out, methodInfoOutput, true);
        
        out.println();

        //Inherited methods.
        Method[] methods = new Method[0];
        try {            
            methods = Class.forName(xmlcDom.getObjectBaseClass()).getMethods();
        }
        catch(ClassNotFoundException ex) {}                                  
        for(int i=0;i<methods.length;i++)
        {
          int modifier = methods[i].getModifiers();
          if(Modifier.isStatic(modifier) || Modifier.isFinal(modifier)) continue;
          StringBuffer methodSig = new StringBuffer("public ");
          methodSig.append(methods[i].getReturnType().getName());
          methodSig.append(" ");
          methodSig.append(methods[i].getName());
          methodSig.append("(");
          Class[] params = methods[i].getParameterTypes();
          for(int j=0;j<params.length;j++)
          {
            methodSig.append(params[j].getName() + " param" + (j+1));
            if(j<params.length-1)
              methodSig.append(", ");
          }
          methodSig.append(")");
          Class[] exceptions = methods[i].getExceptionTypes();
          for(int j=0;j<exceptions.length;j++)
          {
            if(j==0) methodSig.append(" throws ");
            methodSig.append(exceptions[j].getName());
            if(j<exceptions.length-1)
              methodSig.append(", ");
          }
          methodSig.append(";");
          out.println(methodSig.toString());
        }
        out.println("}");
    }
}
