/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: Attribute.java,v 1.5 1999/02/12 12:18:03 andy Exp $
 *-----------------------------------------------------------------------------
 */



package dods.data;

import java.io.Serializable; 

/**
 * This class holds all of the information about an Attribute for a DataObject.
 *
 * @author  Philip Martin
 * @version $Revision: 1.5 $
 *
 * Modifications: 
 *   09 August 1999: DODSTypes & Initial Values split by Chris Ryan (cryan@plugged.net.au) 
 */
public class Attribute implements Serializable {

  /**
   * String for the initialJavaValue of the Attribute.
   */
  protected String initialJavaValue;

  /**
   * String for the initialDBValue of the Attribute.
   */
  protected String initialDBValue;

  /**
   * True when there is an error in some part of the Attribute (ie invalid
   * initial value.
   */
  protected boolean error;

  /**
   * True if this attribute is used as an index in the database.
   */
  protected boolean index;

  /**
   * True if this attribute is a reference to another object instead of
   * its own atttribute.
   */
  protected boolean objectRef;

  /**
   * True if this is an array instead of a single instance.
   */
  protected boolean array;

  /**
   * The DODS Type - information about the Java, JDBC and DB type can
   * be found from this string...
   */
  protected String dodsType;

  /**
   * True if this Attribute can be null in the database.
   */
  protected boolean CanBeNull;

  /**
   * True if this Attribute is a referential constraint.
   * The Attribute is a reference to another DO which must exist in the DB.
   */
  protected boolean refConstraint;

  /**
   * True if this Attribute has a constant value.
   */
  protected boolean constant;

  /**
   * True if this Attribute can be used in queries.
   */
  protected boolean usedForQuery;

  /**
   * the name to discribe this Attribute with.
   */
  protected String name;

  /**
   * The size of this attribute in the database.
   */
  protected String size;

  /**
   * Marks the attribute as unique in the database
   * Leigh Dodds (ldodds@ingenta.com)
   */
  protected boolean unique ;

  /**
   * The string which will be used for the javadoc tag in the DataObject.
   */
  protected String javadoc;

  /**
   * if this is an array, this is its size.
   */
  protected int arraySize;

  /**
   * The text related to the error if there is one.
   */
  protected String errorText;

  protected DataObject objectRefDO;


/**
  * Creates an Attribute with the name "NewAttribute"
  */
  public Attribute() {

    this("NewAttribute");

  }


/**
 * Creates an Attribute with the given name.
 *
 * @param attribute_name  The name of the attribute
 */
  public Attribute(String attribute_name) {

    name = attribute_name;
    initialJavaValue = "";
    initialDBValue = "";
    error = false;
    index = false;
    objectRef = false;
    array = false;
    CanBeNull = false;
    usedForQuery = true;
    size = "";
    javadoc = ("/**\n * \n */");
    arraySize = 1;
    errorText = "";
    objectRefDO = null;

  }


/**
 * Set whether the attribute is unique.
 * @author Leigh Dodds (ldodds@ingenta.com)
 *
 * @param unique True if this attribute is unique
 */
  public void setUnique(boolean unique) {

    this.unique = unique ;

  }
   

/**
 * Query whether the attribute is defined as unique
 * @author Leigh Dodds (ldodds@ingenta.com)
 *
 * @return see comment
 */
  public boolean isUnique() {

     return unique ;

  }


/**
 * Sets the error text
 *
 * @param txt  The error message
 */
    public void setErrorText(String txt) {
        errorText = txt;
    }
    public String getErrorText() {
        return errorText;
    }
    public void setJavadoc(String jdoc) {
        javadoc = jdoc;
    }
    public String getJavadoc() {
        return javadoc;
    }
    public void setArraySize(int asize) {
        arraySize = asize;
    }
    public int getArraySize() {
        return arraySize;
    }
    public void setError(boolean error) {
        this.error = error;
    }
    public boolean hasError() {
        return this.error;
    }
    public void setObjectRef(boolean objectRef) {
        this.objectRef = objectRef;
    }
    public boolean isObjectRef() {
        return this.objectRef;
    }
    public void setArray(boolean array) {
        this.array = array;
    }
    public boolean isArray() {
        return this.array;
    }
    public void setCanBeNull(boolean CanBeNull) {
        this.CanBeNull = CanBeNull;
    }
    public boolean canBeNull() {
        return this.CanBeNull;
    }
    public void setUsedForQuery(boolean usedForQuery) {
        this.usedForQuery = usedForQuery;
    }
    public boolean isUsedForQuery() {
        return this.usedForQuery;
    }
    public void setRefConstraint(boolean refConstraint) {
        this.refConstraint = refConstraint;
    }
    public boolean isRefConstraint() {
        return this.refConstraint;
    }
    public void setConstant(boolean constant) {
        this.constant = constant;
    }
    public boolean isConstant() {
        return this.constant;
    }
    public void setIndex(boolean index) {
        this.index = index;
    }
    public boolean isIndex() {
        return this.index;
    }
    public void setName(String name) {
        this.name = name;
    }
    public String getName() {
        return this.name;
    }
    public void setDODSType(java.lang.String dodsType) {
        this.dodsType = dodsType;
    }
    public String getDODSType() {
        return this.dodsType;
    }
    public void setSize(String size) {
        this.size = size;
    }
    public String getSize() {
        return this.size;
    }

    public void setInitialJavaValue(String initialJavaValue) {
        this.initialJavaValue = initialJavaValue;
    }
    public String getInitialJavaValue() {
        return this.initialJavaValue;
    }

    public void setInitialDBValue(String initialDBValue) {
        this.initialDBValue = initialDBValue;
    }
    public String getInitialDBValue() {
        return this.initialDBValue;
    }

    public DataObject getObjectRefDO() {
        return objectRefDO;
    }
    public void setObjectRefDO(DataObject orDO) {
        this.objectRefDO = orDO;
    }

    /**
     * @return the Attribute as a string of the form "<I>name</I> Attribute"
     */
    public String toString() {
        return (name + " Attribute");
    }

    /**
     * @return true if all of the things describing this Attribute are equal.
     */
    public boolean equals(Attribute att) {
        if(!name.equals(att.getName()))
            return false;
        if(!dodsType.equals(att.getDODSType()))
            return false;
        if(CanBeNull != att.canBeNull())
            return false;
        if(array != att.isArray())
            return false;
        if(arraySize != att.getArraySize())
            return false;
        if(error != att.hasError())
            return false;
        if(refConstraint != att.isRefConstraint())
            return false;
        if(constant != att.isConstant())
            return false;
        if(usedForQuery != att.isUsedForQuery())
            return false;
        if(!errorText.equals(att.getErrorText()))
            return false;
        if(index != att.isIndex())
            return false;
        if(!initialJavaValue.equals(att.getInitialJavaValue()))
            return false;
        if(!initialDBValue.equals(att.getInitialDBValue()))
            return false;
        if(!javadoc.equals(att.getJavadoc()))
            return false;
        if(objectRef != att.isObjectRef())
            return false;
        if(!size.equals(att.getSize()))
            return false;
        if(unique != att.isUnique())
            return false;
        return true;
    }
}
