package org.enhydra.xml.binding;

import java.io.IOException;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.input.SAXBuilder;

/**
 * <p>
 *  <code>Unmarshaller</code> takes an XML instance document, which should
 *    conform to some XML Schema, and creates a Java object from the XML
 *    document. This object is an instance of the implementation class, which
 *    in turn implements the interface, created from using 
 *    <code>{@link SchemaMapper#generateClasses}</code> upon the XML Schema.
 * </p>
 * <p>
 *  It also assumes that the generated interface and implementation classes
 *    are in the classpath, as it will perform a <code>Class.forName</code> on
 *    the classes.
 * </p>
 *
 * @author Brett McLaughlin
 */
public class Unmarshaller {

    /**
     * <p>
     *  Simple constructor.
     * </p>
     */
    public Unmarshaller() {
        // Currently, do nothing
    }
   
    /**
     * <p>
     *  This method is the public entry point for unmarshalling an object from
     *    an XML instance document.
     * </p>
     *
     * @param instanceURL <code>URL</code> for the instance document.
     * @return <code>Object</code> - the created Java object, or 
     *         <code>null</code> if problems occur in a way that does not
     *         generate an <code>Exception</code>.
     * @throws <code>IOException</code> when errors in binding occur.
     */
    public static Object unmarshall(URL instanceURL) throws IOException {
        // Read in the document
        SAXBuilder builder = new SAXBuilder(); 

        try {
            Document doc = builder.build(instanceURL); 
            Element rootElement = doc.getRootElement();
            Unmarshaller unmarshaller = new Unmarshaller();
            return unmarshaller.getJavaRepresentation(rootElement);
        } catch (JDOMException e) {
            throw new IOException (e.getMessage());
        }
    }

    /**
     * <p>
     *  This will take an XML document fragment, starting with the provided
     *    <code>Element</code>, and convert it to a Java representation.
     * </p>
     *
     * @param rootElement <code>Element</code> to begin processing on.
     * @return <code>Object</code> - unmarshalled Java object.
     * @throws <code>IOException</code> - when XML cannot be converted to Java.
     */
    private Object getJavaRepresentation(Element rootElement) 
        throws IOException {

        Object obj = null;
        String className = null;

        try {
            Namespace ns = rootElement.getNamespace();

            // Class name is root element name, with initial caps, plus "Impl"
            className = new StringBuffer()
                .append(BindingUtils.initialCaps(rootElement.getName()))
                .append("Impl")
                .toString();

            Class objectClass;

            // Get the class
            try {
                objectClass = Class.forName(className);
            } catch (ClassNotFoundException e) {
                /**
                 * Account for the case where the class was generated implicitly,
                 *   and the name is XXXTypeImpl, not just XXXImpl.
                 */
                className = new StringBuffer()
                    .append(BindingUtils.initialCaps(rootElement.getName()))
                    .append("TypeImpl")
                    .toString();
                objectClass = Class.forName(className);
            }

            // Get a new instance of this class
            obj = objectClass.newInstance();

            // For each attribute, get its name and call mutator
            List attributes = rootElement.getAttributes();
            Method[] methods = objectClass.getMethods(); 

            for (Iterator i = attributes.iterator(); i.hasNext(); ) {
                Attribute att = (Attribute)i.next();

                // Only want attributes for this namespace
                if ((!att.getNamespace().equals(ns)) &&
                    (!att.getNamespace().equals(Namespace.NO_NAMESPACE))) {
                    continue;
                }

                // Determine method to call
                String methodName = new StringBuffer()
                    .append("set")
                    .append(BindingUtils.initialCaps(att.getName()))
                    .toString();

                // Find the method to call, and its parameter type
                for (int j=0; j<methods.length; j++) {
                    if (methods[j].getName().equals(methodName)) {
                        // Since all mutators have one param, get the first one
                        Class[] paramTypes = methods[j].getParameterTypes();
                        Class paramType = paramTypes[0];

                        // Convert the type we have to the correct type
                        Object param = 
                            DataMapping.getParameter(att.getValue(), paramType);

                        // Invoke the method
                        methods[j].invoke(obj, new Object[] { param });
                    }
                }
            }

            // Now do complex objects
            List elements = rootElement.getChildren();

            for (Iterator i = elements.iterator(); i.hasNext(); ) {
                Element element = (Element)i.next();

                // Only want elements for this namespace
                if ((!element.getNamespace().equals(ns)) &&
                    (!element.getNamespace().equals(Namespace.NO_NAMESPACE))) {
                    continue;
                }

                // Determine method to call
                String methodName = new StringBuffer()
                    .append("set")
                    .append(BindingUtils.initialCaps(element.getName()))
                    .toString();

                // Find the method to call, and its parameter type
                for (int j=0; j<methods.length; j++) {
                    if (methods[j].getName().equals(methodName)) {
                        // Since all mutators have one param, get the first one
                        Class[] paramTypes = methods[j].getParameterTypes();
                        Class paramType = paramTypes[0];

                        // Convert the type we have to the correct type
                        Object param = 
                          getJavaRepresentation(element);

                        // Invoke the method
                        methods[j].invoke(obj, new Object[] { param });
                    }
                }
            }
        } catch (ClassNotFoundException e) {
            throw new IOException("The class that the XML instance document " +
                "should be unmarshalled as an instance of, " + className +
                ", must be generated and on the classpath.");
        } catch (InstantiationException e) {
            throw new IOException("Could not create a new instance of the " +
                "class " + className);
        } catch (IllegalAccessException e) {
            throw new IOException("Could not access the class " + className);
        } catch (IllegalArgumentException e) {
            throw new IOException("Could not pass arguments to mutator " +
                "method on new instance.");
        } catch (InvocationTargetException e) {
            throw new IOException("Error in accessing mutator on new " +
                "instance of class.");
        }

        return obj;
    } 

}
