/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * /home/proj/phoneList1/phoneList/tmp3/phoneList/data/PersonBDO.java
 *-----------------------------------------------------------------------------
 */

package phoneList.data;

import java.sql.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;

import dods.builder.sourceGenerators.Query.DataObjectException;
import dods.builder.sourceGenerators.Query.NonUniqueQueryException;

/**
 * PersonBDO contains the same set and get methods as
 * the PersonDO class.
 * Business Object (BO) classes typically need these set and get methods.
 * So by deriving a BO from a BDO, or by implementing a BO that 
 * contains a BDO, the developer of the BO is spared some work.
 *
 * @author <AUTHOR>
 * @version $Revision: 1.6 $
 */
public class PersonBDO {

    /**
     * The PersonDO object upon which the set and get methods operate.
     * This member is protected so that classes derived from PersonBDO
     * can access the underlying Data Object.
     */
    protected PersonDO DO;

    /**
     * Like the class <CODE>PersonDO</CODE>, 
     * this class acts as a factory.
     * Business Object (BO) classes typically need these set and get methods.
     * So by deriving a BO from a BDO, or by implementing a BO that 
     * contains one or more BDOs, the developer of the BO is spared some work.
     */
    public static PersonBDO createVirgin() throws Exception { 
	PersonBDO bdo = new PersonBDO ();
	bdo.DO = PersonDO.createVirgin();
	return bdo;
    }

    /**
     * The createExisting method is used to create a <CODE>PersonBDO</CODE>
     * from a <CODE>PersonDO</CODE> that was returned by 
     * the <CODE>PersonQuery</CODE> class.
     */
    public static PersonBDO createExisting( PersonDO DO ) { 
	PersonBDO bdo = new PersonBDO ();
	bdo.DO = DO;
	return bdo;
    }

    /** 
     * The getBDOarray method performs a database query to
     * return an array of <CODE>PersonBDO</CODE> objects
     * representing all the rows in the <CODE>Person</CODE> table.
     *
     * This method is a minimal example of using a Query class.
     * To restrict the set of objects returned,  you could
     * invoke <CODE>query.setXxx()</CODE>, where <CODE>Xxx</CODE>
     * is an Attribute of <CODE>PersonDO</CODE> which was 
     * marked as "Can be queried" in the DODS Attribute Editor.
     */
    public static PersonBDO[] getBDOarray()
    throws DataObjectException {
        PersonDO[] DOarray = null;
        try {
            PersonQuery query = new PersonQuery();
	    // To restrict the set of objects returned,
	    // you could invoke query.setXxx(), where Xxx
	    // is an Attribute of <CODE>PersonDO</CODE> which was 
	    // marked as "Can be queried" in the DODS Attribute Editor.
            DOarray = query.getDOArray();
        } catch ( NonUniqueQueryException e1 ) {
            // PersonQuery will throw NonUniqueQueryException
            // only if query.requireUniqueInstance() is called
	    // and more than one DO was found.
        } catch ( DataObjectException e2 ) {
            // This could happen if the database server dies, etc.
            throw e2;
        }
        PersonBDO[] BDOarray = new PersonBDO[ DOarray.length ];
        for ( int i = 0; i < DOarray.length; i++ )
            BDOarray[i] = PersonBDO.createExisting( DOarray[i] );
 
        return BDOarray;
    }


    /**
     * The developer of a Business Object that derives from this class
     * can override the methods:<CODE>
     *     beforeAnyGet
     *     beforeAnySet
     *     afterAnySet
     * </CODE> to handle any general assertions or cleanup needed
     * for <CODE>get</CODE> and <CODE>set</CODE> methods.
     */
    protected void beforeAnyGet() {}
    protected void beforeAnySet() {}
    protected void afterAnySet() {}

   /**
    * Get firstName of the PersonDO
    *
    * @return firstName of the PersonDO
    */
   public String getFirstName () 
   throws DataObjectException {
      return DO.getFirstName ();
   }

   /**
    * Set firstName of the PersonDO
    *
    * @param firstName of the PersonDO
    */
   
   public void setFirstName ( String firstName ) 
   throws DataObjectException {
      DO.setFirstName ( firstName );
   }
   

   /**
    * Get lastName of the PersonDO
    *
    * @return lastName of the PersonDO
    */
   public String getLastName () 
   throws DataObjectException {
      return DO.getLastName ();
   }

   /**
    * Set lastName of the PersonDO
    *
    * @param lastName of the PersonDO
    */
   
   public void setLastName ( String lastName ) 
   throws DataObjectException {
      DO.setLastName ( lastName );
   }
   

   /**
    * Get phoneNumber of the PersonDO
    *
    * @return phoneNumber of the PersonDO
    */
   public String getPhoneNumber () 
   throws DataObjectException {
      return DO.getPhoneNumber ();
   }

   /**
    * Set phoneNumber of the PersonDO
    *
    * @param phoneNumber of the PersonDO
    */
   
   public void setPhoneNumber ( String phoneNumber ) 
   throws DataObjectException {
      DO.setPhoneNumber ( phoneNumber );
   }
   


  /**
   * Inserts/Updates the DO into its table.
   *
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void commit() throws SQLException, DatabaseManagerException {

    modifyDO( false );

  }


  /**
   * Deletes the DO from its table.
   *
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void delete() throws SQLException, DatabaseManagerException {

    modifyDO( true );

  }


  /**
   * Modifies the DO within its table.
   * Created by Chris Ryan (cryan@plugged.net.au)
   *
   * NOTE:  This method is available only for DO's that have no
   * references to other DO's.
   * Such references are implemented as foreign keys in the database,
   * imposing an order for the insertion/deletion of referenced objects.
   * DODS currently does not determine this order, and so this method
   * is not (yet) available for DO's with references to other DO's.
   *
   * @param operation See dods.builder.sourceGenerators.DO.GenericDO for values
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  protected void modifyDO( boolean delete )
  throws SQLException, DatabaseManagerException {

    DatabaseManager dbm = null;
    DBTransaction dbt = null;

    try {

      // get the Database manager
      dbm = Enhydra.getDatabaseManager();

      // create a transaction
      dbt = dbm.createTransaction();

      if ( delete )
          dbt.delete( DO );
      else 
          dbt.insert( DO );   // dbt.insert() handles insertions and updates

      // commit the transaction
      dbt.commit();

    } catch (SQLException sqle) {

      StringBuffer message = new StringBuffer("Failed to insert/update DO: ");

      message.append(sqle.getMessage());

      // rollback, if necessary
      if (dbt != null) {

        try {

          dbt.rollback();

        } catch (SQLException sqle2) {

          message.insert(0,"\n");
          message.insert(0,sqle2.getMessage());
          message.insert(0,"Rollback failed: ");

        }

      }

      throw new SQLException(message.toString());

    } finally {

      // release the transaction, if any
      if (dbt != null) {

        dbt.release();

      }

    }

  }


}
