package com.pisoftware.platypus.newApp;
 
import com.lutris.devsys.newApp.*;
 
import java.io.*;
import com.lutris.util.*;
import com.lutris.servlet.multiServer.MultiServer;

/**
 *  Author: <A HREF="MAILTO:cryan@plugged.net.au">Chris Ryan</A>
 *<P>
 * Company: Plugged In Software<BR>
 *          P.O. Box 4130<BR>
 *          St Lucia South<BR>
 *          Brisbane Queensland 4067<BR>
 *          Australia<BR>
 * <A HREF="http://www.plugged.net.au/">http://www.plugged.net.au</A><BR>
 *<P>
 *    Date: 06 October 1999
 *<P>
 * Purpose: PlatypusAppTree is a convenience mechanism for creating a new
 * Platypus/Enhydra application development tree.
 * It can be used as a standalone application or by instantiating
 * an instance, calling the various set methods, and then calling
 * the create method.
 * The default config file is newapp.conf.
 * Uses '/' instead of File.separator since the make environment expects it.
 * <P>
 * This code has used the com.lutris.devsys.newApp.LutrisAppTree.java
 * as the main source.  It's easier this way...
 * <P>
 * Significant differences include:<BR>
 * <MENU>
 * <LI> No usage/setup of JOLT files.
 * <LI> Different directory structure - Platypus uses a 'static' 
 *       (under "src") and 'dynamic' (under "<appname>") area.
 * </MENU>
 *
 * TODO: Change this system to use templates instead of embedded
 *       code.
 */

/*
 *  Copyright (C) 1999 Plugged In Software Pty Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Plugged In Software
 *  PO Box 4130
 *  St. Lucia South Queensland
 *  Australia  4067
 *  info@PIsoftware.com
 */

public class PlatypusAppTree {

    // === Data ===

    protected Config config = null;
    protected static final String defaultConfFile = "newapp.conf";
    protected String rootDir = null;
    protected String appName = null;      // As entered by user
    protected String appNameLC = null;    // Lowercase for package names etc..
    protected String appNameUC = null;    // Uppercase for class names etc...
    protected String packageDir = null;
    protected String packageName = null;
    protected String copyrightName = "Plugged In Software";

    protected String enhydraDir = "/usr/local/enhydra";
    protected String xmlcDir = "/usr/local/XMLC";
    protected String javaHome = "/usr/local/java";
    protected String javaClassPath = javaHome + "/lib/classes.zip:.";
    protected String FS = "/";
    protected String PS = ":";

    // Argument Names
    public static final String APP_NAME = "appName";
    public static final String ROOT_DIR = "rootDir";
    public static final String PACKAGE_DIR = "packageDir";
    public static final String COPYRIGHT_NAME = "copyrightName";
    public static final String ENHYDRA_DIR = "enhydraDir";
    public static final String XMLC_DIR = "xmlcDir";
    public static final String CONF_FILE = "confFile";
    public static final String MULTISERVER_CONF = "multiserver.conf";
    public static final String MULTISERVER_CONF_IN = "multiserver.conf.in";
    public static final String START_SCRIPT = "start";
    public static final String DASH = "-";

    // File Extensions
    public static final String EXT_SH = ".sh";
    public static final String EXT_CONF = ".conf";
    public static final String EXT_JAVA = ".java";

    // Directories
    public static final String PRESENTATION = "presentation";
    public static final String BUSINESS = "business";
    public static final String DATA = "data";
    public static final String MEDIA = "media";
    public static final String SRC = "src";
    public static final String UTIL = "util";
    public static final String DOC = "doc";
    public static final String JAVASCRIPT = "javascript";
    public static final String DB = "db";

    // File Names
    public static final String MAKEFILE = "Makefile";
    public static final String CONFIG_MK = "config.mk";

    // Classes
    public static final String SERVLET_CLASS =
            "com.lutris.appserver.server.httpPresentation.servlet." +
            "HttpPresentationServlet";


    // === Constructors ===

    /**
     * Constructor for use as an instantiated utility class.
     * You must set the app name using setAppName()
     * @see #setAppName
     */
    public PlatypusAppTree()
    {
        setRootDir(".");
    }

    /**
     * Constructor for use as an instantiated utility class.
     * @parameter appName The application name.
     */
    public PlatypusAppTree(String appName)
    {
        setAppName(appName);
        setRootDir(".");
    }

    /**
     * Constructor for use by main() as a standalone utility or
     * as a way of passing configuration arguments in a single
     * KeywordValueTable.
     * @param argTable The keyword value table of config args.
     */
    public PlatypusAppTree(KeywordValueTable argTable)
    {
        //  This should be changed to take a Config object instead.
        String tmpval = null;

        // appName
        try {
            tmpval = argTable.getString(APP_NAME);
        } catch (KeywordValueException e) {}
        if (tmpval != null) {
            setAppName(tmpval);
        }

        // rootDir
        tmpval = null;
        try {
            tmpval = argTable.getString(ROOT_DIR);
            
        } catch (KeywordValueException e) {}
        if (tmpval != null) {
            setRootDir(tmpval);
        }
        if (this.rootDir == null) {
            setRootDir(".");
        }

        // packageDir
        tmpval = null;
        try {
            tmpval = argTable.getString(PACKAGE_DIR);
        } catch (KeywordValueException e) {}
        if (tmpval != null) {
            setPackageDir(tmpval);
        }

        // copyrightName
        tmpval = null;
        try {
            tmpval = argTable.getString(COPYRIGHT_NAME);
        } catch (KeywordValueException e) {}
        if (tmpval != null) {
            setCopyrightName(tmpval);
        }

        // enhydraDir
        tmpval = null;
        try {
            tmpval = argTable.getString(ENHYDRA_DIR);
        } catch (KeywordValueException e) {}
        if (tmpval != null) {
            setEnhydraDir(tmpval);
        }

        // xmlcDir
        tmpval = null;
        try {
            tmpval = argTable.getString(XMLC_DIR);
        } catch (KeywordValueException e) {}
        if (tmpval != null) {
            setXMLCDir(tmpval);
        }
    }

    // === Main ===

    /**
     * The entry point for use as a standalone utility.
     * The utility will process command line arguments for data
     * that is required to create the application. Optionaly the data 
     * can be stored in a config file (The default file name is newapp.conf) 
     * in the form:
     *
     * # comments
     * rootDir = /applications/root
     * enhydraDir = ??
     * ... other data
     *
     * @param argv An array of input parameters.
     */
    public static void main(String[] argv)
    {
        if (argv.length == 0) {
            usage();
            return;
        }

        KeywordValueTable argTable = null;
        File dcf = new File(defaultConfFile);
        if (dcf.exists()) {
            try {
                FileInputStream confFIS = new FileInputStream(dcf);
                ConfigFile configFile = new ConfigFile(confFIS);
                argTable = configFile.getConfig();
            } catch (Exception e) {
                System.err.println("Error in newapp: " + e);
            }
        } else {
            argTable = new KeywordValueTable();
        }  
        parseArgs(argv, argTable);
        PlatypusAppTree latree = new PlatypusAppTree(argTable);
        try {
            latree.create();
        }
        catch(NewAppException e) {
            usage();
            return;
        }
    }

    // === Methods ===

    private static void usage()
    {
        System.err.println("Platypus App Tree usage:");
        System.err.println(
                "java com.pisoftware.platypus.newApp.PlatypusAppTree [args] appname");
        System.err.println("where args can be any of:");
        System.err.println("-confFile filename");
        System.err.println("-rootDir dirname");
        System.err.println("-packageDir dirname");
        System.err.println("-copyrightName name");
        System.err.println("-enhydraDir dirname");
        System.err.println("-xmlcDir dirname");
    }

    /**
     * parseArgs sets values in the KeywordValueTable for elements of the
     * string array.  It also reads any specified config files into the
     * KeywordValueTable.
     * @param argv The command line arguments.
     * @param argTable The KeywordValueTable in which to put values.
     */
    private static void parseArgs(String[] argv, KeywordValueTable argTable)
    {
        // FIX: this should probably use a generic arg parsing utility class
        for (int i = 0; i < argv.length - 1; i++) {
            if (argv[i].equals(DASH + CONF_FILE)) {
                try {
                    FileInputStream confFIS = new FileInputStream(argv[++i]);
                    ConfigFile configFile = new ConfigFile(confFIS);
                    argTable = configFile.getConfig();
                } catch (Exception e) {}
            } else if (argv[i].equals(DASH + ROOT_DIR)) {
                try {
                    argTable.set(ROOT_DIR, argv[++i]);
                } catch (Exception e) {}
            } else if (argv[i].equals(DASH + PACKAGE_DIR)) {
                try {
                    String tmpstr = argv[++i];
                    if (argv[i].startsWith("/")) {
                        tmpstr = argv[i].substring(1);
                    }
                    argTable.set(PACKAGE_DIR, tmpstr);
                } catch (Exception e) {}
            } else if (argv[i].equals(DASH + COPYRIGHT_NAME)) {
                try {
                    argTable.set(COPYRIGHT_NAME, argv[++i]);
                } catch (Exception e) {}
            } else if (argv[i].equals(DASH + ENHYDRA_DIR)) {
                try {
                    argTable.set(ENHYDRA_DIR, argv[++i]);
                } catch (Exception e) {}
            } else if (argv[i].equals(DASH + XMLC_DIR)) {
                try {
                    argTable.set(XMLC_DIR, argv[++i]);
                } catch (Exception e) {}
            }
        }
        try {
            argTable.set(APP_NAME, argv[argv.length - 1]);
        } catch (Exception e) {}
    }

    private String fixFS(String dir)
    {
        if (FS.equals("\\")) {
            return dir.replace('\\', '/');
        } else {
            return dir;
        }
    }

    private String fixDrive(String path)
    {
        if (path.charAt(1) == ':') {
            return "//" + path.charAt(0) + path.substring(2);
        } else {
            return path;
        }
    }

    private String makefileList( String listName, String [] items, String comment )
    {
        String code ="" ; 
        code += comment ; 
        code += listName + " = \\\n" ; 
        for (int i = 0; i < items.length ; i++) {
            code += "\t" + items[i] + " \\\n" ;
        }
        code += "# End of List\n" ; 
        code += "\n" ; 
        return code ; 
    }

    /**
     * Creates a new Enhydra application development tree.
     */
    public void create() throws NewAppException
    {
        if (appName == null) {
            System.err.println("PlatypusAppTree:create(): no app name specified");
            throw new NewAppException("Error: No appName specified");
        }

        // System properties
        try {
            FS = System.getProperty("file.separator");
        } catch (SecurityException e) {
            FS = "/";
        }

        System.out.println("Creating new tree for Platypus/Enhydra application " +
                appName + " in " + this.rootDir + FS + appName + " ...");

        try {
            PS = System.getProperty("path.separator");
        } catch (SecurityException e) {
            PS = ":";
        }

        try {
            javaHome = System.getProperty("java.home");
        } catch (SecurityException e) {}
        javaHome = fixFS(javaHome);

        try {
            javaClassPath = System.getProperty("java.class.path");
        } catch (SecurityException e) {
            javaClassPath = javaHome + "/lib/classes.zip" + PS + ".";
        }
        javaClassPath = fixFS(javaClassPath);

        // appDir
        File appDir = new File(this.rootDir, appName);
        appDir.mkdirs();

        // Makefile
        String[] subdirs = {SRC};
        String preInc = "\n" +
            "#\n" +
            "# build_all is normally the top-level-rule. This makes it so the jar file\n" +
            "# will be created after the build is all done.\n" +
            "#\n" +
            "top_build_all: build_all jar\n\n";
        String postinclude = "clean::\n\trm -rf $(CLASS_OUTPUT) $(OUTPUT)";
        try {
            writeMakefile(MAKEFILE, appDir, null, subdirs, null, preInc, postinclude);
        } catch (IOException e) {
            System.err.println("Error writing Makefile in " + appDir);
        }

        // README
        try {
            writeReadme(appDir);
        } catch (IOException e) {
            System.err.println("Error writing README in " + appDir);
        }

        // config.mk
        try {
            writeConfigMk(appDir);
        } catch (IOException e) {
            System.err.println("Error writing config.mk in " + appDir);
        }

        // doc subdir
        File docDir = new File(this.rootDir, DOC);
        docDir.mkdirs();

        // packageDir
        File pkgDir = new File(appDir, SRC);
        pkgDir.mkdirs();

        // Makefile
        String[] pkg_subdirs = {UTIL, DATA, BUSINESS, PRESENTATION};
        String[] pkg_classes = {appNameUC};
        String confin = appName + EXT_CONF;
        String confout = "${OUTPUT}/" + appName + EXT_CONF;
        String unused_subdirs_comment = "#\n# Temporarily removed from SUBDIRS list\n#\n" ; 
        String pkg_preinclude = "" ;
        pkg_preinclude += makefileList( "UNUSED_SUBDIRS", pkg_subdirs, unused_subdirs_comment ); 
        pkg_preinclude += "MISC_BUILD = conf_file ${OUTPUT}/" + MULTISERVER_CONF + " ${OUTPUT}/" + START_SCRIPT;

        String pkg_postinclude = "conf_file:\n"
            + "\t@[ -d ${OUTPUT} ] || mkdir -p ${OUTPUT}\n"
            + "\tcp " + confin + " " + confout + "\n"
            + "\n"
            + "${OUTPUT}/" + MULTISERVER_CONF + ": " + MULTISERVER_CONF_IN + "\n"
            + "\t@[ -d ${OUTPUT} ] || mkdir -p ${OUTPUT}\n"
            + "\trm -rf ${OUTPUT}/" + MULTISERVER_CONF + "\n"
            + "\tsed -e s%@OUTPUT@%`cd $(OUTPUT);pwd`%g \\\n"
            + "\t    < " + MULTISERVER_CONF_IN + " > ${OUTPUT}/" + MULTISERVER_CONF + "\n"
            + "\n"
            + "${OUTPUT}/" + START_SCRIPT + ": " + START_SCRIPT + "\n"
            + "\t@[ -d ${OUTPUT} ] || mkdir -p ${OUTPUT}\n"
            + "\trm -f ${OUTPUT}/" + START_SCRIPT + "\n" 
            + "\tcp " + START_SCRIPT + " ${OUTPUT}/" + START_SCRIPT + "\n" 
            + "\tchmod +x ${OUTPUT}/" + START_SCRIPT + "\n"
            + "\n"
            + "clean::\n\trm -f "  + confout + "\n";
        
        try {
            writeMakefile(MAKEFILE, pkgDir, this.packageDir, new String[0], pkg_classes, pkg_preinclude, pkg_postinclude);
        } catch (IOException e) {
            System.err.println("Error writing Makefile in " + pkgDir);
        }

        // appName.conf
        try {
            writeAppConf(pkgDir);
        } catch (IOException e) {
            System.err.println("Error writing " + appName + ".conf in "
                    + pkgDir);
        }

        try {
            writeStartScript(pkgDir);
        } catch (IOException e) {
            System.err.println("Error writing start script in " + pkgDir);
        }

        try {
            writeMultiserverConf(pkgDir);
        } catch (IOException e) {
            System.err.println("Error writing multiserver config in " + pkgDir);
        }

        // appName.java
        try {
            writeAppJava(pkgDir);
        } catch (IOException e) {
            System.err.println("Error writing " + appNameUC + ".java in "
                    + pkgDir);
        }


        // presentation
        File presDir = new File(pkgDir, PRESENTATION);
        presDir.mkdirs();

        // Makefile
        String[] pres_subdirs = {MEDIA, JAVASCRIPT};
        String[] java_classes = {"ErrorHandler", "Debug", "AppTemplate"};
        String pres_packagedir = this.packageDir + "/" + PRESENTATION;
        String xmlcStuff = "HTML_DIR = .\n\nHTML_CLASSES = \\\n\tErrorHandlerHTML\\\n\tDebugHTML\\\n\tAppTemplateHTML\\\n# End of List\n";
        try {
            writeMakefile(MAKEFILE, true, presDir, pres_packagedir, pres_subdirs, java_classes, xmlcStuff, null);
        } catch (IOException e) {
            System.err.println("Error writing Makefile in " + presDir);
        }

        // media
        File mediaDir = new File(presDir, MEDIA);
        mediaDir.mkdirs();

        // Makefile
        String media_packagedir = this.packageDir + "/" + PRESENTATION +
                "/" + MEDIA;

        String media_preinclude = "JAR_INSTALL = Enhydra.gif";
        String media_postinclude = "";


        try {
            writeMakefile(MAKEFILE, mediaDir, media_packagedir, null, null, media_preinclude, media_postinclude);
        } catch (IOException e) {
            System.err.println("Error writing Makefile in " + mediaDir);
        }

        try {
            writeEnhydraGif(mediaDir);
        } catch (Exception e) {
            System.err.println("Error writing Enhydra.gif in " + mediaDir +
                               ": " + e.getMessage());
        }


        // javascript
        File javascriptDir = new File(presDir, JAVASCRIPT);
        javascriptDir.mkdirs();

        // Makefile
        String javascript_packagedir = this.packageDir + "/" + PRESENTATION +
                "/" + JAVASCRIPT;

        String javascript_preinclude = "";
        String javascript_postinclude = "";


        try {
            writeMakefile(MAKEFILE, javascriptDir, javascript_packagedir, null, null, javascript_preinclude, javascript_postinclude);
        } catch (IOException e) {
            System.err.println("Error writing Makefile in " + javascriptDir);
        }


        // business
        File busDir = new File(pkgDir, BUSINESS);
        busDir.mkdirs();

        // Makefile
        String bus_packagedir = this.packageDir + "/" + BUSINESS;
        preInc = "CLASSES =";
        try {
            writeMakefile(MAKEFILE, busDir, bus_packagedir, null, null, preInc, null);
        } catch (IOException e) {
            System.err.println("Error writing Makefile in " + busDir);
        }

        // data
        File dataDir = new File(pkgDir, DATA);
        dataDir.mkdirs();

        // Makefile
        String data_packagedir = this.packageDir + "/" + DATA;
        String data_preinc = "CLASSES = \n\n"+
                             "DODS = $(ENHYDRA_DIR)/output/bin/dods\n\n"+
                             "PRE_MISC_BUILD = $(ROOT)/"+appName+"/Makefile\n\n";
        String data_postinc= "$(ROOT)/"+appName+"/Makefile: "+appName+".doa\n"+
                             "	- mkdir -p $(ROOT)/"+appName+"/data\n"+
                             "	- $(DODS) "+appName+".doa $(ROOT)/"+appName+" regen\n\n";
        try {
            writeMakefile(MAKEFILE, dataDir, data_packagedir, null, null, data_preinc, data_postinc);
        } catch (IOException e) {
            System.err.println("Error writing Makefile in " + dataDir);
        }
 
        // util
        File utilDir = new File(pkgDir, UTIL);
        utilDir.mkdirs();

        // Makefile
        String util_packagedir = this.packageDir + "/" + UTIL;
        try {
            writeMakefile(MAKEFILE, utilDir, util_packagedir, null, null, preInc, null);
        } catch (IOException e) {
            System.err.println("Error writing Makefile in " + utilDir);
        }
 
        System.out.println("Done.");
    }

    /**
     * Finds the relative path to the rootDir from the provided directory.
     */
    protected String findRoot(String dir)
    {
        if (dir == null) {
            return ".";
        }
        String root = "..";
        for (int i = 0; i < dir.length(); i++) {
            if (dir.charAt(i) == '/') {
                root = root + "/..";
            }
        }
        return root;
    }
 
    /**
     * Writes a copyright message to out, starting each line with prefix.
     * @param out A PrintWriter on which to write the copyright message.
     * @param prefix A String to prepend to each line.
     */
    protected void writeCopyright(PrintWriter out, String prefix)
    {
        out.println(prefix + " Copyright (C) 1998 " + this.copyrightName +
                "\tAll Rights Reserved.");
    }

    /**
     * Writes a CVS Id to out, starting each line with prefix.
     * @param out A PrintWriter on which to write the copyright message.
     * @param prefix A String to prepend to each line.
     */
    protected void writeCvsId(PrintWriter out, String prefix)
    {
        out.println(prefix + " $Id: Filename,v 1.1 1998/03/26 21:08:25" +
                " lutris Exp $");
    }

    /**
     * Writes a Makefile.
     * @param dir The directory in which to write the Makefile.
     * @param packagedir The path from the root directory to here (PACKAGEDIR).
     * @param subdirs An array of strings listing the subdirectories (SUBDIRS).
     * @param classes An array of strings listing the java files (CLASSES).
     * @param preinclude Anything else to put before the include statement.
     * @param postinclude Anything else to put after the include statement.
     */
    public void writeMakefile(String makeFileName, File dir, String packagedir, String[] subdirs,
            String[] classes, String preinclude,
            String postinclude) throws IOException
    {
        writeMakefile( makeFileName, false, dir, packagedir, subdirs,
            classes, preinclude, postinclude);
    }

    /**
     * Writes PO list in a Makefile.
     * @param varname Name of variable
     */
    public void writePOList(PrintWriter out, String varname, String comment) throws IOException
    {
        out.println("#");
        out.println("#  " + comment);
        out.println("#");
        out.println(varname + " = \\");
        out.println("# End of List");
        out.println();
    }

    /**
     * Writes a Makefile.
     * @param dir The directory in which to write the Makefile.
     * @param polists If true, write out empty PO_XXXX lists.
     * @param packagedir The path from the root directory to here (PACKAGEDIR).
     * @param subdirs An array of strings listing the subdirectories (SUBDIRS).
     * @param classes An array of strings listing the java files (CLASSES).
     * @param preinclude Anything else to put before the include statement.
     * @param postinclude Anything else to put after the include statement.
     */
    public void writeMakefile(String makeFileName, boolean polists, File dir, String packagedir, String[] subdirs,
            String[] classes, String preinclude,
            String postinclude) throws IOException
    {
        File f = new File(dir, makeFileName);
        PrintWriter out = new PrintWriter(new FileWriter(f));

        // Header
        out.println("#");
        out.println("#  " + appName);
        out.println("#  Enhydra Application Makefile");
        out.println("#");
        // writeCopyright(out, "#");
        out.println("#");
        out.println();

        // PO Lists
        if (polists) { 
            writePOList( out, "PO_UNUSED", "POs not yet started" ); 
            writePOList( out, "PO_DOM", "POs up to DOM class stage" ); 
            writePOList( out, "PO_CONF", "POs up to config file stage" ); 
            writePOList( out, "PO_JAVA", "POs up to Java generation" ); 
            writePOList( out, "PO_CLASS", "POs up to Java compilation" ); 
            writePOList( out, "PO_DONE", "POs finished (nothing done by Makefile)" ); 
        }

        // ROOT
        String root = findRoot(packagedir);
        out.println("#");
        out.println("#  Where is the root of the source tree?");
        out.println("#");
        out.println("ROOT =\t" + root);
        out.println();

        // PACKAGEDIR
        if (packagedir != null) {
            int slashLocn ; 
            slashLocn = packagedir.indexOf( '/' ); 
            if( slashLocn >= 0 ){ 
                out.println("#");
                out.println("#  What is this directory relative to src/, view/, etc");
                out.println("#");
                out.println("REL_DIR = " + packagedir.substring( slashLocn+1 ));
                out.println();
            }
            out.println("#");
            out.println("#  What package does this directory represent? (use / instead of .)");
            out.println("#");
            out.println("PACKAGEDIR = " + packagedir);
            out.println();
        }

        // SUBDIRS
        if (subdirs != null) {
            out.println("#");
            out.println("# To add new directories, create the directories and add their names");
            out.println("# this list. Be sure to copy a Makefile into each directory. Then add a");
            out.println("# \"../\" to the ROOT variable, and add the directory name to the end of.");
            out.println("# the PACKAGEDIR variable.");
            out.println("#");
            out.println("SUBDIRS = \\");
            for (int i = 0; i < (subdirs.length - 1); i++) {
                out.println("\t" + subdirs[i] + " \\");
            }
            if (subdirs.length > 0) {
                out.println("\t" + subdirs[subdirs.length - 1]);
            }
            out.println();
        }

        // CLASSES
        if (classes != null) {
            out.println("#");
            out.println("# To add more Java classes, create the .java files and add their names to");
            out.println("# this list (without the .java extension).");
            out.println("#");
            out.println("CLASSES = \\");
            for (int i = 0; i < (classes.length - 1); i++) {
                out.println("\t" + classes[i] + " \\");
            }
            if (classes.length > 0) {
                out.println("\t" + classes[classes.length - 1]);
            }
            out.println();
        }

        // pre include
        if (preinclude != null) {
            out.println(preinclude);
            out.println();
        }

        // include
        out.println("include $(ROOT)/config.mk");
        out.println();

        // post include
        if (postinclude != null) {
            out.println(postinclude);
            out.println();
        }

        out.close();
    }

    /**
     * Writes the top level README file.
     * @param dir a File which is the directory where the README goes.
     */
    public void writeReadme(File dir) throws IOException
    {
        File f = new File(dir, "README");
        PrintWriter out = new PrintWriter(new FileWriter(f));

        // Header
        out.println("\n To build and run " + appName + ":");
        out.println();

        // Instructions
        out.println("1) Type \"make\" in this directory.\n");
        out.println("2) The application will be compiled, with the resulting class files put in");
        out.println("   the classes directory. Then a jar file will be created in output/lib.");
        out.println("   the application's config file will be placed in the diredtory");
        out.println("   output. An Enhydra Multiserver config file will also be placed");
        out.println("   in the output directory, along with a start script.");
        out.println("3) Type \"cd output\".");
        out.println("4) Type \"./start\".");
        out.println("5) Use your web browser to access " + appName + " at:  http://localhost:9000\n\n");
        out.println("NOTES:");
        out.println(" localhost can also be your host name or 127.0.0.1 (best for Win95).\n");
        out.println();
        out.close();
    }

    /**
     * Writes the top level config.mk file.
     * @param dir a File which is the directory where the config.mk goes.
     */
    public void writeConfigMk(File dir) throws IOException
    {
        File f = new File(dir, CONFIG_MK);
        PrintWriter out = new PrintWriter(new FileWriter(f));

        // Header
        out.println("#");
        out.println("# " + appName);
        out.println("# Platypus/Enhydra Application Makefile Configuration");
        out.println("#");
        out.println("#");
        out.println();

        // ENHYDRA_DIR
        out.println("#");
        out.println("# The directory containing the Enhydra distribution.");
        out.println("#");
        out.println("ENHYDRA_DIR = " + enhydraDir);
        out.println();
        out.println();

        // XMLC_DIR
        out.println("#");
        out.println("# The directory containing the XMLC distribution.");
        out.println("#");
        out.println("XMLC_DIR = " + xmlcDir);
        out.println();
        out.println();
 
        out.println("#");
        out.println("# You might want to change/override some of the following variables:");
        out.println("#");
        out.println("# Is JDKDIR correct?");
        out.println("JDKDIR = /usr/local/java");
        out.println("#");
        out.println("# Is PLATYPUSDIR correct?");
        out.println("PLATYPUSDIR = /usr/local/platypus");
        out.println("#");
        out.println("# Is ENHYDRA_CLASSES correct?");
        out.println("ENHYDRA_CLASSES = $(ENHYDRA_DIR)/output/lib/enhydra.jar");
        out.println("#");
        out.println("# Is XMLC correct?");
        out.println("XMLC = $(XMLC_DIR)/output/bin/xmlc");
        out.println("#");
        out.println("# Is JOLTC correct?");
        out.println("JOLTC = $(ENHYDRA_DIR)/output/bin/joltc");
        out.println("#");
        out.println("# See $(PLATYPUSDIR)/stdrules.mk in the Platypus installation for more");
        out.println("# configuration settings.");
        out.println("#");
        out.println();
        out.println();

        // app details
        out.println("#");
        out.println("APP_NAME = "+appName);
        out.println("APP_DIR = $(shell echo $(APP_NAME) | tr . /)");
        out.println("GEN_DIR = $(ROOT)/$(APP_DIR)");
        out.println("SRC_DIR = $(ROOT)/src");
        out.println("CLASS_DIR = $(ROOT)/classes/$(APP_DIR)");
        out.println("VIEW_DIR = $(ROOT)/view");
        out.println("HTML_XMLC_OPTS_FILE = $(ROOT)/config.xmlc");
        out.println();
        out.println("# NOTE: Some rules in stdrules.mk have HTML_XMLC_OPTS_FILE as a");
        out.println("# prerequisite. If it isn't defined, their targets will always be 'out");
        out.println("# of date'!");
        out.println();

        // javadoc flags
        out.println("JAVADOCFLAGS = -private -author -version");

        // extra classpath information
        out.println();
        out.println("# Extra classpath information required");
        out.println("XTRA_CLASSPATH=$(PLATYPUSDIR)/output/lib/platypus.jar:$(XMLC_DIR)/output/lib/xmlc.jar");
        out.println();

        // jar file
        out.println("#");
        out.println("# What is the name of the jar file to create?");
        out.println("#");
        out.println("JARNAME = " + appName);
        out.println();

        // platypus rules
        out.println("#");
        out.println("# Include the special platypus rules");
        out.println("#");
        out.println("include $(PLATYPUSDIR)/stdrules.mk");
        out.println();

        // Error checking
        out.println("#\n# Only used if ENHYDRA_DIR is not correct.\n#");
        out.println("$(ROOT)/stdrules.mk:");
        out.println("\t@echo");
        out.println("\t@echo \"Please edit config.mk. ENHYDRA_DIR must point to your installation of Enhydra.\"");
        out.println("\t@echo");

        out.close();
    }

    /**
     * Writes the application config file.
     * @param dir a File which is the directory where the file goes.
     */
    public void writeAppConf(File dir) throws IOException
    {
        File f = new File(dir, appName + EXT_CONF);
        PrintWriter out = new PrintWriter(new FileWriter(f));
        
        // Header
        out.println("# " + appName);
        out.println("# Enhydra Application Configuration");
        out.println("#");
        out.println();

        // Server.ClassPath
        String appClassPath = rootDir + "/" + appName + "/classes";
        String jarPath = rootDir + "/" + appName + 
                         "/output/lib/" + appName + ".jar";
        out.println("#");
        out.println("# Comma separated CLASSPATH directories and files"
                    + " used by this application.");
        out.println("# Assumes run from the output directory for debugging.");
        out.println("# If you run from the jar you must rebuild after every change to the app.");
        out.println("#");
        out.println(MultiServer.SERVER + "." + MultiServer.CLASS_PATH +
                    "[] = " + appClassPath);
        out.println("#" + MultiServer.SERVER + "." + MultiServer.CLASS_PATH +
                    "[] = " + jarPath);
        out.println();

        // Server.AppClass
        out.println("#");
        out.println("# The fully qualified name of the application class.");
        out.println("#");
        out.println(MultiServer.SERVER + "." + MultiServer.APP_CLASS +
                " = " + this.packageName + "." + appNameUC );
        out.println();

        // Server.PresentationPrefix
        out.println("#");
        out.println("# Prefix used to derive presentation object class");
        out.println("# names and paths from URLs.");
        out.println("# Assumes run from the output directory for debugging.");
        out.println("#");
        out.println(MultiServer.SERVER + "." +
                    MultiServer.PRESENTATION_PREFIX + " = \"" +
                    this.packageDir + "/" + PRESENTATION + "\"");
        out.println();

        // Server.AutoReload
        out.println("#");
        out.println("# Flag to indicate that application classes and resources should be reloaded");
        out.println("# automatically if ANY file in the CLASSPATH changes.  NOTE:  This is a");
        out.println("# debugging option and may slow down the performance of your application.");
        out.println("# The CLASSPATH should NOT contain any directories (or underlying directories)");
        out.println("# that contain constantly changing files, i.e. a log file.  For example, do");
        out.println("# NOT include the application's output directory in the CLASSPATH because it");
        out.println("# contains the application log files in the underlying log directory.");
        out.println("#");
        out.println("Server.AutoReload = false");
        out.println();

        // sessionManager.Lifetime
        out.println("#");
        out.println("# Maximum number of minutes a user session can last.");
        out.println("#");
        out.println("SessionManager.Lifetime = 60");
        out.println();

        // SessionManager.MaxIdle
        out.println("#");
        out.println("# Maximum number of minutes a user may be"
                + " idle before being logged off.");
        out.println("#");
        out.println("SessionManager.MaxIdleTime = 2");
        out.println();

        // 
        out.println("#");
        out.println("# If the URL \"/\" for this application is accessed, the user will be");
        out.println("# redirected to this URL. This should be a relative URL.");
        out.println("#");
        out.println("Application.DefaultUrl = \"Welcome.po\"");
        out.println();

        out.close();
    }


    /**
     * Writes the multiserver config file.
     * @param dir a File which is the directory where the file goes.
     */
    public void writeMultiserverConf(File dir) throws IOException
    {
        File f = new File(dir, MULTISERVER_CONF_IN);
        PrintWriter out = new PrintWriter(new FileWriter(f));
        
        out.println("#");
        out.println("#  Simple Enhydra Multiserver config file that runs one application.");
        out.println("#  See multiserver.config in the Enhydra installation for a");
        out.println("#  description of the Multiserver config file format.");
        out.println("#");
        out.println();
        out.println("#");
        out.println("# Where the config files are.");
        out.println("#");
        out.println("Server.ConfDir = @OUTPUT@");
        out.println();
        out.println("#");
        out.println("# Set up logging.");
        out.println("# For more information, try adding DEBUG or CLASSLOAD.");
        out.println("#");
        out.println("Server.LogFile = @OUTPUT@/" + appName + ".log");
        out.println("Server.LogToFile[] = EMERGENCY, ALERT, CRITICAL, ERROR, WARNING, INFO");
        out.println("Server.LogToStderr[] = EMERGENCY, ALERT, CRITICAL, ERROR, WARNING, INFO");
        out.println();
        out.println();
        out.println();
        out.println("#");
        out.println("# Specify only one application (no graphical administration).");
        out.println("#");
        out.println("Application." + appName + ".ConfFile = " + appName + ".conf");
        out.println("Application." + appName + ".Description = \"An Enhydra Application\"");
        out.println("Application." + appName + ".Running = yes");
        out.println();
        out.println();
        out.println("#");
        out.println("# Open up a port.");
        out.println("#");
        out.println("Connection.p9000.Type = http");
        out.println("Connection.p9000.Port = 9000");
        out.println();
        out.println();
        out.println("#");
        out.println("# Connect the port to the application.");
        out.println("#");
        out.println("Channel.p9000.aChan.Servlet = " + appName);
        out.println("Channel.p9000.aChan.Url = /");
        out.println("Channel.p9000.aChan.Enabled = yes");
        out.println();
        out.println();
        out.println();
        out.println("#");
        out.println("# ----------------------------------------------------------------------");
        out.println("#");
        out.println("# If you want to use the Multiserver's graphical administration program,");
        out.println("# first copy it's config file to the directory Server.confDir is set to:");
        out.println("#");
        out.println("#     cp " + enhydraDir + "/apps/multiserverAdmin.conf @OUTPUT@");
        out.println("#");
        out.println("# Then uncomment the following lines, re-run the Multiserver (./start),");
        out.println("# and connect to port 9001. The default username is \"admin\" and the default");
        out.println("# password is \"lutris\". This can be changed by editing the local copy of");
        out.println("# multiserverAdmin.conf.");
        out.println("#");
        out.println("# If you are going to be adding other applications, other servlets, or");
        out.println("# other connections, it is highly recommended that you use the graphical");
        out.println("# admin rather than edit this file by hand (remember to save your changes");
        out.println("# by clicking on the \"Save State\" button!).");
        out.println("#");
        out.println("# Application.MultiserverAdmin.ConfFile = multiserverAdmin.conf");
        out.println("# Application.MultiserverAdmin.Description = \"The Enhydra Multiserver Management Console.\"");
        out.println("# Application.MultiserverAdmin.Running = yes");
        out.println("#");
        out.println("# Connection.p9001.Type = http");
        out.println("# Connection.p9001.Port = 9001");
        out.println("#");
        out.println("# Channel.p9001.aChan.Servlet = MultiserverAdmin");
        out.println("# Channel.p9001.aChan.Url = /");
        out.println("# Channel.p9001.aChan.Enabled = yes");
        out.println("#");
        out.println("# ----------------------------------------------------------------------");
        out.println();

        out.close();
    }


    public void writeStartScript(File dir) throws IOException
    {
        File f = new File(dir, START_SCRIPT);
        PrintWriter out = new PrintWriter(new FileWriter(f));
        
        out.println("#!/bin/sh\n");
        out.println("echo \"***\"");
        out.println("echo \"***  Connect to http://`hostname`:9000/\"");
        out.println("echo \"***\"\n");
        out.println(enhydraDir + "/bin/multiserver ./multiserver.conf");
        out.close();
    }


    /**
     * Writes the application main file.
     * @param dir a File which is the directory where the file goes.
     */
    public void writeAppJava(File dir) throws IOException
    {
        File f = new File(dir, appNameUC + EXT_JAVA);
        PrintWriter out = new PrintWriter(new FileWriter(f));

        // Header
        out.println("/*");
        out.println(" * " + appName);
        out.println(" * A sample Enhydra Application");
        out.println(" * ");
        out.println(" */");
        out.println();

        // package/imports
        out.println("package " + this.packageName + ";");
        out.println();
        out.println("import com.lutris.appserver.server.*;");
        out.println("import com.lutris.appserver.server.httpPresentation.*;");
        out.println("import com.lutris.appserver.server.session.*;");
        out.println("import com.lutris.util.*;");
        out.println();

        // class definition
        out.println("/**");
        out.println(" * The application object.");
        out.println(" * ");
        out.println(" * Application-wide data would go here.");
        out.println(" */");
        out.println("public class " + appNameUC +
                " extends StandardApplication {");
        out.println();

        // methods
        out.println("    /*");
        out.println("     *  A few methods you might want to add to.");
        out.println("     *  See StandardApplication for more details.");
        out.println("     */");
        out.println("    public void startup(Config appConfig) throws ApplicationException {");
        out.println("        super.startup(appConfig);");
        out.println("        //  Here is where you would read application-specific settings from");
        out.println("        //  your config file.");
        out.println("    }");
        out.println("    public boolean requestPreprocessor(HttpPresentationComms comms)");
        out.println("                   throws Exception {");
        out.println("        return super.requestPreprocessor(comms);");
        out.println("    }");
        out.println();
  


        // toHtml
        out.println("    /**");
        out.println("     * This is an optional function, used only by the Multiserver's graphical");
        out.println("     * administration. This bit of HTML appears in the status page for this");
        out.println("     * application. You could add extra status info, for example");
        out.println("     * a list of currently logged in users.");
        out.println("     *");
        out.println("     * @return HTML that is displayed in the status page of the Multiserver.");
        out.println("     */");
        out.println("    public String toHtml() {");
        out.println("        return \"This is <I>" + appName + "</I>\";");
        out.println("    }");

        // end class definition
        out.println("}");
        out.println();

        out.close();
    }


    void writeEnhydraGif(File dir) throws Exception {
        File f = new File(dir, "Enhydra.gif");
        OutputStream out = new FileOutputStream(f);
        InputStream in;
        in = ClassLoader.getSystemResourceAsStream(
                                     "com/lutris/devsys/newApp/Enhydra.gif");
        if (in == null)
            throw new Exception("Unable to access Enhydra.gif");
        byte[] buf = new byte[1024];
        while (true) {
            int n = in.read(buf);
            if (n < 0)
                break;
            out.write(buf, 0, n);
        }
        in.close();
        out.close();
    }


    /**
     * Sets the application name for the newly created
     * Enhyudra application development tree.
     * If packageDir is not already set, it is set to appName with the
     * first letter lowercased.
     * @param appName The application name.
     * @see #setPackageDir
     */
    public void setAppName(String appName)
    {
        this.appName = appName;
        appNameLC = appName.substring(0, 1).toLowerCase() +
                    appName.substring(1);
        appNameUC = appName.substring(0, 1).toUpperCase() +
                    appName.substring(1);
        if (this.packageDir == null)
            setPackageDir(appNameLC);
    }

    /**
     * Sets the root directory under which the new
     * app development directory is created.
     * @param dir The directory path name.
     */
    public void setRootDir(String dir)
    {
        File f = new File(dir);
        this.rootDir = f.getAbsolutePath();
    }

    /**
     * Sets the package directory which is the top level for source files.
     * @param dir The directory path name.
     */
    public void setPackageDir(String dir)
    {
        this.packageDir = dir;
        this.packageName = this.packageDir.replace('/', '.');
    }

    /**
     * Sets the name to be used in the copyright message.
     * @param copyrightName The name to use in the copyright message.
     */
    public void setCopyrightName(String copyrightName)
    {
        this.copyrightName = copyrightName;
    }

    /**
     * Sets the class directory where Enhydra is installed.
     * @param dir The directory path name.
     */
    public void setEnhydraDir(String dir)
    {
        enhydraDir = dir;
    }

    /**
     * Sets the class directory where XMLC is installed.
     * @param dir The directory path name.
     */
    public void setXMLCDir(String dir)
    {
        xmlcDir = dir;
    }
}



