package com.pisoftware.platypus.pond;

import java.io.*;
import java.lang.*;
import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import com.lutris.util.*;
import com.lutris.xml.xmlc.html.HTMLObject;
import com.lutris.xml.xmlc.*;
import com.lutris.appserver.server.ApplicationException;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.Enhydra;
import com.lutris.logging.*;
import com.pisoftware.platypus.util.*; 
import com.pisoftware.platypus.production.*; 

/**
 * Class Pond generates code for the presentation layer of Enhydra
 * applications. Its input is the developer configuration information,
 * usually in <CODE>dev.conf</CODE> and a PO specific conf file. It
 * writes out two files of code: (1) an abstract "BASE" manipulation
 * class, and (2) a skeleton manipulation class extending the abstract
 * class.
 *
 * <BR>
 *
 * See the documentation on <EM>Platypus</EM> in the platypus
 * <CODE>doc</CODE> directory for more detail.
 **/

/*
 *  Copyright (C) 1999 Plugged In Software Pty Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Plugged In Software
 *  PO Box 4130
 *  St. Lucia South Queensland
 *  Australia  4067
 *  info@PIsoftware.com
 */

public class Pond{

  /** Configuration information for content of application pages. **/
  KeywordValueFiles confData ; 

  /** Configuration information for content of this presentation object. **/
  KeywordValueTable poData ; 

  /** Config info for presentation layer access to the database **/
  KeywordValueTable accessesData ; 

  /** Config info for clientside FORMS. **/
  KeywordValueTable formsData ; 

  /** Config info for database adds/updates. **/
  KeywordValueTable commitsData ; 

  /** Name of application, capitalised **/
  String appNameCap ; 

  /** Name of application, in lower case. Supplied by invocation (ie, command line argument). **/
  String appName ; 

  /** Name of presentation object, capitalised **/
  String poNameCap ; 

  /** Name of presentation object, in lower case **/
  String poName ; 

  /** 
   * Part of package name of PO -- specifically, that part after the
   * application name. Contains "" if the PO is in the presentation
   * directory.
   **/
  String poRelativePackageName ; 

  /** 
   * Path from presentation object's directory to root of Platypus
   * tree. For example, for a PO in the presentation directory, this
   * would have value "../..".
   **/
  String rootPath ; 

  /** 
   * Directory containing presentation object, relative to the
   * application or src directory. For example, "presentation" or
   * "presentation/reports".
   **/
  String poPackageDir ; 

  /** Java package containing presentation object **/
  String poPackageName ; 

  /** Java package containing business object for this PO **/
  String poBusinessPackageName ; 

  /** Fully qualified class name for DOM class for this PO. **/
  String domQualifiedClassName ; 

  /** Name of application (or PO) specific HTML template file and Java class. **/
  String appTemplateName ;

  /** Name of PO which invoked this one. (See INVOKEDBY in dev.conf doco for more details.) **/
  String invokedbyString ;

  /** DOM object containing template Java code. **/
  CodeTemplateHTML po ;

  /** Name value pairs from query string **/
  private KeywordValueTable args ; 

  /** In manipulation class skeleton: query string variable declarations **/
  String sHandCGIDecls = "" ; 

  /** In manipulation class skeleton: query string variable initialisation **/
  String sHandCGIInits = "" ; 

  /** In manipulation class skeleton: query string variable initialisation DBG logging **/
  String sHandCGIInitDBG = "" ; 

  /** In base manipulation class: variable declarations **/
  String sBaseDecls = "" ; 

  /** In base manipulation class: initialisation **/
  String sBaseInits = "" ; 

  /** 
   * File name of PO, excluding file extensions, but including the
   * path from presentation/ directory.
   **/
  String poPathFile ; 

  /** 
   * Directory into which the skeleton manipulation class and business
   * stub class is written.  <BR> The path is relative to the
   * <CODE>output/</CODE> directory.
   **/
  String handClassDir ; 

  /** 
   * Directory into which the abstract BASE manipulation class is
   * written.  <BR> The path is relative to the <CODE>output/</CODE>
   * directory.
   **/
  String baseClassDir ; 

  /** Stores Access objects, indexed by their access name. **/
  Hashtable accessObjects = new Hashtable() ; 


  /** 
   * Process one FORM node, and its embedded FORM elements, which
   * might invoke this page (and put stuff into its query string!).
   **/
  void processInvokingFormNode( String invokingPOName, String formName, KeywordValueTable formData, Hashtable definedQSNames ) throws ApplicationException { 
    String formAction ; // Action attribute of form

    //////  Initialise variables for node
    try{ 
      formAction = formData.getString( "ACTION", null ); 
    }catch( KeywordValueException kve ){ 
      String tmpStr = "Getting ACTION from " + formName + " in " + invokingPOName ; 
      throw new ApplicationException( tmpStr, kve ); 
    }

    //////  Tell programmer that this FORM exists
    sHandCGIDecls +=
      "  ////  Form  '" + formName + "' in '" + invokingPOName + "' PO invokes URL  '" + formAction + "'\n"; 

    //////  Process FORM elements (and prepare arg decls).
    processInvokingFormInputNodes( formName, formData, definedQSNames ); 
  }


  /** Process an invoking FORM's element nodes, ie all INPUT, SELECT
      and TEXTAREA nodes in the FORM. **/
  // TODOONEDAY: Should also handle BUTTON nodes. 
  void processInvokingFormInputNodes( String formName, KeywordValueTable formData, Hashtable definedQSNames ) throws ApplicationException { 
    String [] keys ; 
    KeywordValueTable inputsData ; 
    KeywordValueTable inputData ; 
    ////  Used in loop
    int i ; 
    String inputName ; 
    try{ 
      inputsData = formData.getSection( "ELEMENTS" ); 
    }catch( KeywordValueException kve ){ 
      String tmpStr = "Getting ELEMENTS section from " + formName ; 
      throw new ApplicationException( tmpStr, kve ); 
    }
    keys = inputsData.keys(); 
    for( i=0 ; i<keys.length ; i++ ){ 
      inputName = keys[i] ; 
      if( ! definedQSNames.containsKey( inputName ) ){ 
        try{ 
          inputData = inputsData.getSection( inputName ); 
        }catch( KeywordValueException kve ){ 
          String tmpStr = "Processing " + inputName + " in " + formName ; 
          throw new ApplicationException( tmpStr, kve ); 
        }
        processInvokingFormInputNode( formName, formData, inputName, inputData ); 
        definedQSNames.put( inputName, "What did the hobbit with a cold say?" );
      }
    }
  }


  /** 
   * Process one FORM element node from FORM which might invoke this PO. 
   * 
   * <BR> Note: The forms on the pages invoking this page might
   * contain several elements with the same name attribute. This
   * method is called once for each <EM>distinct</EM> name.
   **/
  void processInvokingFormInputNode( String formName, KeywordValueTable formData, String inputName, KeywordValueTable inputData ) throws ApplicationException { 
    String typeAttr ; // TYPE attribute of INPUT tag, or tag type of other FORM elements.
    String multiple ; // If non-null, this INPUT may be repeated in served page. 
                      // Contains name of repeated ID node.

    //////  Initialise FORM node variables
    try{
      typeAttr = inputData.getString( "TYPE" ); 
      multiple = inputData.getString( "MULTIPLE", null ); 
    }catch( KeywordValueException kve ){ 
      String tmpStr = "Getting strings from " + formName ; 
      throw new ApplicationException( tmpStr, kve ); 
    }


    //////  Generate variable to a single value
    sHandCGIDecls += 
      "\n" + 
      "  /** Value of '" + inputName + "' on query string, from " + formName + " form on " + poName + " PO **/\n" ; 

    //////  Generate code for handling String values
    if( null != multiple ){ 
      // Name can occur multiple times on the query string.
      sHandCGIDecls += 
        "  " + "String [] qslist" + inputName + " ; \n" + 
        "  " + "String qs" + inputName + " ; \n" ; 
      sHandCGIInits += 
        "    qslist" + inputName + " = ProdUtil.getArrayFromQueryString( args, \"" + inputName + "\" );\n" + 
        "    qs" + inputName + " = qslist" + inputName + "[i" + multiple + "] ; \n" ; 
      sHandCGIInitDBG +=  /* Caution: this line is triply-quoted, and may cause sanity haemorage :-) */
        "    for( i" + multiple + "=0 ; i" + multiple + "<qslist" + inputName + ".length ; i" + multiple + "++ ){ \n" + 
        "      ProdUtil.log( \"Query string pair: '" + inputName + "' #i" + multiple + " = '\" + qslist" + inputName + "[i" + multiple + "] + \"'\" ); \n" +
        "    } \n" ; 
    }else if( "IMAGE".equalsIgnoreCase( typeAttr ) ){
      // Image form elements generate x & y co-ordinates on the query string.
      if( inputName.startsWith( "_" ) ){ // Note: Currently (Sep99), inputName should be "_noattr"
        // Image with no NAME attribute generates x & y on query string
        // The name "_noattr" is inserted by ConfigGen.
        sHandCGIDecls += 
          "  " + "String qsx ; \n" + 
          "  " + "String qsy ; \n" ; 
        sHandCGIInits += 
          "    qsx = ProdUtil.getFromQueryString( args, \"x\" );\n" + 
          "    qsy = ProdUtil.getFromQueryString( args, \"y\" );\n" ; 
        sHandCGIInitDBG +=  /* Caution: this line is triply-quoted, and may cause sanity haemorage :-) */
          "    ProdUtil.log( \"Query string coordinates: '" + inputName + "' = '\" + qsx + \"','\" + qsy + \"'\" );\n" ; 
      }else{
        // Image with NAME="Frodo" generates Frodo.x and Frodo.y on query string
        sHandCGIDecls += 
          "  " + "String qsx" + inputName + " ; \n" + 
          "  " + "String qsy" + inputName + " ; \n" ; 
        sHandCGIInits += 
          "    qsx" + inputName + " = ProdUtil.getFromQueryString( args, \"" + inputName + ".x\" );\n" + 
          "    qsy" + inputName + " = ProdUtil.getFromQueryString( args, \"" + inputName + ".y\" );\n" ; 
        sHandCGIInitDBG +=  /* Caution: this line is triply-quoted, and may cause sanity haemorage :-) */
          "    ProdUtil.log( \"Query string coordinates: '" + inputName + "' = '\" + qsx" + inputName + " + \"','\" + qsy" + inputName + " + \"'\" );\n" ; 
      }
    }else{
      // Normal case: name appears only once in query string.
      sHandCGIDecls += 
        "  " + "String qs" + inputName + " ; \n" ; 
      sHandCGIInits += 
        "    qs" + inputName + " = ProdUtil.getFromQueryString( args, \"" + inputName + "\" );\n" ; 
      sHandCGIInitDBG +=  /* Caution: this line is triply-quoted, and may cause sanity haemorage :-) */
        "    ProdUtil.log( \"Query string pair: '" + inputName + "' = '\" + qs" + inputName + " + \"'\" );\n" ; 
    }
    if( "CHECKBOX".equalsIgnoreCase( typeAttr ) ){ 
      // CHECKBOXES should be given to the BO layer as "TRUE"/"FALSE" (or "yes"/"no").
      sHandCGIInits += 
        "    qs" + inputName + " = ProdUtil.stringHasContent( qs" + inputName + " ) ? \"TRUE\" : \"FALSE\" ; // CHECKBOX INPUT tag \n" ; 
    }
  }


// OLD IA() CODE   /** 
// OLD IA() CODE    * Initialise an 'access' to the business and database layers. 
// OLD IA() CODE    **/
// OLD IA() CODE     // TODO: The arguments to the Access constructor could be replaced
// OLD IA() CODE     // by accessData, and the fields extracted in the
// OLD IA() CODE     // constructor. Perhaps the whole method can be elided.
// OLD IA() CODE   void initialiseAccess( String accessName, KeywordValueTable accessData ) throws ApplicationException { 
// OLD IA() CODE     Access access ; 
// OLD IA() CODE     //// Create Access object
// OLD IA() CODE     // NB: Constructor for Access object actually does a lot of work.
// OLD IA() CODE     access = new Access( po, confData, poName, poData, accessName, accessData ); 
// OLD IA() CODE     //// Store access object in Hashtable so it can be found later
// OLD IA() CODE     accessObjects.put( accessName, access ); 
// OLD IA() CODE   }


  /** 
   * Initialise presentation layer 'accesses' to the database. 
   * @throws ApplicationException
   **/
  void initialiseAccesses() throws ApplicationException { 
    String [] keys ; 
    KeywordValueTable accessData ; 
    Access access ; 
    ////  Used in loop
    int i ; 
    String accessName ;
    keys = accessesData.keys(); 
    for( i=0 ; i<keys.length ; i++ ){ 
      accessName = keys[i] ; 
      try{ 
        accessData = accessesData.getSection( accessName ); 
      }catch( KeywordValueException kve ){ 
        String tmpStr = "Getting " + accessName + " section" ; 
        throw new ApplicationException( tmpStr, kve ); 
      }
      // OLD IA() CODE initialiseAccess( accessName, accessData ); 
      // TODO: 04Oct99 Delete OLD IA() CODE, once the following has worked.
      access = new Access( po, confData, poName, poData, accessName, accessData ); 
      accessObjects.put( accessName, access ); 
    }
  }


  /** 
   * Process all nodes with an ID attribute. 
   * @throws ApplicationException
   **/
  void processIDNodes( KeywordValueTable idsData ) throws ApplicationException { 
    String [] keys ; 
    KeywordValueTable idData ; 
    ////  Used in loop
    int i ; 
    String idName ;
      keys = idsData.keys(); 
    for( i=0 ; i<keys.length ; i++ ){ 
      idName = keys[i] ; 
      try{ 
        // OLD CODE 29Sep99 idData = idsData.getSection( idName ); 
        idData = KeywordValueFiles.getFilledSection( confData, idsData, idName ); 
      }catch( KeywordValueException kve ){ 
        String tmpStr = "Can't get keys!" ; 
        throw new ApplicationException( tmpStr, kve ); 
      }
      processIdNode( idName, idData ); 
    }
  }


  /**
   * Generates code for reading values from the query string.
   * @throws ApplicationException
   **/
  void processQueryString() throws ApplicationException { 
    Hashtable definedQSNames ; // Records QS names we've already generated code for.
    String [] invokedbyList ; // List of POs which might invoke this PO.
    //// Used in outer loop
    int iPO ; 
    String invokedby ; // Qualified class name of invoking PO. Eg, "reports.Sales". 
    KeywordValueTable invokingFormsData ; // Configuration info on all FORMs in invoking PO.
    String [] invokingFormsList ; // List of FORMs in invoking PO.
    String message = "Problem in processing INVOKEDBY" ; // Message to put in exception, if thrown
    //// Used in inner loop
    int iForm ; 
    String invokingFormName ; // List of FORMs in invoking PO.
    KeywordValueTable invokingFormData ; // Configuration info on a FORM in invoking PO.
    try{ 

      definedQSNames = new Hashtable(); 
      invokedbyList = Util.getStringList( invokedbyString, ',' ); 
      for( iPO=0 ; iPO<invokedbyList.length ; iPO++ ){ 
        invokedby = invokedbyList[iPO] ; 
        message = "Problem in getting '" + "PO." + invokedby + "'" ; 
        confData.readSection( "PO." + invokedby ); 
        message = "Problem in getting '" + "PO." + invokedby + ".FORMS" + "'" ; 
        invokingFormsData = confData.getSection( "PO." + invokedby + ".FORMS" ); 
        message = "Problem in processing INVOKEDBY element '" + invokedby + "'" ; 
        invokingFormsList = invokingFormsData.keys(); 
        for( iForm=0 ; iForm<invokingFormsList.length ; iForm++ ){ 
          invokingFormName = invokingFormsList[iForm] ; 
          invokingFormData = invokingFormsData.getSection( invokingFormName ); 
          processInvokingFormNode( invokedby, invokingFormName, invokingFormData, definedQSNames ); 
        }
        message = "Problem in processing INVOKEDBY" ; 
      }
    }catch( Exception e ){ 
      throw new ApplicationException( message, e ); 
    }
  }


  /** 
   * Generate BASE manipulation code for HTML element nodes.
   **/
  void buildBaseForNode( String idName, String tagName, String typeName ){ 
    String varName ; 
    varName = "proto" + idName ; 
    sBaseDecls += 
      "  /** Variable to hold prototype " + tagName + " node " + idName + " **/\n" + 
      "  " + typeName + " " + varName + " ; \n" +
      "\n" ; 
    sBaseInits += 
      "    " + varName + " = po.getElement" + idName + "(); \n" ;
  }


  /** 
   * Process one ID node.
   * @throws ApplicationException
   **/
  void processIdNode( String idName, KeywordValueTable idData ) throws ApplicationException { 
    String tagName ; // Type of tag. Eg, SPAN, TABLE, TR.
    String domTypeName ; // Name of DOM type for tag
    String dataFrom ; // How do we instantiate this node?
    String accessName ; // Name of the associated ACCESS, if any.
    Access access ; // The associated ACCESS, if any.
    String skelCode ; // Code, if any, for skeleton of hand-crafted class. 
    Wrapper removeCode ; // Code to perhaps remove node, wrapped around code to fill node.
    boolean isPrototypeServed ; // True if prototype itself is served, false if clone of prototype is served.
    String alwaysIn ; // "TRUE" ==> don't need to generate code for removing this node.
    try{ 

      //////  Initialise
      tagName = idData.getString( "TAG" ); 
      domTypeName = GenUtil.tagToDOMType( tagName ); 
      dataFrom = idData.getString( "DATAFROM", "BYHAND" ); 
      accessName = idData.getString( "ACCESS", null ); 
      access = ( null != accessName ) ? (Access)(accessObjects.get( accessName )) : null ; 
      alwaysIn = idData.getString( "ALWAYSIN", "TRUE" ); 
      isPrototypeServed = (null != access) ? access.expectsUnique : true ; 
      skelCode = "" ; 

      //////  Build code for BASE manipulation class
      buildBaseForNode( idName, tagName, domTypeName ); 

      // TODOONEDAY: Ideally, we should know what dynamic nodes are
      // contained inside this node, and not bother filling them if this
      // is removed.

      //////  Build code to remove dynamic node, if needed.
      if( "TRUE".equalsIgnoreCase( alwaysIn ) ){ 
        // Node is always in served page. Don't need to generate deletion code.
        removeCode = new Wrapper(); 
      }else{
        Wrapper removeCall ; // Call to remove a node. Wraps around expression returning node.
        if( "FALSE".equalsIgnoreCase( alwaysIn ) ){ 
          removeCall = new Wrapper( "ProdUtil.removeThisNode( ", " )" ); 
        }else{ 
          removeCall = new Wrapper( "ProdUtil.removeThisNodeAndContext( ", ", " + GenUtil.tagToDOMType( alwaysIn ) + " )" ); 
        }
        // TODO: The Platypus code in ( isPrototypeServed ) 'then'
        // clause, and in access.removeClone() in 'else' clause is
        // similar, and should be parametrised and merged. (We're
        // talking the code in Pond.java & Access.java here - not the
        // code in the generated app!).
        if( isPrototypeServed ){ 
          // Code for removing *prototype* is needed.
          removeCode = new Wrapper( 
            "    // TODO: Code for getting values (if any) - move from inside the 'then' clause to before the 'if()'.\n" + 
            "    // TODO: Check the if() condition! Perhaps ( bo.isIdValue( \"" + idName + "\" ) )?\n" + 
            "    if( ProdUtil.stringHasContent( field" + idName + " ) ){\n", 
            "    }else{ \n" + 
            "      " + removeCall.wrap( "proto" + idName ) + "; \n" + 
            "    }\n" ); 
        }else{
          // Code for removing *clone* of prototype is needed.
          access.removeClone( removeCall, idName, accessName ); 
          removeCode = new Wrapper(); 
        }
      }

      //////  Build code to fill dynamic node with a value
      // Note: this is almost, but not quite, just a switch on dataFrom
      if( "NONE".equalsIgnoreCase( dataFrom ) ){
        // Absolutely nothing generated for DATAFROM = NONE!
      }else if( "BYHAND".equalsIgnoreCase( dataFrom ) ){
        DynamicNode dynamicNode = new DynamicNode( poData, idName, idData ); 
        skelCode += "\n" + "    //////  Processing ID " + idName + "\n" ; 
        skelCode += dynamicNode.insertValueIntoDOM( "/*TODO: code value expression*/", ProdUtil.indentString( 4 ) ); 
      }else if( "IGNORE".equalsIgnoreCase( dataFrom ) ){
        // TODOONEDAY Deprecated 2Sep99. Delete after current app.
        skelCode += "\n" + "    //////  Processing ID " + idName + "\n" ; 
        skelCode += "    // TODO: remember to process ID " + idName + "\n" ; 
      }else if( null != accessName ){
        // Accessing fields from database, via business layer.
        processGetFieldCall( idName, idData ); 
      }else if( "PULLDOWN".equalsIgnoreCase( dataFrom ) ){ 
        // We're filling a static SELECT. The SELECTED option will be
        // determined by the PulldownItem.isDefault() method -- not by a
        // query to a normal DB table.
        skelCode += processStaticPulldown( idName, idData ); 
      }else if( "EXPR".equalsIgnoreCase( dataFrom ) ){ 
        // The config file contains the expression for the rhs.
        skelCode += "\n" + "    //////  Processing ID " + idName + "\n" ; 
        skelCode += processExpression( idName, idData );  
      }else if( "APPLICATION".equalsIgnoreCase( dataFrom ) ){ 
        skelCode += "\n" + "    //////  Processing ID " + idName + "\n" ; 
        skelCode += processApplicationMethod( idName, idData );  
      }else if( "BUSINESS".equalsIgnoreCase( dataFrom ) ){ 
        skelCode += "\n" + "    //////  Processing ID " + idName + "\n" ; 
        skelCode += processBusinessMethod( idName, idData );  
      }else if( "DATABASE".equalsIgnoreCase( dataFrom ) ){ 
        // Note: there is a check for accessName above (in this if-else-if switch).
        throw new ApplicationException( "Expect ACCESS to be present for database values" ); 
        // OLD CODE // Accessing fields from database, via business layer.
        // OLD CODE processGetFieldCall( idName, idData, true ); 
      }else{ 
        throw new ApplicationException( "Unexpected DATAFROM value in configuration: '" + dataFrom +"'" ); 
      }

      //////  Add this processing code into skeleton java.
      skelCode = removeCode.wrap( skelCode ); 
      if( 0 < skelCode.length() ){ 
        GenUtil.appendTextTo( po.getElementPOFillDynamicSkeletons(), skelCode ); 
      }
    }catch( Exception e ){ 
      // Catch, encapsulate and rethrow all exceptions so ID appears in stack trace.
      throw new ApplicationException( "Problem with ID " + idName, e ); 
    }
  }


  /** 
   * (1) Generates a call to a BO get() method, used in hand
   * manipulation class skeleton, (2) generates code to put the DB
   * value into the prototype DOM node, and (3) adds to the list of
   * arguments to declare in a BO add() method. 
   * @throws ApplicationException
   * @throws KeywordValueException
   **/
  void processGetFieldCall( String idName, KeywordValueTable idData ) throws ApplicationException, KeywordValueException { 
    String accessName ; 
    Access access ; 

    //////  Initialise
    accessName = idData.getString( "ACCESS" ); 

    //////  Get Access object
    access = (Access) accessObjects.get( accessName ) ; 
    if( null == access ){
      // Handle error condition
      String message = "Processing ID '" + idName + "': couldn't get '" + accessName + "'." ; 
      throw new ApplicationException( message ); 
    }

    //////  Process call in Access object
    access.processGetFieldCall( idName, idData ); 
  }


  /** 
   * Generate code which fills a pulldown. It's 'static', because the
   * default value is taken the PulldownItem array, not passed in as a
   * separate argument to method <CODE>fillSelect</CODE> at run time.
   * @throws ApplicationException
   **/
  String processStaticPulldown( String idName, KeywordValueTable idData ) throws ApplicationException {
    String pulldownKey ; 
    String tagName ; 
    try{ 
      pulldownKey = idData.getString( "PULLDOWNKEY" ); 
      tagName = idData.getString( "TAG" ); 
    }catch( KeywordValueException kve ){ 
      throw new ApplicationException( "Problem processing ID '" + idName + "'", kve ); 
    }
    if( "SELECT".equalsIgnoreCase( tagName ) ){ 
      return
        "\n" + 
        "    //////  Processing ID " + idName + "\n" + 
        "    ProdUtil.drainSelect( proto" + idName + ", \"" + pulldownKey + "\" ); // TODO: Really delete all OPTIONS?\n" + 
        "    ProdUtil.fillSelect( proto" + idName + ", \"" + pulldownKey + "\", bo.getPulldownItems( \"" + pulldownKey + "\" ) ); \n" ; 
    }else{
      return 
        "    // TODO: Fix ID " + idName + " in config file. \"DATAFROM = PULLDOWN\" should only occur in Select tags.\n" ;
    }
  }


  /** 
   * Generate code which fills a node with the supplied expression.
   * @throws KeywordValueException
   **/
  String processExpression( String idName, KeywordValueTable idData ) throws KeywordValueException {
    DynamicNode dynamicNode ; 
    String rhsExpr ; 

    rhsExpr = idData.getString( "LITERAL" ); 
    dynamicNode = new DynamicNode( poData, idName, idData ); 
    return dynamicNode.insertValueIntoDOM( rhsExpr, ProdUtil.indentString( 4 ) ); 
  }


  /** 
   * Generate code which fills a node with an application method.
   * @throws KeywordValueException
   **/
  String processApplicationMethod( String idName, KeywordValueTable idData ) throws KeywordValueException {
    DynamicNode dynamicNode ; 
    dynamicNode = new DynamicNode( poData, idName, idData ); 
    return dynamicNode.insertValueIntoDOM( "bo.getApplicationIdValue( \"" + idName + "\" )", ProdUtil.indentString( 4 ) ); 
  }


  /** 
   * Generate code which fills a node with a business object method.
   * @throws KeywordValueException
   **/
  String processBusinessMethod( String idName, KeywordValueTable idData ) throws KeywordValueException, ApplicationException {
    DynamicNode dynamicNode ; 
    String formatExpr ;
    dynamicNode = new DynamicNode( poData, idName, idData ); 
    formatExpr = dynamicNode.formatExpr(); 
    return dynamicNode.insertValueIntoDOM( "bo.getBusinessIdValue( \"" + idName + "\", " + formatExpr + " )", ProdUtil.indentString( 4 ) ); 
  }


  /** 
   * Process all DB commits. 
   * @throws KeywordValueException
   **/
  void processCommits( KeywordValueTable commitsData ) throws Exception { 
    String [] keys ; 
    KeywordValueTable commitData ; 
    ////  Used in loop
    int i ; 
    String commitName ;
    keys = commitsData.keys(); 
    for( i=0 ; i<keys.length ; i++ ){ 
      commitName = keys[i] ; 
      processCommit( commitName, commitsData.getSection( commitName ) ); 
    }
  }


  /** 
   * Process a DB commit. 
   * @throws Exception
   **/
  void processCommit( String commitName, KeywordValueTable commitData ) throws Exception { 
    String commitType ; // Either "add" or "update"
    String [] dataArgList ; // List of data arguments to generated method.
    String dataArgs ; // Data arguments to generated method, as generated.
    String code ; // The generated code.

    //////  Build code
    if( commitData.containsKey( "DBINDEX" ) ){ 
      commitType = "update" ; 
    }else{ 
      commitType = "add" ; 
    }
    dataArgList = Util.commitArgs( confData, commitData ); 
    dataArgs = Util.expandStrings( "qs", ", ", "", dataArgList ); 
    code = 
      "\n" + 
      "    //////  Database " + commitType + " for " + commitName + "\n" +
      "    String oid" + commitName + " ; // ObjectId for new/updated record\n" +
      "    try{ \n" + 
      "      // Note: commit" + commitName + "() args must be consistent with BO code. \n" + 
      "      // Edit the conf file rather than java code. \n" + 
      "      oid" + commitName + " = bo.commit" + commitName + "( " + dataArgs + " ); \n" + 
      "    }catch( UserDuplicateRecordException udre ){ \n" + 
      "      try{ \n" + 
      "        presentationData.set( \"RESTOREQUERYSTRING\", args ); \n" + 
      "      }catch( KeywordValueException kve ){ \n" + 
      "        ProdUtil.log( \"Failed insert into presentationData: RESTOREQUERYSTRING\" ); \n" + 
      "      }\n" + 
      "      String backURLPO ; \n" + 
      "      backURLPO = \"" + ProdUtil.replaceChar( invokedbyString, '.', '/' ) + "\" ; // TODO: Check backURLPO is correct. Should be relative PO URL, without suffix.\n" +
      "      udre.backURL = backURLPO + \".po?RESTOREQUERYSTRING=TRUE\" ; \n" + 
      "      throw udre ; \n" + 
      "    }\n" + 
      "    // DBG\n" + 
      "    ProdUtil.log( \"oid" + commitName + " is '\" + oid" + commitName + " + \"'\" ); \n" + 
      "" ; 

    //////  Insert into DOM
    GenUtil.appendTextTo( po.getElementPOCommitCalls(), code ); 
  }


  /** 
   * Writes the text in an HTML fragment to a file. 
   * @throws ApplicationException
   **/
  private void writeCodeToFile( String filePath,  HTMLElement codeDiv ) throws ApplicationException { 
    int i ; 
    String codeText ; 
    codeText = GenUtil.toText( codeDiv ); 
    try{ 
      Util.writeFile( filePath, codeText ); 
    }catch( IOException ioe ){ 
      throw new ApplicationException( "Error writing to file " + filePath, ioe ); 
    }
  }


  /**
   * Handles command line invocation of Pond class. Note that method
   * <CODE>main</CODE> must be static, but most of the code in this
   * class relies on instance variables. Hence, this method must
   * invoke an instance method -- specifically method
   * <CODE>go</CODE>. Method <CODE>main</CODE> only grabs (and cleans)
   * the command line arguments before invoking the instance method.
   **/
  public static void main( String[] args ){
    Pond pond ; // Need an instance of the class --  method main() is static!
    MainArgs mainArgs ; // Parses command line arguments. 

    //////  Initialise MainArgs object
    mainArgs = new MainArgs( args ); 
    mainArgs.setExpectedCount( 4 ); 
    //// Return a usage statement if they do not appear to be correct.
    if( ! mainArgs.valid() ){ 
      System.out.println("  Usage:  java com.pisoftware.platypus.Pond <AppName> <POPath> <HandClassDir> <BaseClassDir>");
      System.exit(1);
    }

    //////  Create an instance of the Pond class.
    pond = new Pond(); 

    //////  Parse args
    try{ 
      pond.appName = mainArgs.getPlainArg( 0 );
      pond.poPathFile = mainArgs.getPlainArg( 1 );
      pond.handClassDir = mainArgs.getPlainArg( 2 );
      pond.baseClassDir = mainArgs.getPlainArg( 3 );
    }catch( Exception e ){ 
      System.out.println("Exception while processing command line arguments:");
      System.out.println(e.getMessage());
      e.printStackTrace(); 
    }

    //////  Clean data from arguments
    if( pond.poPathFile.startsWith( "presentation/" ) ){ 
      // We need to remove the 'presentation/' part of the name. It is added elsewhere.
      pond.poPathFile = pond.poPathFile.substring( "presentation/".length() ); 
    }

    // DBG
    //System.out.println("  POPath = " + pond.poPathFile);
    //System.out.println("  handClassDir = " + pond.handClassDir);
    //System.out.println("  BaseClassDir = " + pond.baseClassDir);
    //System.exit(1);

    //////  Start the run.
    try {
      pond.go();
    } catch (Exception e) {
      System.out.println("Acckkk!  Got an exception which we won't be handling here:");
      System.out.println(e.getMessage());
      e.printStackTrace(); 
    }
  }


  /**
   * Write out the names for the PO being generated. For example,
   * write out the directory, package names, capitalised PO name,
   * etc.
   * @param exitAtEnd if true, exit the program at the end of this method.
   * @throws ApplicationException
   **/
  public void writeOutNames( boolean exitAtEnd ) throws ApplicationException { 
    System.err.println( "Variable 'poPathFile' has value '" + poPathFile + "'" ); 
    System.err.println( "Variable 'poRelativePackageName' has value '" + poRelativePackageName + "'" ); 
    System.err.println( "Variable 'rootPathlue '" + rootPath + "'" ); 
    System.err.println( "Variable 'poNameCap' has value '" + poNameCap + "'" ); 
    System.err.println( "Variable 'appNameCap' has value '" + appNameCap + "'" ); 
    System.err.println( "Variable 'poName' has value '" + poName + "'" ); 
    System.err.println( "Variable 'poPackageName' has value '" + poPackageName + "'" ); 
    System.err.println( "Variable 'poBusinessPackageName' has value '" + poBusinessPackageName + "'" ); 
    System.err.println( "Variable 'poPackageDir' has value '" + poPackageDir + "'" ); 
    System.err.println( "Variable 'domQualifiedClassName' has value '" + domQualifiedClassName + "'" ); 
    if( exitAtEnd ) throw new ApplicationException( "Exit requested in writeOutNames()" );
  }


  /**
   * @throws ApplicationException
   **/
  private void setPackageAndClassRelatedNames() throws ApplicationException { 
    String poBaseName ; // Name of PO, computed from CGI variable. Excludes package-name / directory.
    String poPath ; // Path to PO, relative to presentation/. May be "."
    //// Temporary variables
    int tmpSlashIndex ; 
    StringBuffer fileName ; 
    int i ; 

    //////  Extract directory and base names from full pathname.
    tmpSlashIndex = poPathFile.lastIndexOf( "/" ); 
    if( tmpSlashIndex == -1 ){ 
      // No directory component in given path name
      poPath = "." ; 
      poRelativePackageName = "" ; 
      poBaseName = poPathFile ; 
    }else{ 
      // Extract directory part of given path name. 
      poPath = poPathFile.substring( 0, tmpSlashIndex ); 
      poRelativePackageName = ProdUtil.replaceChar( poPath, '/', '.' ) + "." ; 
      poBaseName = poPathFile.substring( tmpSlashIndex+1 ); 
    }

    //////  Derive a whole lotta other names
    poNameCap = poBaseName ; // Name of presentation object (capitalised)
    appNameCap = appName.substring( 0, 1 ).toUpperCase() + appName.substring( 1 ) ; 
    poName = poNameCap.substring( 0, 1 ).toLowerCase() + poNameCap.substring( 1 ); 
    poPackageName = appName + ".presentation" ; 
    poBusinessPackageName = appName + ".business" ; 
    poPackageDir = "presentation" ; 
    if( ! ".".equals( poPath ) ){
      poPackageName += "." + poPath ; 
      poBusinessPackageName += "." + poPath ; 
      poPackageDir += File.separator + poPath ; 
    }
    domQualifiedClassName = poPackageName + "." + poNameCap + "HTML" ; 
    if( poPackageName.indexOf( "/" ) >= 0 ){
      throw new ApplicationException( "Programmer oversight: must change '/' to '.'" );
    }

    //////  Compute the path to the root config file
    fileName = new StringBuffer( ".." ); 
    for( i=0 ; i>=0 ; i=poPackageDir.indexOf( '/', i+1 ) ) fileName.append( "/.." ); 
    rootPath = fileName.toString(); 

    //DBG
    // writeOutNames( false ); 
  }


  /** 
   * Set the application name and presentation object name throughout the DOM.
   **/
  private void setSimpleDOMNames(){ 
    po.setTextAppName2( appName ); 
    po.setTextAppName4( appName ); 
    po.setTextAppName5( appName ); 
    po.setTextAppName6( appName ); 
    po.setTextAppName7( appName ); 
    po.setTextAppName8( appName ); 
    po.setTextAppName9( appName ); 
    po.setTextAppName10( appName ); 
    po.setTextPONameCap1( poNameCap ); 
    po.setTextPONameCap2( poNameCap ); 
    po.setTextPONameCap3( poNameCap ); 
    po.setTextPONameCap4( poNameCap ); 
    po.setTextPONameCap5( poNameCap ); 
    po.setTextPONameCap8( poNameCap ); 
    po.setTextPONameCap9( poNameCap ); 
    po.setTextPONameCap10( poNameCap ); 
    po.setTextPONameCap12( poNameCap ); 
    po.setTextPONameCap13( poNameCap ); 
    po.setTextPONameCap16( poNameCap ); 
    po.setTextPONameCap17( poNameCap ); 
    po.setTextPONameCap27( poNameCap ); 
    po.setTextPONameCap28( poNameCap ); 
    po.setTextPONameCap29( poNameCap ); 
    po.setTextPackageName1( poPackageName ); 
    po.setTextPackageName2( poPackageName ); 
    po.setTextPackageName3( poPackageName ); 
    po.setTextPackageName4( poPackageName ); 
    po.setTextBusinessPackageName3( poBusinessPackageName ); 
    po.setTextBusinessPackageName4( poBusinessPackageName ); 
    po.setTextAppTemplate1( appTemplateName ); 
    po.setTextAppTemplate2( appTemplateName ); 
    po.setTextVersionLocn1( VersionInfo.version() ); 
    po.setTextVersionLocn2( VersionInfo.version() ); 
  }



  /**
   * Top level method of the class. This method assumes the invocation
   * arguments have been put in their appropriate variables, but
   * otherwise oversees all the work of the class.
   * @throws Exception
   **/
  public void go () throws Exception {
    //// Variables containing extracts of configuration info
    String redirectTo ; 
    String redirectList ; 
    //// Other variables 
    boolean wantVarForRedirect ; // True if we generate redirection code.
    // idsData can't be global, 'coz its set twice: from poData and from confData.
    KeywordValueTable idsData ; 

    //////  Initialise 
    po = new CodeTemplateHTML(); 
    setPackageAndClassRelatedNames(); 
    confData = new KeywordValueFiles( rootPath + File.separator + "dev.conf", false ); 
    confData.readSection( "PO." + poRelativePackageName + poNameCap ); 
    poData = confData.getSection( "PO." + poRelativePackageName + poNameCap ); 
    if( null == poData ){ 
      throw new Exception( "PO section of config file did not have section '" + "PO." + poRelativePackageName + poNameCap + "'" ); 
    }
    appTemplateName = confData.getString( "TEMPLATES", "AppTemplate" ); 
    appTemplateName = poData.getString( "TEMPLATES", appTemplateName ); 

    //////  Generate or delete code for redirecting to another PO.
    redirectTo = poData.getString( "REDIRECTTO", null ); 
    redirectList = poData.getString( "REDIRECTLIST", null ); 
    wantVarForRedirect = false ; 
    if( null == redirectTo ){ 
      ProdUtil.removeThisNode( po.getElementPORedirectAlwaysCall() ); 
    }else{
      wantVarForRedirect = true ; 
      po.setTextPORedirectAlwaysURL( redirectTo ); 
    }
    if( null == redirectList ){ 
      ProdUtil.removeThisNode( po.getElementPORedirectListCall() ); 
    }else{
      wantVarForRedirect = true ; 
      po.setTextPORedirectListURL( redirectList ); 
    }
    if( ! wantVarForRedirect ){ 
      ProdUtil.removeThisNode( po.getElementPORedirectDecl() ); 
    }

    //////  Set the application name and presentation object name throughout the DOM.
    setSimpleDOMNames(); 

    //////  Initialise all 'accesses'.
    accessesData = poData.getSection( "ACCESSES" ); 
    if( null != accessesData ) initialiseAccesses(); 

    //////  Process query string names from invoking pages.
    invokedbyString = poData.getString( "INVOKEDBY", null ); 
    if( null != invokedbyString ) processQueryString();

    //////  Process all dynamic content nodes (and other dynamic values)
    idsData = poData.getSection( "IDS" ); 
    if( null != idsData ) processIDNodes( idsData ); 

    //////  Process all database adds and updates
    commitsData = poData.getSection( "COMMITS" ); 
    if( null != commitsData ) processCommits( commitsData ); 

    //////  Incorporate the generated code into the DOM.
    po.setTextHandCGIDecls( sHandCGIDecls ); 
    po.setTextHandCGIInits( sHandCGIInits ); 
    po.setTextHandCGIInitDBG( sHandCGIInitDBG ); 
    po.setTextBaseDecls( sBaseDecls ); 
    po.setTextBaseInits( sBaseInits ); 

    ////// Modules ???  
    // NOTE: we intend to introduce a concept of optional modules. For
    // example, a module that checks for application wide
    // administration notices like "Going down for maintenance 5pm
    // this evening". We haven't implemetned them yet, and this
    // section removes code for the AdminNotice module.
    ProdUtil.removeThisNode( po.getElementModuleAdminNotice1() ); 

    //////  Write out generated code
    writeCodeToFile( handClassDir + File.separator + poNameCap + ".java", po.getElementHand() ); 
    writeCodeToFile( baseClassDir + File.separator + poNameCap + "BASE.java", po.getElementBase() );
  }
}
