package com.pisoftware.platypus.production;

import java.io.*;
import java.lang.*;
import java.util.*;
import java.text.*;
import com.lutris.util.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.ApplicationException;
import com.lutris.appserver.server.Enhydra;
import com.pisoftware.platypus.production.*;

/** 
 * This class contains utilities used when the code is in
 * production. They might also be used while the code is being
 * generated.
 * <P>
 * The utilities in this file are related to display formatting
 * issues only.
 *<P>
 *
 **/


/*
 *  Copyright (C) 1999 Plugged In Software Pty Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Plugged In Software
 *  PO Box 4130
 *  St. Lucia South Queensland
 *  Australia  4067
 *  info@PIsoftware.com
 */


public class FormatUtil{

  /*
   * Display format types supported.
   *
   * These static Strings can be passed by the presentation layer
   * to the business layer for data to be formatted correctly.
   *
   * MONEY_LOCALE - monetary value formatted in locale specific format
   * DATE_LOCALE - date formatted in locale specific format
   * NONE - no formatting applied, string representation of data returned
   * BOOLEAN_YN - a boolean value displayed as 'Yes' or 'No'
   * DRILL - a partial format indicator.  Everything after the ':' in the format
   *         is returned, with the value passed added to the end.  This is
   *         usually used for 'drill-downs' on screens, where the value passed
   *         is an object id.
   * DATE - a partial format indicator.  Everything after the ':' in the format
   *        is used as the pattern in java.text.SimpleDateFormat to format the date
   *        value passed.
   */
  public static final String MONEY_LOCALE = "MONEY";
  public static final String DATE_LOCALE = "DATE";
  public static final String NONE = "NONE"; 
  public static final String BOOLEAN_YN = "BOOLEAN";
  public static final String DRILL = "DRILL:";
  public static final String DATE = "DATE:";


  // the SQL date format
  public static final String SQL_DATE_FORMAT = "yyyy-MM-dd";

/**
 * Formats the passed string using the method
 * indicated.  If the format method is empty (or "NONE"),
 * then the string is passed back unchanged.
 * If the format method is unknown, an
 * ApplicationException is thrown. 
 *
 * @param str  String to format
 * @param format_method  The type of formatting to do
 * @return see comment
 * @exception ApplicationException Thrown if the format method is unknown
 */
  public static String format(String str, String format_method) throws ApplicationException {

    // DBG
    System.err.println("STRING: "+str+", format is: <"+format_method+">");

    // check the format method is ok
    if (!ProdUtil.stringHasContent(format_method) || format_method.equalsIgnoreCase(NONE)) {

      // empty/NONE format method - nothing to do
      return str;

    }

    // Find the format method and process as needed...

    // Date?
    if (format_method.equalsIgnoreCase(DATE_LOCALE)) {

      return SQLDateToDisplay(str, null);

    }

    // Have we got a 'user' specified date format?
    if (format_method.startsWith(DATE)) {

      String date_format_str = format_method.substring(DATE.length());

      return SQLDateToDisplay(str, date_format_str);

    }

    // format method is unknown
    throw new ApplicationException("Unable to format data as this format method is unknown: "+format_method);

  }


/**
 * Formats the passed number using the method
 * indicated.  If the format method is empty (or "NONE"),
 * then the number is passed back unchanged, as a String.
 * If the format method is unknown, an
 * ApplicationException is thrown. 
 *
 * @param dbl   The double to format
 * @param format_method  The type of formatting to do
 * @return see comment
 * @exception ApplicationException Thrown if the format method is unknown
 */
  public static String format(double dbl, String format_method) throws ApplicationException {

    // DBG
    System.err.println("double: "+dbl+", format is: <"+format_method+">");

    // check the format method is ok
    if (!ProdUtil.stringHasContent(format_method) || format_method.equalsIgnoreCase(NONE)) {

      // empty/NONE format method - nothing to do
      return ""+dbl;

    }

    // Find the format method and process as needed...

    // Money?
    if (format_method.equalsIgnoreCase(MONEY_LOCALE)) {

      return doubleToDollarsDisplay(dbl);

    }

    // format method is unknown
    throw new ApplicationException("Unable to format data as this format method is unknown: "+format_method);

  }


/**
 * Formats the passed number using the method
 * indicated.  If the format method is empty (or "NONE"),
 * then the number is passed back unchanged, as a String.
 * If the format method is unknown, an
 * ApplicationException is thrown. 
 *
 * @param nr   The int to format
 * @param format_method  The type of formatting to do
 * @return see comment
 * @exception ApplicationException Thrown if the format method is unknown
 */
  public static String format(int nr, String format_method) throws ApplicationException {

    // DBG
    System.err.println("int: "+nr+", format is: <"+format_method+">");

    // check the format method is ok
    if (!ProdUtil.stringHasContent(format_method) || format_method.equalsIgnoreCase(NONE)) {

      // empty/NONE format method - nothing to do
      return ""+nr;

    }

    // Find the format method and process as needed...

    // format method is unknown
    throw new ApplicationException("Unable to format data as this format method is unknown: "+format_method);

  }


/**
 * Formats the passed number using the method
 * indicated.  If the format method is empty (or "NONE"),
 * then the number is passed back unchanged, as a String.
 * If the format method is unknown, an
 * ApplicationException is thrown. 
 *
 * @param nr   The long to format
 * @param format_method  The type of formatting to do
 * @return see comment
 * @exception ApplicationException Thrown if the format method is unknown
 */
  public static String format(long nr, String format_method) throws ApplicationException {

    // DBG
    System.err.println("long: "+nr+", format is: <"+format_method+">");

    // check the format method is ok
    if (!ProdUtil.stringHasContent(format_method) || format_method.equalsIgnoreCase(NONE)) {

      // empty/NONE format method - nothing to do
      return ""+nr;

    }

    // Find the format method and process as needed...

    // format method is unknown
    throw new ApplicationException("Unable to format data as this format method is unknown: "+format_method);

  }


/**
 * Formats the passed boolean using the method
 * indicated.  If the format method is empty (or "NONE"),
 * then the boolean is passed back unchanged, as a String.
 * If the format method is unknown, an
 * ApplicationException is thrown. 
 *
 * @param bool   The boolean value to format
 * @param format_method  The type of formatting to do
 * @return see comment
 * @exception ApplicationException Thrown if the format method is unknown
 */
  public static String format(boolean bool, String format_method) throws ApplicationException {

    // DBG
    System.err.println("boolean: "+bool+", format is: <"+format_method+">");

    // check the format method is ok
    if (!ProdUtil.stringHasContent(format_method) || format_method.equalsIgnoreCase(NONE)) {

      // empty/NONE format method - nothing to do
      return ""+bool;

    }

    // Find the format method and process as needed...

    // Boolean?
    if (format_method.equalsIgnoreCase(BOOLEAN_YN)) {

      return booleanToDisplay(bool);

    }

    // format method is unknown
    throw new ApplicationException("Unable to format data as this format method is unknown: "+format_method);

  }


/**
 * Formats the passed BigDecimal using the method
 * indicated.  If the format method is empty (or "NONE"),
 * then the number is passed back unchanged, as a string.
 * If the format method is unknown, an
 * ApplicationException is thrown. 
 *
 * @param bignr  Number to format
 * @param format_method  The type of formatting to do
 * @return see comment
 * @exception ApplicationException Thrown if the format method is unknown
 */
  public static String format(java.math.BigDecimal bignr, String format_method) throws ApplicationException {

    // DBG
    System.err.println("java.math.BigDecimal: "+bignr+", format is: <"+format_method+">");

    // check the format method is ok
    if (!ProdUtil.stringHasContent(format_method) || format_method.equalsIgnoreCase(NONE)) {

      // empty/NONE format method - nothing to do
      return bignr.toString();

    }

    // Find the format method and process as needed...

    // Have we got a drill down?
    if (format_method.startsWith(DRILL)) {

      return format_method.substring(DRILL.length()) + bignr.toString();

    }

    // format method is unknown
    throw new ApplicationException("Unable to format data as this format method is unknown: "+format_method);

  }


/**
 * Formats the passed date using the method
 * indicated.  If the format method is empty (or "NONE"),
 * then the date is passed back unchanged, as a string.
 * If the format method is unknown, an
 * ApplicationException is thrown. 
 *
 * @param date  Date to format
 * @param format_method  The type of formatting to do
 * @return see comment
 * @exception ApplicationException Thrown if the format method is unknown
 */
  public static String format(java.util.Calendar date, String format_method) throws ApplicationException {

    // DBG
    System.err.println("java.util.Calendar: "+date+", format is: <"+format_method+">");

    // check the format method is ok
    if (!ProdUtil.stringHasContent(format_method) || format_method.equalsIgnoreCase(NONE)) {

      // empty/NONE format method - make sure it's it SQL format as default
      return SQLDateToDisplay(date, SQL_DATE_FORMAT);

    }

    // Find the format method and process as needed...

    // Date?
    if (format_method.equalsIgnoreCase(DATE_LOCALE)) {

      return SQLDateToDisplay(date, null);

    }

    // Have we got a 'user' specified date format?
    if (format_method.startsWith(DATE)) {

      String date_format_str = format_method.substring(DATE.length());

      return SQLDateToDisplay(date, date_format_str);

    }

    // format method is unknown
    throw new ApplicationException("Unable to format data as this format method is unknown: "+format_method);

  }


/**
 * Formats the passed date using the method
 * indicated.  If the format method is empty (or "NONE"),
 * then the date is passed back unchanged, as a string.
 * If the format method is unknown, an
 * ApplicationException is thrown. 
 *
 * @param date  Date to format
 * @param format_method  The type of formatting to do
 * @return see comment
 * @exception ApplicationException Thrown if the format method is unknown
 */
  public static String format(java.sql.Date date, String format_method) throws ApplicationException {

    // DBG
    System.err.println("java.sql.Date: "+date+", format is: <"+format_method+">");

    // check the format method is ok
    if (!ProdUtil.stringHasContent(format_method) || format_method.equalsIgnoreCase(NONE)) {

      // empty/NONE format method - nothing to do
      return date.toString();

    }

    // Find the format method and process as needed...

    // Date?
    if (format_method.equalsIgnoreCase(DATE_LOCALE)) {

      return SQLDateToDisplay(date, null);

    }

    // Have we got a 'user' specified date format?
    if (format_method.startsWith(DATE)) {

      String date_format_str = format_method.substring(DATE.length());

      return SQLDateToDisplay(date, date_format_str);

    }

    // format method is unknown
    throw new ApplicationException("Unable to format data as this format method is unknown: "+format_method);

  }


/**
 * Converts a date to * yyyy-mm-dd, which is required for java.sql.Date.
 *
 * @param datestr  String holding the date value to convert.
 * @return see comment
 */
  public static String displayDateToSQL(String datestr) {

    return displayDateToSQL(datestr, null);

  }


/**
 * Converts a date in the format passed to
 * yyyy-mm-dd, which is required for java.sql.Date.
 * If the format passed is null or empty, the locale
 * specific format is used.
 *
 * @param datestr  String holding the date value to convert.
 * @param format_str  String holding the date format
 * @return see comment
 */
  public static String displayDateToSQL(String datestr, String format_str) {

    try {

      Date date = null;

      SimpleDateFormat sdf = new SimpleDateFormat();

      String result_str = "";

      // any format string passed?
      if (ProdUtil.stringHasContent(format_str)) {

        // apply the pattern to the formatter
        sdf.applyPattern(format_str);
  
        // parse the string passed
        date = sdf.parse(datestr);

      } else {

        // default country is US, if not set
        String country = System.getProperty("user.region","US");

        // default language is english, if not set
        String language = System.getProperty("user.language","EN");

        Locale lc = new Locale(language, country);

        DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT,lc);

        // parse the string passed
        date = df.parse(datestr);

      }

      // apply the pattern to the formatter
      sdf.applyPattern("yyyy-MM-dd");
  
      // format the date 
      result_str = sdf.format(date).toString();

      // DBG
      System.err.println("displayDateToSQL result: "+result_str+", format_str is: <"+format_str+">");
  
      // return the result, whatever it is
      return result_str;

    } catch (Exception e) {

      // ignore this...

    }

    return "";

  }


/**
 * Converts a java.sql.Date to a String for display in HTML
 * using the format supplied.  If the format is empty or null,
 * the locale specific format is used.
 *
 * @param date  String holding the date value to convert.
 * @param format_str  String holding the date format to use
 * @return see comment
 */
  public static String SQLDateToDisplay(java.sql.Date date, String format_str) {
    if( null == date ) return "" ; 

    String result_str = "";

    // any format string passed?
    if (ProdUtil.stringHasContent(format_str)) {

      SimpleDateFormat df = new SimpleDateFormat();

      // apply the pattern to the formatter
      df.applyPattern(format_str);

      // format the date passed
      result_str = df.format(date).toString();

    } else {

      // default country is US, if not set
      String country = System.getProperty("user.region","US");

      // default language is english, if not set
      String language = System.getProperty("user.language","EN");

      Locale lc = new Locale(language, country);

      System.err.println("LOCALE: Country: "+lc.getDisplayCountry()+", Language: "+lc.getDisplayLanguage());
      System.err.println("user.region is: "+System.getProperty("user.region","UNKNOWN!"));

      DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT, lc);

      // format the date passed
      result_str = df.format(date).toString();

    }

    // DBG
    System.err.println("SQLDateToDisplay result is: "+result_str+", from format: <"+format_str+">");

    // return the result, whatever it is
    return result_str;

  }


/**
 * Converts a date in the format yyyy-mm-dd 
 * for display in HTML.
 *
 * @param datestr  String holding the date value to convert.
 * @return see comment
 */
  public static String SQLDateToDisplay(String datestr, String format_str) {

    try {

      // convert this date to java.sql.Date...
      java.sql.Date date = java.sql.Date.valueOf(datestr);

      return SQLDateToDisplay(date, format_str);

    } catch (Exception e) {

      // ignore this...

    }

    return "";

  }


/**
 * Converts a java.util.Calendar for display in HTML.
 *
 * @param date  String holding the date value to convert.
 * @param format_str  String holding the date format to use
 * @return see comment
 */
  public static String SQLDateToDisplay(java.util.Calendar date, String format_str) {
    if( null == date ) return "" ; 

    int year = date.get(date.YEAR);
    int month = date.get(date.MONTH);
    int day = date.get(date.DATE);

    return SQLDateToDisplay( new java.sql.Date(year, month, day), format_str ); 
  }


/**
 * Converts the amount into a monetary amount.  The
 * cents part of the amount is ignored.
 *
 * @param amount  String holding the value to convert.
 * @return see comment
 */
  public static String doubleToDollarsDisplay(double amount) {

    java.text.NumberFormat dollarsFormat = java.text.NumberFormat.getCurrencyInstance();

    // format the amount
    return dollarsFormat.format(amount);

  }


/**
 * Returns a "Yes" or "No" for a boolean. 
 * 
 * @param b  Value to display
 * @return see comment.
 */
  static public String booleanToDisplay( boolean b ){ 

    return b ? "Yes" : "No" ; 

  }

}
