// $Id: QuickSort.java,v 1.2 1999/10/20 01:00:03 cryan Exp $

package AU.net.plugged.sort;

import java.lang.*;
import java.util.*;
import java.io.*;

import AU.net.plugged.sort.event.*;

// Class:	QuickSort

/**
 *
 *  Version: 0.1<p>
 *
 *  Author: <A HREF="MAILTO:cryan@plugged.net.au">Chris Ryan</A><p>
 *
 *  Company: <blocquote>Plugged In Software<br>
 *           P.O. Box 4130<br>
 *	     St Lucia South<br>
 *           Brisbane Qld Australia 4067<br>
 *           <a href=http://www.plugged.net.au>http://www.plugged.net.au</a><br>
 *<p>
 *  Date:       28 January 1997
 *<p>
 *<dl><dt>Purpose:
 *	<dd>This class sorts a vector using the QuickSort algorithm.
 * </dl>
 *
 *  Usage:      
 *
 *<dl><dt> Modification
 *</dl>
*/

/*
 *  Copyright (C) 1999 Plugged In Software Pty Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Plugged In Software
 *  PO Box 4130
 *  St. Lucia South Queensland
 *  Australia  4067
 *  info@PIsoftware.com
 */


public class QuickSort implements Runnable {

  // comparison types

  /** For sorting Strings. */
  public final static int STRING = 0;

  /** For sorting ints. */
  public final static int INT = 1;

  /** For sorting by "extension" Strings, ie everything after the
      '.' in a filename. */
  public final static int EXTENSION = 2;

  /** For sorting by using the SortInterface callback method. */
  public final static int SORT_INTERFACE = 7;


  /* The following variables are used for when this
   * sort object is instantiated.  QuickSorting is then
   * done via a separate thread and any listeners
   * are notified of completion.
   */
  /** The vector to sort */
  protected Vector sort_vector;
  /** The sorting thread */
  protected Thread sort_thread;
  /** The vector of listeners for this sort */
  protected Vector sort_listeners = new Vector();

  /** Sort flags, passed in some comparison tests */
  public static Object sort_flags = null;


/** 
 * Constructor - This can be used to run the sorting
 * in a separate thread.  It assumes that the
 * SORT_INTERFACE comparison style is being used.
 *
 * @param v         The vector containing the elements to sort
 * @param priority  the priority to run the sorting at
 * @param flags     any flags to pass to the comparison test
 */
  public QuickSort(Vector v, int priority, Object flags) {

    // keep the passed vector
    sort_vector = v;

    // keep the passed flags
    sort_flags = flags;

    // create the sort thread
    sort_thread = new Thread(this);

    // set the priority
    sort_thread.setPriority(priority);

  }


/** 
 * Starts the thread
 */
  public void start() {

    // start the thread
    sort_thread.start();

  }


/** 
 * The sort thread run method
 */
  public void run() {

    // call the quicksort routine
    quickSort(sort_vector);

    // sorting is done, fire a sort completed event
    
    // create a sort event
    SortEvent se = new SortEvent(this, SortEvent.SORT_COMPLETED, sort_vector);

    // process the event
    processSortEvent(se);

  }


/** 
 * Swaps elements in a vector
 *
 * @param v      The vector containing the elements
 * @param index1 The position of the first element
 * @param index2 The position of the second element
 */
  static void swapElements(Vector v, int index1, int index2) {

    // get the objects
    Object obj1 = v.elementAt(index1);
    Object obj2 = v.elementAt(index2);

    // swap them
    v.setElementAt(obj1,index2);
    v.setElementAt(obj2,index1);

  }


/** 
 * Compares two objects.  Returns 0 if they are equal, -1 if
 * object 1 is less than object 2, and 1 if object 1 is greater
 * than object 2.
 *
 * @param object1          First object
 * @param object2          Second object
 * @param comparison_type  Type of comparison to perform
 */
/* Local Variables
 * <none>
 *
 */
  static int compareObjects(Object object1, Object object2, int comparison_type) {

    // switch to the right comparison type
    switch (comparison_type) {

      case STRING:

        // ensure neither string is null
        if (object1 == null) {

          return -1;

        }

        // ensure neither string is null
        if (object2 == null) {

          return 1;

        }

        // just do a normal string comparison
        return ((String) object1).compareTo((String) object2);

      case INT:

        // compare the two ints
        return ((Integer) object1).intValue() - ((Integer) object2).intValue();

      case EXTENSION:

        // convert objects to strings
        String str1 = (String) object1;
        String str2 = (String) object2;

        // get the positions of the first '.' in each string
        int dot1 = str1.indexOf('.');
        int dot2 = str2.indexOf('.');

        // DBG
        //System.out.println("str1: "+str1+", str2: "+str2+", dot1: "+dot1+", dot2: "+dot2);

        // do both strings have '.' in them?
        if (dot1 >= 0 && dot2 >= 0) {

          // both strings have '.', compare extensions
          int compare_val = str1.substring(dot1).compareTo(str2.substring(dot2));

          // DBG
          //System.out.println("compare_val: "+compare_val);

          // are they equal?
          if (compare_val == 0) {

            // yes, just do a normal compare on the strings
            return str1.compareTo(str2);

          } else {

            // not equal, return result
            return compare_val;

          }

        } else {

          // does the first string have a '.'
          if (dot1 >= 0) {

            // yes, therefore, the second string doesn't
            return 1;

          } 

          // does the second string have a '.'
          if (dot2 >= 0) {

            // yes, therefore, the first string doesn't
            return -1;

          } 

          // both strings don't have a '.', just do a normal compare
          return str1.compareTo(str2);

        }

      case SORT_INTERFACE:

        // invoke the callback method on the SortInterface
        return ((SortInterface) object1).compareTo(object2, sort_flags);

      default:
        break;

    }

    // should never get here
    return 0;

  }


/** 
 * Recursively sorts a vector containing strings
 *
 * @param v      The vector containing the elements
 * @param start  The start point in the vector
 * @param finish The finish point in the vector
 * @param comparison_type   The type of comparison for sorting
 */
/* Local Variables
 * <none>
 *
 */
  static void quickSort(Vector v, int start, int finish, int comparison_type) {

    int lo = start;
    int hi = finish;
    Object mid;

    if (finish > start) {

      /* Arbitrarily establishing partition element as the midpoint of
       * the vector.
       */
      mid = (Object) v.elementAt(( start + finish ) / 2);

      // loop through the array until indices cross
      while (lo <= hi) {

        /* find the first element that is greater than or equal to 
         * the partition element starting from the left Index.
         */
        while ((lo < finish) && (compareObjects(v.elementAt(lo),mid,comparison_type) < 0))
          ++lo;

        /* find an element that is smaller than or equal to 
         * the partition element starting from the right Index.
         */
        while ((hi > start) && (compareObjects(v.elementAt(hi),mid,comparison_type) > 0))
          --hi;

        // if the indexes have not crossed, swap
        if (lo <= hi) {

          swapElements(v, lo, hi);

          ++lo;
          --hi;

        }
      }

      /* If the right index has not reached the left side of array
       * must now sort the left partition.
       */
      if (start < hi)
        quickSort(v, start, hi,comparison_type);

      /* If the left index has not reached the right side of array
       * must now sort the right partition.
       */
      if (lo < finish)
        quickSort(v, lo, finish,comparison_type);

    }
  }


/** 
 * Quicksorts a vector, using the comparison type passed to 
 * determine the vector's contents.
 *
 * @param v      The vector containing the elements
 * @param comparison_type   The type of comparison for sorting
 */
  public static void quickSort(Vector v, int comparison_type) {

      // DBG
      // System.out.println("Doing this type of comparison: "+comparison_type);

      quickSort(v, 0, v.size() - 1, comparison_type);
  }


/** 
 * Quicksorts a vector.  The objects within the vector must
 * implement the SortInterface.
 *
 * @param v      The vector containing the elements
 */
  public static void quickSort(Vector v) {

      // DBG
      //System.out.println("Doing SORT_INTERFACE comparison: ");

      quickSort(v, 0, v.size() - 1, SORT_INTERFACE);
  }



/**
 * Adds a SortListener who will receive the sort events that occur
 *
 * @param listener  Listener who wants to catch events
 */
  public void addSortListener(SortListener listener) {

    synchronized (sort_listeners) {
      sort_listeners.addElement(listener);
    }

  }


/**
 * Removes a SortListener who no longer wants to catch
 * QuickSort events.
 *
 * @param listener   Listener who no longer wants to catch events.
 */
  public void removeSortListener(SortListener listener) {

    synchronized (sort_listeners) {
      sort_listeners.removeElement(listener);
    }

  }


/**
 * Processes a SortEvent.  
 * @param e   SortEvent to process
 */
  protected void processSortEvent(SortEvent e) {
    Vector listeners;

    synchronized(sort_listeners) {
      listeners = (Vector) sort_listeners.clone();
    }

    // For each SortListener, run the appropriate method
    // based on the event passed.
    for (int i=0; i<listeners.size(); i++) {

      SortListener this_listener =
        (SortListener) listeners.elementAt(0);

      switch(e.getEventType()) {

        case SortEvent.SORT_COMPLETED:
          this_listener.sortCompleted(e);
          break;

        default:
          // Unknown event type!

      }

    }

  }



  // The String gets changed to the current version by RCS
  private static final String version_string = "$Revision: 1.2 $";

  /**
  *    Returns the RCS version information for this class.
  *
  * @return Returns the RCS Revision string for this class
  */
  public static String getVersionString() {

    return version_string;

  }

}
