package com.pisoftware.platypus.util;

import java.io.*;
import java.lang.*;
import java.util.*;
import com.lutris.util.*;
import com.pisoftware.platypus.production.*;
import com.lutris.appserver.server.ApplicationException;

/**
 * Class <CODE>KeywordValueFiles</CODE> extends the functionality
 * offered by class <CODE>KeywordValueTable</CODE>. It implements the
 * reading and writing of configuraton information spread over several
 * files, as specified in the Platypus documentation (actually,
 * currently, as specified in
 * <CODE>platypus/doc/devconf.txt</CODE>). If the
 * configuration information is written back to disk, only those files
 * which must be changed are written. This is a <EM>good thing</EM> when you're
 * using <CODE>make</CODE>.
 * 
 * <P>
 * 
 * Currently, comments (and order) are not preserved in the
 * configuration files.
 * 
 * <P>
 * 
 * Do not modify the table with calls in subsections. Why? 'Coz
 * subsections won't set the flags which make the relevent files get
 * written by method <CODE>writeBack()</CODE>. If you don't care about
 * this, then you can modify the table with calls in subsections.
 * 
 * <P>
 * 
 * The method <CODE>calcSectionFile</CODE> determines the mapping from
 * section names to files. If a different mapping is required, you
 * could override this method, or even replace it with a helper class
 * passed to the constructor.
 **/

/*
 *  Copyright (C) 1999 Plugged In Software Pty Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Plugged In Software
 *  PO Box 4130
 *  St. Lucia South Queensland
 *  Australia  4067
 *  info@PIsoftware.com
 */

public class KeywordValueFiles extends KeywordValueTable {

  /** True if configuration file(s) might be written out again. **/
  // Writing back to file requires quite a bit of overhead. This
  // variable allows some of that over head to be skipped if it's not
  // needed.
  protected boolean writable ; 

  /** Name of root configuration file. If it is relative, it's
      relative to the current working direcotry. **/
  String rootFile ; 

  /** Contains list of sections which have been edited. This is needed
      when deciding which files need to be written. The root section
      is stored under key "" (ie, the empty string).**/
  protected Hashtable dirtySections = null ; 

  /** Contains list of sections which have been read from file. Maps
      from section name to file name (relative to root file). Does not
      include the root file. **/
  protected Hashtable fileSections = null ; 


  /**
   * Constructs a KeywordValueFiles object from a configuration
   * file. None of the optional sections are loaded. <B>Note</B>:
   * actually, a <CODE>Config</CODE> object is contructed. See note in
   * class Javadoc about the difference between the
   * <CODE>Config</CODE> and <CODE>KeywordValueTable</CODE> classes.
   * 
   * @param rootName File containing root of configuration information
   * @param writable If true, keep the necessary info to write back to file.
   **/
  public KeywordValueFiles( String rootName, boolean writable ) throws KeywordValueException, FileNotFoundException {
    Config config ; // Temporary store for initial configuration info
    String [] keys ; // Keys in initial configuration info
    ////  Used in copy loop
    int i ; 
    String oneKey ; 

    //////  Initialise class variables
    rootFile = rootName ; 
    this.writable = writable ; 
    fileSections = new Hashtable(); 
    if( writable ) dirtySections = new Hashtable(); 

    //////  Read root section and copy it into this instance
    config = readConf( rootName ); 
    keys = config.keys(); 
    for( i=0 ; i<keys.length ; i++ ){ 
      oneKey = keys[i] ; 
      super.set( oneKey, config.get( oneKey ) ); 
    }

    // DBG 
    // System.err.println( "Constructing KeywordValueFiles( " + rootName + " )" ); 
    // System.err.println( "Initial keys: " + Util.expandStrings( "", ", ", "", keys ) ); 
  }


  /**
   * Calls main constructor, with <CODE>writable</CODE> set to <CODE>true</CODE>. 
   **/
  public KeywordValueFiles( String rootName ) throws KeywordValueException, FileNotFoundException {
    this( rootName, true ); 
  }


  /**
   * Reads in the entrie configuration information, from all files. 
   **/
  public void readAllSections( boolean ignoreMissingFile ) throws KeywordValueException, ApplicationException {
    String [] sectionNames ; // Names of all sections in other files.
    int i ; 
    sectionNames = getAllSectionNames(); 
    for( i=0 ; i<sectionNames.length ; i++ ) readSection( sectionNames[i], ignoreMissingFile ); 
  }


  /**
   * Reads in the entrie configuration information, from all files. 
   **/
  public void readAllSections() throws KeywordValueException, ApplicationException {
    readAllSections( false ); 
  }


  /**
   * Reads in a section from a file.
   * @param Exact name of section in file.
   **/
  public void readSection( String sectionName, boolean ignoreMissingFile ) throws ApplicationException {
    // NOTE: method can return before end of body.
    String rootDirPath ; // Path to root file's directory.
    String sectionFile ; // Path & file containing new section. Relative to root file.
    Config sectionConf ; // Contains the new section
    try{ 

      //////  If the section has already been read, don't read it again!
      if( fileSections.containsKey( sectionName ) ) return ; 
      
      //////  Read in the section, incorporate into this instance
      rootDirPath = (new File( rootFile )).getParent(); 
      if( null == rootDirPath ) rootDirPath = "." ; 
      sectionFile = calcSectionFile( sectionName, false ); 
      // DBG 
      // System.err.println( "Reading config file: " + rootDirPath + " / " + sectionFile ); 
      try{ 
        sectionConf = readConf( rootDirPath + File.separator + sectionFile ); 
        ProdUtil.testForNull( sectionName, "sectionName is null" ); 
        ProdUtil.testForNull( sectionConf, "sectionConf is null" ); 
        super.set( sectionName, sectionConf.get( sectionName ) ); // Don't mark root as changed

        //////  Record section in instance variables
        fileSections.put( sectionName, sectionFile ); 
      }catch( FileNotFoundException fnfe ){ 
        if( ! ignoreMissingFile ) throw fnfe ; 
      }
    }catch( Exception e ){ 
      throw new ApplicationException( "Problem while reading section '" + sectionName + "'", e ); 
    }
  }


  /**
   * Reads in a section from a file.
   * @param Exact name of section in file.
   **/
  public void readSection( String sectionName ) throws ApplicationException {
    readSection( sectionName, false ); 
  }


  /** 
   * Returns a Config from a file.
   **/
  Config readConf( String fileName ) throws KeywordValueException, FileNotFoundException {
    InputStream underlyinginfilestream ;
    BufferedInputStream infilestream ; 
    ConfigFile configFile ;
    Config conf ; 
    // DBG 
    // System.err.println( "Reading configuration file '" + fileName + "'" ); 
    underlyinginfilestream = new FileInputStream( fileName );
    infilestream = new BufferedInputStream( underlyinginfilestream ); 
    // NOTE: Once, I got a NullPointerException from within 'new
    // ConfigFile( ... )'. It was caused by a syntax error in the
    // config file. (Specifically, a quoted value contained a newline,
    // splitting it over two lines.) Nasty.
    configFile = new ConfigFile( infilestream ); 
    conf = configFile.getConfig(); 
    return conf ; 
  } 


  /** 
   * Writes a Config to a file.
   **/
  void writeConf( Config config, String fileName ) throws KeywordValueException, FileNotFoundException, IOException {
    File file ; 
    OutputStream underlyingoutfilestream ;
    BufferedOutputStream outfilestream ; 
    ConfigFile configFile ;
    Config conf ; 
    file = new File( fileName ); 
    if( file.isDirectory() ) 
      throw new KeywordValueException( "Config file '" + fileName + "' is a directory" ); 
    underlyingoutfilestream = new FileOutputStream( file );
    outfilestream = new BufferedOutputStream( underlyingoutfilestream ); 
    configFile = new ConfigFile( config ); 
    configFile.write( outfilestream ); 
  } 


  /**
   * Writes configuration files back to where they were read from. Not
   * all sections are actually written -- only those which have
   * actually been changed are written.
   * 
   * <P>
   * 
   * <B>WARNING</B>: This method alters the instance's
   * KeywordValueTable. It removes all file sections if the root
   * section needs to be written. This may change in future versions
   * (ie, yes, it's a bug).
   **/
  void writeBack() throws KeywordValueException, FileNotFoundException, IOException {
    Enumeration dirtySectionNames ; // Names of sections that have changed.
    //// Used in loop
    String dirtySectionName ; // Name of this changed section
    String dirtySectionFile ; // Name of file for this section
    Config tmpConfig ; // Contains section to write to this file
    boolean rootIsDirty ; // True if root section is dirty.

    if( ! writable ) 
      throw new KeywordValueException( "Object instantiated with 'writable' flag false." ); 

    dirtySectionNames = dirtySections.keys(); 
    rootIsDirty = false ; // Perhaps set to true in loop.
    while( dirtySectionNames.hasMoreElements() ){ 
      dirtySectionName = (String) dirtySectionNames.nextElement(); 
      // DBG
      // System.out.println( "Dirty section: '" + dirtySectionName + "'" ); 
      if( dirtySectionName.length() != 0 ){ 
        // Writing non-root section
        dirtySectionFile = calcSectionFile( dirtySectionName, true ); 
        tmpConfig = new Config(); 
        tmpConfig.set( dirtySectionName, get( dirtySectionName ) ); 
        writeConf( tmpConfig, dirtySectionFile ); 
      }else{
        // Root section needs special handling
        rootIsDirty = true ; 
      }
    }
    if( rootIsDirty ) writeBackRootSection(); 
  }


  /**
   * 
   **/
  // Currently, this method removes all file sections from the KeywordValueTable. 
  void writeBackRootSection() throws KeywordValueException, FileNotFoundException, IOException {
    Config tmpConfig ; // Temporary copy -- all file sections will be removed!
    Enumeration fileSectionNames ; // Names of all sections read in from file
    //// Used in loop
    String fileSectionName ; 

    tmpConfig = new Config( this ); 
    fileSectionNames = fileSections.keys(); 
    while( fileSectionNames.hasMoreElements() ){ 
      fileSectionName = (String) fileSectionNames.nextElement(); 
      tmpConfig.remove( fileSectionName ); 
    }
    writeConf( tmpConfig, rootFile ); 
  }


  /**
   * Same functionality as superclass method. 
   * @see KeywordValueTable#set
   **/
  // This method must be overridden to allow writing of altered files only.
  public void set( String keyword, Object value) throws KeywordValueException {
    //////  Implement superclass functionality
    super.set( keyword, value ); 

    //////  Record which file has been altered
    if( writable ) markSectionAsDirty( keyword ); 
  }


  /**
   * Same functionality as superclass method. 
   * @see KeywordValueTable#remove
   **/
  // This method must be overridden to allow writing of altered files only.
  public void remove( String keyword ) throws KeywordValueException{
    //////  Implement superclass functionality
    super.remove( keyword ); 

    //////  Record which file has been altered
    if( writable ) markSectionAsDirty( keyword ); 
  }


  /**
   * Same functionality as superclass method. 
   * @see KeywordValueTable#setDefault
   **/
  public void setDefault( String keyword, Object defaultValue ) throws KeywordValueException {
    //////  Implement superclass functionality
    super.setDefault( keyword, defaultValue ); 

    //////  Record which file has been altered
    // Hmmm. Has this section actually been changed? 
    // I don't know. Better safe than sorry.
    if( writable ) markSectionAsDirty( keyword ); 
  }


  /**
   * Records that file containing <CODE>dirtySectionName</CODE> needs
   * to be written. The section <CODE>dirtySectionName</CODE> does
   * <EM>not</EM> need to be the outermost section of the file.
   **/
  protected void markSectionAsDirty( String dirtySectionName ){
    Enumeration fileSectionNames ; 
    String dirtyFileSectionName ; 
    //// Used in loop 
    String fileSectionName ; 

    fileSectionNames = fileSections.keys(); 
    dirtyFileSectionName = "" ; // Initialise with value for root config file.
    while( fileSectionNames.hasMoreElements() ){ 
      // Loop can exit before whole list processed!
      fileSectionName = (String) fileSectionNames.nextElement() ; 
      if( dirtySectionName.equals( fileSectionName ) ||
          dirtySectionName.startsWith( fileSectionName + "." ) ){
        dirtyFileSectionName = fileSectionName ;
        break ; 
      }
    }
    dirtySections.put( dirtyFileSectionName, "" ); 
  }


  /**
   * Computes the name of the file containing a section. The name path
   * is relative to the root configuration file. 
   * 
   * <BR>
   * 
   * <B>WARNING</B>: The exact name of the outermost section in a file
   * must be given. This method will not detect sections nested within
   * a file.
   * @param sectionName Name of section to be read
   * @param allowRoot If false, throw an exception instead of returning the root file.
   **/
  protected String calcSectionFile( String sectionName, boolean allowRoot ) throws KeywordValueException {
    String sectionFile ; 
    if( sectionName.startsWith( "PO." ) ){ 
      // Look in CORE.PO.... for file name
      sectionFile = getString( "CORE." + sectionName ); 
      if( null == sectionFile ){ 
        System.err.println( "" ); 
        System.err.println( "Value of KeywordValueFiles object just before throwing exception ..." ); 
        System.err.println( super.toString() ); 
        throw new KeywordValueException( "Couldn't find entry 'CORE." + sectionName + "' in KeywordValueFiles object" ); 
      }
      return "src/presentation/" + sectionFile + ".conf" ; 
    }else{
      // Perhaps it's in CORE.XTRACONF...?
      try{ 
        sectionFile = getString( "CORE.XTRACONF." + sectionName ); 
      }catch( KeywordValueException kve ){ 
        sectionFile = null ; 
      }
      if( null != sectionFile ){ 
        // It was in XTRACONF!
        return sectionFile ; 
      }else{ 
        // Couldn't find it. Belongs in root file.
        if( allowRoot ){ 
          return (new File( rootFile )).getName(); 
        }else{ 
          throw new KeywordValueException( "Section " + sectionName + " is in root file" ); 
        }
      }
    }
  }


  /** 
   * Returns list of all leaf keys, with the fully qualified path name to the leaves!
   **/
  static public void leafKeysUtility( KeywordValueTable kvt, Vector leafKeys, String currentPrefix ){ 
    String [] currentKeys ; 
    int i ; 
    currentKeys = kvt.keys(); 
    KeywordValueTable subsection ; 
    for( i=0 ; i<currentKeys.length ; i++ ){ 
      try{ 
        subsection = kvt.getSection( currentKeys[i] ); 
        leafKeysUtility( subsection, leafKeys, currentPrefix + currentKeys[i] + "." ); 
      }catch( KeywordValueException kve ){ 
        leafKeys.addElement( currentPrefix + currentKeys[i] ); 
      }
    }
  }


  /** 
   * Returns list of all leaf keys in a subsection of kvt, with the fully qualified path name to the leaves!
   **/
  static public String [] leafKeysWithPath( KeywordValueTable kvt, String sectionName, String prefix ) throws KeywordValueException { 
    Vector vec = new Vector(); 
    String [] keysArray ; 
    int i ; 
    leafKeysUtility( kvt.getSection( sectionName ), vec, "" ); 
    keysArray = new String[ vec.size()] ; 
    vec.copyInto( keysArray ); 
    for( i=0 ; i<keysArray.length ; i++ ){ 
      keysArray[i] = prefix + keysArray[i] ; 
    }
    return keysArray ; 
  }


  /** 
   * Reads in all the sections from other files, as specified in the
   * CORE.PO and CORE.XTRACONF sections.
   **/
  public String [] getAllSectionNames() throws KeywordValueException {
    String [] poNames = null ; 
    String [] xtraNames = null ; 
    String [] allNames ;
    KeywordValueTable coreData ; 
    coreData = getSection( "CORE" ); 
    if( coreData.containsKey( "PO" ) ){ 
      poNames = leafKeysWithPath( coreData, "PO", "PO." ); 
    }
    if( coreData.containsKey( "XTRACONF" ) ){ 
      xtraNames = leafKeysWithPath( coreData, "XTRACONF", "" ); 
    }
    if( null != poNames ){ 
      if( null != xtraNames ){ 
        allNames = new String[ poNames.length + xtraNames.length ]; 
        System.arraycopy( poNames, 0, allNames, 0, poNames.length ); 
        System.arraycopy( xtraNames, 0, allNames, poNames.length, xtraNames.length ); 
        return allNames ; 
      }else{ 
        return poNames ; 
      }
    }else{ 
      if( null != xtraNames ){ 
        return xtraNames ; 
      }else{ 
        return new String[0] ; 
      }
    }
  }


  /** 
   * Returns a copy of the named section, with defaults set. 
   *
   * <BR> Returns a copy of a section, containing values as
   * follows. First values are copied from section
   * <CODE>thisSection.sectionName</CODE>. If this includes a
   * <CODE>REFERS</CODE> subsection, this is treated as the section
   * name (within <CODE>rootSection</CODE>) of default values. Any
   * names which occur in the default section, but not in the named
   * subsection, are copied into the section to be returned.
   *
   * <BR> Note: <CODE>REFERS</CODE> subsections are recursively
   * followed. Also, they are removed from the final copied section.
   *
   * <BR> Note: Although the returned section is copied, its values
   * might or might not be, copies.
   **/
  public static KeywordValueTable getFilledSection( KeywordValueTable rootSection, KeywordValueTable thisSection, String sectionName ) throws KeywordValueException {
    KeywordValueTable subsection ; // Desired subsection, before cloning and without defaults
    KeywordValueTable copy ; // Subsection to return
    String defaultSectionName ; // Name of section containing defaults
    // Temporary variables
    int i ; 
    String [] names ;
    String name ; 

    //////  Get desired section, before copying and without defaults
    subsection = thisSection.getSection( sectionName ); 
    if( null == subsection ) return new KeywordValueTable();

    //////  Create copy filled with defaults, if any.
    if( subsection.containsKey( "REFERS" ) ){ 
      defaultSectionName = subsection.getString( "REFERS" ); 
      // ProdUtil.log( "About to copy defaults from " + defaultSectionName ); 
      copy = getFilledSection( rootSection, rootSection, defaultSectionName ); 
      // ProdUtil.log( "Copied defaults from        " + defaultSectionName ); 
    }else{
      copy = new KeywordValueTable(); 
    }

    //////  Copy desired section's values into the return copy.
    // ProdUtil.log( "About to copy names from " + sectionName ); 
    names = subsection.keys(); 
    for( i=0 ; i<names.length ; i++ ){
      name = names[i] ; 
      // ProdUtil.log( "Copying name " + name ); 
      copy.set( name, subsection.get( name ) ); 
    }
    // ProdUtil.log( "Copied names from        " + sectionName ); 

    //////  Leave
    return copy ; 
  }


  /** 
   * Used for testing the class.
   **/
  public static void main (String[] args) throws Exception {
    KeywordValueFiles kvf ; 
    kvf = new KeywordValueFiles( "dev.conf", true ); 
    //kvf.readSection( "PO.Welcome", false ); 
    //kvf.set( "OOWAH", "Where am I" ); 
    //kvf.set( "PO.Welcome.elsewhere", "Where am I" ); 
    kvf.readAllSections(); 
    System.out.println( kvf.toString() ); 
    //kvf.writeBack(); 
    //System.out.println( kvf.toString() ); 
  }
}
