package org.enhydra.xml.binding;

import java.util.Map;
import java.util.HashMap;

/**
 * <p>
 *  <code>DataMapping</code> is a singleton that handles the mappings from
 *    XML Schema data types to Java as well as Java data types to XML Schema.
 * </p>
 *
 * @author Brett McLaughlin
 */
public class DataMapping {

    /** Singleton instance */
    private static DataMapping instance = null;

    /** Mappings from Java to XML Schema */
    private Map schemaMappings;

    /** Mappings from XML Schema to Java */
    private Map javaMappings;

    /**
     * <p>
     *  This (intentionally left) private constructor handles initialization
     *    of the data mappings. It can only be created internally, and forces
     *    the singleton pattern to be used.
     * </p> 
     */
    private DataMapping() {
        schemaMappings = getSchemaMappings();
        javaMappings = getJavaMappings();
    } 

    /**
     * <p>
     *  This will retrieve the singleton instance of this class, allowing
     *    it to be used across applications.
     * </p>
     *
     * @return <code>DataMapping</code> - the singleton instance to use.
     */
    public static DataMapping getInstance() {
        if (instance == null) {
            synchronized (DataMapping.class) {
                if (instance == null) {
                    instance = new DataMapping();
                }
            }
        }
        return instance;
    }

    /**
     * <p>
     *  This will take the <code>String</code> value supplied and convert it
     *    to an <code>Object</code> of the type specified in <code>paramType</code>.
     * </p>
     *
     * @param value <code>String</code> value to convert.
     * @param paramType <code>Class</code> with type to convert to.
     * @return <code>Object</code> - value in correct type.
     */
    public static Object getParameter(String value, Class paramType) {
        Object ob = null;
        String type = paramType.getName();

        if (type.equals("java.lang.String")) {
            ob = value;
        } else if ((type.equals("int")) || (type.equals("java.lang.Integer"))) {
            ob = Integer.valueOf(value);
        } else if ((type.equals("long")) || (type.equals("java.lang.Long"))) {
            ob = Long.valueOf(value);
        } else if ((type.equals("float")) || (type.equals("java.lang.Float"))) {
            ob = Float.valueOf(value);
        } else if ((type.equals("double")) || (type.equals("java.lang.Double"))) {
            ob = Double.valueOf(value);
        } else if ((type.equals("boolean")) || (type.equals("java.lang.Boolean"))) {
            ob = Boolean.valueOf(value);
        }

        return ob;
    }

    /**
     * <p>
     *  This will return the Java data type given an XML Schema data type.
     * </p>
     *
     * @param schemaType XML Schema data type (<code>String</code> format).
     * @return <code>String</code> - Java data type that is comparable.
     */
    public String getJavaType(String schemaType) {
        return (String)javaMappings.get(schemaType);
    }

    /**
     * <p>
     *  This will return the XML Schema data type given a Java data type.
     * </p>
     *
     * @param javaType Java data type (<code>String</code> format).
     * @return <code>String</code> - XML Schema data type that is comparable.
     */
    public String getSchemaType(String javaType) {
        return (String)schemaMappings.get(javaType);
    }

    /**
     * <p>
     *  This will generate the data mappings from XML Schema to Java.
     * </p>
     *
     * @return <code>Map</code> - data type mappings.
     */
    private Map getSchemaMappings() {
        Map map = new HashMap();

        // Key is Java type, value is XML Schema type
        map.put("String", "string");
        map.put("boolean", "boolean");
        map.put("float", "float");
        map.put("double", "double");

        // XXX: Need to map these extra data types
        // map.put("", "decimal");
        // map.put("", "timeDuration");
        // map.put("", "recurringDuration");
        // map.put("", "binary");
        // map.put("", "uriReference");
        // map.put("", "ID");
        // map.put("", "IDREF");
        // map.put("", "ENTITY");
        // map.put("", "NOTATION");
        // map.put("", "QName");
        // map.put("", "language");
        // map.put("", "IDREFS");
        // map.put("", "ENTITIES");
        // map.put("", "NMTOKEN");
        // map.put("", "Name");
        // map.put("", "NCName");
        // map.put("", "integer");
        // map.put("", "nonPositiveInteger");
        // map.put("", "negativeInteger");
        map.put("long", "long");
        map.put("int", "int");
        map.put("short", "short");
        map.put("byte", "byte");
        // map.put("", "nonNegativeInteger");
        // map.put("", "negativeInteger");
        // map.put("", "unsignedLong");
        // map.put("", "unsignedInt");
        // map.put("", "unsignedShort");
        // map.put("", "unsignedByte");
        // map.put("", "positiveInteger");
        // map.put("", "timeInstant");
        // map.put("", "time");
        // map.put("", "timePeriod");
        // map.put("", "date");
        // map.put("", "month");
        // map.put("", "year");
        // map.put("", "century");
        // map.put("", "recurringDate");
        // map.put("", "recurringDay");

        return map;
    }

    /**
     * <p>
     *  This will generate the data mappings from Java to XML Schema.
     * </p>
     *
     * @return <code>Map</code> - data type mappings.
     */
    private Map getJavaMappings() {
        Map map = new HashMap();

        // Key is XML Schema type, value is Java type
        map.put("string", "String");
        map.put("boolean", "boolean");
        map.put("float", "float");
        map.put("double", "double");

        // XXX: Need to map these extra data types
        // map.put("decimal", "");
        // map.put("timeDuration", "");
        // map.put("recurringDuration", "");
        // map.put("binary", "");
        // map.put("uriReference", "");
        // map.put("ID", "");
        // map.put("IDREF", "");
        // map.put("ENTITY", "");
        // map.put("NOTATION", "");
        // map.put("QName", "");
        // map.put("language", "");
        // map.put("IDREFS", "");
        // map.put("ENTITIES", "");
        // map.put("NMTOKEN", "");
        // map.put("Name", "");
        // map.put("NCName", "");
        map.put("integer", "int");
        // map.put("nonPositiveInteger", "");
        // map.put("negativeInteger", "");
        map.put("long", "long");
        map.put("int", "int");
        map.put("short", "short");
        map.put("byte", "byte");
        // map.put("nonNegativeInteger", "");
        // map.put("negativeInteger", "");
        // map.put("unsignedLong", "");
        // map.put("unsignedInt", "");
        // map.put("unsignedShort", "");
        // map.put("unsignedByte", "");
        // map.put("positiveInteger", "");
        // map.put("timeInstant", "");
        // map.put("time", "");
        // map.put("timePeriod", "");
        // map.put("date", "");
        // map.put("month", "");
        // map.put("year", "");
        // map.put("century", "");
        // map.put("recurringDate", "");
        // map.put("recurringDay", "");

        return map;
    }

}
