package org.enhydra.xml.binding;

import java.io.IOException;
import java.io.OutputStream;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.XMLOutputter;

/**
 * <p>
 *  <code>Marshaller</code> takes a Java <code>Object</code> instance and writes
 *    out an XML representation of that object, with each property (variable)
 *    of the instance and its value.
 * </p>
 *
 * @author Brett McLaughlin
 */
public class Marshaller {

    /**
     * <p>
     *  Simple constructor.
     * </p>
     */
    public Marshaller() {
        // Currently, do nothing
    }
   
    /**
     * <p>
     *  This method is the public entry point for marshalling an object into
     *    an XML instance document.
     * </p>
     *
     * @param obj <code>Object</code> to convert to XML.
     * @param out <code>OutputStream</code> to write XML to.
     * @throws <code>IOException</code> when errors in output occur.
     */
    public static void marshall(Object obj, OutputStream out) throws IOException {
        Marshaller marshaller = new Marshaller();
        marshaller.writeXMLRepresentation(obj, out);
    }

    /**
     * <p>
     *  This will take a Java object instance, and convert it into an
     *    XML document, and write that document to the supplied output stream.
     * </p>
     *
     * @param obj <code>Object</code> to convert to XML.
     * @param out <code>OutputStream</code> to write XML to.
     * @throws <code>IOException</code> when errors in output occur.
     */
    private void writeXMLRepresentation(Object obj, OutputStream out) 
        throws IOException {

        // Root Element is the start of recursion
        Element root = getXMLRepresentation(obj);
        Document doc = new Document(root);

        // Use 2 space indentation and line feeds
        XMLOutputter outputter = new XMLOutputter("  ", true);
        outputter.output(doc, out);
    } 

    /**
     * <p>
     *  This is the granular portion of binding; a Java <code>Object</code> is
     *    converted into an XML element (in JDOM form), using recursion for any
     *    children.
     * </p>
     * 
     * @param obj <code>Object</code> to get the XML element representation for.
     * @return <code>Element</code> - representation of <code>Object</code> in XML.
     * @throws <code>IOException</code> when errors occur in binding.
     */
    private Element getXMLRepresentation(Object obj) throws IOException {
        Class objectClass = obj.getClass();

        // Get the name of the element for this object
        String objectName = BindingUtils.initialLowercase(objectClass.getName());

        // If this is an "Impl" class, remove that from the name
        int index = -1;
        if ((index = objectName.indexOf("Impl")) != -1) {
            objectName = objectName.substring(0, index);
        }

        Element element = new Element(objectName);

        Method[] methods = objectClass.getMethods();
        for (int i=0; i<methods.length; i++) {
            // Only want accessor methods, but not the getClass() method in Object
            Method method = methods[i];
            if ((method.getName().startsWith("get")) && 
                (!method.getName().equals("getClass"))) {

                // Get the value for this method
                try {
                    Object o = method.invoke(obj, new Object[] { });

                    // For the name, remove the "get" and lower the initial letter
                    String propertyName = 
                        BindingUtils.initialLowercase(method.getName().substring(3));
                    
                    // Determine if it's primitive by seeing if it's a java.lang type
                    if (o.getClass().getName().startsWith("java.lang.")) {
                        // If it's a primitive, add as an attribute
                        element.addAttribute(propertyName, o.toString());
                    } else {
                        // ... otherwise, recurse and add new element as child
                        element.addContent(getXMLRepresentation(o));
                    }

                } catch (IllegalAccessException e) {
                    throw new IOException(e.getMessage());
                } catch (InvocationTargetException e) {
                    throw new IOException(e.getMessage());
                }
            }
        }

        return element;
    }

}
