/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * ../../timesheet/data/CompanyQuery.java
 *-----------------------------------------------------------------------------
 */

package timesheet.data;

import com.lutris.dods.builder.sourceGenerators.Query.*;

//import com.lutris.business.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.session.*;

import java.sql.*;
import java.util.*;
import java.util.Date;  // when I say Date, I don't mean java.sql.Date

/**
 * CompanyQuery is used to query the t_company table in the database.<BR>
 * It returns objects of type CompanyDO.
 * <P>
 * General usage:
 * <P>
 *     In DODS:
 *        Create a Data Object named "Dog",
 *        and create an Attribute named "Name",
 *        and set that Attribute to "Can be queried." 
 *        DODS will then generate the method DogQuery.setQueryName(). 
 * <P>
 *     In your Business Layer, prepare the query:<BR>
 * <P><PRE>
 *             DogQuery dq = new DogQuery();
 *             dq.setQueryName("Rex") 
 *             if ( Rex is a reserved name )
 *                 dq.requireUniqueInstance();
 * </PRE>
 *     Then, get the query results one of two ways:
 * <P>
 *         #1:<PRE>
 *             String names = "";
 *             DogBDO[] dogs = dq.getBDOArray();
 *             for ( int i = 0; i < dogs.length; i++ ) {
 *                 names += dogs[i].getName() + " ";
 *             }
 * </PRE>
 *      or #2:<PRE>
 *             String names = "";
 *             DogBDO dog;
 *             while ( null != ( dog = dq.getNextBDO() ) ) {
 *                 names += dog.getName() + " ";
 *             }
 * </PRE>
 *     Note:   Methods <CODE>getDOArray()</CODE> and <CODE>getNextDO()</CODE>
 *             do exist, but they are not generally used
 *             in the Business or Presentation layers of an application.
 *             <B>All access to the Data Layer (DO classes) should occur via
 *             the Business Layer (BDO classes).
 *             Only the Business Layer (BDO classes and classes extending them)
 *             should need to manipulate the Data Layer (DO classes).</B>
 *             See also the comments in the BDO constructors.
 * <P>
 *     Note:   If <CODE>requireUniqueInstance()</CODE> was called,
 *             then <CODE>getBDOArray()</CODE> or <CODE>getNextBDO()</CODE>
 *             will throw an exception if more than one "Rex" was found.
 * <P>
 *     Note:   Results of the query will come from the Data Object cache if:
 *             -  The cache is available.
 *             -  Matches were found in the cache.
 *             -  No other tables (Data Objects of other types) were involved
 *                in the query.
 *                This can happen if you extend the <CODE>DogQuery</CODE> class
 *                and you make calls to the <CODE>QueryBuilder</CODE> object
 *                to add SQL involving other tables.
 *             If any of these conditions is not true,
 *             then any results from the query will come from the database.
 * <P>
 *     To reuse the query object, call:
 * <P><PRE>
 *             dq.reset();
 * </PRE>
 * @author dwood
 * @version $Revision: 1.12 $
 */
final public class CompanyQuery implements Query {

    private QueryBuilder builder;

    /**
     * Public constructor.
     */
    public CompanyQuery() { 
	builder = new QueryBuilder( "t_company", "t_company.*" );
	builder.setDatabaseVendor( "PostgreSQL" );
	builder.setStringMatchDetails( "LIKE", "%" );
	reset();
    }


/**
 * Public constructor.  This version can be used to create
 * arbitrary queries on the table.  Note that the SELECT &
 * FROM clauses are still set as per the normal constructor
 * above.  Do not use the setXXX methods with this constructor.
 * Using requireUniqueInstance is ok.  Use get(B)DOArray/getNext(B)DO
 * as normal.
 *
 * Created by Chris Ryan (cryan@plugged.net.au)
 *
 * @param where_clause  Arbitrary where clause for the query. This string should contain everything required after the 'SELECT ... FROM ... ' clauses. 
 */
    public CompanyQuery(String where_clause) { 

	builder = new QueryBuilder( "t_company", "t_company.*" );
	builder.setDatabaseVendor( "PostgreSQL" );
	builder.setStringMatchDetails( "LIKE", "%" );
	reset();

    // caching is a bit broken - ensure the cache is cleared completely
    cacheHits = new Vector();
    hitDatabase();

  builder.addEndClause(where_clause);
    }


/**
 * Set the WHERE clause for an arbitrary queries on the table.  
 * Note that the SELECT & FROM clauses are still set as normal.
 *
 * Created by Chris Ryan (cryan@plugged.net.au)
 *
 * @param where_clause  Arbitrary where clause for the query. This string should contain everything required after the 'SELECT ... FROM ... ' clauses. 
 */
  public void setArbitraryWhereClause(String where_clause) {

    // clear any residual query setup
    reset();

    // caching is a bit broken - ensure the cache is cleared completely
    cacheHits = new Vector();
    hitDatabase();

    // add the clause.
    builder.addEndClause(where_clause);

  }

    private ResultSet		resultSet	= null;
    private boolean 		uniqueInstance	= false;
    private CompanyDO[]	DOs		= null;
    private int			arrayIndex	= -1;
    private boolean		needToRun	= true;
    private Vector		cacheHits	= null;

    private boolean partialCache = false;
    private boolean hitDb = true;
    public void hitDatabase() { hitDb = true; }

    /**
     * Perform the query on the database, and prepare the 
     * array of returned DO objects.
     *
     * @exception DataObjectException If a database access error occurs. 
     * @exception NonUniqueQueryException If too many rows were found. 
     */
    private void runQuery()
    throws DataObjectException, NonUniqueQueryException {
	needToRun = false;
	arrayIndex = -1;

	DBQuery dbQuery = null;
	try {
	    Vector results;
	    /*
	    if ( cacheHits.size() > 0 ) {
		// The setQuery methods build up the cacheHits.
		// If the cache had our desired objects, 
		// we don't query the database, so we have no ResultSet.
		results = cacheHits;	 // executeQuery saw cache hits
	    } else {
		// If the cache doesn't exist, or could not handle the query,
		// then we actually query the database.
		dbQuery = Enhydra.getDatabaseManager().createQuery();
		dbQuery.query( this );    // invokes executeQuery
	        results = new Vector();
	        while ( resultSet.next() ) {
		    results.addElement( new CompanyDO ( resultSet ) );
	        }
	    }
	    */
	    if ( partialCache && 0 == cacheHits.size() )
		hitDb = true;
	    if ( hitDb ) {
		dbQuery = Enhydra.getDatabaseManager().createQuery();
		dbQuery.query( this );    // invokes executeQuery
	        results = new Vector();
	        while ( resultSet.next() ) {
		    results.addElement( new CompanyDO ( resultSet ) );
	        }
	    } else {
		results = cacheHits;	 // executeQuery saw cache hits
	    }

	    if ( results.size() > 1 && uniqueInstance )
		throw new NonUniqueQueryException( 
		    "Too many rows returned from database" );
	    DOs = new CompanyDO [ results.size() ];
	    for ( int i = 0; i < results.size(); i++ ) {
		DOs[i] = ( CompanyDO ) results.elementAt( i );
	    }
	    arrayIndex = 0;
	} catch ( SQLException se ) {
	    if (null == se.getSQLState() ) {
		throw new DataObjectException( 
		    "Unknown SQLException", se );
	    }
	    if (	se.getSQLState().startsWith("02") &&
			se.getErrorCode() == 100 ) {
		throw new DataObjectException( 
		    "Update or delete DO is out of synch", se );
	    } else if (	se.getSQLState().equals("S1000") &&
			se.getErrorCode() == -268 ) {
		throw new DataObjectException( 
		    "Integrity constraint violation", se );
	    } else {
		throw new DataObjectException( "Data Object Error", se );
	    }
	} catch ( ObjectIdException oe ) { 
	    throw new DataObjectException( "Object ID Error", oe );
	} catch ( DatabaseManagerException de ) {
	    throw new DataObjectException( "Database connection Error", de );
	} finally {
	    if ( null != dbQuery )
		dbQuery.release();
	}
    }


    /**
     * Return array of DOs constructed from ResultSet returned by query.
     * @exception DataObjectException If a database access error occurs. 
     * @exception NonUniqueQueryException If too many rows were found. 
     */
    public CompanyDO[] getDOArray()
    throws DataObjectException, NonUniqueQueryException {
	if ( needToRun )
	    runQuery();
	return DOs;
    }

    /**
     * Return successive DOs from array built from ResultSet returned by query.
     * @exception DataObjectException If a database access error occurs. 
     * @exception NonUniqueQueryException If too many rows were found. 
     */
    public CompanyDO getNextDO()
    throws DataObjectException, NonUniqueQueryException {
	if ( needToRun )
	    runQuery();
	if ( null == DOs ) {
	    /* This should never happen.  
	     * runQuery() should either throw an exception
	     * or create an array of DOs, possibly of zero length.
	     */
	    return null;
	}
	if ( arrayIndex < DOs.length )
	    return DOs[ arrayIndex++ ];
	return null;
    }

    /**
     * Return array of BDOs constructed from ResultSet returned by query.
     * @exception DataObjectException If a database access error occurs. 
     * @exception NonUniqueQueryException If too many rows were found. 
     */
    public CompanyBDO[] getBDOArray()
    throws DataObjectException, NonUniqueQueryException {
	if ( needToRun )
	    runQuery();
	CompanyBDO[] BDOs = new CompanyBDO[ DOs.length ];
	for ( int i = 0; i < DOs.length; i++ )
	    BDOs[ i ] = CompanyBDO.createExisting( DOs[ i ] );
	return BDOs;
    }

    /**
     * Return successive BDOs from array built from ResultSet returned by query.
     * @exception DataObjectException If a database access error occurs. 
     * @exception NonUniqueQueryException If too many rows were found. 
     */
    public CompanyBDO getNextBDO()
    throws DataObjectException, NonUniqueQueryException {
	CompanyDO DO = getNextDO();
	if ( null == DO )
	    return null;
	return CompanyBDO.createExisting( DO );
    }


    /**
     * Set the OID to query.
     * WARNING!  This method assumes that table <CODE>t_company</CODE>
     * has a column named <CODE>"objectID"</CODE>.
     * This method is called from the DO classes to retrieve an object by id.
     *
     * @param oid The object id to query.
     */
    public void setQueryOId(ObjectId oid) {
        // Remove from cacheHits any DOs that do not meet this
        // setQuery requirement.
	if ( null == oid ) 
	    return;
	requireUniqueInstance();
        for ( int i = 0; i < cacheHits.size(); i++ ) {
            CompanyDO DO = ( CompanyDO ) cacheHits.elementAt( i );
            if ( null == DO ) continue;
            boolean equals = true;
	    ObjectId id = DO.getOId();
	    if ( null == id || ! id.equals( oid ) )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database
        // in case there is no cache, or the query involves other tables.
	builder.addWhereClause( "objectID",  oid.toBigDecimal(),
						QueryBuilder.NOT_NULL );
    }


    /**
     * Set "unique instance" assertion bit.
     * The first call to the next() method will throw an exception 
     * if more than one object was found.
     */
    public void requireUniqueInstance()
    {
	uniqueInstance = true;
    }

    /**
     * Reset the query parameters.
     */
    public void reset() {
	cacheHits	= new Vector();
	DOs		= null;
	uniqueInstance	= false;
	needToRun	= true;
        builder.reset();
    }

    /**
     * Return the appropriate QueryBuilder flag for selecting
     * exact matches (SQL '=') or inexact matches (SQL 'matches').
     */
    private boolean exactFlag( boolean exact ) {
        return exact ? QueryBuilder.EXACT_MATCH : QueryBuilder.NOT_EXACT;
    }



    //
    // Implementation of Query interface
    //

    /**
     * Method to query objects from the database.
     * The following call in runQuery()
     *	    dbQuery.query( this );
     * causes the dbQuery object to invoke 
     *      executeQuery()
     *
     * @param conn Handle to database connection.
     * @exception java.sql.SQLException If a database access error occurs.
     */
    public ResultSet executeQuery(DBConnection conn)
        throws SQLException
    {
	resultSet = builder.executeQuery( conn );
        return resultSet;
    }
 
 
    /**
     * WARNING!  This method is disabled.
     * It's implementation is forced by the Query interface.
     * This method is disabled for 2 reasons:
     * 1)  the getDOArray() and getNextDO() methods are better
     *     because they return DOs instead of JDBC objects.
     * 2)  the createExisting() method throws an exception
     *     that we cannot reasonably handle here,
     *     and that we cannot throw from here.
     *
     * @param rs JDBC result set from which the next object
     *   will be instantiated.
     * @exception java.sql.SQLException
     *   If a database access error occurs.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an invalid object id was queried from the database.
     */
    public Object next(ResultSet rs)
    throws SQLException, ObjectIdException {
	/*
        if (!rs.next())
            return null;
 
        return CompanyDO.createExisting(rs);
	*/
	throw new ObjectIdException( 
	    "next() should not be used.  Use getNextDO() instead." );
	//return null;
    }


    /**
     * Set the Name to query.
     *
     * @param x The Name of the t_company to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryName(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    CompanyDO DO = ( CompanyDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getName();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "Name", x, "VARCHAR",
                QueryBuilder.NOT_NULL, exactFlag( exact ) );
    }

    /**
     * Set the Name to query
     * @param x The Name of the t_company to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryName( 
				String x )
    throws DataObjectException {
	setQueryName( x, true );
    }

    /**
     * Add Name to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByName(boolean direction_flag) {
        builder.addOrderByColumn("Name",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add Name to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByName() {
        builder.addOrderByColumn("Name","ASC");
    }


    /**
     * Set the PhysicalStreet to query.
     *
     * @param x The PhysicalStreet of the t_company to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPhysicalStreet(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    CompanyDO DO = ( CompanyDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getPhysicalStreet();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "PhysicalStreet", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the PhysicalStreet to query
     * @param x The PhysicalStreet of the t_company to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPhysicalStreet( 
				String x )
    throws DataObjectException {
	setQueryPhysicalStreet( x, true );
    }

    /**
     * Add PhysicalStreet to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByPhysicalStreet(boolean direction_flag) {
        builder.addOrderByColumn("PhysicalStreet",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add PhysicalStreet to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByPhysicalStreet() {
        builder.addOrderByColumn("PhysicalStreet","ASC");
    }


    /**
     * Set the PhysicalCity to query.
     *
     * @param x The PhysicalCity of the t_company to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPhysicalCity(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    CompanyDO DO = ( CompanyDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getPhysicalCity();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "PhysicalCity", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the PhysicalCity to query
     * @param x The PhysicalCity of the t_company to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPhysicalCity( 
				String x )
    throws DataObjectException {
	setQueryPhysicalCity( x, true );
    }

    /**
     * Add PhysicalCity to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByPhysicalCity(boolean direction_flag) {
        builder.addOrderByColumn("PhysicalCity",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add PhysicalCity to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByPhysicalCity() {
        builder.addOrderByColumn("PhysicalCity","ASC");
    }


    /**
     * Set the PhysicalPostCode to query.
     *
     * @param x The PhysicalPostCode of the t_company to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPhysicalPostCode(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    CompanyDO DO = ( CompanyDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getPhysicalPostCode();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "PhysicalPostCode", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the PhysicalPostCode to query
     * @param x The PhysicalPostCode of the t_company to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPhysicalPostCode( 
				String x )
    throws DataObjectException {
	setQueryPhysicalPostCode( x, true );
    }

    /**
     * Add PhysicalPostCode to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByPhysicalPostCode(boolean direction_flag) {
        builder.addOrderByColumn("PhysicalPostCode",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add PhysicalPostCode to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByPhysicalPostCode() {
        builder.addOrderByColumn("PhysicalPostCode","ASC");
    }


    /**
     * Set the PhysicalState to query.
     *
     * @param x The PhysicalState of the t_company to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPhysicalState(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    CompanyDO DO = ( CompanyDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getPhysicalState();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "PhysicalState", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the PhysicalState to query
     * @param x The PhysicalState of the t_company to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPhysicalState( 
				String x )
    throws DataObjectException {
	setQueryPhysicalState( x, true );
    }

    /**
     * Add PhysicalState to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByPhysicalState(boolean direction_flag) {
        builder.addOrderByColumn("PhysicalState",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add PhysicalState to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByPhysicalState() {
        builder.addOrderByColumn("PhysicalState","ASC");
    }


    /**
     * Set the PostalStreet to query.
     *
     * @param x The PostalStreet of the t_company to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPostalStreet(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    CompanyDO DO = ( CompanyDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getPostalStreet();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "PostalStreet", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the PostalStreet to query
     * @param x The PostalStreet of the t_company to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPostalStreet( 
				String x )
    throws DataObjectException {
	setQueryPostalStreet( x, true );
    }

    /**
     * Add PostalStreet to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByPostalStreet(boolean direction_flag) {
        builder.addOrderByColumn("PostalStreet",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add PostalStreet to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByPostalStreet() {
        builder.addOrderByColumn("PostalStreet","ASC");
    }


    /**
     * Set the PostalCity to query.
     *
     * @param x The PostalCity of the t_company to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPostalCity(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    CompanyDO DO = ( CompanyDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getPostalCity();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "PostalCity", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the PostalCity to query
     * @param x The PostalCity of the t_company to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPostalCity( 
				String x )
    throws DataObjectException {
	setQueryPostalCity( x, true );
    }

    /**
     * Add PostalCity to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByPostalCity(boolean direction_flag) {
        builder.addOrderByColumn("PostalCity",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add PostalCity to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByPostalCity() {
        builder.addOrderByColumn("PostalCity","ASC");
    }


    /**
     * Set the PostalPostCode to query.
     *
     * @param x The PostalPostCode of the t_company to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPostalPostCode(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    CompanyDO DO = ( CompanyDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getPostalPostCode();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "PostalPostCode", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the PostalPostCode to query
     * @param x The PostalPostCode of the t_company to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPostalPostCode( 
				String x )
    throws DataObjectException {
	setQueryPostalPostCode( x, true );
    }

    /**
     * Add PostalPostCode to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByPostalPostCode(boolean direction_flag) {
        builder.addOrderByColumn("PostalPostCode",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add PostalPostCode to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByPostalPostCode() {
        builder.addOrderByColumn("PostalPostCode","ASC");
    }


    /**
     * Set the PostalState to query.
     *
     * @param x The PostalState of the t_company to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPostalState(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    CompanyDO DO = ( CompanyDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getPostalState();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "PostalState", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the PostalState to query
     * @param x The PostalState of the t_company to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPostalState( 
				String x )
    throws DataObjectException {
	setQueryPostalState( x, true );
    }

    /**
     * Add PostalState to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByPostalState(boolean direction_flag) {
        builder.addOrderByColumn("PostalState",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add PostalState to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByPostalState() {
        builder.addOrderByColumn("PostalState","ASC");
    }


    /**
     * Set the Country to query.
     *
     * @param x The Country of the t_company to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryCountry(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    CompanyDO DO = ( CompanyDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getCountry();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "Country", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the Country to query
     * @param x The Country of the t_company to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryCountry( 
				String x )
    throws DataObjectException {
	setQueryCountry( x, true );
    }

    /**
     * Add Country to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByCountry(boolean direction_flag) {
        builder.addOrderByColumn("Country",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add Country to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByCountry() {
        builder.addOrderByColumn("Country","ASC");
    }


    /**
     * Set the Phone to query.
     *
     * @param x The Phone of the t_company to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPhone(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    CompanyDO DO = ( CompanyDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getPhone();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "Phone", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the Phone to query
     * @param x The Phone of the t_company to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPhone( 
				String x )
    throws DataObjectException {
	setQueryPhone( x, true );
    }

    /**
     * Add Phone to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByPhone(boolean direction_flag) {
        builder.addOrderByColumn("Phone",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add Phone to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByPhone() {
        builder.addOrderByColumn("Phone","ASC");
    }


    /**
     * Set the Fax to query.
     *
     * @param x The Fax of the t_company to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryFax(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    CompanyDO DO = ( CompanyDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getFax();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "Fax", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the Fax to query
     * @param x The Fax of the t_company to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryFax( 
				String x )
    throws DataObjectException {
	setQueryFax( x, true );
    }

    /**
     * Add Fax to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByFax(boolean direction_flag) {
        builder.addOrderByColumn("Fax",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add Fax to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByFax() {
        builder.addOrderByColumn("Fax","ASC");
    }


    /**
     * Set the URL to query.
     *
     * @param x The URL of the t_company to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryURL(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    CompanyDO DO = ( CompanyDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getURL();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "URL", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the URL to query
     * @param x The URL of the t_company to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryURL( 
				String x )
    throws DataObjectException {
	setQueryURL( x, true );
    }

    /**
     * Add URL to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByURL(boolean direction_flag) {
        builder.addOrderByColumn("URL",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add URL to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByURL() {
        builder.addOrderByColumn("URL","ASC");
    }

    /**
    * Returns the <code>QueryBuilder</code> object that this class uses to
    * construct and execute database queries.
    * <code>CompanyQuery.set</code> methods use the <code>QueryBuilder</code> to
    * append SQL expressions to the <code>"where"</code> clause to be executed.
    * The <code>QueryBuilder.addEndClause method.</code> can be used to
    * append freeform SQL expressions to the <code>where</code> clause,
    * e.g. "order by name".
    */
    public QueryBuilder getQueryBuilder() {
	return builder;
    }
}
