/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * ../../timesheet/data/ProjectBDO.java
 *-----------------------------------------------------------------------------
 */

package timesheet.data;

import java.sql.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;

import com.lutris.dods.builder.sourceGenerators.Query.DataObjectException;
import com.lutris.dods.builder.sourceGenerators.Query.NonUniqueQueryException;
import com.lutris.dods.builder.sourceGenerators.Query.RefAssertionException;

/**
 * ProjectBDO contains the same set and get methods as
 * the ProjectDO class.
 * Business Object (BO) classes typically need these set and get methods.
 * So by deriving a BO from a BDO, or by implementing a BO that 
 * contains a BDO, the developer of the BO is spared some work.
 *
 * @author dwood
 * @version $Revision: 1.12.2.1 $
 */
public class ProjectBDO {

    /**
     * The ProjectDO object upon which the set and get methods operate.
     * This member is protected so that classes derived from ProjectBDO
     * can access the underlying Data Object.
     */
    protected ProjectDO DO;

    /**
     * Note:  This method is intended for use only by other BDO classes.
     * Presentation Layer classes should (theoretically) always use the
     * Business Layer (BDO) to create/access Data Layer (DO) objects.
     * The overhead for using BDO objects is small
     * (the BDO classes are fairly lightweight.)
     *
     * @return The DO object held by this BDO object.
     */
    public ProjectDO getDO() { 
	return DO;
    }

    /**
     * Like the class <CODE>ProjectDO</CODE>, 
     * this class acts as a factory.
     * Business Object (BO) classes typically need these set and get methods.
     * So by deriving a BO from a BDO, or by implementing a BO that 
     * contains one or more BDOs, the developer of the BO is spared some work.
     *
     * @exception Exception
     *   If an error occurs.
     */
    public static ProjectBDO createVirgin() throws Exception { 
	ProjectBDO bdo = new ProjectBDO ();
	return bdo;
    }

    /**
     * Constructor for use by classes derived from <CODE>ProjectBDO</CODE>.
     * Example usage:<CODE>
     *      class CustomerBO extends CustomerBDO {
     *          // a BDO class is commonly extended in order to implement:
     *          public void beforeAnySet() throws Exception {
     *              if ( CustomerDO should not be altered )
     *                  throw new ApplicationException( "ERROR" );
     *          }
     *          public CustomerBO( CustomerDO DO ) {
     *              super( DO );
     *          }
     *      }
     *      class SomePresentationLayerClass {
     *          public CustomerBO findCustomer( String name ) {
     *              CustomerQuery q = new CustomerQuery();
     *              q.setQueryName( name );
     *              CustomerDO DO = q.getNextDO();
     *              // Here the CustomerBO ctor fires the CustomerBDO ctor.
     *              return new CustomerBO( DO );
     *          }
     *      }
     * </CODE>
     */
    public ProjectBDO( ProjectDO DO ) { 
	this.DO = DO;
    }

    /**
     * Constructor required by <CODE>ProjectBDO.create</CODE> methods.
     */
    public ProjectBDO() throws Exception { 
	this.DO = ProjectDO.createVirgin();
    }

    /**
     * The createExisting method is used to create a <CODE>ProjectBDO</CODE>
     * from a <CODE>ProjectDO</CODE> that was returned by 
     * the <CODE>ProjectQuery</CODE> class.
     */
    public static ProjectBDO createExisting( ProjectDO DO ) { 
	ProjectBDO bdo = new ProjectBDO ( DO );
	return bdo;
    }

    /** 
     * The getBDOarray method performs a database query to
     * return an array of <CODE>ProjectBDO</CODE> objects
     * representing all the rows in the <CODE>t_project</CODE> table.
     *
     * This method is a minimal example of using a Query class.
     * To restrict the set of objects returned,  you could
     * invoke <CODE>query.setXxx()</CODE>, where <CODE>Xxx</CODE>
     * is an Attribute of <CODE>ProjectDO</CODE> which was 
     * marked as "Can be queried" in the DODS Attribute Editor.
     *
     * @exception DataObjectException
     *   If an object is not found in the database.
     */
    public static ProjectBDO[] getBDOarray()
    throws DataObjectException {
        ProjectDO[] DOarray = null;
        try {
            ProjectQuery query = new ProjectQuery();
	    // To restrict the set of objects returned,
	    // you could invoke query.setXxx(), where Xxx
	    // is an Attribute of <CODE>ProjectDO</CODE> which was 
	    // marked as "Can be queried" in the DODS Attribute Editor.
            DOarray = query.getDOArray();
        } catch ( NonUniqueQueryException e1 ) {
            // ProjectQuery will throw NonUniqueQueryException
            // only if query.requireUniqueInstance() is called
	    // and more than one DO was found.
        } catch ( DataObjectException e2 ) {
            // This could happen if the database server dies, etc.
            throw e2;
        }
        ProjectBDO[] BDOarray = new ProjectBDO[ DOarray.length ];
        for ( int i = 0; i < DOarray.length; i++ )
            BDOarray[i] = ProjectBDO.createExisting( DOarray[i] );
 
        return BDOarray;
    }


    /**
     * The developer of a Business Object that derives from this class
     * can override the methods:<CODE>
     *     beforeAnyGet
     *     beforeAnySet
     *     afterAnySet
     * </CODE> to handle any general assertions or cleanup needed
     * for <CODE>get</CODE> and <CODE>set</CODE> methods.
     */
    protected void beforeAnyGet() {
    }
    // beforeAnySet throws plain Exception to allow overriding implementations
    // to throw any flavor needed.
    protected void beforeAnySet() throws Exception {
	if ( DO.isReadOnly() )
	    throw new DataObjectException( DO + " is read-only." ); 
    }
    protected void afterAnySet() {
    }

    /**
     * The methods <CODE>
     *     getHandle
     *     hasMatchingHandle
     * </CODE> are used by Presentation Objects that need to populate
     * HTML select lists with <CODE>ProjectBDO</CODE> objects as options.
     *
     * The <CODE>getHandle()</CODE> method is used
     * to set the value for each option,
     * and the <CODE>hasMatchingHandle()<CODE>
     * methods are used to lookup the Data Object when the selection has
     * been made.
     *
     * This ProjectBDO object holds a reference to a ProjectDO object.
     * The id of this ProjectBDO is the id of its ProjectDO.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @return id of this BDO as a string
     *   If an object id can't be allocated for this object.
     */
    public String getHandle()
    throws DatabaseManagerException {
	return DO.getHandle();
    }

    /**
     * @param handle
     *   <CODE>String</CODE> representation of the id for this BDO
     * @return boolean
     *   True if the string version of the id of this DO matches passed handle
     * @see getHandle
     */
    public boolean hasMatchingHandle( String handle ) {
	return DO.hasMatchingHandle( handle );
    }


    /**
     * for debugging
     */
    public String toString() {
	if ( null == DO )
		return "NULL";
	return DO.toString();
    }

   /**
    * Get Name of the ProjectDO
    *
    * @return Name of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getName () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getName ();
   }

   
   /**
    * Set Name of the ProjectDO
    *
    * @param Name of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setName ( String Name ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setName ( Name );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get CompanyId of the ProjectDO
    *
    * @return CompanyId of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public timesheet.data.CompanyDO getCompanyId () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getCompanyId ();
   }

   
   /**
    * Set CompanyId of the ProjectDO
    *
    * @param CompanyId of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setCompanyId ( timesheet.data.CompanyDO CompanyId ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setCompanyId ( CompanyId );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get Status of the ProjectDO
    *
    * @return Status of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getStatus () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getStatus ();
   }

   
   /**
    * Set Status of the ProjectDO
    *
    * @param Status of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setStatus ( String Status ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setStatus ( Status );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get OutstandingWorkState of the ProjectDO
    *
    * @return OutstandingWorkState of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public boolean getOutstandingWorkState () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getOutstandingWorkState ();
   }

   
   /**
    * Set OutstandingWorkState of the ProjectDO
    *
    * @param OutstandingWorkState of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setOutstandingWorkState ( boolean OutstandingWorkState ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setOutstandingWorkState ( OutstandingWorkState );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get DevelopmentURL of the ProjectDO
    *
    * @return DevelopmentURL of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getDevelopmentURL () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getDevelopmentURL ();
   }

   
   /**
    * Set DevelopmentURL of the ProjectDO
    *
    * @param DevelopmentURL of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setDevelopmentURL ( String DevelopmentURL ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setDevelopmentURL ( DevelopmentURL );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get ProductionURL of the ProjectDO
    *
    * @return ProductionURL of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getProductionURL () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getProductionURL ();
   }

   
   /**
    * Set ProductionURL of the ProjectDO
    *
    * @param ProductionURL of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setProductionURL ( String ProductionURL ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setProductionURL ( ProductionURL );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get CurrentVersion of the ProjectDO
    *
    * @return CurrentVersion of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getCurrentVersion () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getCurrentVersion ();
   }

   
   /**
    * Set CurrentVersion of the ProjectDO
    *
    * @param CurrentVersion of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setCurrentVersion ( String CurrentVersion ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setCurrentVersion ( CurrentVersion );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get AdditionalContactDetails of the ProjectDO
    *
    * @return AdditionalContactDetails of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getAdditionalContactDetails () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getAdditionalContactDetails ();
   }

   
   /**
    * Set AdditionalContactDetails of the ProjectDO
    *
    * @param AdditionalContactDetails of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setAdditionalContactDetails ( String AdditionalContactDetails ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setAdditionalContactDetails ( AdditionalContactDetails );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get DevelopmentDirectory of the ProjectDO
    *
    * @return DevelopmentDirectory of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getDevelopmentDirectory () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getDevelopmentDirectory ();
   }

   
   /**
    * Set DevelopmentDirectory of the ProjectDO
    *
    * @param DevelopmentDirectory of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setDevelopmentDirectory ( String DevelopmentDirectory ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setDevelopmentDirectory ( DevelopmentDirectory );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get ProductionDirectory of the ProjectDO
    *
    * @return ProductionDirectory of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getProductionDirectory () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getProductionDirectory ();
   }

   
   /**
    * Set ProductionDirectory of the ProjectDO
    *
    * @param ProductionDirectory of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setProductionDirectory ( String ProductionDirectory ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setProductionDirectory ( ProductionDirectory );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get SuppliedContentDirectory of the ProjectDO
    *
    * @return SuppliedContentDirectory of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getSuppliedContentDirectory () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getSuppliedContentDirectory ();
   }

   
   /**
    * Set SuppliedContentDirectory of the ProjectDO
    *
    * @param SuppliedContentDirectory of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setSuppliedContentDirectory ( String SuppliedContentDirectory ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setSuppliedContentDirectory ( SuppliedContentDirectory );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get BaseDirectory of the ProjectDO
    *
    * @return BaseDirectory of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getBaseDirectory () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getBaseDirectory ();
   }

   
   /**
    * Set BaseDirectory of the ProjectDO
    *
    * @param BaseDirectory of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setBaseDirectory ( String BaseDirectory ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setBaseDirectory ( BaseDirectory );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get PurchaseHistoryFile of the ProjectDO
    *
    * @return PurchaseHistoryFile of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getPurchaseHistoryFile () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getPurchaseHistoryFile ();
   }

   
   /**
    * Set PurchaseHistoryFile of the ProjectDO
    *
    * @param PurchaseHistoryFile of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setPurchaseHistoryFile ( String PurchaseHistoryFile ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setPurchaseHistoryFile ( PurchaseHistoryFile );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get Relationship of the ProjectDO
    *
    * @return Relationship of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getRelationship () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getRelationship ();
   }

   
   /**
    * Set Relationship of the ProjectDO
    *
    * @param Relationship of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setRelationship ( String Relationship ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setRelationship ( Relationship );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get ServiceLevelAgreementFile of the ProjectDO
    *
    * @return ServiceLevelAgreementFile of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getServiceLevelAgreementFile () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getServiceLevelAgreementFile ();
   }

   
   /**
    * Set ServiceLevelAgreementFile of the ProjectDO
    *
    * @param ServiceLevelAgreementFile of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setServiceLevelAgreementFile ( String ServiceLevelAgreementFile ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setServiceLevelAgreementFile ( ServiceLevelAgreementFile );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get OutstandingWorkInformationFile of the ProjectDO
    *
    * @return OutstandingWorkInformationFile of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getOutstandingWorkInformationFile () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getOutstandingWorkInformationFile ();
   }

   
   /**
    * Set OutstandingWorkInformationFile of the ProjectDO
    *
    * @param OutstandingWorkInformationFile of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setOutstandingWorkInformationFile ( String OutstandingWorkInformationFile ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setOutstandingWorkInformationFile ( OutstandingWorkInformationFile );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get NotesFile of the ProjectDO
    *
    * @return NotesFile of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getNotesFile () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getNotesFile ();
   }

   
   /**
    * Set NotesFile of the ProjectDO
    *
    * @param NotesFile of the ProjectDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setNotesFile ( String NotesFile ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setNotesFile ( NotesFile );
      afterAnySet();	// business actions/assertions after data assignment
   }


   



  /**
   * Inserts/Updates the DO into its table.
   *
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void commit() 
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException {
    modifyDO( null, false );
  }

  /**
   * Inserts/Updates the DO into its table.
   * The transaction is likely provided by the commit() method of another BDO
   * whose DO references this DO.
   * 
   * @param dbt The transaction object to use for this operation.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void commit(DBTransaction dbt)
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException {
    modifyDO( dbt, false );
  }

  /**
   * Deletes the DO from its table.
   *
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void delete() 
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException {
    modifyDO( null, true );
  }

  /**
   * Deletes the DO from its table.
   * The transaction is likely provided by the delete() method of another BDO
   * whose DO references this DO.
   *
   * @param dbt The transaction object to use for this operation.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void delete(DBTransaction dbt)
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException {
    modifyDO( dbt, true );
  }

  

  /**
   * Modifies the DO within its table.
   * Performs recursive commit/delete on referenced DOs;
   * all operations occur within a single transaction
   * to allow rollback in the event of error.
   * Only the creator of the transaction releases it.
   *
   * @param dbt The transaction object to use for this operation.
   * @param delete True if doing a delete, otherwise doing insert/update.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  protected void modifyDO( DBTransaction dbt, boolean delete )
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException {
    boolean ownTransaction = false;
    try {
      if ( null == dbt ) {
	DatabaseManager dbm = Enhydra.getDatabaseManager();
        dbt = dbm.createTransaction();      // create a transaction
	ownTransaction = true;
      }
      if ( null == dbt )
	throw new DatabaseManagerException(
		"DatabaseManager.createTransaction returned null." );
      if ( delete ) {
	  
      }
      
      if ( false ) throw new DataObjectException( "FIX" );
      if ( delete ) {
//System.err.println("deleting DO ="+DO );
          dbt.delete( DO );
      } else {
	  if ( DO.isLoaded() )
	      dbt.insert( DO );   // dbt.insert() handles insertions and updates
      }
      if (ownTransaction) {
//System.err.println("\nDBT="+dbt);
	  dbt.commit(); // commit the transaction
      }
    } catch (DataObjectException doe) {
      throw doe;
    } catch (SQLException sqle) {
      StringBuffer message = new StringBuffer("Failed to insert/update DO: ");
      message.append(sqle.getMessage());

      // rollback, if necessary
      if (ownTransaction) {
        try {
          dbt.rollback();
        } catch (SQLException sqle2) {
          message.insert(0,"\n");
          message.insert(0,sqle2.getMessage());
          message.insert(0,"Rollback failed: ");
        }
      }
      throw new SQLException(message.toString());
    } finally {
      // release the transaction, if any
      if (ownTransaction) {
        dbt.release();
      }
    }
  }

}
