/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * ../../timesheet/data/ProjectQuery.java
 *-----------------------------------------------------------------------------
 */

package timesheet.data;

import com.lutris.dods.builder.sourceGenerators.Query.*;

//import com.lutris.business.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.session.*;

import java.sql.*;
import java.util.*;
import java.util.Date;  // when I say Date, I don't mean java.sql.Date

/**
 * ProjectQuery is used to query the t_project table in the database.<BR>
 * It returns objects of type ProjectDO.
 * <P>
 * General usage:
 * <P>
 *     In DODS:
 *        Create a Data Object named "Dog",
 *        and create an Attribute named "Name",
 *        and set that Attribute to "Can be queried." 
 *        DODS will then generate the method DogQuery.setQueryName(). 
 * <P>
 *     In your Business Layer, prepare the query:<BR>
 * <P><PRE>
 *             DogQuery dq = new DogQuery();
 *             dq.setQueryName("Rex") 
 *             if ( Rex is a reserved name )
 *                 dq.requireUniqueInstance();
 * </PRE>
 *     Then, get the query results one of two ways:
 * <P>
 *         #1:<PRE>
 *             String names = "";
 *             DogBDO[] dogs = dq.getBDOArray();
 *             for ( int i = 0; i < dogs.length; i++ ) {
 *                 names += dogs[i].getName() + " ";
 *             }
 * </PRE>
 *      or #2:<PRE>
 *             String names = "";
 *             DogBDO dog;
 *             while ( null != ( dog = dq.getNextBDO() ) ) {
 *                 names += dog.getName() + " ";
 *             }
 * </PRE>
 *     Note:   Methods <CODE>getDOArray()</CODE> and <CODE>getNextDO()</CODE>
 *             do exist, but they are not generally used
 *             in the Business or Presentation layers of an application.
 *             <B>All access to the Data Layer (DO classes) should occur via
 *             the Business Layer (BDO classes).
 *             Only the Business Layer (BDO classes and classes extending them)
 *             should need to manipulate the Data Layer (DO classes).</B>
 *             See also the comments in the BDO constructors.
 * <P>
 *     Note:   If <CODE>requireUniqueInstance()</CODE> was called,
 *             then <CODE>getBDOArray()</CODE> or <CODE>getNextBDO()</CODE>
 *             will throw an exception if more than one "Rex" was found.
 * <P>
 *     Note:   Results of the query will come from the Data Object cache if:
 *             -  The cache is available.
 *             -  Matches were found in the cache.
 *             -  No other tables (Data Objects of other types) were involved
 *                in the query.
 *                This can happen if you extend the <CODE>DogQuery</CODE> class
 *                and you make calls to the <CODE>QueryBuilder</CODE> object
 *                to add SQL involving other tables.
 *             If any of these conditions is not true,
 *             then any results from the query will come from the database.
 * <P>
 *     To reuse the query object, call:
 * <P><PRE>
 *             dq.reset();
 * </PRE>
 * @author dwood
 * @version $Revision: 1.12 $
 */
final public class ProjectQuery implements Query {

    private QueryBuilder builder;

    /**
     * Public constructor.
     */
    public ProjectQuery() { 
	builder = new QueryBuilder( "t_project", "t_project.*" );
	builder.setDatabaseVendor( "PostgreSQL" );
	builder.setStringMatchDetails( "LIKE", "%" );
	reset();
    }


/**
 * Public constructor.  This version can be used to create
 * arbitrary queries on the table.  Note that the SELECT &
 * FROM clauses are still set as per the normal constructor
 * above.  Do not use the setXXX methods with this constructor.
 * Using requireUniqueInstance is ok.  Use get(B)DOArray/getNext(B)DO
 * as normal.
 *
 * Created by Chris Ryan (cryan@plugged.net.au)
 *
 * @param where_clause  Arbitrary where clause for the query. This string should contain everything required after the 'SELECT ... FROM ... ' clauses. 
 */
    public ProjectQuery(String where_clause) { 

	builder = new QueryBuilder( "t_project", "t_project.*" );
	builder.setDatabaseVendor( "PostgreSQL" );
	builder.setStringMatchDetails( "LIKE", "%" );
	reset();

    // caching is a bit broken - ensure the cache is cleared completely
    cacheHits = new Vector();
    hitDatabase();

  builder.addEndClause(where_clause);
    }


/**
 * Set the WHERE clause for an arbitrary queries on the table.  
 * Note that the SELECT & FROM clauses are still set as normal.
 *
 * Created by Chris Ryan (cryan@plugged.net.au)
 *
 * @param where_clause  Arbitrary where clause for the query. This string should contain everything required after the 'SELECT ... FROM ... ' clauses. 
 */
  public void setArbitraryWhereClause(String where_clause) {

    // clear any residual query setup
    reset();

    // caching is a bit broken - ensure the cache is cleared completely
    cacheHits = new Vector();
    hitDatabase();

    // add the clause.
    builder.addEndClause(where_clause);

  }

    private ResultSet		resultSet	= null;
    private boolean 		uniqueInstance	= false;
    private ProjectDO[]	DOs		= null;
    private int			arrayIndex	= -1;
    private boolean		needToRun	= true;
    private Vector		cacheHits	= null;

    private boolean partialCache = false;
    private boolean hitDb = true;
    public void hitDatabase() { hitDb = true; }

    /**
     * Perform the query on the database, and prepare the 
     * array of returned DO objects.
     *
     * @exception DataObjectException If a database access error occurs. 
     * @exception NonUniqueQueryException If too many rows were found. 
     */
    private void runQuery()
    throws DataObjectException, NonUniqueQueryException {
	needToRun = false;
	arrayIndex = -1;

	DBQuery dbQuery = null;
	try {
	    Vector results;
	    /*
	    if ( cacheHits.size() > 0 ) {
		// The setQuery methods build up the cacheHits.
		// If the cache had our desired objects, 
		// we don't query the database, so we have no ResultSet.
		results = cacheHits;	 // executeQuery saw cache hits
	    } else {
		// If the cache doesn't exist, or could not handle the query,
		// then we actually query the database.
		dbQuery = Enhydra.getDatabaseManager().createQuery();
		dbQuery.query( this );    // invokes executeQuery
	        results = new Vector();
	        while ( resultSet.next() ) {
		    results.addElement( new ProjectDO ( resultSet ) );
	        }
	    }
	    */
	    if ( partialCache && 0 == cacheHits.size() )
		hitDb = true;
	    if ( hitDb ) {
		dbQuery = Enhydra.getDatabaseManager().createQuery();
		dbQuery.query( this );    // invokes executeQuery
	        results = new Vector();
	        while ( resultSet.next() ) {
		    results.addElement( new ProjectDO ( resultSet ) );
	        }
	    } else {
		results = cacheHits;	 // executeQuery saw cache hits
	    }

	    if ( results.size() > 1 && uniqueInstance )
		throw new NonUniqueQueryException( 
		    "Too many rows returned from database" );
	    DOs = new ProjectDO [ results.size() ];
	    for ( int i = 0; i < results.size(); i++ ) {
		DOs[i] = ( ProjectDO ) results.elementAt( i );
	    }
	    arrayIndex = 0;
	} catch ( SQLException se ) {
	    if (null == se.getSQLState() ) {
		throw new DataObjectException( 
		    "Unknown SQLException", se );
	    }
	    if (	se.getSQLState().startsWith("02") &&
			se.getErrorCode() == 100 ) {
		throw new DataObjectException( 
		    "Update or delete DO is out of synch", se );
	    } else if (	se.getSQLState().equals("S1000") &&
			se.getErrorCode() == -268 ) {
		throw new DataObjectException( 
		    "Integrity constraint violation", se );
	    } else {
		throw new DataObjectException( "Data Object Error", se );
	    }
	} catch ( ObjectIdException oe ) { 
	    throw new DataObjectException( "Object ID Error", oe );
	} catch ( DatabaseManagerException de ) {
	    throw new DataObjectException( "Database connection Error", de );
	} finally {
	    if ( null != dbQuery )
		dbQuery.release();
	}
    }


    /**
     * Return array of DOs constructed from ResultSet returned by query.
     * @exception DataObjectException If a database access error occurs. 
     * @exception NonUniqueQueryException If too many rows were found. 
     */
    public ProjectDO[] getDOArray()
    throws DataObjectException, NonUniqueQueryException {
	if ( needToRun )
	    runQuery();
	return DOs;
    }

    /**
     * Return successive DOs from array built from ResultSet returned by query.
     * @exception DataObjectException If a database access error occurs. 
     * @exception NonUniqueQueryException If too many rows were found. 
     */
    public ProjectDO getNextDO()
    throws DataObjectException, NonUniqueQueryException {
	if ( needToRun )
	    runQuery();
	if ( null == DOs ) {
	    /* This should never happen.  
	     * runQuery() should either throw an exception
	     * or create an array of DOs, possibly of zero length.
	     */
	    return null;
	}
	if ( arrayIndex < DOs.length )
	    return DOs[ arrayIndex++ ];
	return null;
    }

    /**
     * Return array of BDOs constructed from ResultSet returned by query.
     * @exception DataObjectException If a database access error occurs. 
     * @exception NonUniqueQueryException If too many rows were found. 
     */
    public ProjectBDO[] getBDOArray()
    throws DataObjectException, NonUniqueQueryException {
	if ( needToRun )
	    runQuery();
	ProjectBDO[] BDOs = new ProjectBDO[ DOs.length ];
	for ( int i = 0; i < DOs.length; i++ )
	    BDOs[ i ] = ProjectBDO.createExisting( DOs[ i ] );
	return BDOs;
    }

    /**
     * Return successive BDOs from array built from ResultSet returned by query.
     * @exception DataObjectException If a database access error occurs. 
     * @exception NonUniqueQueryException If too many rows were found. 
     */
    public ProjectBDO getNextBDO()
    throws DataObjectException, NonUniqueQueryException {
	ProjectDO DO = getNextDO();
	if ( null == DO )
	    return null;
	return ProjectBDO.createExisting( DO );
    }


    /**
     * Set the OID to query.
     * WARNING!  This method assumes that table <CODE>t_project</CODE>
     * has a column named <CODE>"objectID"</CODE>.
     * This method is called from the DO classes to retrieve an object by id.
     *
     * @param oid The object id to query.
     */
    public void setQueryOId(ObjectId oid) {
        // Remove from cacheHits any DOs that do not meet this
        // setQuery requirement.
	if ( null == oid ) 
	    return;
	requireUniqueInstance();
        for ( int i = 0; i < cacheHits.size(); i++ ) {
            ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
            if ( null == DO ) continue;
            boolean equals = true;
	    ObjectId id = DO.getOId();
	    if ( null == id || ! id.equals( oid ) )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database
        // in case there is no cache, or the query involves other tables.
	builder.addWhereClause( "objectID",  oid.toBigDecimal(),
						QueryBuilder.NOT_NULL );
    }


    /**
     * Set "unique instance" assertion bit.
     * The first call to the next() method will throw an exception 
     * if more than one object was found.
     */
    public void requireUniqueInstance()
    {
	uniqueInstance = true;
    }

    /**
     * Reset the query parameters.
     */
    public void reset() {
	cacheHits	= new Vector();
	DOs		= null;
	uniqueInstance	= false;
	needToRun	= true;
        builder.reset();
    }

    /**
     * Return the appropriate QueryBuilder flag for selecting
     * exact matches (SQL '=') or inexact matches (SQL 'matches').
     */
    private boolean exactFlag( boolean exact ) {
        return exact ? QueryBuilder.EXACT_MATCH : QueryBuilder.NOT_EXACT;
    }



    //
    // Implementation of Query interface
    //

    /**
     * Method to query objects from the database.
     * The following call in runQuery()
     *	    dbQuery.query( this );
     * causes the dbQuery object to invoke 
     *      executeQuery()
     *
     * @param conn Handle to database connection.
     * @exception java.sql.SQLException If a database access error occurs.
     */
    public ResultSet executeQuery(DBConnection conn)
        throws SQLException
    {
	resultSet = builder.executeQuery( conn );
        return resultSet;
    }
 
 
    /**
     * WARNING!  This method is disabled.
     * It's implementation is forced by the Query interface.
     * This method is disabled for 2 reasons:
     * 1)  the getDOArray() and getNextDO() methods are better
     *     because they return DOs instead of JDBC objects.
     * 2)  the createExisting() method throws an exception
     *     that we cannot reasonably handle here,
     *     and that we cannot throw from here.
     *
     * @param rs JDBC result set from which the next object
     *   will be instantiated.
     * @exception java.sql.SQLException
     *   If a database access error occurs.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an invalid object id was queried from the database.
     */
    public Object next(ResultSet rs)
    throws SQLException, ObjectIdException {
	/*
        if (!rs.next())
            return null;
 
        return ProjectDO.createExisting(rs);
	*/
	throw new ObjectIdException( 
	    "next() should not be used.  Use getNextDO() instead." );
	//return null;
    }


    /**
     * Set the Name to query.
     *
     * @param x The Name of the t_project to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryName(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getName();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "Name", x, "VARCHAR",
                QueryBuilder.NOT_NULL, exactFlag( exact ) );
    }

    /**
     * Set the Name to query
     * @param x The Name of the t_project to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryName( 
				String x )
    throws DataObjectException {
	setQueryName( x, true );
    }

    /**
     * Add Name to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByName(boolean direction_flag) {
        builder.addOrderByColumn("Name",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add Name to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByName() {
        builder.addOrderByColumn("Name","ASC");
    }


    /**
     * Set the CompanyId to query.
     *
     * @param x The CompanyId of the t_project to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryCompanyId(
				timesheet.data.CompanyDO x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		timesheet.data.CompanyDO y = DO.getCompanyId();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = (x == y);

      } else {

        equals = (x == y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "CompanyId", x, "INT8",
                QueryBuilder.NOT_NULL, exactFlag( exact ) );
    }

    /**
     * Set the CompanyId to query
     * @param x The CompanyId of the t_project to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryCompanyId( 
				timesheet.data.CompanyDO x )
    throws DataObjectException {
	setQueryCompanyId( x, true );
    }

    /**
     * Add CompanyId to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByCompanyId(boolean direction_flag) {
        builder.addOrderByColumn("CompanyId",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add CompanyId to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByCompanyId() {
        builder.addOrderByColumn("CompanyId","ASC");
    }


    /**
     * Set the Status to query.
     *
     * @param x The Status of the t_project to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryStatus(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getStatus();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "Status", x, "VARCHAR",
                QueryBuilder.NOT_NULL, exactFlag( exact ) );
    }

    /**
     * Set the Status to query
     * @param x The Status of the t_project to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryStatus( 
				String x )
    throws DataObjectException {
	setQueryStatus( x, true );
    }

    /**
     * Add Status to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByStatus(boolean direction_flag) {
        builder.addOrderByColumn("Status",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add Status to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByStatus() {
        builder.addOrderByColumn("Status","ASC");
    }


    /**
     * Set the OutstandingWorkState to query.
     *
     * @param x The OutstandingWorkState of the t_project to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryOutstandingWorkState(
				boolean x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		boolean y = DO.getOutstandingWorkState();


    
    if (exact) {

      equals = (x == y);

    } else {

      equals = (x == y);

    }
    

	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "OutstandingWorkState", x, "BOOL",
                QueryBuilder.NOT_NULL, exactFlag( exact ) );
    }

    /**
     * Set the OutstandingWorkState to query
     * @param x The OutstandingWorkState of the t_project to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryOutstandingWorkState( 
				boolean x )
    throws DataObjectException {
	setQueryOutstandingWorkState( x, true );
    }

    /**
     * Add OutstandingWorkState to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByOutstandingWorkState(boolean direction_flag) {
        builder.addOrderByColumn("OutstandingWorkState",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add OutstandingWorkState to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByOutstandingWorkState() {
        builder.addOrderByColumn("OutstandingWorkState","ASC");
    }


    /**
     * Set the DevelopmentURL to query.
     *
     * @param x The DevelopmentURL of the t_project to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryDevelopmentURL(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getDevelopmentURL();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "DevelopmentURL", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the DevelopmentURL to query
     * @param x The DevelopmentURL of the t_project to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryDevelopmentURL( 
				String x )
    throws DataObjectException {
	setQueryDevelopmentURL( x, true );
    }

    /**
     * Add DevelopmentURL to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByDevelopmentURL(boolean direction_flag) {
        builder.addOrderByColumn("DevelopmentURL",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add DevelopmentURL to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByDevelopmentURL() {
        builder.addOrderByColumn("DevelopmentURL","ASC");
    }


    /**
     * Set the ProductionURL to query.
     *
     * @param x The ProductionURL of the t_project to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryProductionURL(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getProductionURL();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "ProductionURL", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the ProductionURL to query
     * @param x The ProductionURL of the t_project to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryProductionURL( 
				String x )
    throws DataObjectException {
	setQueryProductionURL( x, true );
    }

    /**
     * Add ProductionURL to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByProductionURL(boolean direction_flag) {
        builder.addOrderByColumn("ProductionURL",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add ProductionURL to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByProductionURL() {
        builder.addOrderByColumn("ProductionURL","ASC");
    }


    /**
     * Set the CurrentVersion to query.
     *
     * @param x The CurrentVersion of the t_project to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryCurrentVersion(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getCurrentVersion();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "CurrentVersion", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the CurrentVersion to query
     * @param x The CurrentVersion of the t_project to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryCurrentVersion( 
				String x )
    throws DataObjectException {
	setQueryCurrentVersion( x, true );
    }

    /**
     * Add CurrentVersion to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByCurrentVersion(boolean direction_flag) {
        builder.addOrderByColumn("CurrentVersion",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add CurrentVersion to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByCurrentVersion() {
        builder.addOrderByColumn("CurrentVersion","ASC");
    }


    /**
     * Set the AdditionalContactDetails to query.
     *
     * @param x The AdditionalContactDetails of the t_project to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryAdditionalContactDetails(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getAdditionalContactDetails();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "AdditionalContactDetails", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the AdditionalContactDetails to query
     * @param x The AdditionalContactDetails of the t_project to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryAdditionalContactDetails( 
				String x )
    throws DataObjectException {
	setQueryAdditionalContactDetails( x, true );
    }

    /**
     * Add AdditionalContactDetails to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByAdditionalContactDetails(boolean direction_flag) {
        builder.addOrderByColumn("AdditionalContactDetails",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add AdditionalContactDetails to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByAdditionalContactDetails() {
        builder.addOrderByColumn("AdditionalContactDetails","ASC");
    }


    /**
     * Set the DevelopmentDirectory to query.
     *
     * @param x The DevelopmentDirectory of the t_project to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryDevelopmentDirectory(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getDevelopmentDirectory();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "DevelopmentDirectory", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the DevelopmentDirectory to query
     * @param x The DevelopmentDirectory of the t_project to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryDevelopmentDirectory( 
				String x )
    throws DataObjectException {
	setQueryDevelopmentDirectory( x, true );
    }

    /**
     * Add DevelopmentDirectory to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByDevelopmentDirectory(boolean direction_flag) {
        builder.addOrderByColumn("DevelopmentDirectory",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add DevelopmentDirectory to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByDevelopmentDirectory() {
        builder.addOrderByColumn("DevelopmentDirectory","ASC");
    }


    /**
     * Set the ProductionDirectory to query.
     *
     * @param x The ProductionDirectory of the t_project to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryProductionDirectory(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getProductionDirectory();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "ProductionDirectory", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the ProductionDirectory to query
     * @param x The ProductionDirectory of the t_project to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryProductionDirectory( 
				String x )
    throws DataObjectException {
	setQueryProductionDirectory( x, true );
    }

    /**
     * Add ProductionDirectory to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByProductionDirectory(boolean direction_flag) {
        builder.addOrderByColumn("ProductionDirectory",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add ProductionDirectory to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByProductionDirectory() {
        builder.addOrderByColumn("ProductionDirectory","ASC");
    }


    /**
     * Set the SuppliedContentDirectory to query.
     *
     * @param x The SuppliedContentDirectory of the t_project to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQuerySuppliedContentDirectory(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getSuppliedContentDirectory();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "SuppliedContentDirectory", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the SuppliedContentDirectory to query
     * @param x The SuppliedContentDirectory of the t_project to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQuerySuppliedContentDirectory( 
				String x )
    throws DataObjectException {
	setQuerySuppliedContentDirectory( x, true );
    }

    /**
     * Add SuppliedContentDirectory to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderBySuppliedContentDirectory(boolean direction_flag) {
        builder.addOrderByColumn("SuppliedContentDirectory",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add SuppliedContentDirectory to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderBySuppliedContentDirectory() {
        builder.addOrderByColumn("SuppliedContentDirectory","ASC");
    }


    /**
     * Set the BaseDirectory to query.
     *
     * @param x The BaseDirectory of the t_project to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryBaseDirectory(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getBaseDirectory();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "BaseDirectory", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the BaseDirectory to query
     * @param x The BaseDirectory of the t_project to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryBaseDirectory( 
				String x )
    throws DataObjectException {
	setQueryBaseDirectory( x, true );
    }

    /**
     * Add BaseDirectory to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByBaseDirectory(boolean direction_flag) {
        builder.addOrderByColumn("BaseDirectory",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add BaseDirectory to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByBaseDirectory() {
        builder.addOrderByColumn("BaseDirectory","ASC");
    }


    /**
     * Set the PurchaseHistoryFile to query.
     *
     * @param x The PurchaseHistoryFile of the t_project to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPurchaseHistoryFile(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getPurchaseHistoryFile();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "PurchaseHistoryFile", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the PurchaseHistoryFile to query
     * @param x The PurchaseHistoryFile of the t_project to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryPurchaseHistoryFile( 
				String x )
    throws DataObjectException {
	setQueryPurchaseHistoryFile( x, true );
    }

    /**
     * Add PurchaseHistoryFile to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByPurchaseHistoryFile(boolean direction_flag) {
        builder.addOrderByColumn("PurchaseHistoryFile",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add PurchaseHistoryFile to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByPurchaseHistoryFile() {
        builder.addOrderByColumn("PurchaseHistoryFile","ASC");
    }


    /**
     * Set the Relationship to query.
     *
     * @param x The Relationship of the t_project to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryRelationship(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getRelationship();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "Relationship", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the Relationship to query
     * @param x The Relationship of the t_project to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryRelationship( 
				String x )
    throws DataObjectException {
	setQueryRelationship( x, true );
    }

    /**
     * Add Relationship to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByRelationship(boolean direction_flag) {
        builder.addOrderByColumn("Relationship",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add Relationship to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByRelationship() {
        builder.addOrderByColumn("Relationship","ASC");
    }


    /**
     * Set the ServiceLevelAgreementFile to query.
     *
     * @param x The ServiceLevelAgreementFile of the t_project to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryServiceLevelAgreementFile(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getServiceLevelAgreementFile();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "ServiceLevelAgreementFile", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the ServiceLevelAgreementFile to query
     * @param x The ServiceLevelAgreementFile of the t_project to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryServiceLevelAgreementFile( 
				String x )
    throws DataObjectException {
	setQueryServiceLevelAgreementFile( x, true );
    }

    /**
     * Add ServiceLevelAgreementFile to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByServiceLevelAgreementFile(boolean direction_flag) {
        builder.addOrderByColumn("ServiceLevelAgreementFile",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add ServiceLevelAgreementFile to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByServiceLevelAgreementFile() {
        builder.addOrderByColumn("ServiceLevelAgreementFile","ASC");
    }


    /**
     * Set the OutstandingWorkInformationFile to query.
     *
     * @param x The OutstandingWorkInformationFile of the t_project to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryOutstandingWorkInformationFile(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getOutstandingWorkInformationFile();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "OutstandingWorkInformationFile", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the OutstandingWorkInformationFile to query
     * @param x The OutstandingWorkInformationFile of the t_project to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryOutstandingWorkInformationFile( 
				String x )
    throws DataObjectException {
	setQueryOutstandingWorkInformationFile( x, true );
    }

    /**
     * Add OutstandingWorkInformationFile to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByOutstandingWorkInformationFile(boolean direction_flag) {
        builder.addOrderByColumn("OutstandingWorkInformationFile",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add OutstandingWorkInformationFile to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByOutstandingWorkInformationFile() {
        builder.addOrderByColumn("OutstandingWorkInformationFile","ASC");
    }


    /**
     * Set the NotesFile to query.
     *
     * @param x The NotesFile of the t_project to query.
     * @param exact to use matches or not
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryNotesFile(
				String x, boolean exact)
    throws DataObjectException
    {
	// Remove from cacheHits any DOs that do not meet this
	// setQuery requirement.
	for ( int i = 0; i < cacheHits.size() && ! hitDb; i++ ) {
	    ProjectDO DO = ( ProjectDO ) cacheHits.elementAt( i );
	    if ( null == DO ) continue;
	    boolean equals = true;
	    
		String y = DO.getNotesFile();

    
    if (x == null && y == null) {

      equals = true;

    } else if (x == null || y == null) {

      equals = false;

    } else {

      if (exact) {

        equals = x.equals(y);

      } else {

        equals = x.equalsIgnoreCase(y);

      }

    }
    


	    if ( ! equals )
		cacheHits.removeElementAt( i-- );
	}

	// Also prepare the SQL needed to query the database 
	// in case there is no cache, or the query involves other tables.
	if ( partialCache || hitDb )
	    builder.addWhereClause( "NotesFile", x, "VARCHAR",
                QueryBuilder.NULL_OK, exactFlag( exact ) );
    }

    /**
     * Set the NotesFile to query
     * @param x The NotesFile of the t_project to query.
     * @exception DataObjectException If a database access error occurs.
     */
    public void setQueryNotesFile( 
				String x )
    throws DataObjectException {
	setQueryNotesFile( x, true );
    }

    /**
     * Add NotesFile to the ORDER BY clause.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderByNotesFile(boolean direction_flag) {
        builder.addOrderByColumn("NotesFile",
					(direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add NotesFile to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     */
    public void addOrderByNotesFile() {
        builder.addOrderByColumn("NotesFile","ASC");
    }

    /**
    * Returns the <code>QueryBuilder</code> object that this class uses to
    * construct and execute database queries.
    * <code>ProjectQuery.set</code> methods use the <code>QueryBuilder</code> to
    * append SQL expressions to the <code>"where"</code> clause to be executed.
    * The <code>QueryBuilder.addEndClause method.</code> can be used to
    * append freeform SQL expressions to the <code>where</code> clause,
    * e.g. "order by name".
    */
    public QueryBuilder getQueryBuilder() {
	return builder;
    }
}
