/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * ../../timesheet/data/CompanyBDO.java
 *-----------------------------------------------------------------------------
 */

package timesheet.data;

import java.sql.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;

import com.lutris.dods.builder.sourceGenerators.Query.DataObjectException;
import com.lutris.dods.builder.sourceGenerators.Query.NonUniqueQueryException;
import com.lutris.dods.builder.sourceGenerators.Query.RefAssertionException;

/**
 * CompanyBDO contains the same set and get methods as
 * the CompanyDO class.
 * Business Object (BO) classes typically need these set and get methods.
 * So by deriving a BO from a BDO, or by implementing a BO that 
 * contains a BDO, the developer of the BO is spared some work.
 *
 * @author dwood
 * @version $Revision: 1.12.2.1 $
 */
public class CompanyBDO {

    /**
     * The CompanyDO object upon which the set and get methods operate.
     * This member is protected so that classes derived from CompanyBDO
     * can access the underlying Data Object.
     */
    protected CompanyDO DO;

    /**
     * Note:  This method is intended for use only by other BDO classes.
     * Presentation Layer classes should (theoretically) always use the
     * Business Layer (BDO) to create/access Data Layer (DO) objects.
     * The overhead for using BDO objects is small
     * (the BDO classes are fairly lightweight.)
     *
     * @return The DO object held by this BDO object.
     */
    public CompanyDO getDO() { 
	return DO;
    }

    /**
     * Like the class <CODE>CompanyDO</CODE>, 
     * this class acts as a factory.
     * Business Object (BO) classes typically need these set and get methods.
     * So by deriving a BO from a BDO, or by implementing a BO that 
     * contains one or more BDOs, the developer of the BO is spared some work.
     *
     * @exception Exception
     *   If an error occurs.
     */
    public static CompanyBDO createVirgin() throws Exception { 
	CompanyBDO bdo = new CompanyBDO ();
	return bdo;
    }

    /**
     * Constructor for use by classes derived from <CODE>CompanyBDO</CODE>.
     * Example usage:<CODE>
     *      class CustomerBO extends CustomerBDO {
     *          // a BDO class is commonly extended in order to implement:
     *          public void beforeAnySet() throws Exception {
     *              if ( CustomerDO should not be altered )
     *                  throw new ApplicationException( "ERROR" );
     *          }
     *          public CustomerBO( CustomerDO DO ) {
     *              super( DO );
     *          }
     *      }
     *      class SomePresentationLayerClass {
     *          public CustomerBO findCustomer( String name ) {
     *              CustomerQuery q = new CustomerQuery();
     *              q.setQueryName( name );
     *              CustomerDO DO = q.getNextDO();
     *              // Here the CustomerBO ctor fires the CustomerBDO ctor.
     *              return new CustomerBO( DO );
     *          }
     *      }
     * </CODE>
     */
    public CompanyBDO( CompanyDO DO ) { 
	this.DO = DO;
    }

    /**
     * Constructor required by <CODE>CompanyBDO.create</CODE> methods.
     */
    public CompanyBDO() throws Exception { 
	this.DO = CompanyDO.createVirgin();
    }

    /**
     * The createExisting method is used to create a <CODE>CompanyBDO</CODE>
     * from a <CODE>CompanyDO</CODE> that was returned by 
     * the <CODE>CompanyQuery</CODE> class.
     */
    public static CompanyBDO createExisting( CompanyDO DO ) { 
	CompanyBDO bdo = new CompanyBDO ( DO );
	return bdo;
    }

    /** 
     * The getBDOarray method performs a database query to
     * return an array of <CODE>CompanyBDO</CODE> objects
     * representing all the rows in the <CODE>t_company</CODE> table.
     *
     * This method is a minimal example of using a Query class.
     * To restrict the set of objects returned,  you could
     * invoke <CODE>query.setXxx()</CODE>, where <CODE>Xxx</CODE>
     * is an Attribute of <CODE>CompanyDO</CODE> which was 
     * marked as "Can be queried" in the DODS Attribute Editor.
     *
     * @exception DataObjectException
     *   If an object is not found in the database.
     */
    public static CompanyBDO[] getBDOarray()
    throws DataObjectException {
        CompanyDO[] DOarray = null;
        try {
            CompanyQuery query = new CompanyQuery();
	    // To restrict the set of objects returned,
	    // you could invoke query.setXxx(), where Xxx
	    // is an Attribute of <CODE>CompanyDO</CODE> which was 
	    // marked as "Can be queried" in the DODS Attribute Editor.
            DOarray = query.getDOArray();
        } catch ( NonUniqueQueryException e1 ) {
            // CompanyQuery will throw NonUniqueQueryException
            // only if query.requireUniqueInstance() is called
	    // and more than one DO was found.
        } catch ( DataObjectException e2 ) {
            // This could happen if the database server dies, etc.
            throw e2;
        }
        CompanyBDO[] BDOarray = new CompanyBDO[ DOarray.length ];
        for ( int i = 0; i < DOarray.length; i++ )
            BDOarray[i] = CompanyBDO.createExisting( DOarray[i] );
 
        return BDOarray;
    }


    /**
     * The developer of a Business Object that derives from this class
     * can override the methods:<CODE>
     *     beforeAnyGet
     *     beforeAnySet
     *     afterAnySet
     * </CODE> to handle any general assertions or cleanup needed
     * for <CODE>get</CODE> and <CODE>set</CODE> methods.
     */
    protected void beforeAnyGet() {
    }
    // beforeAnySet throws plain Exception to allow overriding implementations
    // to throw any flavor needed.
    protected void beforeAnySet() throws Exception {
	if ( DO.isReadOnly() )
	    throw new DataObjectException( DO + " is read-only." ); 
    }
    protected void afterAnySet() {
    }

    /**
     * The methods <CODE>
     *     getHandle
     *     hasMatchingHandle
     * </CODE> are used by Presentation Objects that need to populate
     * HTML select lists with <CODE>CompanyBDO</CODE> objects as options.
     *
     * The <CODE>getHandle()</CODE> method is used
     * to set the value for each option,
     * and the <CODE>hasMatchingHandle()<CODE>
     * methods are used to lookup the Data Object when the selection has
     * been made.
     *
     * This CompanyBDO object holds a reference to a CompanyDO object.
     * The id of this CompanyBDO is the id of its CompanyDO.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @return id of this BDO as a string
     *   If an object id can't be allocated for this object.
     */
    public String getHandle()
    throws DatabaseManagerException {
	return DO.getHandle();
    }

    /**
     * @param handle
     *   <CODE>String</CODE> representation of the id for this BDO
     * @return boolean
     *   True if the string version of the id of this DO matches passed handle
     * @see getHandle
     */
    public boolean hasMatchingHandle( String handle ) {
	return DO.hasMatchingHandle( handle );
    }


    /**
     * for debugging
     */
    public String toString() {
	if ( null == DO )
		return "NULL";
	return DO.toString();
    }

   /**
    * Get Name of the CompanyDO
    *
    * @return Name of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getName () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getName ();
   }

   
   /**
    * Set Name of the CompanyDO
    *
    * @param Name of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setName ( String Name ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setName ( Name );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get PhysicalStreet of the CompanyDO
    *
    * @return PhysicalStreet of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getPhysicalStreet () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getPhysicalStreet ();
   }

   
   /**
    * Set PhysicalStreet of the CompanyDO
    *
    * @param PhysicalStreet of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setPhysicalStreet ( String PhysicalStreet ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setPhysicalStreet ( PhysicalStreet );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get PhysicalCity of the CompanyDO
    *
    * @return PhysicalCity of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getPhysicalCity () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getPhysicalCity ();
   }

   
   /**
    * Set PhysicalCity of the CompanyDO
    *
    * @param PhysicalCity of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setPhysicalCity ( String PhysicalCity ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setPhysicalCity ( PhysicalCity );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get PhysicalPostCode of the CompanyDO
    *
    * @return PhysicalPostCode of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getPhysicalPostCode () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getPhysicalPostCode ();
   }

   
   /**
    * Set PhysicalPostCode of the CompanyDO
    *
    * @param PhysicalPostCode of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setPhysicalPostCode ( String PhysicalPostCode ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setPhysicalPostCode ( PhysicalPostCode );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get PhysicalState of the CompanyDO
    *
    * @return PhysicalState of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getPhysicalState () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getPhysicalState ();
   }

   
   /**
    * Set PhysicalState of the CompanyDO
    *
    * @param PhysicalState of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setPhysicalState ( String PhysicalState ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setPhysicalState ( PhysicalState );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get PostalStreet of the CompanyDO
    *
    * @return PostalStreet of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getPostalStreet () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getPostalStreet ();
   }

   
   /**
    * Set PostalStreet of the CompanyDO
    *
    * @param PostalStreet of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setPostalStreet ( String PostalStreet ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setPostalStreet ( PostalStreet );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get PostalCity of the CompanyDO
    *
    * @return PostalCity of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getPostalCity () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getPostalCity ();
   }

   
   /**
    * Set PostalCity of the CompanyDO
    *
    * @param PostalCity of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setPostalCity ( String PostalCity ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setPostalCity ( PostalCity );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get PostalPostCode of the CompanyDO
    *
    * @return PostalPostCode of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getPostalPostCode () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getPostalPostCode ();
   }

   
   /**
    * Set PostalPostCode of the CompanyDO
    *
    * @param PostalPostCode of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setPostalPostCode ( String PostalPostCode ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setPostalPostCode ( PostalPostCode );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get PostalState of the CompanyDO
    *
    * @return PostalState of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getPostalState () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getPostalState ();
   }

   
   /**
    * Set PostalState of the CompanyDO
    *
    * @param PostalState of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setPostalState ( String PostalState ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setPostalState ( PostalState );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get Country of the CompanyDO
    *
    * @return Country of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getCountry () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getCountry ();
   }

   
   /**
    * Set Country of the CompanyDO
    *
    * @param Country of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setCountry ( String Country ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setCountry ( Country );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get Phone of the CompanyDO
    *
    * @return Phone of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getPhone () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getPhone ();
   }

   
   /**
    * Set Phone of the CompanyDO
    *
    * @param Phone of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setPhone ( String Phone ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setPhone ( Phone );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get Fax of the CompanyDO
    *
    * @return Fax of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getFax () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getFax ();
   }

   
   /**
    * Set Fax of the CompanyDO
    *
    * @param Fax of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setFax ( String Fax ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setFax ( Fax );
      afterAnySet();	// business actions/assertions after data assignment
   }


   

   /**
    * Get URL of the CompanyDO
    *
    * @return URL of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getURL () 
   throws DataObjectException {
      beforeAnyGet();	// business actions/assertions prior to data return
      return DO.getURL ();
   }

   
   /**
    * Set URL of the CompanyDO
    *
    * @param URL of the CompanyDO
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setURL ( String URL ) 
   throws DataObjectException {
      try {
	  // business actions/assertions prior to data assignment
	  beforeAnySet();
      } catch ( Exception e ) { 
	  throw new DataObjectException( "beforeAnySet: " + e.getMessage() );
      }
      DO.setURL ( URL );
      afterAnySet();	// business actions/assertions after data assignment
   }


   



  /**
   * Inserts/Updates the DO into its table.
   *
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void commit() 
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException {
    modifyDO( null, false );
  }

  /**
   * Inserts/Updates the DO into its table.
   * The transaction is likely provided by the commit() method of another BDO
   * whose DO references this DO.
   * 
   * @param dbt The transaction object to use for this operation.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void commit(DBTransaction dbt)
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException {
    modifyDO( dbt, false );
  }

  /**
   * Deletes the DO from its table.
   *
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void delete() 
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException {
    modifyDO( null, true );
  }

  /**
   * Deletes the DO from its table.
   * The transaction is likely provided by the delete() method of another BDO
   * whose DO references this DO.
   *
   * @param dbt The transaction object to use for this operation.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void delete(DBTransaction dbt)
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException {
    modifyDO( dbt, true );
  }

  

  /**
   * Modifies the DO within its table.
   * Performs recursive commit/delete on referenced DOs;
   * all operations occur within a single transaction
   * to allow rollback in the event of error.
   * Only the creator of the transaction releases it.
   *
   * @param dbt The transaction object to use for this operation.
   * @param delete True if doing a delete, otherwise doing insert/update.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  protected void modifyDO( DBTransaction dbt, boolean delete )
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException {
    boolean ownTransaction = false;
    try {
      if ( null == dbt ) {
	DatabaseManager dbm = Enhydra.getDatabaseManager();
        dbt = dbm.createTransaction();      // create a transaction
	ownTransaction = true;
      }
      if ( null == dbt )
	throw new DatabaseManagerException(
		"DatabaseManager.createTransaction returned null." );
      if ( delete ) {
	  
      }
      
      if ( false ) throw new DataObjectException( "FIX" );
      if ( delete ) {
//System.err.println("deleting DO ="+DO );
          dbt.delete( DO );
      } else {
	  if ( DO.isLoaded() )
	      dbt.insert( DO );   // dbt.insert() handles insertions and updates
      }
      if (ownTransaction) {
//System.err.println("\nDBT="+dbt);
	  dbt.commit(); // commit the transaction
      }
    } catch (DataObjectException doe) {
      throw doe;
    } catch (SQLException sqle) {
      StringBuffer message = new StringBuffer("Failed to insert/update DO: ");
      message.append(sqle.getMessage());

      // rollback, if necessary
      if (ownTransaction) {
        try {
          dbt.rollback();
        } catch (SQLException sqle2) {
          message.insert(0,"\n");
          message.insert(0,sqle2.getMessage());
          message.insert(0,"Rollback failed: ");
        }
      }
      throw new SQLException(message.toString());
    } finally {
      // release the transaction, if any
      if (ownTransaction) {
        dbt.release();
      }
    }
  }

}
