package timesheet.business;

import com.lutris.appserver.server.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.sql.ObjectId;
import com.lutris.util.*;
import timesheet.*;
import timesheet.data.*;
import java.util.*;
import java.math.*;
import com.lutris.dods.builder.sourceGenerators.Query.*;
import com.pisoftware.platypus.production.*;

/**
 * Date Created: Oct 22, 1999
 * <P>
 * Purpose:  This class is the base BO for the application.
 *           All BOs that related to a PO eventually extend
 *           this class, possibly via subdirectory level BO
 *           classes.
 *
 */
public class TimesheetBO {


  /*
   * The following are the BOs in the system that extend
   * BDOs.  They're the business objects that deal with the
   * DOs in a close manner.  This class ensures that we
   * only create these once per session.
   */
  public CompanyBO companyBO = null;
  public PersonBO personBO = null;
  public ActivityBO activityBO = null;
  public ProjectBO projectBO = null;
  public TimeEntryBO timeEntryBO = null;


  // The KeywordValueTable for the business objects
  protected KeywordValueTable BO_KeywordValueTable = null;
  protected final static String BO_KEYWORD = "business";

  /*
   * The following list is the keyword values for all
   * the BOs listed above.  This will allow us to
   * save and restore those BOs with the sessionData.
   */
  protected final static String BO_COMPANY = "company";
  protected final static String BO_PERSON = "person";
  protected final static String BO_ACTIVITY = "activity";
  protected final static String BO_PROJECT = "project";
  protected final static String BO_TIMEENTRY = "timeentry";



/**
 * Constructor.  Creates the object, and initialises the appropriate variables, etc.
 *
 * @param comms  Used to get the session data required
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public TimesheetBO(HttpPresentationComms comms) throws ApplicationException {


    try {

      // Can we restore any of the BOs from the sessionData?
      BO_KeywordValueTable = comms.sessionData.getSection(BO_KEYWORD);

      if (BO_KeywordValueTable != null) {

        // yes, restore the objects
        companyBO = (CompanyBO) BO_KeywordValueTable.get(BO_COMPANY);
        personBO = (PersonBO) BO_KeywordValueTable.get(BO_PERSON);
        activityBO = (ActivityBO) BO_KeywordValueTable.get(BO_ACTIVITY);
        projectBO = (ProjectBO) BO_KeywordValueTable.get(BO_PROJECT);
        timeEntryBO = (TimeEntryBO) BO_KeywordValueTable.get(BO_TIMEENTRY);


      } else {

        // no, create and save the objects
        companyBO = new CompanyBO();
        comms.sessionData.set(BO_KEYWORD+"."+BO_COMPANY,companyBO);

        personBO = new PersonBO();
        comms.sessionData.set(BO_KEYWORD+"."+BO_PERSON,personBO);

        activityBO = new ActivityBO();
        comms.sessionData.set(BO_KEYWORD+"."+BO_ACTIVITY,activityBO);

        projectBO = new ProjectBO();
        comms.sessionData.set(BO_KEYWORD+"."+BO_PROJECT,projectBO);

        timeEntryBO = new TimeEntryBO();
        comms.sessionData.set(BO_KEYWORD+"."+BO_TIMEENTRY,timeEntryBO);



      }

    } catch (KeywordValueException kve) {

      throw new ApplicationException("Unable to save/restore BOs to/from the sessionData!",kve);

    } catch (Exception e) {

      // creating BOs that extend a BDO could throw an exception
      throw new ApplicationException("Unable to create BO object!",e);

    }



    /*
     * Insert your code here, e.g. User authentication.
     */

  }
    
 
  // cache for static pulldowns
  private static Hashtable pulldowns = new Hashtable();
    
   
/**   
 * Returns the list of pulldown items for the name passed.
 *  
 * @param pulldown_name  The Pulldown name to query for
 * @return an array of PulldownDOs containing the list of values for this pulldown name, null or empty array if none found.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */   
  public PulldownItem[] getPulldownItems(String pulldown_name) throws ApplicationException {
  
    // Have we got this pulldown cached? 
    if (pulldowns.containsKey(pulldown_name)) {
      
      // yes, return it
      return (PulldownItem[]) pulldowns.get(pulldown_name);
  
    }
    
/*
 * Uncomment this code if you've created the Pulldown table in the
 * database.  The structure of this table is:
 *
 * create table t_pulldown {
 *  Name VARCHAR(255) DEFAULT '' NOT NULL,
 *  Value VARCHAR(255) DEFAULT '' NOT NULL,
 *  DisplayName VARCHAR(255) DEFAULT '' NOT NULL,
 *  IsDefaultValue BOOLEAN DEFAULT FALSE NOT NULL,
 *  Position SMALLINT DEFAULT 0 NOT NULL,
 *  ObjectId DECIMAL(19,0) NOT NULL PRIMARY KEY,
 *  ObjectVersion INTEGER NOT NULL
 * };

    // get the static pulldown
    PulldownDO[] pulldownDOs = pulldownBO.getPulldownDOs(pulldown_name);
      
    // any results?
    if (pulldownDOs == null || pulldownDOs.length == 0) {
    
      // no - get out of here
      return null;
    
    }
  
    // go through results and put them into the result array
    PulldownItem[] pi = new PulldownItem[pulldownDOs.length];
  
    try { 
  
      for (int i=0; i<pulldownDOs.length; i++) {

        pi[i] = new PulldownItem(pulldown_name,pulldownDOs[i].getValue(),pulldownDOs[i].getDisplayValue(),pulldownDOs[i].getIsDefaultValue());
 
      }
 
    } catch (DataObjectException doe) {

      throw new ApplicationException("Couldn't get values from the PulldownDO!",doe);

    }

    // store these pulldown items in the cache
    pulldowns.put(pulldown_name,pi);

    // return the items
    return pi;
*/

    // DBG
    throw new ApplicationException("getPulldownItems has not been coded yet!\nI can't get any values for: "+pulldown_name);

  }


/**
 * Gets information from the DO layer for the ID passed.
 * Note that a previous ACCESS method must have been
 * called for this function to work.
 *
 * @param index  The record to use
 * @param id  The id who's value is to be returned.
 * @param format  Determines what formatting is required for the id's value
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getIdValue(int index, String id, String format) throws ApplicationException {

    /*
     * This method will be overridden by subclasses that have
     * an ACCESS method.  This default method is provided to
     * allow the PO layer to compile easily and for debugging
     * purposes.
     */

    // DBG
    return "ID Value passed: "+id+", index is: "+index+", format is: <"+format+">";

  }


/**
 * Gets information from the DO layer for the ID passed.
 * Note that a previous ACCESS method must have been
 * called for this function to work.
 * Just calls the same method with no format info.
 *
 * @param index  The record to use
 * @param id  The id who's value is to be returned.
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getIdValue(int index, String id) throws ApplicationException {

    return getIdValue(index, id, "");

  }


/**
 * Gets information from the DO layer for the ID passed.
 *
 * @param oidstr  The database oid for the record
 * @param id  The id who's value is to be returned.
 * @param format  Determines what formatting is required for the id's value
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getIdValue(String oidstr, String id, String format) throws ApplicationException {

    /*
     * This method may be overridden by subclasses.
     * This default method is provided to
     * allow the PO layer to compile easily and for debugging
     * purposes.
     */

    // DBG
    return "ID Value passed: "+id+", oidstr is: "+oidstr+", format is: <"+format+">";

  }


/**
 * Gets information from the DO layer for the ID passed.
 * Just calls the same method with no formatting info.
 *
 * @param oidstr  The database oid for the record
 * @param id  The id who's value is to be returned.
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getIdValue(String oidstr, String id) throws ApplicationException {

    return getIdValue(oidstr, id, "");

  }



/**
 * Provides information for the ID passed.
 * 
 * @param id  The id who's value is to be returned.
 * @param format  Determines what formatting is required for the id's value
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getBusinessIdValue(String id, String format) throws ApplicationException {

    String result_str = "";
    boolean id_found = false;

    if (id.equalsIgnoreCase("TodayDate")) {

      /* TODO: Handle this id */

    } else     if (id.equalsIgnoreCase("TodayDate1")) {

      /* TODO: Handle this id */

    }

    // was the id found?
    if (!id_found) {

      // no
      return "ID value is: "+id+", format method is: <"+format+">";

    }

    // return the (formatted) result
    return result_str;

  }



/**
 * Provides information for the ID passed.  Just calls
 * the same method with no formatting information.
 * 
 * @param id  The id who's value is to be returned.
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getBusinessIdValue(String id) throws ApplicationException {

    return getBusinessIdValue(id, "");

  }


/**
 * Allows the presentation layer to give the business layer
 * some information, e.g. perhaps for user logins.
 * 
 * @param id  The agreed upon id
 * @param data  The data related to this id
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public void setBusinessIdValue(String id, Object data) throws ApplicationException {

    /*
     * Insert your code here to handle the id and its data.
     */

  }


}


