package timesheet.business;

import com.lutris.appserver.server.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.util.*;
import timesheet.*;
import timesheet.data.*;
import java.util.*;
import java.math.*;
import com.lutris.dods.builder.sourceGenerators.Query.*;
import com.pisoftware.platypus.production.*;

/**
 * Date Created: Oct 15, 1999
 * <P>
 * Purpose:  This class is the base BO for the application.
 *           All BOs that related to a PO eventually extend
 *           this class, possibly via subdirectory level BO
 *           classes.
 *
 */
public class TimesheetBO {


  /*
   * The following are the BOs in the system that extend
   * BDOs.  They're the business objects that deal with the
   * DOs in a close manner.  This class ensures that we
   * only create these once per session.
   */
  public CompanyBO companyBO = null;
  public PersonBO personBO = null;
  public ActivityBO activityBO = null;
  public ProjectBO projectBO = null;
  public TimeEntryBO timeEntryBO = null;


  // The KeywordValueTable for the business objects
  protected KeywordValueTable BO_KeywordValueTable = null;
  protected final static String BO_KEYWORD = "business";

  /*
   * The following list is the keyword values for all
   * the BOs listed above.  This will allow us to
   * save and restore those BOs with the sessionData.
   */
  protected final static String BO_COMPANY = "company";
  protected final static String BO_PERSON = "person";
  protected final static String BO_ACTIVITY = "activity";
  protected final static String BO_PROJECT = "project";
  protected final static String BO_TIMEENTRY = "timeentry";


  // User information
  protected String operator_username = null;
  protected String operator_id = null;
  protected PersonDO operatorDO = null;


/**
 * Constructor.  Creates the object, and initialises the appropriate variables, etc.
 *
 * @param comms  Used to get the session data required
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public TimesheetBO(HttpPresentationComms comms) throws ApplicationException {


    try {

      // Can we restore any of the BOs from the sessionData?
      BO_KeywordValueTable = comms.sessionData.getSection(BO_KEYWORD);

      if (BO_KeywordValueTable != null) {

        // yes, restore the objects
        companyBO = (CompanyBO) BO_KeywordValueTable.get(BO_COMPANY);
        personBO = (PersonBO) BO_KeywordValueTable.get(BO_PERSON);
        activityBO = (ActivityBO) BO_KeywordValueTable.get(BO_ACTIVITY);
        projectBO = (ProjectBO) BO_KeywordValueTable.get(BO_PROJECT);
        timeEntryBO = (TimeEntryBO) BO_KeywordValueTable.get(BO_TIMEENTRY);


      } else {

        // no, create and save the objects
        companyBO = new CompanyBO();
        comms.sessionData.set(BO_KEYWORD+"."+BO_COMPANY,companyBO);

        personBO = new PersonBO();
        comms.sessionData.set(BO_KEYWORD+"."+BO_PERSON,personBO);

        activityBO = new ActivityBO();
        comms.sessionData.set(BO_KEYWORD+"."+BO_ACTIVITY,activityBO);

        projectBO = new ProjectBO();
        comms.sessionData.set(BO_KEYWORD+"."+BO_PROJECT,projectBO);

        timeEntryBO = new TimeEntryBO();
        comms.sessionData.set(BO_KEYWORD+"."+BO_TIMEENTRY,timeEntryBO);

      }


    } catch (KeywordValueException kve) {

      throw new ApplicationException("Unable to save/restore BOs to/from the sessionData!",kve);

    } catch (Exception e) {

      // creating BOs that extend a BDO could throw an exception
      throw new ApplicationException("Unable to create BO object!",e);

    }



    /*
     * Insert your code here, e.g. User authentication.
     */


    /*
     * Get the operator information from the web server.
     */

    // Get the operator's login from the HttpPresentationComms object.
    try {

      operator_username = comms.request.getRemoteUser();

    } catch (HttpPresentationException hpe) {

      throw new ApplicationException("Unable to get REMOTE_USER parameter!",hpe);

    }

    PersonDO[] operators = null;

    // get the Person representing this operator
    operators = personBO.getPersonByUserName(operator_username);

    if (operators == null)
      throw new ApplicationException("Unable to get any Person details for this operator: <"+operator_username+">!");

    if (operators.length > 1)
      throw new ApplicationException("Too many people with this operator name: <"+operator_username+">!");

    // keep the operator info for use elsewhere
    operatorDO = operators[0];

    try {

      operator_id = operatorDO.getHandle();

    } catch (DatabaseManagerException dme) {

      throw new ApplicationException("Unable to get handle to the operator!", dme);

    }

    // if we get here, then the user's ok for now...

  }
    
 
  // cache for static pulldowns
  private static Hashtable pulldowns = new Hashtable();
    
   
/**   
 * Returns the list of pulldown items for the name passed.
 *  
 * @param pulldown_name  The Pulldown name to query for
 * @return an array of PulldownDOs containing the list of values for this pulldown name, null or empty array if none found.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */   
  public PulldownItem[] getPulldownItems(String pulldown_name) throws ApplicationException {

    PulldownItem[] pi = null;
  
    // Have we got this pulldown cached? 
    if (pulldowns.containsKey(pulldown_name)) {
      
      // yes, return it
      return (PulldownItem[]) pulldowns.get(pulldown_name);
  
    }

    if (pulldown_name.equalsIgnoreCase("authorisationlevel")) {

      pi = new PulldownItem[2];
      pi[0] = new PulldownItem(pulldown_name,"none","No special authorisation",true);
      pi[1] = new PulldownItem(pulldown_name,"pm","Project Manager",true);

      // store these pulldown items in the cache
      pulldowns.put(pulldown_name,pi);

    } else if (pulldown_name.equalsIgnoreCase("companyid")) {

      try {

        // get all the valid companies
        CompanyDO[] companies = companyBO.getAllCompanies();

        pi = new PulldownItem[companies.length];

        for (int i=0; i<companies.length; i++) {

          pi[i] = new PulldownItem(pulldown_name, companies[i].getHandle(), companies[i].getName(), false);

        }

      } catch (Exception e) {

        // exceptions should only be DatabaseManagerException or DataObjectException
        throw new ApplicationException("Unable to create pulldown for: "+pulldown_name, e);

      }

    } else if (pulldown_name.equalsIgnoreCase("title")) {

      pi = new PulldownItem[7];
      pi[0] = new PulldownItem(pulldown_name,"mr","Mr",false);
      pi[1] = new PulldownItem(pulldown_name,"mrs","Mrs",false);
      pi[2] = new PulldownItem(pulldown_name,"miss","Miss",false);
      pi[3] = new PulldownItem(pulldown_name,"dr","Dr",false);
      pi[4] = new PulldownItem(pulldown_name,"ms","Ms",false);
      pi[5] = new PulldownItem(pulldown_name,"eo","Evil Overlord",false);
      pi[6] = new PulldownItem(pulldown_name,"gm","Grovelling Minion",false);

      // store these pulldown items in the cache
      pulldowns.put(pulldown_name,pi);

    } else {
    
      throw new ApplicationException("Unknown pulldown name: "+pulldown_name);

    }


    // return the items
    return pi;

  }


/**
 * Gets information from the DO layer for the ID passed.
 * Note that a previous ACCESS method must have been
 * called for this function to work.
 *
 * @param index  The record to use
 * @param id  The id who's value is to be returned.
 * @param format  Determines what formatting is required for the id's value
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getIdValue(int index, String id, String format) throws ApplicationException {

    /*
     * This method will be overridden by subclasses that have
     * an ACCESS method.  This default method is provided to
     * allow the PO layer to compile easily and for debugging
     * purposes.
     */

    // DBG
    return "ID Value passed: "+id+", index is: "+index+", format is: <"+format+">";

  }


/**
 * Gets information from the DO layer for the ID passed.
 * Note that a previous ACCESS method must have been
 * called for this function to work.
 * Just calls the same method with no format info.
 *
 * @param index  The record to use
 * @param id  The id who's value is to be returned.
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getIdValue(int index, String id) throws ApplicationException {

    return getIdValue(index, id, "");

  }


/**
 * Gets information from the DO layer for the ID passed.
 *
 * @param oidstr  The database oid for the record
 * @param id  The id who's value is to be returned.
 * @param format  Determines what formatting is required for the id's value
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getIdValue(String oidstr, String id, String format) throws ApplicationException {

    /*
     * This method may be overridden by subclasses.
     * This default method is provided to
     * allow the PO layer to compile easily and for debugging
     * purposes.
     */

    // DBG
    return "ID Value passed: "+id+", oidstr is: "+oidstr+", format is: <"+format+">";

  }


/**
 * Gets information from the DO layer for the ID passed.
 * Just calls the same method with no formatting info.
 *
 * @param oidstr  The database oid for the record
 * @param id  The id who's value is to be returned.
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getIdValue(String oidstr, String id) throws ApplicationException {

    return getIdValue(oidstr, id, "");

  }



/**
 * Provides information for the ID passed.
 * 
 * @param id  The id who's value is to be returned.
 * @param format  Determines what formatting is required for the id's value
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getBusinessIdValue(String id, String format) throws ApplicationException {

    String result_str = "";

    if (id.equalsIgnoreCase("TodayDate")) {

      /* TODO: Handle this id */

    } else     if (id.equalsIgnoreCase("DayHours")) {

      /* TODO: Handle this id */

    } else     if (id.equalsIgnoreCase("TodayDate1")) {

      /* TODO: Handle this id */

    } else     if (id.equalsIgnoreCase("DisplayDate")) {

      /* TODO: Handle this id */

    }

    // was the id found?
    if (result_str.equals("")) {

      // no
      return "ID value is: "+id+", format method is: <"+format+">";

    } else {

      // return the formatted result
      return FormatUtil.format(result_str, format);

    }

  }



/**
 * Provides information for the ID passed.  Just calls
 * the same method with no formatting information.
 * 
 * @param id  The id who's value is to be returned.
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getBusinessIdValue(String id) throws ApplicationException {

    return getBusinessIdValue(id, "");

  }


/**
 * Allows the presentation layer to give the business layer
 * some information, e.g. perhaps for user logins.
 * 
 * @param id  The agreed upon id
 * @param data  The data related to this id
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public void setBusinessIdValue(String id, Object data) throws ApplicationException {

    /*
     * Insert your code here to handle the id and its data.
     */

  }


}


