/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * ../../timesheet/data/PersonDO.java
 *-----------------------------------------------------------------------------
 */

package timesheet.data;

import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.*;
import com.lutris.util.Config;
import com.lutris.util.KeywordValueTable;
import com.lutris.dods.builder.sourceGenerators.Query.NonUniqueQueryException;
import com.lutris.dods.builder.sourceGenerators.Query.DataObjectException;
import java.sql.*;
import java.math.*;
import java.util.Hashtable;
import java.util.Enumeration;


/**
 * Data core class, used to set, retrieve the PersonDO information.
 *
 * @version $Revision: 1.19.2.1 $
 * @author  dwood
 * @since   timesheet
 */
 public class PersonDO extends com.lutris.dods.builder.sourceGenerators.DO.GenericDO {

    /**
     * The ObjectId is the unique key for the DO in the database.
     */

    /**
     * A DO class contains a reference to a DataStruct class.
     * This reference can be null (when the data for the DO 
     * has not yet been retrieved from the database),
     * allowing a DO object to be a lightweight placeholder
     * until its data is needed.
     */
    private PersonDataStruct data = null;

    /**
     * isReadOnly() 
     * Returns true if the data for this object has been marked read-only.
     */
    public boolean isReadOnly() {
	if ( null == data ) return false;
	return data.readOnly;
    }

    /**
     * Protected constructor.  Only derived classes should call it.
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     */
    protected PersonDO ()
    throws ObjectIdException, DatabaseManagerException {
        super();
    }

    /**
     * isLoaded() 
     * Returns true if the data for this objects has been retrieved
     * from the database.
     */
    public boolean isLoaded() {
	return ( null != data );
    }

    /**
     * loadData() 
     * Load the fields for the DO from the database.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    public void loadData() 
    throws SQLException, ObjectIdException, DataObjectException 
    {
	if ( null == data ) {
	    
	    data = new PersonDataStruct ();
	}

	

	ObjectId id = getOId();
	if ( null == id )
	    return;

	// an id means we complain if no record in database
	PersonQuery query = new PersonQuery ();
	query.setQueryOId( id );
	query.requireUniqueInstance();
	PersonDO obj;
	try {
	    while ( null != ( obj = query.getNextDO() ) ) {
	        data = obj.data;
		setPersistent( true );
                setVersion(obj.getVersion());
	        // obj.data.readOnly = true;
	    }
	} catch ( NonUniqueQueryException e ) { 
	    throw new ObjectIdException( "Duplicate ObjectId" );
	}

	
    }

    


    /**
     * createVirgin() 
     * Creates a DO that has no ObjectId or data.
     * Such a DO is used to insert a new database entry
     * after its data has been set.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    
    public static PersonDO createVirgin()
    throws DatabaseManagerException, ObjectIdException {
	return new PersonDO ();
    }
    

    /**
     * createExisting() 
     * Creates a DO that represents an existing entry in the database.
     * Such a DO is used to examine and possibly update such an entry.
     * createExisting() is called only from the code that retrieves
     * an ObjectId from a ResultSet (database query result).
     * createExisting() is protected because no other DO or BO should ever
     * need to call it.
     * FIX unfortunately the createExisting(BigDecimal) form *does* need 
     * to be public because it is called by the public ctors of other DOs.
     * For example,
     *    AaaDO contains a ref to a BbbDO,
     *    so there is a method AaaDO.setBbb(BbbDO).
     *    In the ctor AaaDO(ResultSet), we have the call
     *       setBbb( BbbDO.createExisting( rs.getBigDecimal( "bbb", 0 )));
     * Since AaaDO is not in the same package as BbbDO,
     * BbbDO.createExisting(BigDecimal) must be public, not protected.
     * Java needs the C++ 'friend' idea.
     *
     * @param bd The BigDecimal representation of the ObjectId for the object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    
    public static PersonDO createExisting( BigDecimal bd ) 
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException
    {
	if ( null == bd )
	    return null;
	return createExisting( new ObjectId( bd ) );
    }

    /**
     * createExisting() 
     *
     * Factory method used to create an instance of this class to 
     * represent a Data Object already existing in the database.
     *
     * @param id The ObjectId for the object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static PersonDO createExisting( ObjectId id ) 
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException
    {
	PersonDO ret = null;


	ret = new PersonDO ();

	  ret.setOId( id );
	if ( ! false )
	    ret.loadData();
//System.err.println("PersonDO.createExisting("+id+"): data="+ret.data);
	// unset the GenericDO.dirty flag.
	ret.markClean();
	return ret;
    }

    /**
     * createExisting() 
     *
     * Factory method used to create an instance of this class to 
     * represent a Data Object already existing in the database.
     *
     * @param rs The ResultSet returned by the Query class for 
     * an existing Data Object stored in the database.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static PersonDO createExisting( ResultSet rs )
    throws SQLException, ObjectIdException, DataObjectException
		, DatabaseManagerException
    {
	PersonDO ret = new PersonDO ( rs );
	return ret;
    }
    

    /**
     * createCopy() 
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     *
     * @param data The data struct to copy values from.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    
    public static PersonDO createCopy( PersonDataStruct data )
    throws DatabaseManagerException, ObjectIdException {
	PersonDO ret = new PersonDO ();
	ret.data = ( PersonDataStruct ) data.duplicate();
	return ret;
    }
    

    /**
     * createCopy() 
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     *
     * @param orig The original DO to copy.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    
    public static PersonDO createCopy( PersonDO orig )
    throws DatabaseManagerException, ObjectIdException {
	PersonDO ret = new PersonDO ();
	if ( null != orig.data ) {
	    ret.data = ( PersonDataStruct ) orig.data.duplicate();
	}
	return ret;
    }
    

    /**
     * reload() 
     * Causes the DO to refresh itself from the database
     * the next time a set or get method is called.
     */
    
    public void reload() {
	data = null;
    }
    

    /** 
     * The methods <CODE>
     *     getHandle
     *     hasMatchingHandle
     * </CODE> are used by Presentation Objects that need to populate
     * HTML select lists with Data Objects as options.
     * The <CODE>getHandle()</CODE> method is used
     * to set the value for each option,
     * and the <CODE>hasMatchingHandle()<CODE>
     * methods are used to lookup the Data Object when the selection has
     * been made.
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     *
     * @return id of this DO as a string
     *   If an object id can't be allocated for this object.
     */
    
    public        String  getHandle() 
    throws DatabaseManagerException {
	String ret = null;
	    if ( null == getOId() )
		   throw new DatabaseManagerException( "ID not set" );
	    ret = getOId().toString();
        return ret;
    }
    

   /**
     * hasMatchingHandle
     *
     * @param handle
     *   <CODE>String</CODE> version of DO id
     * @return boolean
     *   True if the string version of the id of this DO matches passed handle
     * @see getHandle
     */
    
    public        boolean hasMatchingHandle( String handle ) {
	boolean ret = false;
	    if ( null == getOId() )
		   ret = false;
	    else
		   ret = getOId().toString().equals( handle );
        return ret;
    }
    


    


   /**
    * Get CompanyId of the t_person
    *
    * @return CompanyId of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public timesheet.data.CompanyDO getCompanyId () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PersonDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.CompanyId;
   }

   /**
    * Set CompanyId of the t_person
    *
    * @param CompanyId of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   
   public void setCompanyId ( timesheet.data.CompanyDO CompanyId )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PersonDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.CompanyId = (timesheet.data.CompanyDO) markNewValue(
	data.CompanyId, CompanyId  );
   }
   

   /**
    * Get FirstName of the t_person
    *
    * @return FirstName of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getFirstName () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PersonDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.FirstName;
   }

   /**
    * Set FirstName of the t_person
    *
    * @param FirstName of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   
   public void setFirstName ( String FirstName )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PersonDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.FirstName = (String) markNewValue(
	data.FirstName, FirstName , 0, 255, false );
   }
   

   /**
    * Get Surname of the t_person
    *
    * @return Surname of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getSurname () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PersonDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.Surname;
   }

   /**
    * Set Surname of the t_person
    *
    * @param Surname of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   
   public void setSurname ( String Surname )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PersonDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.Surname = (String) markNewValue(
	data.Surname, Surname , 0, 255, false );
   }
   

   /**
    * Get Title of the t_person
    *
    * @return Title of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getTitle () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PersonDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.Title;
   }

   /**
    * Set Title of the t_person
    *
    * @param Title of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   
   public void setTitle ( String Title )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PersonDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.Title = (String) markNewValue(
	data.Title, Title , 0, 255, true );
   }
   

   /**
    * Get Email of the t_person
    *
    * @return Email of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getEmail () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PersonDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.Email;
   }

   /**
    * Set Email of the t_person
    *
    * @param Email of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   
   public void setEmail ( String Email )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PersonDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.Email = (String) markNewValue(
	data.Email, Email , 0, 255, true );
   }
   

   /**
    * Get Phone of the t_person
    *
    * @return Phone of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getPhone () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PersonDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.Phone;
   }

   /**
    * Set Phone of the t_person
    *
    * @param Phone of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   
   public void setPhone ( String Phone )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PersonDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.Phone = (String) markNewValue(
	data.Phone, Phone , 0, 255, true );
   }
   

   /**
    * Get Fax of the t_person
    *
    * @return Fax of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getFax () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PersonDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.Fax;
   }

   /**
    * Set Fax of the t_person
    *
    * @param Fax of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   
   public void setFax ( String Fax )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PersonDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.Fax = (String) markNewValue(
	data.Fax, Fax , 0, 255, true );
   }
   

   /**
    * Get Type of the t_person
    *
    * @return Type of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getType () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PersonDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.Type;
   }

   /**
    * Set Type of the t_person
    *
    * @param Type of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   
   public void setType ( String Type )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PersonDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.Type = (String) markNewValue(
	data.Type, Type , 0, 255, false );
   }
   

   /**
    * Get AuthorisationLevel of the t_person
    *
    * @return AuthorisationLevel of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getAuthorisationLevel () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PersonDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.AuthorisationLevel;
   }

   /**
    * Set AuthorisationLevel of the t_person
    *
    * @param AuthorisationLevel of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   
   public void setAuthorisationLevel ( String AuthorisationLevel )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PersonDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.AuthorisationLevel = (String) markNewValue(
	data.AuthorisationLevel, AuthorisationLevel , 0, 255, false );
   }
   

   /**
    * Get PublicPassword of the t_person
    *
    * @return PublicPassword of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getPublicPassword () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PersonDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.PublicPassword;
   }

   /**
    * Set PublicPassword of the t_person
    *
    * @param PublicPassword of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   
   public void setPublicPassword ( String PublicPassword )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PersonDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.PublicPassword = (String) markNewValue(
	data.PublicPassword, PublicPassword , 0, 255, true );
   }
   

   /**
    * Get UserName of the t_person
    *
    * @return UserName of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public String getUserName () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PersonDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.UserName;
   }

   /**
    * Set UserName of the t_person
    *
    * @param UserName of the t_person
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   
   public void setUserName ( String UserName )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PersonDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.UserName = (String) markNewValue(
	data.UserName, UserName , 0, 255, false );
   }
   
    /**
     * Protected constructor.
     *
     * @param rs
     *   Result set from which to obtain product data.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected PersonDO(ResultSet rs)
            throws SQLException, ObjectIdException, DataObjectException
 	    , DatabaseManagerException
/*  	      */
    {
        super(rs);

	// Constructing a DO from a ResultSet means we definitely need the 
	// DataStruct ready for the setXxx methods invoked below.
	data = new PersonDataStruct ();
 
	// writeMemberStuff uses the ResultSetExtraction.template
	// to build up the value for this tag:
	// the value is a series of calls to the DO set methods.
		
	setCompanyId( 
	    timesheet.data.CompanyDO.createExisting( 
		rs.getBigDecimal( 
			"CompanyId" , 0 )
	     )
      
	);
	
	
	setFirstName( 
	    /*
		rs.getString( 
			"FirstName"  )
	    */
      rs.getString("FirstName" )
	);
	
	
	setSurname( 
	    /*
		rs.getString( 
			"Surname"  )
	    */
      rs.getString("Surname" )
	);
	
	
	setTitle( 
	    /*
		rs.getString( 
			"Title"  )
	    */
      rs.getString("Title" )
	);
	
	
	setEmail( 
	    /*
		rs.getString( 
			"Email"  )
	    */
      rs.getString("Email" )
	);
	
	
	setPhone( 
	    /*
		rs.getString( 
			"Phone"  )
	    */
      rs.getString("Phone" )
	);
	
	
	setFax( 
	    /*
		rs.getString( 
			"Fax"  )
	    */
      rs.getString("Fax" )
	);
	
	
	setType( 
	    /*
		rs.getString( 
			"Type"  )
	    */
      rs.getString("Type" )
	);
	
	
	setAuthorisationLevel( 
	    /*
		rs.getString( 
			"AuthorisationLevel"  )
	    */
      rs.getString("AuthorisationLevel" )
	);
	
	
	setPublicPassword( 
	    /*
		rs.getString( 
			"PublicPassword"  )
	    */
      rs.getString("PublicPassword" )
	);
	
	
	setUserName( 
	    /*
		rs.getString( 
			"UserName"  )
	    */
      rs.getString("UserName" )
	);
	

 
        markClean();
    }

    

    



    private int[] param = null;

    /**
     * Prepares the statement used to insert this object
     * into the database.
     * @param conn the database connection.
     * @return the insert statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getInsertStatement(DBConnection conn)
            throws SQLException {
	/* FIX:  here I've hacked in some cache maintenance code.
	 * It would probably be better to have CoreDO implement
	 * 	void addToCache(CoreDO DO) {}
	 * and have each DO that has caching enabled override it as
	 *      void addToCache(CoreDO DO) { cache.put( DO.getOId(), DO ); }
	 * and change CoreDO to invoke addToCache()
	 * when it invokes getInsertStatement().
	 */

        ObjectId oid;

        PreparedStatement stmt = conn.prepareStatement( 
	    "insert into t_person values ( ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? )" );

	param = new int[1]; param[0] = 1;
	// writeMemberStuff uses the JDBCsetCalls.template
	// to build up the value for this tag:
	// the value is a series of calls to setPrepStmtParam_TYPE methods.
	// Those methods are defined in GenericDO.
	try {
	    setPrepStmtParam_DO( stmt, param, getCompanyId() );
setPrepStmtParam_String( stmt, param, getFirstName() );
setPrepStmtParam_String( stmt, param, getSurname() );
setPrepStmtParam_String( stmt, param, getTitle() );
setPrepStmtParam_String( stmt, param, getEmail() );
setPrepStmtParam_String( stmt, param, getPhone() );
setPrepStmtParam_String( stmt, param, getFax() );
setPrepStmtParam_String( stmt, param, getType() );
setPrepStmtParam_String( stmt, param, getAuthorisationLevel() );
setPrepStmtParam_String( stmt, param, getPublicPassword() );
setPrepStmtParam_String( stmt, param, getUserName() );


	    /* The order of the values being inserted must match
	     * the order of the columns listed in the CREATE TABLE command
	     * that appears in the .sql file for this DO.
	     * Since the id and version number are always the last columns
	     * listed in the CREATE TABLE command, their values are added
	     * to the PreparedStatement after all other values.
	     */
	    setPrepStmtParam_BigDecimal( stmt, param, getOId().toBigDecimal() );
	    setPrepStmtParam_Int(        stmt, param, getNewVersion() );

	} catch ( Exception e ) {
	    throw new SQLException( "Data Object error: " + e.getMessage() );
	}

        return stmt;
    }

    /**
     * Prepares the statement used to update this object
     * in the database.
     * @param conn the database connection
     * @return the update statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getUpdateStatement(DBConnection conn)
            throws SQLException {

        ObjectId oid;

        PreparedStatement stmt = conn.prepareStatement(
	    "update t_person set objectVersion = ?, CompanyId = ?, FirstName = ?, Surname = ?, Title = ?, Email = ?, Phone = ?, Fax = ?, Type = ?, AuthorisationLevel = ?, PublicPassword = ?, UserName = ? " +
	    "where objectID= ? and objectVersion = ?" );

	param = new int[1]; param[0] = 1;
	//int[] param = {1};
	// writeMemberStuff builds up the value for this tag:
	// the value is a series of calls to setPrepStmtParam_TYPE methods.
	// Those methods are defined below.
	try {
	    setPrepStmtParam_Int( stmt, param, getNewVersion() );
	    setPrepStmtParam_DO( stmt, param, getCompanyId() );
setPrepStmtParam_String( stmt, param, getFirstName() );
setPrepStmtParam_String( stmt, param, getSurname() );
setPrepStmtParam_String( stmt, param, getTitle() );
setPrepStmtParam_String( stmt, param, getEmail() );
setPrepStmtParam_String( stmt, param, getPhone() );
setPrepStmtParam_String( stmt, param, getFax() );
setPrepStmtParam_String( stmt, param, getType() );
setPrepStmtParam_String( stmt, param, getAuthorisationLevel() );
setPrepStmtParam_String( stmt, param, getPublicPassword() );
setPrepStmtParam_String( stmt, param, getUserName() );


	    /* When updating a persistent object, the UPDATE_WHERE_CLAUSE tag
	     * used to build the SQL WHERE clause (above) uses the 
	     * DO's id and version number to locate the correct row in
	     * the database table.
	     */
	    setPrepStmtParam_BigDecimal( stmt, param, getOId().toBigDecimal() );
	    setPrepStmtParam_Int(        stmt, param, getVersion() );

	} catch ( Exception e ) {
	    throw new SQLException( "Data Object error: " + e.getMessage() );
	}

        return stmt;
    }

    /**
     * Prepares the statement used to delete this object
     * from the database.
     * @param conn the database connection
     * @return the delete statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getDeleteStatement(DBConnection conn)
            throws SQLException {

        String sql =
            "delete from t_person \n" +
            "where objectID = ?";
        PreparedStatement stmt = conn.prepareStatement(sql);
        stmt.setBigDecimal(1, getOId().toBigDecimal());
        return stmt;
    }

    

    /*
     * toString - for debugging
     */
    public String toString(){
	String str = "PersonDO:";
	ObjectId oid = getOId();
	String id = "virgin";
	if ( null != oid ) 
	    id = oid.toString();
	str += " objectID=" + id;
	if ( null != data ) 
	    str = str + " CompanyId=" + data.CompanyId
+ " FirstName=" + data.FirstName
+ " Surname=" + data.Surname
+ " Title=" + data.Title
+ " Email=" + data.Email
+ " Phone=" + data.Phone
+ " Fax=" + data.Fax
+ " Type=" + data.Type
+ " AuthorisationLevel=" + data.AuthorisationLevel
+ " PublicPassword=" + data.PublicPassword
+ " UserName=" + data.UserName
;
        return str + "; " + super.toString();
    }

}
